/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CWarning.java,v 1.1 1999/10/12 15:40:05 vgp Exp $
 */

package at.dms.compiler;

import java.io.File;
import java.io.FileInputStream;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

import at.dms.util.Message;

/**
 * This class represents warnings in error hierarchy
 */
public class CWarning extends PositionedError {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * An always catchable error in source file
   * @param	where		the line of this node in the source code
   * @param	msg		the message to display for this error
   * @param	par1		the first parameter of message (name of a var by example)
   * @param	par2		the second parameter of message
   */
  public CWarning(TokenReference where, String msg, Object par1, Object par2) {
    super(where, msg, par1, par2);
  }

  /**
   * An always catchable error in source file
   * @param	where		the line of this node in the source code
   * @param	msg		the message to display for this error
   * @param	par1		the first parameter of message (name of a var by example)
   */
  public CWarning(TokenReference where, String msg, Object par1) {
    super(where, msg, par1);
  }

  /**
   * An always catchable error in source file
   * @param	where		the line of this node in the source code
   * @param	msg		the message to display for this error
   */
  public CWarning(TokenReference where, String msg) {
    super(where, msg);
  }

  /**
   * An always catchable error in source file
   * @param	where		the line of this node in the source code
   * @param	msg		the message to display for this error
   */
  public CWarning(TokenReference where, Message msg) {
    super(where, msg);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Return the string explaining the error and the token reference
   */
  public String getErrorMessage() {
    return where.getFile() + ":" + where.getLine() + ": " + "warning: " + message.getMessage();
  }

  /**
   * Returns true if this warning is enabled
   */
  public boolean isEnabled() {
    return isEnabled(getKey());
  }

  // ----------------------------------------------------------------------
  // PUBLIC UTILITIES
  // ----------------------------------------------------------------------

  /**
   * Returns true if this warning is enabled
   * @param	fullIdent	"type-name" ("dataflow-unreachable")
   */
  public static boolean isEnabled(String fullIdent) {
    if (!resourceLoaded) {
      loadResources();
    }

    if (fullIdent.equals("method-unthrown-exception")) {
      return false; // !!!
    }
    if (resources != null) {
      String	val = getMessage(fullIdent);

      if ("true".equals(val)) {
	return true;
      } else if ("false".equals(val)) {
	return false;
      }

      int	indexOfMinus = fullIdent.indexOf("-");
      if (indexOfMinus != -1) {
	String	sub = fullIdent.substring(0, indexOfMinus);
	if ("false".equals(getMessage(sub))) {
	  return false;
	}
      }

      return true;
    }

    return true;
  }

  // ----------------------------------------------------------------------
  // PRIVATE UTILITIES
  // ----------------------------------------------------------------------

  private static void loadResources() {
    try {
      resourceLoaded = true;
      String	config = System.getProperty("user.home") + File.separatorChar + ".kjcrc";

      resources = new PropertyResourceBundle(new FileInputStream(new File(config)));
    } catch (Exception e) {
      //e.printStackTrace();
    }
  }

  private static String getMessage(String key) {
    try {
      return resources == null ? null : resources.getString(key);
    } catch (Exception e) {
      return null;
    }
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private static	boolean		resourceLoaded;
  private static	ResourceBundle	resources;
}
