/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Console.java,v 1.10 1999/10/08 11:18:08 vgp Exp $
 */

package at.dms.compiler;

import java.io.PrintWriter;

import at.dms.util.Message;

/**
 * This class implements a console that display information about
 * compile process
 */
public class Console {

  // ----------------------------------------------------------------------
  // CONSTRUCTOR
  // ----------------------------------------------------------------------

  /**
   * Create a text console
   */
  public Console(PrintWriter err, boolean verbose) {
    this.err = err;
    this.verbose = verbose;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Starts the compiling process
   */
  public void compilationStarted(int files) {
    err.println(Message.format("verbose-start", null));
  }

  /**
   * file parsed
   */
  public void parseFile(long lastTime, String name) {
    printTime(lastTime, Message.format("verbose-parse", null) + " " + name);
  }

  /**
   * file checked (interface)
   */
  public void checkInterface(long lastTime) {
    printTime(lastTime, Message.format("verbose-check", null));
  }

  /**
   * file checked (code)
   */
  public void checkBody(long lastTime, String name) {
    printTime(lastTime, Message.format("verbose-check-body", null) + " " + name);
  }

  /**
   * class file generated
   */
  public void generateClassFile(long lastTime, String name) {
    printTime(lastTime, Message.format("verbose-optimize", null) + " " + name);
  }

  /**
   * file generated
   */
  public void generateJavaCode(String name) {
    err.println("[ " + Message.format("verbose-generate", null) + name + " ]");
  }

  /**
   * Ends the compiling process (if successful)
   */
  public void loadClass(String name) {
    // DONT SHOW
  }

  /**
   * Ends the compiling process (if successful)
   */
  public void compilationEnded() {
  }

  // ----------------------------------------------------------------------
  // PRIVATE UTILITIES
  // ----------------------------------------------------------------------

  /**
   * Print a verbose message about time to execute a certain task
   */
  protected void printTime(long lastTime, String message) {
    if (verbose) {
      long		time = System.currentTimeMillis() - lastTime;
      String		mesg = "[ ";

      for (long i = time == 0 ? 1 : time; i < 100000; i *= 10) {
	mesg += " ";
      }

      mesg += time + message + " ]";

      err.println(mesg);
      err.flush();
    }
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA
  // ----------------------------------------------------------------------

  private PrintWriter		err;
  private final boolean		verbose;
}
