/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: FastParser.java,v 1.2 1999/10/08 11:18:08 vgp Exp $
 */

package at.dms.compiler;

import java.util.Vector;

import antlr.Token;
import antlr.TokenQueue;
import antlr.ParserException;
import java.io.IOException;
import antlr.debug.MessageListener;
import antlr.debug.ParserListener;
import antlr.debug.ParserMatchListener;
import antlr.debug.ParserTokenListener;
import antlr.debug.SemanticPredicateListener;
import antlr.debug.SyntacticPredicateListener;
import antlr.debug.TraceListener;

import antlr.collections.impl.BitSet;
import antlr.TokenStream;
import antlr.TokenBuffer;
import antlr.collections.AST;
import antlr.ASTFactory;
import antlr.ParserSharedInputState;

import at.dms.compiler.Compiler;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * This class describes the capabilities of parsers.
 */
public abstract class FastParser {

  // --------------------------------------------------------------------
  // CONSTRUCTORS
  // --------------------------------------------------------------------

  /**
   * Constructs a new parser instance.
   * @param	compiler	the invoking compiler.
   * @param	lexer		the token stream generator
   * @param	file		the file name
   */
  protected FastParser(HandLexer lexer, int lookahead) {
    k = lookahead;
    this.lexer = lexer;
  }

  /**
   * Make sure current lookahead symbol matches token type <tt>t</tt>.
   * Throw an exception upon mismatch, which is catch by either the
   * error handler or by the syntactic predicate.
   */
  public final void match(int t) throws ParserException, IOException {
    if (LA(1) != t) {
      throw new ParserException();
    } else {
      // mark token as consumed -- fetch next token deferred until LA/LT
      consume();
    }
  }
	
  /**
   * Make sure current lookahead symbol matches the given set
   * Throw an exception upon mismatch, which is catch by either the
   * error handler or by the syntactic predicate.
   */
  public final void match(BitSet b) throws ParserException, IOException {
    if ( !b.member(LA(1)) ) {
      throw new ParserException();
    } else {
      // mark token as consumed -- fetch next token deferred until LA/LT
      consume();
    }
  }
  
  public final void matchNot(int t) throws ParserException, IOException {
    if (LA(1) == t) {
      // Throws inverted-sense exception
      throw new ParserException();
    } else {
      // mark token as consumed -- fetch next token deferred until LA/LT
      consume();
    }
  }

  public final void rewind(int pos) {
    throw new RuntimeException();
  }

  public final int mark() {
    return 0;
  }

  /**
   * Consume another token from the input stream.  Can only write sequentially!
   * If you need 3 tokens ahead, you must consume() 3 times.
   * <p>
   * Note that it is possible to overwrite tokens that have not been matched.
   * For example, calling consume() 3 times when k=2, means that the first token
   * consumed will be overwritten with the 3rd.
   */
  private final void consume() {
    current = next;
    next = null;
  }

  public final int LA(int i) throws IOException {
    if (i == 1) {
      if (current == null) {
	current = lexer.nextToken();
      }
      return current.getType();
    } else {
      if (next == null) {
	next = lexer.nextToken();
      }
      return next.getType();
    }
  }

  public final Token LT(int i) throws IOException {
    if (i == 1) {
      if (current == null) {
	current = lexer.nextToken();
      }
      return current;
    } else {
      if (next == null) {
	next = lexer.nextToken();
      }
      return next;
    }
  }

  class ParserException extends antlr.ParserException {
    public int getColumn() { return 0; }
    public String getErrorMessage () { return "error"; }
    public int getLine() { return lexer.getLine(); }
  }

  // --------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // --------------------------------------------------------------------

  private final int		k;
  private final HandLexer	lexer;

  private Token	current;
  private Token	next;
  
  protected ParserSharedInputState inputState = new ParserSharedInputState();
  protected Object		tokenNames; //unused but generated by antlr
}
