/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Parser.java,v 1.5 1999/10/28 14:03:11 vgp Exp $
 */

package at.dms.compiler;

import java.util.Vector;

import antlr.LLkParser;

/**
 * This class describes the capabilities of parsers.
 */
public abstract class Parser extends LLkParser {
  
  /**
   * Constructs a new parser instance.
   * @param	compiler	the invoking compiler.
   * @param	lexer		the token stream generator
   * @param	lookahead	lookahead
   */
  protected Parser(Compiler compiler, HandLexer lexer, int lookahead) {
    super(lexer, lookahead);
    this.compiler = compiler;
    this.lexer = lexer;
  }
  // --------------------------------------------------------------------
  // ACCESSORS
  // --------------------------------------------------------------------

  /**
   * Returns a reference to the current position in the source file.
   */
  protected final TokenReference buildTokenReference() {
    return lexer.getTokenReference();
  }

  /**
   *
   */
  protected final JavaStyleComment[] getStatementComment() {
    return lexer.getStatementComment();
  }

  /**
   *
   */
  protected Vector getComment() {
    return null; // !!!lexer.getComment();
  }

  /**
   * Add an error into the list and eat it
   * This method should be called after a try catch block after catching exception
   * or directly without exception thrown
   * @param	error		the error
   */
  protected final void reportTrouble(PositionedError trouble) {
    compiler.reportTrouble(trouble);
  }

  // --------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // --------------------------------------------------------------------

  private final Compiler		compiler;
  protected final HandLexer		lexer;
}
