/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: UnpositionedError.java,v 1.6 1999/10/08 11:18:08 vgp Exp $
 */

package at.dms.compiler;

import at.dms.util.Message;

/**
 * This class represents the root class for all kopic errors
 */
public class UnpositionedError extends Throwable {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * An error with a message as parameter.
   * @param	message		the message to be displayed
   */
  public UnpositionedError(Message message) {
    this.message	= message;
  }

  /**
   * An error with with an arbitrary number of parameters.
   * @param	key		the message ident to be displayed
   * @param	params		the array of parameters
   */
  public UnpositionedError(String key, Object[] params) {
    this(new Message(key, params));
  }

  /**
   * An error with two parameters.
   * @param	key		the message ident to be displayed
   * @param	param1		the first parameter
   * @param	param2		the second parameter
   */
  public UnpositionedError(String key, Object param1, Object param2) {
    this(key, new Object[] { param1, param2 });
  }

  /**
   * An error with one parameter.
   * @param	key		the message ident to be displayed
   * @param	param		the parameter
   */
  public UnpositionedError(String key, Object param) {
    this(key, new Object[] { param });
  }

  /**
   * An error without parameters.
   * @param	key		the message ident to be displayed
   */
  public UnpositionedError(String key) {
    this(key, null);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Returns an error with a reference to the source file.
   * @param	ref		the position in the source file of the catcher
   */
  public PositionedError getError(TokenReference ref) {
    return new PositionedError(ref, message);	// !!! was CLineError
  }

  /**
   *
   */
  public String getKey() {
    return message.getKey();
  }

  /**
   *
   */
  public Message getText() {
    return message;
  }

  /**
   *
   */
  public String getMessage() {
    return message.getMessage();
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  protected Message		message;
}
