/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Main.java,v 1.37 1999/10/16 23:25:35 vgp Exp $
 */

package at.dms.dis;

import gnu.getopt.Getopt;
import gnu.getopt.LongOpt;

import at.dms.classfile.ClassPath;
import at.dms.util.Message;
import at.dms.compiler.UnpositionedError;

/**
 * This class is the entry point for the KOPI disassembler.
 */
public class Main implements Constants {

  // --------------------------------------------------------------------
  // CONSTRUCTORS
  // --------------------------------------------------------------------

  /**
   * Only main can construct Main
   */
  private Main(String[] args) {
    parseArgs(args);

    if (sourceFiles == null) {
      printUsage();
      System.exit(1);
    } else {
      boolean	errorsFound = false;

      for (int i = 0; i < sourceFiles.length; i++) {
	if (optionVerbose) {
	  System.err.println("Processing " + sourceFiles[i] + ".");
	}

	try {
	  disassembleClass(sourceFiles[i]);
	} catch (UnpositionedError e) {
	  System.err.println("Error: " + e.getMessage());
	  errorsFound = true;
	}
      }

      System.exit(errorsFound ? 1 : 0);
    }
  }

  // --------------------------------------------------------------------
  // ACTIONS
  // --------------------------------------------------------------------

  /**
   * Reads, and disassembles a class file
   * @exception	UnpositionedError	an error occurred
   */
  private void disassembleClass(String fileName) throws UnpositionedError {
    Disassembler.disassemble(fileName, destination, options);
  }

  // --------------------------------------------------------------------
  // ENTRY POINT
  // --------------------------------------------------------------------

  /**
   * Entry point to the assembler
   */
  public static void main(String[] args) {
    Message.addBundle("dis");

    String		path = System.getProperty("java.class.path");
    ClassPath.init(path == null ? "." : path);

    new Main(args);
  }

  // --------------------------------------------------------------------
  // ACCESSORS
  // --------------------------------------------------------------------

  /*
   * Parse command line arguments.
   */
  private void parseArgs(String[] args) {
    Getopt	g = new Getopt("dis", args, shortopts, longopts, true);

  _loop_:
    for (;;) {
      switch (g.getopt()) {
      case 'c':
	options |= OPT_STDOUT;
	break;

      case 'd':
	this.destination = g.getOptarg();
	break;

      case 'h':
	printUsage();
	System.exit(0);

      case 'i':
	options |= OPT_NO_CODE;
	break;

      case 's':
	options |= OPT_SORT_MEMBERS;
	break;

      case 'V':
	printVersion();
	break;

      case 'v':
	optionVerbose = true;
	break;

      case 'x':
	options |= OPT_SHOW_STACK;
	System.err.println("option --show-stack currently not available");
	break;

      case -1:
	break _loop_;
      }
    }

    if (args.length > g.getOptind()) {
      this.sourceFiles = new String[args.length - g.getOptind()];
      System.arraycopy(args, g.getOptind(), this.sourceFiles, 0, this.sourceFiles.length);
    }
  }

  private static void printUsage() {
    System.err.println(Message.format("usage-line-dis", null));
    System.err.println(Message.format("usage-stack", null));
    System.err.println(Message.format("usage-sorted", null));
    System.err.println(Message.format("usage-stdout", null));
    System.err.println(Message.format("usage-version", null));
    System.err.println(Message.format("usage-verbose", null));
    System.err.println();
    System.err.println(Message.format("info1", null));
    System.err.println(Message.format("info2", null));
    System.err.println(Message.format("info3", null));
    System.err.println(Message.format("info4", null));
  }

  private static void printVersion() {
    System.err.println("KOPI Disassembler Version: " + VERSION);
  }

  // --------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // --------------------------------------------------------------------

  private static final String	VERSION	= "1.3D";

  private static final String	shortopts = "cd:hisVvx";
  private static final
    LongOpt[]			longopts = {
      new LongOpt("stdout", LongOpt.NO_ARGUMENT, null, 'c'),
      new LongOpt("destination", LongOpt.REQUIRED_ARGUMENT, null, 'd'),
      new LongOpt("help", LongOpt.NO_ARGUMENT, null, 'h'),
      new LongOpt("interface", LongOpt.NO_ARGUMENT, null, 'i'),
      new LongOpt("sorted", LongOpt.NO_ARGUMENT, null, 's'),
      new LongOpt("version", LongOpt.NO_ARGUMENT, null, 'V'),
      new LongOpt("verbose", LongOpt.NO_ARGUMENT, null, 'v'),
      new LongOpt("stack", LongOpt.NO_ARGUMENT, null, 'x')
    };

  private String		destination;
  private boolean		optionVerbose;
  private int			options;
  private String[]		sourceFiles;
}
