/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: IkjcServer.java,v 1.30 1999/10/08 11:18:57 vgp Exp $
 */

package at.dms.ikjc;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.IOException;
import java.net.Socket;
import java.net.ServerSocket;
import java.util.Vector;

import at.dms.compiler.Compiler;
import at.dms.compiler.Console;

/**
 * This class implements the server side of the kjc incremental compiler
 */
public class IkjcServer {

  /**
   * Launch the server
   */
  public static void main(final String[] args) {
    port = 4444;

    parseArguments(args);

    // create the socket
    try {
      if (serverSocket == null) {
	serverSocket = new ServerSocket(port);
      }
    } catch (IOException e) {
      System.err.println("Could not listen on port: " + port);
      System.exit(1);
    }

    if (console && window == null) {
      window = new Console(new PrintWriter(System.err), true);
    }

    // call our compiler with changed output
    Thread t = new Thread(new ThreadGroup("Worker"), new Runnable() {
      public void run() {
	try {
	  new IkjcServer().run();
	} catch (Exception e) {
	  e.printStackTrace();
	  try {
	    clientSocket.close();
	  } catch (Exception ef) {}
	  main(args);
	}
      }});
    t.setPriority(Thread.MIN_PRIORITY);
    t.start();
  }

  /**
   * The run process
   */
  public void run() throws Exception {
    System.err.println("Server launched.");

    while (true) {
      System.err.println("Waiting for clients...");
      // wait for clients
      try {
	clientSocket = serverSocket.accept();
      } catch (IOException e) {
	System.err.println("Accept failed.");
	continue;
      }

      processClientRequest(clientSocket);

      System.err.println("Compilation processed, cleaning up ...");
      clientSocket.close();
      //System.gc();
    }

    // serverSocket.close();
  }

  /*
   *
   */
  private void processClientRequest(Socket client) throws Exception {
    BufferedReader	in = new BufferedReader(new InputStreamReader(client.getInputStream()));
    PrintWriter		out = new PrintWriter(client.getOutputStream(), true);

    System.err.println("Processing compilation...");

    try {
      // read arguments
      Vector		args = new Vector();
      String		line;
      Compiler		compiler;
      String		directory;
      String[]		arguments;
      boolean		success;

      System.err.print("Calling ikjc with arguments: ");
      while ((line = in.readLine()) != null && !line.equals("")) {
	args.addElement(line);
	System.err.print(line + " ");
      }
      System.err.println();

      if (args.size() <= 2) {
	System.err.println("Error: not enough arguments given.");
	return;
      }

      // get working directory
      directory = (String)args.elementAt(0);
      args.removeElementAt(0);

      // get compiler
      if (((String)args.elementAt(0)).equals("kjc")) {
	compiler = new at.dms.kjc.Main();
      } else if (((String)args.elementAt(0)).equals("xkjc")) {
	compiler = (Compiler)Class.forName("at.dms.xkjc.Main").newInstance();
      } else if (((String)args.elementAt(0)).equals("vkjc")
		 || ((String)args.elementAt(0)).equals("kopic")) {
	compiler = (Compiler)Class.forName("at.dms.vkjc.Main").newInstance();
      } else if (((String)args.elementAt(0)).equals("dbi")) {
	compiler = (Compiler)Class.forName("at.dms.dbi.Main").newInstance();
      } else {
	System.err.println("Error: unknown compiler: " + args.elementAt(0));
	return;
      }
      args.removeElementAt(0);

      // get compiler arguments
      args.copyInto(arguments = new String[args.size()]);

      // run the compiler
      success = compiler.run(directory, out, window, arguments);

      if (window != null) {
	window.compilationEnded();
      }

      out.println(IkjcClient.TERM_STRING);
      out.println(success);
    } finally {
      in.close();
      out.close();
    }
  }

  /**
   * parse the argument list and set flags
   * @return parsing error occur
   */
  private static void parseArguments(String[] args) {
    for (int i = 0; i < args.length; i++) {
      if (args[i].equals("-port")) {
	i += 1;
	if (i == args.length) {
	  System.err.println("argument for option -port is missing");
	  System.exit(1);
	}
	try {
	  port = Integer.parseInt(args[i]);
	} catch (Exception e) {
	  System.err.println("Bad port number: " + args[i]);
	}
      } else if (args[i].equals("-console")) {
	console = true;
      } else {
	help();
      }
    }
  }

  /**
   * shows a help message
   */
  private static void help() {
    System.err.println("usage: IkjcServer [-help] [-port p]");
    System.err.println("\t -port p: \tset the port number of server (int)");
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private static int		port;
  private static boolean	console;
  private static Console	window;
  private static ServerSocket	serverSocket;
  private static Socket		clientSocket;
}
