/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CBinaryClass.java,v 1.5 1999/10/21 12:23:08 vgp Exp $
 */

package at.dms.kjc;

import java.util.Hashtable;

import at.dms.classfile.ClassInfo;
import at.dms.classfile.FieldInfo;
import at.dms.classfile.InnerClassInfo;
import at.dms.classfile.MethodInfo;

/**
 * This class represents the exported members of a class (inner classes, methods and fields)
 */
public class CBinaryClass extends CClass {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a class export from file
   */
  public CBinaryClass(ClassInfo classInfo) {
    super(getOwner(classInfo.getName()),
	  classInfo.getMajorVersion(),
	  classInfo.getSourceFile(),
	  classInfo.getModifiers(),
	  getIdent(classInfo.getName()),
	  classInfo.getName(),
	  classInfo.getSuperClass() == null ? null : CClassType.lookup(classInfo.getSuperClass()),
	  classInfo.isDeprecated());

    CClassType[]	interfaces = loadInterfaces(classInfo.getInterfaces());

    FieldInfo[]	fields = classInfo.getFields();
    Hashtable	fields_H = null;
    CField[]	fields_V = null;
    boolean	hasOuterThis = false;

    if (fields.length <= 10) {
      fields_V = new CField[fields.length];
      for (int i = 0; i < fields.length; i++) {
	fields_V[i] = new CField(this, fields[i]);
	if (fields_V[i].getIdent().equals(JAV_OUTER_THIS)) {
	  hasOuterThis = true;
	}
      } 
    } else {
      fields_H = new Hashtable(fields.length + 1, 1.0f);
      
      for (int i = 0; i < fields.length; i++) {
	CField field = new CField(this, fields[i]);

	if (field.getIdent().equals(JAV_OUTER_THIS)) {
	  hasOuterThis = true;
	}
	fields_H.put(field.getIdent(), field);
      }
    }

    MethodInfo[]	methods = classInfo.getMethods();
    Hashtable		methods_H = null;
    CMethod[]		methods_V = new CMethod[methods.length];
    if (methods.length <= 10) {
      for (int i = 0; i < methods.length; i++) {
	methods_V[i] = new CMethod(this, methods[i]);
      }
    } else {
      methods_H = new Hashtable(methods.length + 1, 1.0f);
      
      for (int i = 0; i < methods.length; i++) {
	CMethod method = new CMethod(this, methods[i]);
	String	ident = method.getIdent();
	
	methods_V[i] = method;
	if (ident != JAV_CONSTRUCTOR && ident != JAV_INIT && ident != JAV_STATIC_INIT) {
	  methods_H.put(method.getIdent(), "loaded");
	}
      }
    }

    setInnerClasses(loadInnerClasses(classInfo.getInnerClasses()));

    setHasOuterThis(hasOuterThis);
    close(interfaces, fields_V, fields_H, methods_V, methods_H);
  }

  /**
   *
   */
  protected CClassType[] loadInterfaces(String[] interfaces) {
    if (interfaces != null) {
      CClassType[]	ret;
      ret = new CClassType[interfaces.length];
      for (int i = 0; i < interfaces.length; i++) {
	ret[i] = CClassType.lookup(interfaces[i]);
      }
      return ret;
    } else {
      return CClassType.EMPTY;
    }
  }

  /**
   *
   */
  protected CClassType[] loadInnerClasses(InnerClassInfo[] inners) {
    if (inners != null) {
      CClassType[] innerClasses = new CClassType[inners.length];
      for (int i = 0; i < inners.length; i++) {
	innerClasses[i] = CClassType.lookup(inners[i].getQualifiedName());
      }
      return innerClasses;
    } else {
      return CClassType.EMPTY;
    }
  }

  /**
   * Returns the field infos
   */
  protected FieldInfo[] getFieldInfo(Hashtable fields_H, CField[] fields) {
    FieldInfo[]	pfields = new FieldInfo[fields.length];

    for (int i = 0; i < fields.length; i++) {
      pfields[i] = fields[i].genFieldInfo();	
    }
    
    return pfields;
  }

  private static CClass getOwner(String clazz) {
    int               index = clazz.lastIndexOf("$");
    
    return index == -1 ? null : CClassType.lookup(clazz.substring(0, index)).getCClass();
  }
}
