/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CBlockError.java,v 1.7 1999/10/08 11:18:57 vgp Exp $
 */

package at.dms.kjc;

import at.dms.util.Message;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * This class represents Block errors in error hierarchy
 */
public class CBlockError extends PositionedError {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	msg		the message to display for this error
   * @param	par1		the first parameter of message (name of a var by example)
   * @param	par2		the second parameter of message
   */
  public CBlockError(TokenReference where, String msg, Object par1, Object par2) {
    super(where, msg, par1, par2);
  }

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	msg		the message to display for this error
   * @param	par1		the first parameter of message (name of a var by example)
   */
  public CBlockError(TokenReference where, String msg, Object par1) {
    super(where, msg, par1);
  }

  /**
   * An error with with an arbitrary number of parameters.
   * @param	where		the reference to token where error happen
   * @param	key		the message ident to be displayed
   * @param	params		the array of parameters
   */
  public CBlockError(TokenReference where, String key, Object[] params) {
    super(where, key, params);
  }

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	msg		the message to display for this error
   */
  public CBlockError(TokenReference where, String msg) {
    super(where, msg);
  }

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	msg		the message to display for this error
   */
  public CBlockError(TokenReference where, Message msg) {
    super(where, msg);
  }
}
