/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CField.java,v 1.18 1999/09/17 13:30:14 vgp Exp $
 */

package at.dms.kjc;

import at.dms.classfile.FieldInfo;

/**
 * This class represents an exported member of a class (fields)
 */
public class CField extends CMember {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a field export
   * @param	owner		the owner of this field
   * @param	modifiers	the modifiers on this field
   * @param	ident		the name of this field
   * @param	type		the type of this field
   * @param	javadoc	is this field deprecated
   */
  public CField(CClass owner, int modifiers, String ident, CType type, boolean deprecated) {
    super(owner, modifiers, ident, deprecated);
    this.type = type;
  }

  /**
   * Constructs a field export
   * @param	owner		the owner of this field
   * @param	fieldInfo	a field info from a class file
   */
  public CField(CClass owner, FieldInfo fieldInfo) {
    super(owner,
	  fieldInfo.getModifiers(),
	  getIdent(fieldInfo.getName().intern()),
	  fieldInfo.isDeprecated());

    this.type = CType.parseSignature(fieldInfo.getSignature());

    if (isFinal() && isStatic()) {
      Object		value = fieldInfo.getConstantValue();
      if (value != null) {
	setValue(JLiteral.createLiteral(type, value));
      }
    }
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * @return	the interface
   */
  public CField getField() {
    return this;
  }

  /**
   * @return the type of this field
   */
  public CType getType() {
    return type;
  }

  /**
   * @param	value		the value known at third pass
   */
  public void setValue(JExpression value) {
    this.value = value; //JLiteral.createLiteral(value.getType(), value.getValue());
  }

  /**
   * @return	the value of initializer or null
   */
  public JExpression getValue() {
    return value;
  }

  /**
   * @return the short name of this class
   */
  private static String getIdent(String qualifiedName) {
    assert(qualifiedName != null);
    int		cursor = qualifiedName.lastIndexOf('/');
    if (cursor > 0) {
      return qualifiedName.substring(qualifiedName.lastIndexOf('/') + 1);
    } else {
      return qualifiedName;
    }
  }

  // ----------------------------------------------------------------------
  // GENERATE CLASSFILE INFO
  // ----------------------------------------------------------------------

  /**
   * Generate the code in a class file
   */
  public FieldInfo genFieldInfo() {
    Object		value = null;

    if (this.value != null &&
	this.value.isConstant() &&
	isFinal() &&
	isStatic() &&
	!(this.value instanceof JNullLiteral)) {
      value = ((JLiteral)this.value).getValue();
      if (this.value.getType().isOrdinal() && !(value instanceof Long)) {
	value = new Integer(this.value.getOrdinalLiteral().numberValue().intValue());
      } else if (this.value.getType() == CStdType.Float) {
	value = new Float(((Number)value).floatValue());
      }
    }

    return new FieldInfo((short)getModifiers(),
			 getIdent(),
			 type.getSignature(),
			 value,
			 isDeprecated(),
			 getIdent().indexOf('$') >= 0);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private	CType		type;
  private	JExpression	value;
}
