/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CJavaPrettyPrinter.java,v 1.47 1999/10/24 13:58:52 vgp Exp $
 */

package at.dms.kjc;

import java.util.StringTokenizer;
import at.dms.compiler.JavaStyleComment;
import at.dms.compiler.TabbedPrintWriter;

/**
 * This class implements a Java pretty printer
 */
public class CJavaPrettyPrinter extends at.dms.compiler.Utils implements Constants {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * construct a pretty printer object for java code
   */
  public CJavaPrettyPrinter() {
  }

  /**
   * construct a pretty printer object for java code
   * @param	fileName		the file into the code is generated
   */
  public CJavaPrettyPrinter(String fileName) {
    try {
      this.p = new TabbedPrintWriter(
	 new java.io.BufferedWriter(new java.io.FileWriter(fileName)));
    } catch (Exception e) {
      e.printStackTrace();
      System.err.println("cant write: " + fileName + e);
    }
  }

  /**
   * construct a pretty printer object for java code
   * @param	fileName		the file into the code is generated
   */
  public CJavaPrettyPrinter(TabbedPrintWriter p) {
    this.p = p;
    this.pos = 0;
  }

  /**
   * Close the stream at the end
   */
  public void close() {
    p.close();
  }

  public void setPos(int pos) {
    this.pos = pos;
  }

  // ----------------------------------------------------------------------
  // TYPE DECLARATION
  // ----------------------------------------------------------------------

  /**
   * prints a compilation unit
   */
  public void printCompilationUnit(JPackageName packageName,
				   JPackageImport[] importedPackages,
				   JClassImport[] importedClasses,
				   JTypeDeclaration[] typeDeclarations) {
    if (packageName.getName().length() > 0) {
      packageName.genJavaCode(this);
      if (importedPackages.length + importedClasses.length > 0) {
	newLine();
      }
    }

    for (int i = 0; i < importedPackages.length ; i++) {
      if (!importedPackages[i].getName().equals("java/lang")) {
	importedPackages[i].genJavaCode(this);
	newLine();
      }
    }

    for (int i = 0; i < importedClasses.length ; i++) {
      importedClasses[i].genJavaCode(this);
      newLine();
    }

    for (int i = 0; i < typeDeclarations.length ; i++) {
      newLine();
      typeDeclarations[i].genJavaCode(this);
      newLine();
    }
  }

  // ----------------------------------------------------------------------
  // TYPE DECLARATION
  // ----------------------------------------------------------------------

  /**
   * prints a class declaration
   */
  public void printClassDeclaration(int modifiers,
				    String ident,
				    String superName,
				    CClassType[] interfaces,
				    JPhylum[] body,
				    JMethodDeclaration[] methods,
				    JTypeDeclaration[] decls) {
    newLine();
    print(CModifier.toString(modifiers));
    print("class " + ident);

    if (superName != null) {
      print(" extends " + superName.replace('/', '.'));
    }

    if (interfaces.length != 0) {
      print(" implements ");
      for (int i = interfaces.length - 1; i >= 0; i--) {
	print((i == 0 ? "" : ", ") + interfaces[i]);
      }
    }

    print(" ");
    printClassBody(decls, methods, body);
  }

  /**
   *
   */
  public void printClassBody(JTypeDeclaration[] decls, JMethodDeclaration[] methods, JPhylum[] body) {
    print("{");
    pos += TAB_SIZE;
    for (int i = 0; i < decls.length ; i++) {
      decls[i].genJavaCode(this);
    }
    for (int i = 0; i < methods.length ; i++) {
      methods[i].genJavaCode(this);
    }
    for (int i = 0; i < body.length ; i++) {
      if (!(body[i] instanceof JFieldDeclaration)) {
	body[i].genJavaCode(this);
      }
    }
    for (int i = 0; i < body.length ; i++) {
      if (body[i] instanceof JFieldDeclaration) {
	body[i].genJavaCode(this);
      }
    }

    pos -= TAB_SIZE;
    newLine();
    print("}");
  }

  /**
   * prints a class declaration
   */
  public void printInnerClassDeclaration(int modifiers,
					 String ident,
					 String superName,
					 CClassType[] interfaces,
					 JTypeDeclaration[] decls,
					 JPhylum[] body,
					 JMethodDeclaration[] methods) {
    print(" {");
    pos += TAB_SIZE;
    for (int i = 0; i < decls.length ; i++) {
      decls[i].genJavaCode(this);
    }
    for (int i = 0; i < methods.length ; i++) {
      methods[i].genJavaCode(this);
    }
    for (int i = 0; i < body.length ; i++) {
      body[i].genJavaCode(this);
    }
    pos -= TAB_SIZE;
    newLine();
    print("}");
  }

  /**
   * prints an interface declaration
   */
  public void printInterfaceDeclaration(int modifiers,
					String ident,
					CClassType[] interfaces,
					JPhylum[] body,
					JMethodDeclaration[] methods) {
    newLine();
    print(CModifier.toString(modifiers));
    print("interface " + ident);

    if (interfaces.length != 0) {
      print(" extends ");
      for (int i = interfaces.length - 1; i >= 0; i--) {
	print((i == 0 ? "" : ",") + interfaces[i]);
      }
    }

    print(" {");
    pos += TAB_SIZE;
    for (int i = 0; i < body.length; i++) {
      body[i].genJavaCode(this);
    }
    for (int i = 0; i < methods.length; i++) {
      methods[i].genJavaCode(this);
    }
    pos -= TAB_SIZE;
    newLine();
    print("}");
  }

  // ----------------------------------------------------------------------
  // METHODS AND FIELDS
  // ----------------------------------------------------------------------

  /**
   * prints a field declaration
   */
  public void printFieldDeclaration(int modifiers,
				    CType type,
				    String ident,
				    JExpression expr) {
    if (ident.indexOf("$") != -1) {
      return; // dont print generated elements
    }

    newLine();
    print(CModifier.toString(modifiers));
    print(type);
    print("\t");
    print(ident);
    if (expr != null) {
      print("\t= ");
      expr.genJavaCode(this);
    }
    print(";");
  }

  /**
   * prints a method declaration
   */
  public void printMethodDeclaration(int modifiers,
				     CType returnType,
				     String ident,
				     JFormalParameter[] parameters,
				     CClassType[] exceptions,
				     JBlock body) {
    if (ident.equals("<clinit>") || ident.equals("Block$")) {
      return; // we do not want to generate this methods in source code
    }

    newLine();
    print(CModifier.toString(modifiers));
    print(returnType);
    print(" ");
    print(ident);
    print("(");
    int count = 0;

    for (int i = 0; i < parameters.length; i++) {
      if (count != 0) {
	print(", ");
      }

      if (!parameters[i].isGenerated()) {
	parameters[i].genJavaCode(this);
	count++;
      }
    }
    print(")");

    for (int i = 0; i < exceptions.length; i++) {
      if (i != 0) {
	print(", ");
      } else {
	print(" throws ");
      }
      print(exceptions[i].toString());
    }
   
    print(" ");
    if (body != null) {
      body.genJavaCode(this);
    } else {
      print(";");
    }
    newLine();
  }

  /**
   * prints a method declaration
   */
  public void printConstructorDeclaration(int modifiers,
					  String ident,
					  JFormalParameter[] parameters,
					  CClassType[] exceptions,
					  JBlock body) {
    newLine();
    print(CModifier.toString(modifiers));
    print(ident);
    print("(");
    int count = 0;
    for (int i = 0; i < parameters.length; i++) {
      if (count != 0) {
	print(", ");
      }
      if (!parameters[i].isGenerated()) {
	parameters[i].genJavaCode(this);
	count++;
      }
    }
    print(")");

    for (int i = 0; i < exceptions.length; i++) {
      if (i != 0) {
	print(", ");
      } else {
	print(" throws ");
      }
      print(exceptions[i].toString());
    }
    print(" ");
    body.genJavaCode(this);
    newLine();
  }

  // ----------------------------------------------------------------------
  // STATEMENT
  // ----------------------------------------------------------------------

  /**
   * prints a while statement
   */
  public void printWhileStatement(JExpression cond, JStatement body) {
    print("while (");
    cond.genJavaCode(this);
    print(") ");

    body.genJavaCode(this);
  }

  /**
   * prints a variable declaration statement
   */
  public void printVariableDeclarationStatement(JVariableDefinition[] vars) {
    for (int i = 0; i < vars.length; i++) {
      vars[i].genJavaCode(this);
    }
  }

  /**
   * prints a variable declaration statement
   */
  public void printVariableDefinition(int modifiers,
				      CType type,
				      String ident,
				      JExpression expr) {
    print(CModifier.toString(modifiers));
    print(type);
    print("\t");
    print(ident);
    if (expr != null) {
      print("\t= ");
      expr.genJavaCode(this);
    }
    print(";");
  }

  /**
   * prints a try-catch statement
   */
  public void printTryCatchStatement(JBlock tryClause, JCatchClause[] catchClauses) {
    print("try ");
    tryClause.genJavaCode(this);
    for (int i = 0; i < catchClauses.length; i++) {
      catchClauses[i].genJavaCode(this);
    }
  }

  /**
   * prints a try-finally statement
   */
  public void printTryFinallyStatement(JBlock tryClause, JBlock finallyClause) {
    print("try ");
    tryClause.genJavaCode(this);
    if (finallyClause != null) {
      print(" finally ");
      finallyClause.genJavaCode(this);
    }
  }

  /**
   * prints a throw statement
   */
  public void printThrowStatement(JExpression expr) {
    print("throw ");
    expr.genJavaCode(this);
    print(";");
  }

  /**
   * prints a synchronized statement
   */
  public void printSynchronizedStatement(JExpression cond, JStatement body) {
    print("synchronized (");
    cond.genJavaCode(this);
    print(") ");
    body.genJavaCode(this);
  }

  /**
   * prints a switch statement
   */
  public void printSwitchStatement(JExpression expr, JSwitchGroup[] body) {
    print("switch (");
    expr.genJavaCode(this);
    print(") {");
    for (int i = 0; i < body.length; i++) {
      body[i].genJavaCode(this);
    }
    newLine();
    print("}");
  }

  /**
   * prints a return statement
   */
  public void printReturnStatement(JExpression expr) {
    print("return");
    if (expr != null) {
      print(" ");
      expr.genJavaCode(this);
    }
    print(";");
  }

  /**
   * prints a labeled statement
   */
  public void printLabeledStatement(String label, JStatement stmt) {
    print(label + ":");
    stmt.genJavaCode(this);
  }

  /**
   * prints a if statement
   */
  public void printIfStatement(JExpression cond,
			       JStatement thenClause,
			       JStatement elseClause) {
    print("if (");
    cond.genJavaCode(this);
    print(") ");
    pos += thenClause instanceof JBlock ? 0 : TAB_SIZE;
    thenClause.genJavaCode(this);
    pos -= thenClause instanceof JBlock ? 0 : TAB_SIZE;
    if (elseClause != null) {
      if ((elseClause instanceof JBlock) || (elseClause instanceof JIfStatement)) {
	print(" ");
      } else {
	newLine();
      }
      print("else ");
      pos += elseClause instanceof JBlock || elseClause instanceof JIfStatement ? 0 : TAB_SIZE;
      elseClause.genJavaCode(this);
      pos -= elseClause instanceof JBlock || elseClause instanceof JIfStatement ? 0 : TAB_SIZE;
    }
  }

  /**
   * prints a for statement
   */
  public void printForStatement(JStatement init,
				JExpression cond,
				JStatement incr,
				JStatement body) {
    print("for (");
    forInit = true;
    if (init != null) {
      init.genJavaCode(this);
    } else {
      print(";");
    }
    forInit = false;

    print(" ");
    if (cond != null) {
      cond.genJavaCode(this);
    }
    print("; ");

    if (incr != null) {
      incr.genJavaCode(this);
    }
    print(") ");

    print("{");
    pos += TAB_SIZE;
    body.genJavaCode(this);
    pos -= TAB_SIZE;
    newLine();
    print("}");
  }

  /**
   * prints a compound statement
   */
  public void printCompoundStatement(JStatement[] body) {
    for (int i = 0; i < body.length; i++) {
      if (body[i] instanceof JIfStatement && 
	  i < body.length - 1 &&
	  !(body[i + 1] instanceof JReturnStatement)) {
	newLine();
      }
      if (body[i] instanceof JReturnStatement && i > 0) {
	newLine();
      }

      newLine();
      body[i].genJavaCode(this);

      if (body[i] instanceof JVariableDeclarationStatement && 
	  i < body.length - 1 &&
	  !(body[i + 1] instanceof JVariableDeclarationStatement)) {
	newLine();
      }
    }
  }

  /**
   * prints an expression statement
   */
  public void printExpressionStatement(JExpression expr) {
    expr.genJavaCode(this);
    if (!forInit) {
      print(";");
    }
  }

  /**
   * prints an expression list statement
   */
  public void printExpressionListStatement(JExpression[] expr) {
    for (int i = 0; i < expr.length; i++) {
      if (i != 0) {
	print(", ");
      }
      expr[i].genJavaCode(this);
    }
    if (forInit) {
      print(";");
    }
  }

  /**
   * prints a empty statement
   */
  public void printEmptyStatement() {
    newLine();
    print(";");
  }

  /**
   * prints a do statement
   */
  public void printDoStatement(JExpression cond, JStatement body) {
    newLine();
    print("do ");
    body.genJavaCode(this);
    print("");
    print("while (");
    cond.genJavaCode(this);
    print(");");
  }

  /**
   * prints a continue statement
   */
  public void printContinueStatement(String label) {
    newLine();
    print("continue");
    if (label != null) {
      print(" " + label);
    }
    print(";");
  }

  /**
   * prints a break statement
   */
  public void printBreakStatement(String label) {
    newLine();
    print("break");
    if (label != null) {
      print(" " + label);
    }
    print(";");
  }

  /**
   * prints an expression statement
   */
  public void printBlockStatement(JStatement[] body, JavaStyleComment[] comments) {
    print("{");
    pos += TAB_SIZE;
    printCompoundStatement(body);
    if (comments != null) {
      printComments(comments);
    }
    pos -= TAB_SIZE;
    newLine();
    print("}");
  }

  /**
   * prints a type declaration statement
   */
  public void printTypeDeclarationStatement(JTypeDeclaration decl) {
    decl.genJavaCode(this);
  }

  // ----------------------------------------------------------------------
  // EXPRESSION
  // ----------------------------------------------------------------------

  /**
   * prints an unary expression
   */
  public void printUnaryExpression(int oper, JExpression expr) {
    switch (oper) {
    case OPE_PLUS:
      print("+");
      expr.genJavaCode(this);
      break;

    case OPE_MINUS:
      print("-");
      expr.genJavaCode(this);
      break;

    case OPE_BNOT:
      print("~");
      expr.genJavaCode(this);
      break;

    case OPE_LNOT:
      print("!");
      expr.genJavaCode(this);
      break;

    default:
      fail();
    }
  }

  /**
   * prints a type name expression
   */
  public void printTypeNameExpression(CType type) {
    print(type);
  }

  /**
   * prints a this expression
   */
  public void printThisExpression(JExpression prefix) {
    if (prefix != null) {
      prefix.genJavaCode(this);
      print(".this");
    } else {
      print("this");
    }
  }

  /**
   * prints a super expression
   */
  public void printSuperExpression() {
    print("super");
  }

  /**
   * prints a shift expression
   */
  public void printShiftExpression(int oper, JExpression left, JExpression right) {
    left.genJavaCode(this);
    if (oper == OPE_SL) {
      print(" << ");
    } else if (oper == OPE_SR) {
      print(" >> ");
    } else {
      print(" >>> ");
    }
    right.genJavaCode(this);
  }

  /**
   * prints a shift expressiona
   */
  public void printRelationalExpression(int oper, JExpression left, JExpression right) {
    left.genJavaCode(this);
    switch (oper) {
    case OPE_LT:
      print(" < ");
      break;
    case OPE_LE:
      print(" <= ");
      break;
    case OPE_GT:
      print(" > ");
      break;
    case OPE_GE:
      print(" >= ");
      break;
    default:
      fail();
    }
    right.genJavaCode(this);
  }

  /**
   * prints a prefix expression
   */
  public void printPrefixExpression(int oper, JExpression expr) {
    if (oper == OPE_PREINC) {
      print("++");
    } else {
      print("--");
    }
    expr.genJavaCode(this);
  }

  /**
   * prints a postfix expression
   */
  public void printPostfixExpression(int oper, JExpression expr) {
    expr.genJavaCode(this);
    if (oper == OPE_POSTINC) {
      print("++");
    } else {
      print("--");
    }
  }

  /**
   * prints a parenthesed expression
   */
  public void printParenthesedExpression(JExpression expr) {
    print("(");
    expr.genJavaCode(this);
    print(")");
  }

  /**
   * prints an object allocator expression
   */
  public void printNewObjectExpression(CClassType type, JExpression[] params, JClassDeclaration innerBody) {
    print("new " + type + "(");
    printArgs(params);
    print(")");
    if (innerBody != null) {
      innerBody.genInnerJavaCode(this);
    }
  }

  /**
   * prints an array allocator expression
   */
  public void printNewArrayExpression(CType type, JArrayDimsAndInits dims) {
    if (type instanceof CArrayType) {
      print("new " + ((CArrayType)type).getBaseType());
    } else {
      print("new " + type);
    }
    dims.genJavaCode(this);
  }

  /**
   * prints a name expression
   */
  public void printNameExpression(JExpression prefix, String ident) {
    if (prefix != null) {
      prefix.genJavaCode(this);
      print('.');
    }
    print(ident);
  }

  /**
   * prints an array allocator expression
   */
  public void printBinaryExpression(String oper,
				    JExpression left,
				    JExpression right) {
    left.genJavaCode(this);
    print(" ");
    print(oper);
    print(" ");
    right.genJavaCode(this);
  }

  /**
   * prints a method call expression
   */
  public void printMethodCallExpression(JExpression prefix,
					String ident,
					JExpression[] args) {
    if (ident != null && ident.equals("Block$")) {
      return; // we do not want generated methods in source code
    }

    if (prefix != null) {
      prefix.genJavaCode(this);
      print('.');
    }
    print(ident);
    print("(");
    printArgs(args);
    print(")");
  }

  /**
   * prints a local variable expression
   */
  public void printLocalVariableExpression(String ident) {
    print(ident);
  }

  /**
   * prints an instanceof expression
   */
  public void printInstanceofExpression(JExpression expr, CType dest) {
    expr.genJavaCode(this);
    print(" instanceof ");
    print(dest);
  }

  /**
   * prints an equality expression
   */
  public void printEqualityExpression(int oper,
				      JExpression left,
				      JExpression right) {
    left.genJavaCode(this);
    if (oper == OPE_EQ) {
      print(" == ");
    } else {
      print(" != ");
    }
    right.genJavaCode(this);
  }

  /**
   * prints a conditional expression
   */
  public void printConditionalExpression(JExpression cond,
					 JExpression left,
					 JExpression right) {
    cond.genJavaCode(this);
    print(" ? ");
    left.genJavaCode(this);
    print(" : ");
    right.genJavaCode(this);
  }

  /**
   * prints a compound expression
   */
  public void printCompoundAssignmentExpression(int oper,
						JExpression left,
						JExpression right) {
    left.genJavaCode(this);
    switch (oper) {
    case OPE_STAR:
      print(" *= ");
      break;
    case OPE_SLASH:
      print(" /= ");
      break;
    case OPE_PERCENT:
      print(" %= ");
      break;
    case OPE_PLUS:
      print(" += ");
      break;
    case OPE_MINUS:
      print(" -= ");
      break;
    case OPE_SL:
      print(" <<= ");
      break;
    case OPE_SR:
      print(" >>= ");
      break;
    case OPE_BSR:
      print(" >>>= ");
      break;
    case OPE_BAND:
      print(" &= ");
      break;
    case OPE_BXOR:
      print(" ^= ");
      break;
    case OPE_BOR:
      print(" |= ");
      break;
    }
    right.genJavaCode(this);
  }

 /**
   * prints a field expression
   */
  public void printFieldExpression(JExpression left, String ident) {
    if (ident.equals("this$0")) {// don't generate generated fields
      print(left.getType().getCClass().getOwner().getType() + ".this");
      return;
    }
    int		index = ident.indexOf("_$");
    if (index != -1) {
      print(ident.substring(0, index));      // local var
    } else {
      left.genJavaCode(this);
      print("." + ident);
    }
  }

  /**
   * prints a class expression
   */
  public void printClassExpression(CType type) {
    print(type);
    print(".class");
  }

  /**
   * prints a cast expression
   */
  public void printCastExpression(JExpression expr, CType type) {
    print("(");
    print(type);
    print(")");
    expr.genJavaCode(this);
  }

  /**
   * prints a cast expression
   */
  public void printUnaryPromoteExpression(JExpression expr, CType type) {
    print("(");
    print(type);
    print(")");
    print("(");
    expr.genJavaCode(this);
    print(")");
  }

  /**
   * prints a compound assignment expression
   */
  public void printBitwiseExpression(int oper,
				     JExpression left,
				     JExpression right) {
    left.genJavaCode(this);
    switch (oper) {
    case OPE_BAND:
      print(" & ");
      break;
    case OPE_BOR:
      print(" | ");
      break;
    case OPE_BXOR:
      print(" ^ ");
      break;
    default:
      fail();
    }
    right.genJavaCode(this);
  }

  /**
   * prints an assignment expression
   */
  public void printAssignmentExpression(JExpression left, JExpression right) {
    if ((left instanceof JClassFieldExpression) &&
	((JClassFieldExpression)left).getField().getIdent().equals(Constants.JAV_OUTER_THIS)) {
      return;
    }

    left.genJavaCode(this);
    print(" = ");
    right.genJavaCode(this);
  }

  /**
   * prints an array length expression
   */
  public void printArrayLengthExpression(JExpression prefix) {
    prefix.genJavaCode(this);
    print(".length");
  }

  /**
   * prints an array length expression
   */
  public void printArrayAccessExpression(JExpression prefix, JExpression accessor) {
    prefix.genJavaCode(this);
    print("[");
    accessor.genJavaCode(this);
    print("]");
  }

  /**
   * prints an array length expression
   */
  public void printComments(JavaStyleComment[] comments) {
    for (int i = 0; i < comments.length; i++) {
      if (comments[i] != null) {
	printComment(comments[i]);
      }
    }
  }

  /**
   * prints an array length expression
   */
  public void printComment(JavaStyleComment comment) {
    StringTokenizer	tok = new StringTokenizer(comment.getText(), "\n");

    if (comment.hadSpaceBefore()) {
      newLine();
    }

    if (comment.isLineComment()) {
      print("// " + tok.nextToken().trim());
      p.println();
    } else {
      if (p.getLine() > 0) {
	if (!nl) {
	  newLine();
	}
	newLine();
      }
      print("/*");
      while (tok.hasMoreTokens()){
	String comm = tok.nextToken().trim();
	if (comm.startsWith("*")) {
	  comm = comm.substring(1).trim();
	}
	if (tok.hasMoreTokens() || comm.length() > 0) {
	  newLine();
	  print(" * " + comm);
	}
      }
      newLine();
      print(" */");
      newLine();
    }

    if (comment.hadSpaceAfter()) {
      newLine();
    }
  }

  /**
   * prints an array length expression
   */
  public void printJavadoc(JJavadocComment comment) {
    StringTokenizer	tok = new StringTokenizer(comment.getText(), "\n");
    int			count = 0;
    boolean		isFirst = true;

    if (!nl) {
      newLine();
    }
    newLine();
    print("/**");
    while (tok.hasMoreTokens()) {
      String	text = tok.nextToken().trim();
      String	type = null;
      boolean	param = false;
      int	idx = text.indexOf("@param");
      if (idx >= 0) {
	type = "@param";
	param = true;
      }
      if (idx < 0) {
	idx = text.indexOf("@exception");
	if (idx >= 0) {
	  type = "@exception";
	  param = true;
	}
      }
      if (idx < 0) {
	idx = text.indexOf("@exception");
	if (idx >= 0) {
	  type = "@exception";
	  param = true;
	}
      }
      if (idx < 0) {
	idx = text.indexOf("@author");
	if (idx >= 0) {
	  type = "@author";
	}
      }
      if (idx < 0) {
	idx = text.indexOf("@see");
	if (idx >= 0) {
	  type = "@see";
	}
      }
      if (idx < 0) {
	idx = text.indexOf("@version");
	if (idx >= 0) {
	  type = "@version";
	}
      }
      if (idx < 0) {
	idx = text.indexOf("@return");
	if (idx >= 0) {
	  type = "@return";
	}
      }
      if (idx < 0) {
	idx = text.indexOf("@deprecated");
	if (idx >= 0) {
	  type = "@deprecated";
	}
      }
      if (idx >= 0) {
	newLine();
	isFirst = false;
	if (param) {
	  text = text.substring(idx + type.length()).trim();
	  idx = Math.min(text.indexOf(" ") == -1 ? 1000 : text.indexOf(" "),
			 text.indexOf("\t") == -1 ? 1000 : text.indexOf("\t"));
	  String	before = text.substring(0, idx);
	  print(" * " + type);
	  pos += 12;
	  print(before);
	  pos += 20;
	  print(text.substring(idx).trim());
	  pos -= 20;
	  pos -= 12;
	} else {
	  text = text.substring(idx + type.length()).trim();
	  print(" * " + type);
	  pos += 12;
	  print(text);
	  pos -= 12;
	}
      } else {
	text = text.substring(text.indexOf("*") + 1);
	if (tok.hasMoreTokens() || text.length() > 0) {
	  newLine();
	  print(" * ");
	  pos += isFirst ? 0 : 32;
	  print(text.trim());
	  pos -= isFirst ? 0 : 32;
	}
      }
    }
    newLine();
    print(" */");
  }

  // ----------------------------------------------------------------------
  // UTILS
  // ----------------------------------------------------------------------

  /**
   * prints an array length expression
   */
  public void printSwitchLabel(JExpression expr) {
    newLine();
    if (expr != null) {
      print("case ");
      expr.genJavaCode(this);
      print(": ");
    } else {
      print("default: ");
    }
  }

  /**
   * prints an array length expression
   */
  public void printSwitchGroup(JSwitchLabel[] labels, JStatement[] stmts) {
    for (int i = 0; i < labels.length; i++) {
      labels[i].genJavaCode(this);
    }
    pos += TAB_SIZE;
    for (int i = 0; i < stmts.length; i++) {
      newLine();
      stmts[i].genJavaCode(this);
    }
    pos -= TAB_SIZE;
  }

  /**
   * prints an array length expression
   */
  public void printCatchClause(JFormalParameter exception, JBlock body) {
    print(" catch (");
    exception.genJavaCode(this);
    print(") ");
    body.genJavaCode(this);
  }

  /**
   * prints an array length expression
   */
  public void printStringLiteral(String value) {
    print('"' + value + '"');
  }

  /**
   * prints an array length expression
   */
  public void printCharLiteral(Character value) {
    switch(value.charValue()) {
    case '\b':
      print("'\\b'");
      break;
    case '\r':
      print("'\\r'");
      break;
    case '\t':
      print("'\\t'");
      break;
    case '\n':
      print("'\\n'");
      break;
    case '\f':
      print("'\\f'");
      break;
    case '\\':
      print("'\\\\'");
      break;
    case '\'':
      print("'\\''");
      break;
    case '\"':
      print("'\\\"'");
      break;
    default:
      print("'" + value + "'");
    }
  }

  /**
   * prints an array length expression
   */
  public void printBooleanLiteral(boolean value) {
    print(value);
  }

  /**
   * prints an array length expression
   */
  public void printNumberLiteral(Number value) {
    print(value.toString());
    if (value instanceof Long) {
      print("L");
    } else if (value instanceof Float) {
      print("f");
    }
  }

  /**
   * prints an array length expression
   */
  public void printNullLiteral() {
    print("null");
  }

  /**
   * prints an array length expression
   */
  public void printPackageName(String name) {
    print("package " + name + ";");
    newLine();
  }

  /**
   * prints an array length expression
   */
  public void printPackageImport(String name) {
    print("import " + name.replace('/', '.') + ".*;");
  }

  /**
   * prints an array length expression
   */
  public void printClassImport(String name) {
    print("import " + name.replace('/', '.') + ";");
  }

  /**
   * prints an array length expression
   */
  public void printFormalParameters(boolean isFinal, CType type, String ident) {
    if (isFinal) {
      print("final ");
    }
    print(type.toString());
    if (ident.indexOf("$") == -1) {
      print(" ");
      print(ident);
    }
  }

  /**
   * prints an array length expression
   */
  public void printArgs(JExpression[] args) {
    if (args != null) {
      for (int i = 0; i < args.length; i++) {
	if (i != 0) {
	  print(", ");
	}
	args[i].genJavaCode(this);
      }
    }
  }

  /**
   * prints an array length expression
   */
  public void printExplicitConstructorInvocation(JExpression prefix,
						 String ident,
						 JExpression[] params) {
    newLine();
    if (prefix != null) {
      prefix.genJavaCode(this);
    }

    print(ident);

    print("(");
    printArgs(params);
    print(");");
  }

  /**
   * prints an array initializer expression
   */
  public void printArrayInitializer(JExpression[] elems) {
    newLine();
    print("{");
    for (int i = 0; i < elems.length; i++) {
      if (i != 0) {
	print(", ");
      }
      elems[i].genJavaCode(this);
    }
    print("}");
  }

  /**
   * prints an array initializer expression
   */
  public void printArrayDimsAndInit(JExpression[] dims, JArrayInitializer init) {
    for (int i = 0; i < dims.length; i++) {
      print("[");
      if (dims[i] != null) {
	dims[i].genJavaCode(this);
      }
      print("]");
    }
    if (init != null) {
      init.genJavaCode(this);
    }
  }

  // ----------------------------------------------------------------------
  // PROTECTED METHODS
  // ----------------------------------------------------------------------

  protected void newLine() {
    p.println();
  }

  protected void print(Object s) {
    print(s.toString());
  }

  protected void print(String s) {
    p.setPos(pos);
    p.print(s);
  }

  protected void print(boolean s) {
    print("" + s);
  }

  protected void print(int s) {
    print("" + s);
  }

  protected void print(char s) {
    print("" + s);
  }

  protected void print(double s) {
    print("" + s);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  protected boolean			forInit;	// is on a for init
  protected int				TAB_SIZE = 2;
  protected int				WIDTH = 80;
  protected int				pos;

  protected TabbedPrintWriter		p;
  protected boolean			nl = true;
}

