/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CSourceMethod.java,v 1.27 1999/09/10 14:23:43 vgp Exp $
 */

package at.dms.kjc;

import at.dms.classfile.NoArgInstruction;
import at.dms.classfile.MethodInfo;
import at.dms.classfile.CodeInfo;

/**
 * This class represents an exported member of a class (fields)
 */
public class CSourceMethod extends CMethod {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a field export
   * @param	modifiers	the modifiers on this method
   * @param	ident		the ident of this method
   * @param	returnType	the return type of this method
   * @param	paramTypes	the parameters type of this method
   * @param	exceptions	a list of all exceptions of the throws list
   * @param	javadoc		is this method deprecated
   * @param	declaration	the source code
   */
  public CSourceMethod(CClass owner,
		       int modifiers,
		       String ident,
		       CType returnType,
		       CType[] paramTypes,
		       CClassType[] exceptions,
		       boolean deprecated,
		       JBlock body) {
    super(owner, modifiers, ident, returnType, paramTypes, exceptions, deprecated);

    this.body = body;
  }

  // ----------------------------------------------------------------------
  // GENERATE CLASSFILE INFO
  // ----------------------------------------------------------------------

  /**
   * Generate the code in a class file
   */
  public MethodInfo genMethodInfo() {
    CClassType[]	excs = getThrowables();
    String[]		exceptions = new String[excs.length];
    for (int i = 0; i < excs.length; i++) {
      exceptions[i] = excs[i].getQualifiedName();
    }

    if (body != null) {
      return new CMethodInfo((short)getModifiers(),
			     getIdent(),
			     getSignature(),
			     exceptions,
			     this,
			     isDeprecated(),
			     false);
    } else {
      return new MethodInfo((short)getModifiers(),
			    getIdent(),
			    getSignature(),
			    exceptions,
			    null,
			    isDeprecated(),
			    false);
    }
  }

  /**
   * @return the type of this field
   */
  public String getSignature() {
    CType[] params = getParameters();
    if (getOwner().isInnerClass() && isConstructor()) {
      params = ((CSourceClass)getOwner()).genConstructorArray(params);
    }
    return CType.genMethodSignature(getReturnType(), params);
  }

  public CodeInfo genCode() {
    CodeSequence code = CodeSequence.getCodeSequence();
    
    code.setLineNumber(body.getTokenReference().getLine());
    body.genCode(code);
    
    if (code.getPC() == 0 || 
	code.getInstructionAt(code.getPC() - 1).getOpcode() != opc_return &&
	getReturnType() == CStdType.Void) {
      code.plantInstruction(new NoArgInstruction(opc_return));
    }

    CodeInfo info = new CodeInfo(code.getInstructionArray(),
				 code.getHandlers(),
				 Main.optimizeCode() >  1 ? null : code.getLineNumbers(),
				 null);    //seq.getLocalVariableInfos());
    code.release();
    body = null;

    CType[]	parameters = getParameters();
    int		paramCount = 0;

    for (int i = 0; i < parameters.length; i++) {
      paramCount += parameters[i].getSize();
    }
    paramCount += getReturnType().getSize();
    paramCount += isStatic() ? 0 : 1;

    info.setParameterCount(paramCount);

    return Main.optimizeCode() > 0 ?
      at.dms.optimize.Optimizer.optimize(info, Main.optimizeCode()) :
      info;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private	JBlock		body;
}
