/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CToken.java,v 1.8 1999/09/17 19:20:28 graf Exp $
 */

package at.dms.kjc;

import antlr.Token;
import java.util.Hashtable;

public final class CToken extends Token {

  /**
   * Constructs a token
   */
  public CToken(int type, String text) {
    super(type);
    this.text = text;
  }

  public String getText() {
    return text;
  }

  public void setText(String text) {
    this.text = text;
  }

  // --------------------------------------------------------------------
  // LOOKUP
  // --------------------------------------------------------------------

  public static CToken lookupToken(char[] data, int start, int length) {
    CToken	tok = null;

    currentToken.data = data;
    currentToken.start = start;
    currentToken.length = length;
    
    tok = (CToken)table.get(currentToken);
    if (tok == null) { 
      tok = new CToken(JavaTokenTypes.IDENT, String.valueOf(data, start, length).intern());
      table.put(currentToken.store(), tok);
    }
	
    return tok;
  }

  static class LookupToken {
    public char[] data;
    public int    start;
    public int    length;

    public LookupToken store() {
      char[]		temp = new char[length];
      LookupToken	store = new LookupToken();

      System.arraycopy(data, start, temp, 0, length);
      store.data = temp;
      store.length = length;

      return store;
    }

    public int hashCode() {
      int	h = 0;
      int	index1 = start;
      char[]	val = data;

      for (int i = length - 1 ; i >= 0; i--) {
	h = (h * 37) + val[index1++];
      }

      return h;
    }

    public boolean equals(Object o) {
      LookupToken	tok = (LookupToken)o;

      if (length != tok.length) {
	return false;
      }

      char[]	data1 = this.data;
      char[]	data2 = tok.data;
      int	index1 = start;
      int	index2 = tok.start;

      for (int i = 0; i < length; i++) {
	if (data1[index1++] != data2[index2++]) {
	  return false;
	}
      }

      return true;
    }
  }

  // --------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // --------------------------------------------------------------------

  private static final LookupToken	currentToken = new LookupToken();
  private static final Hashtable	table = new Hashtable(1000);

  private String		text;
}

