/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JAssignmentExpression.java,v 1.19 1999/10/12 15:39:47 vgp Exp $
 */

package at.dms.kjc;

import at.dms.compiler.CWarning;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * This class implements the assignment operation
 */
public class JAssignmentExpression extends JBinaryExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	left		the left operand
   * @param	right		the right operand
   */
  public JAssignmentExpression(TokenReference where,
			       JExpression left,
			       JExpression right) {
    super(where, left, right);
  }

  /**
   *
   */
  public static JExpression create(TokenReference where, int oper, JExpression left, JExpression right) {
    if (oper == OPE_SIMPLE) {
      return new JAssignmentExpression(where, left, right);
    } else {
      return new JCompoundAssignmentExpression(where, oper, left, right);
    }
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param	context		the actual context of analyse
   * @return	a pure java expression including promote node
   * @exception	PositionedError		if the check fails
   * @exception	UnpositionedError	this error will be positioned soon
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    context.setLeftSide(true);
    left = left.checkExpression(context);
    check(context, left instanceof CInitializable &&
	  ((CInitializable)left).isLValue(context), "assignment-notlvalue");
    context.setLeftSide(false);

    right = right.checkExpression(context);
    try {
      ((CInitializable)left).initialize(context.getBodyContext());
    } catch (UnpositionedError e) {
      throw e.getError(getTokenReference());
    }

    check(context, right.getType().isAssignableTo(left.getType()),
	  "assignment-badtype", right.getType(), left.getType());
    if (right instanceof JTypeNameExpression) {
      check(context, false, "var-unknown", ((JTypeNameExpression)right).getQualifiedName());
    }

    type = left.getType();
    right = right.convertType(type, context);

    /*
    if ((left instanceof JVariableExpression) && (right instanceof JVariableExpression))
      if (((JVariableExpression)left).getInfo().getKey() == ((JVariableExpression)right).getInfo().getKey()) {
	context.reportTrouble(new CWarning(getTokenReference(), "self-assignment", null));
      }
    */

    return this;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param	p		the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    p.printAssignmentExpression(left, right);
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    boolean	discardValue = code.discardValue();
    code.setLineNumber(getTokenReference().getLine());
    code.setDiscardValue(false);

    ((CInitializable)left).genStartStoreCode(code);

    right.genCode(code);

    ((CInitializable)left).genEndStoreCode(code, discardValue);
  }
}
