/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JBreakStatement.java,v 1.23 1999/10/21 12:23:09 vgp Exp $
 */

package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;

/**
 * JLS 14.13: Break Statement
 *
 * A break statement transfers control out of an enclosing statement.
 */
public class JBreakStatement extends JStatement {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * @param	where		the line of this node in the source code
   * @param	label		the label of the enclosing labeled statement
   */
  public JBreakStatement(TokenReference where, String label, JavaStyleComment[] comments) {
    super(where, comments);
    this.label = label;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check statement.
   * @param	context		the actual context of analyse
   * @exception	PositionedError		if the check fails
   * @exception	UnpositionedError	this error will be positioned soon
   */
  public void checkStatement(CBodyContext context) throws PositionedError {
    if (label != null) {
      check(context, context.getMethodContext().hasLabel(label), "label-unknown", label);
      JStatement lab = context.getMethodContext().getLabeleldStatement(label);
      stmt = (CBreakable)lab;
    } else {
      stmt = context.getNearestBreakableStatement();
    }

    check(context, stmt != null, "cant-break", label);
    stmt.addBreak(context);
    context.addBreak();

    breakLabel = stmt.getEnd();
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param	p		the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    super.genJavaCode(p);
    p.printBreakStatement(label);
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    code.setLineNumber(getTokenReference().getLine());

    code.plantBreak(stmt);

    code.plantLabelRef(opc_goto, breakLabel);

    breakLabel = null;
    stmt = null;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private CodeLabel		breakLabel;
  private String		label;
  private CBreakable		stmt;
}
