/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JClassExpression.java,v 1.15 1999/10/08 11:18:59 vgp Exp $
 */

package at.dms.kjc;

import at.dms.classfile.PushLiteralInstruction;
import at.dms.classfile.MethodRefInstruction;
import at.dms.classfile.FieldRefInstruction;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * A 'int.class' expression
 */
public class JClassExpression extends JExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * @param where the line of this node in the source code
   */
  public JClassExpression(TokenReference where, CType type) {
    super(where);
    this.type = type;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Compute the type of this expression (called after parsing)
   * @return the type of this expression
   */
  public CType getType() {
    return CStdType.Class;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param context the actual context of analyse
   * @return  a pure java expression including promote node
   * @exception	PositionedError		if the check fails
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    try {
      type.checkType(context);
    } catch (UnpositionedError e) {
      throw e.getError(getTokenReference());
    }
    return this;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param p the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    p.printClassExpression(type);
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    code.setLineNumber(getTokenReference().getLine());

    String		className = "UNKNOWN";
    switch (type.getTypeID()) {
    case TID_CLASS:
    case TID_ARRAY: // $$$ may be optimized and exception changed
      code.plantInstruction(new PushLiteralInstruction(((CClassType)type).getQualifiedName().replace('/', '.')));
      code.plantInstruction(new MethodRefInstruction(opc_invokestatic,
						     "java/lang/Class",
						     "forName",
						     "(Ljava/lang/String;)Ljava/lang/Class;"));
      return;
    case TID_INT:
      className = "java/lang/Integer";
      break;
    case TID_LONG:
      className = "java/lang/Long";
      break;
    case TID_FLOAT:
      className = "java/lang/Float";
      break;
    case TID_DOUBLE:
      className = "java/lang/Double";
      break;
    case TID_BYTE:
      className = "java/lang/Byte";
      break;
    case TID_BOOLEAN:
      className = "java/lang/Boolean";
      break;
    case TID_CHAR:
      className = "java/lang/Character";
      break;
    case TID_SHORT:
      className = "java/lang/Short";
      break;
    }

    code.plantInstruction(new FieldRefInstruction(opc_getstatic,
						  className,
						  "TYPE",
						  CStdType.Class.getSignature()));
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private	CType		type;
}
