/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JClassImport.java,v 1.10 1999/10/21 12:23:09 vgp Exp $
 */

package at.dms.kjc;

import at.dms.compiler.CWarning;
import at.dms.compiler.Compiler;
import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;

/**
 * This class represents the "package at.dms.kjc.Main" statement
 */
public class JClassImport extends JPhylum {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * construct a class name
   * @param where the token reference of this node
   * @param name the class name
   */
  public JClassImport(TokenReference where, String name, JavaStyleComment[] comments) {
    super(where);

    this.name = name;
    this.comments = comments;
    this.used = false;

    assert(name.indexOf('.') == -1);
    int		index = name.lastIndexOf('/');

    this.ident = index == -1 ? name : name.substring(index + 1).intern();
  }

  // ----------------------------------------------------------------------
  // ACCESSORS & MUTATORS
  // ----------------------------------------------------------------------

  /**
   * @return the class name defined by this statement
   */
  public String getClassName() {
    return name;
  }

  /**
   * @return the class name defined by this statement
   */
  public String getClassIdent() {
    return ident;
  }

  /**
   * States that specified class is used.
   */
  public void setUsed() {
    used = true;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   */
  public void checkBody(Compiler compiler) {
    if (!used && getTokenReference() != TokenReference.NO_REF) {
      compiler.reportTrouble(new CWarning(getTokenReference(),
					  "unused-class-import",
					  name.replace('/', '.'),
					  null));
    }
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is useful to debug and tune compilation process
   * @param p the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    if (comments != null) {
      p.printComments(comments);
    }
    p.printClassImport(name);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private final String			name;
  private final String			ident;
  private final JavaStyleComment[]	comments;
  private boolean			used;
}
