/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JConstructorBlock.java,v 1.37 1999/10/16 23:25:35 vgp Exp $
 */

package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * JLS 14.2: Block
 *
 * TA block is a sequence of statements and local variable declaration
 * statements within braces.
 */
public class JConstructorBlock extends JBlock {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * @param	where		the line of this node in the source code
   */
  public JConstructorBlock(TokenReference where, JStatement[] body) {
    super(where, body, null);
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check statement.
   * @param	context		the actual context of analyse
   * @param	head		some declarations that take place in block
   *				such as int foo(int i) {}
   * @exception	PositionedError		if the check fails
   * @exception	UnpositionedError	this error will be positioned soon
   */
  public void checkStatement(CBodyContext context) throws PositionedError {
    CBodyContext	inside = context; // !!!context.createBodyContext();

    sourceClass = (CSourceClass)context.getClassContext().getCClass();
    if (body.length == 0 || !(body[0] instanceof JExplicitConstructorInvocation)) {
      if (context.getClassContext().getCClass().getType().equals(CStdType.Object)) {
	if (body.length == 0) {
	  explicitSuper = new JReturnStatement(getTokenReference(), null, null);
	}
      } else {
	explicitSuper = new JExplicitConstructorInvocation(getTokenReference(),
							   null,
							   "super",
							   JExpression.EMPTY,
							   null);
      }
    } else if (body[0] instanceof JExplicitConstructorInvocation) {
      explicitSuper = body[0];
      body[0] = new JEmptyStatement(getTokenReference(), null);
    }

    if (sourceClass.isInnerClass()) {
      paramsLength = context.getMethodContext().getCMethod().getParameters().length;
    }

    if (context.getClassContext().hasInitializer() &&
	!((explicitSuper instanceof JExplicitConstructorInvocation) &&
	  ((JExplicitConstructorInvocation)explicitSuper).isThisInvoke())) {
      // "Block$();" ===>
      JNameExpression ident = new JNameExpression(getTokenReference(), JAV_INIT);
      JExpression expr = new JMethodCallExpression(getTokenReference(), ident, JExpression.EMPTY);
      blockCall = new JExpressionStatement(getTokenReference(), expr, null);
      // "Block$();" <===
    }

    if (explicitSuper != null) {
      explicitSuper.checkStatement(inside);
      if (explicitSuper instanceof JExplicitConstructorInvocation &&
	  ((JExplicitConstructorInvocation)explicitSuper).isThisInvoke()) {
	context.getClassContext().markAllFieldToInitialized();
      }
    }

    if (blockCall != null) {
      blockCall.checkStatement(inside);
    }

    super.checkStatement(inside);

    inside.close(getTokenReference());
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

   /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    code.setLineNumber(getTokenReference().getLine());

    if (explicitSuper != null) {
      explicitSuper.genCode(code);
    }

    if (sourceClass.isInnerClass()) {
      sourceClass.genInit(code, paramsLength);
    }

    if (blockCall != null) {
      blockCall.genCode(code);
    }

    for (int i = 0; i < body.length; i++) {
      body[i].genCode(code);
    }
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  JStatement		explicitSuper;
  JStatement		blockCall;
  CSourceClass		sourceClass;
  int			paramsLength;
}
