/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JMemberDeclaration.java,v 1.20 1999/10/21 12:23:10 vgp Exp $
 */

package at.dms.kjc;

import java.util.StringTokenizer;

import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;


/**
 * This class represents a java class in the syntax tree
 */
public abstract class JMemberDeclaration extends JPhylum {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	modifiers	list of modifiers
   * @param	ident		the name of this method
   * @param	javadoc	is this member deprecated
   */
  public JMemberDeclaration(TokenReference where,
			    JJavadocComment javadoc,
			    JavaStyleComment[] comments) {
    super(where);
    this.comments = comments;
    this.javadoc = javadoc;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS (INTERFACE)
  // ----------------------------------------------------------------------

  /**
   * Returns true if this member is deprecated
   */
  public boolean isDeprecated() {
    return Main.deprecationMode() && javadoc != null && javadoc.isDeprecated();
  }

  /**
   * @return	the interface
   */
  public CField getField() {
    return export.getField();
  }

  /**
   * @return	the interface
   */
  public CMethod getMethod() {
    return export.getMethod();
  }

  /**
   * @return	the interface
   */
  public CClass getCClass() {
    return export.getCClass();
  }

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param	p		the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    genComments(p);
  }

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param	p		the printwriter into the code is generated
   */
  public void genComments(CJavaPrettyPrinter p) {
    if (comments != null) {
      p.printComments(comments);
    }
    if (javadoc != null) {
      p.printJavadoc(javadoc);
    }
  }

  // ----------------------------------------------------------------------
  // PROTECTED ACCESSORS
  // ----------------------------------------------------------------------

  protected void setInterface(CMember export) {
    this.export = export;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private	CMember				export;
  private	JJavadocComment			javadoc;
  private	JavaStyleComment[]		comments;
}
