/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JPackageImport.java,v 1.9 1999/10/21 12:23:10 vgp Exp $
 */

package at.dms.kjc;

import java.util.Vector;
import at.dms.compiler.Compiler;
import at.dms.compiler.CWarning;
import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;

/**
 * This class represents the "package at.dms.kjc" statement
 */
public class JPackageImport extends JPhylum {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * construct a package name
   *
   * @param	where		the token reference of this node
   * @param	name		the package name
   */
  public JPackageImport(TokenReference where, String name, JavaStyleComment[] comments) {
    super(where);

    this.name = name;
    this.comments = comments;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS & MUTATORS
  // ----------------------------------------------------------------------

  /**
   * Returns the package name defined by this declaration.
   *
   * @return	the package name defined by this declaration
   */
  public String getName() {
    return name;
  }

  /**
   * States that specified class in imported package is used.
   * @param	clazz		the class that is used.
   */
  public void setClassUsed(String clazz) {
    if (classesUsed == null) {
      classesUsed = new Vector();
    }
    classesUsed.addElement(clazz);
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   */
  public void checkBody(Compiler compiler) {
    if (getTokenReference() != TokenReference.NO_REF) {
      if (classesUsed == null) {
	compiler.reportTrouble(new CWarning(getTokenReference(),
					    "unused-package-import",
					    name.replace('/', '.'),
					    null));
      } else if (classesUsed.size() < 5) {
	// !!! graf 990815: replace by symbolic constant (or command line argument)
	StringBuffer	buffer = new StringBuffer();

	for (int i = 0; i < classesUsed.size(); i++) {
	  if (i != 0) {
	    buffer.append(", ");
	  }
	  buffer.append((name + "/" + (String)classesUsed.elementAt(i)).replace('/', '.'));
	}
	compiler.reportTrouble(new CWarning(getTokenReference(),
					    "replace-package-import",
					    name.replace('/', '.'),
					    buffer.toString()));
      }
    }
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param p the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    if (comments != null) {
      p.printComments(comments);
    }
    p.printPackageImport(name);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private final String			name;
  private final JavaStyleComment[]	comments;
  private Vector			classesUsed;
}
