/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JStatement.java,v 1.16 1999/10/21 12:23:10 vgp Exp $
 */

package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;

/**
 * JLS 14.4: The root class for all statements
 */
public abstract class JStatement extends JPhylum {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

 /**
   * Construct a node in the parsing tree
   * @param where the line of this node in the source code
   */
  public JStatement(TokenReference where, JavaStyleComment[] comments) {
    super(where);
    this.comments = comments;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check statement
   * @param	context		the actual context of analyse
   * @exception	PositionedError		if the check fails
   */
  public void checkBody(CBodyContext context) throws PositionedError {
    checkStatement(context);
  }

  /**
   * Check statement
   * @param	context		the actual context of analyse
   * @exception	PositionedError		if the check fails
   * @exception	UnpositionedError	this error will be positioned soon
   */
  public abstract void checkStatement(CBodyContext context) throws PositionedError;

  /**
   * Adds a compiler error.
   * @param	context		the context in which the error occurred
   * @param	key		the message ident to be displayed
   * @param	params		the array of parameters
   *
   */
  protected void fail(CContext context, String key, Object[] params)
    throws PositionedError
  {
    throw new CLineError(getTokenReference(), key, params);
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public abstract void genCode(CodeSequence code);

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param	p		the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    if (comments != null) {
      p.printComments(comments);
    }
  }

  /**
   * Returns the comments
   */
  public JavaStyleComment[] getComments() {
    return comments;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  private JavaStyleComment[]	comments;
}
