/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JTypeDeclarationStatement.java,v 1.10 1999/10/14 17:08:56 vgp Exp $
 */

package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * JLS 14.3: Local type declaration declaration statement
 *
 * A local type declaration declaration statement declares one type declaration in a body of a method.
 */
public class JTypeDeclarationStatement extends JStatement {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * @param	where		the line of this node in the source code
   * @param	vars		the variables declared by this statement
   */
  public JTypeDeclarationStatement(TokenReference where, JTypeDeclaration decl) {
    super(where, null);
    this.decl = decl;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check statement.
   * @param	context		the actual context of analyse
   * @exception	PositionedError		if the check fails
   */
  public void checkStatement(CBodyContext context) throws PositionedError {
    CClass	owner = context.getClassContext().getCClass();
    String	prefix = owner.getQualifiedName() + "$" + 
      context.getClassContext().getNextSyntheticIndex();

    decl.generateInterface(owner, prefix);

    try {
      context.getBlockContext().addClass(decl.getCClass());
    } catch (UnpositionedError cue) {
      throw cue.getError(getTokenReference());
    }

    decl.checkInterface(context);
    decl.checkInitializers(context);
    decl.checkTypeBody(context);
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param	p		the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    super.genJavaCode(p);
    p.printTypeDeclarationStatement(decl);
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    // nothing to do here
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private JTypeDeclaration		decl;
}
