/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: ParseManager.java,v 1.27 1999/10/14 17:08:56 vgp Exp $
 */

package at.dms.kjc;

import java.io.File;
import java.io.IOException;

import at.dms.compiler.Compiler;
import at.dms.compiler.InputBuffer;

/**
 * This class manage paser switching between Java and SQL
 */
public class ParseManager {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * This class manages parser switching between Java and ...
   *
   * @param	file		the file to read
   * @param	name		the name of the file
   */
  protected ParseManager(Compiler compiler, File file, String name)
    throws IOException
  {
    this.compiler = compiler;
    this.buffer = new InputBuffer(name, file);
    this.javaLexer = null;
  }

  /**
   * Factory
   */
  public static ParseManager getParseManager(Compiler compiler,
					     File file,
					     String name)
    throws IOException
  {
    return new ParseManager(compiler, file, name);
  }

  /**
   * Release resources
   * @deprecated	!!! REMOVE
   */
  public void close() {
    buffer.release();
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Returns the java lexer
   */
  public CHandLexer getJavaLexer() {
    if (javaLexer == null) {
      javaLexer = new JavaScanner(compiler, buffer);
    }

    return javaLexer;
  }

  /**
   * Returns the java parser
   */
  public JavaParser getJavaParser() {
    return new JavaParser(compiler, getJavaLexer(), buffer.file);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  // !!! make private: currently used by MParseManager ...
  protected Compiler			compiler;
  // !!! make private: currently used by MParseManager ...
  protected InputBuffer			buffer;
  private CHandLexer			javaLexer;
}
