/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: java.g,v 1.46 1999/10/21 12:23:11 vgp Exp $
 */

/*
 * Grammar based on:
 * Java 1.1 Recognizer Grammar
 *
 * Authors:
 *		John Mitchell		johnm@non.net
 *		Terence Parr		parrt@magelang.com
 *		John Lilley		jlilley@empathy.com
 *		Scott Stanchfield	thetick@magelang.com
 */

header { package at.dms.kjc; }
{
import java.util.Vector;

import at.dms.compiler.Compiler;
import at.dms.compiler.CWarning;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;
import at.dms.util.VectorCache;
}

// ----------------------------------------------------------------------
// THE PARSER STARTS HERE
// ----------------------------------------------------------------------

class JavaParser extends Parser;

options {
  k = 2;				// two token lookahead
  importVocab = Java;
  exportVocab = Java;
  codeGenMakeSwitchThreshold = 3;	// Some optimizations
  codeGenBitsetTestThreshold = 4;
  defaultErrorHandler = false;		// Don't generate parser error handlers
  superClass="at.dms.kjc.Parser";
}
{
  public JavaParser(Compiler compiler, CHandLexer lexer, String file) {
    super(compiler, lexer, file, MAX_LOOKAHEAD);
  }
}

// JLS 7.3: CompilationUnit is the goal symbol (2.1) for the syntactic grammar
// In Java, this is a single file.This is the default start rule for this parser
jCompilationUnit
  returns [JCompilationUnit self = null]
{
  TokenReference		sourceRef = buildTokenReference();
  JPackageName			pack = JPackageName.UNNAMED;
  CParseCompilationUnitContext	context = CParseCompilationUnitContext.getInstance();
}
:
  ( pack = jPackageDeclaration[] )?
    {context.setPackage(pack); }
  jDefaultImport[context]
  ( jImportDeclaration[context] )*
  ( jTypeDeclaration[context] )*

  EOF
{
  self = new JCompilationUnit(sourceRef, 
	                      pack,
			      context.getPackageImports(),
			      context.getClassImports(),
			      context.getTypeDeclarations());
  context.release();
}
;

jDefaultImport[CParseCompilationUnitContext context]
:
  {
    context.addPackageImport(CParseCompilationUnitContext.JAVA_LANG);
  }
;

// JLS 7.4: Package Declaration
jPackageDeclaration  []
  returns [JPackageName self = null]
{
  String	name = null;
}
:
  "package" name = jName[] SEMI
    { self = new JPackageName(buildTokenReference(), name, getStatementComment()); }
;

// JLS 7.5: Import Declarations
// We do not use jName here because it can be followed by ".*", thus
// requiring a lookahead of 2
jImportDeclaration [CParseCompilationUnitContext cunit]
{
  String	name = null;
  boolean	star = false;
}
:
  "import" name = jName[]
  ( DOT STAR { star = true; } )?
  SEMI
    {
      if (star) {
	cunit.addPackageImport(new JPackageImport(buildTokenReference(), name, getStatementComment()));
      } else {
	cunit.addClassImport(new JClassImport(buildTokenReference(), name, getStatementComment()));
      }
    }
;

// JLS 7.6: Type Declarations
// The syntax for modifiers is relaxed like in the LALR(1) grammar (JLS 19.1.2, 19.7).
// A later stage of compiler analysis then sorts out the precise role of each modifier
// and whether it is permitted in a given context.
jTypeDeclaration [CParseCompilationUnitContext context]
{
  int			mods = 0;
  JTypeDeclaration	decl = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  (
    mods = jModifiers[]
    (
      decl = jClassDeclaration[mods]
    |
      decl = jInterfaceDeclaration[mods]
    )
      { context.addTypeDeclaration(decl); }
  )
|
  SEMI
    { reportTrouble(new CWarning(sourceRef, "stray-semicolon", null)); }
;

// JLS 19.7: Modifiers
jModifiers []
  returns [int self = 0]
{
  int		mod;
}
:
  (
    mod = jModifier[]
      {
	// !!! warning/error if ((mod & self) != 0)
	self += mod;
      }
  )*
;

jModifier []
  returns [int self = 0]
:
  "public" { self = at.dms.kjc.Constants.ACC_PUBLIC; }
|
  "protected" { self = at.dms.kjc.Constants.ACC_PROTECTED; }
|
  "private" { self = at.dms.kjc.Constants.ACC_PRIVATE; }
|
  "static" { self = at.dms.kjc.Constants.ACC_STATIC; }
|
  "abstract" { self = at.dms.kjc.Constants.ACC_ABSTRACT; }
|
  "final" { self = at.dms.kjc.Constants.ACC_FINAL; }
|
  "native" { self = at.dms.kjc.Constants.ACC_NATIVE; }
|
  "synchronized" { self = at.dms.kjc.Constants.ACC_SYNCHRONIZED; }
|
  "transient" { self = at.dms.kjc.Constants.ACC_TRANSIENT; }
|
  "volatile" { self = at.dms.kjc.Constants.ACC_VOLATILE; }
;

// JLS 8.1 Class Declaration
jClassDeclaration [int mods]
  returns [JClassDeclaration self = null]
{
  String		superClass = null;
  CClassType[]		interfaces = CClassType.EMPTY;
  CParseClassContext	context = CParseClassContext.getInstance();
  TokenReference	sourceRef = buildTokenReference();
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  "class" ident:IDENT
  ( "extends" superClass = jName[] )?
  ( "implements" interfaces = jNameList[] )?
  jClassBody[context]
    {
      self = new JClassDeclaration(sourceRef,
				   mods,
				   ident.getText(),
				   superClass,
				   interfaces,
				   context.getFields(),
				   context.getMethods(),
				   context.getInnerClasses(),
				   context.getBody(),
				   javadoc,
				   comments);
      context.release();
    }
;

// JLS 8.1.4 Class Body and Member Declarations
jClassBody [CParseClassContext context]
:
  LCURLY
  (
    jClassBodyDeclaration[context]
  |
    SEMI
      { reportTrouble(new CWarning(buildTokenReference(), "stray-semicolon", null)); }
  )*
  RCURLY
;

// JLS 8.1.4 Class Body and Member Declarations
jInnerClassBody [String name]
  returns [JClassDeclaration self = null]
{
  TokenReference	sourceRef = buildTokenReference();
  CParseClassContext	context = CParseClassContext.getInstance();
}
:
  LCURLY
  (
    jClassBodyDeclaration[context]
  |
    SEMI
      { reportTrouble(new CWarning(sourceRef, "stray-semicolon", null)); }
  )*
  RCURLY
{
  self = new JClassDeclaration(sourceRef,
			       0,
			       name,
			       null,
			       new CClassType[]{},
			       context.getFields(),
			       context.getMethods(),
			       context.getInnerClasses(),			       
			       context.getBody(),
			       getJavadocComment(),
			       getStatementComment());
  context.release();
}
;

// JLS 8.1.4 + ICS: Class Body and Member Declarations
//
// !!! CHECK COMMENT
// Now the various things that can be defined inside a class or interface...
// Note that not all of these are really valid in an interface (constructors,
// for example), and if this grammar were used for a compiler there would
// need to be some semantic checks to make sure we're doing the right thing...
jClassBodyDeclaration [CParseClassContext context]
{
  int			mods = 0;
  JClassBlock		block = null;
}
:
  mods = jModifiers[]
  (
    jConstructorDeclaration[context, mods]
  |
    jTypeMemberDeclaration[context, mods]
  |
    jInnerTypeDeclaration[context, mods]
  )
|
  // JLS 8.6: Class and Instance Initializer
  "static" block = jClassBlock[]
    { 
      block.setStatic(true);
      context.addBlockInitializer(block); 
    }
|
  block = jClassBlock[]
    { context.addBlockInitializer(block); }
;

// JLS 9.1 Interface Declaration
jInterfaceDeclaration [int mods]
  returns [JInterfaceDeclaration self = null]
{
  CClassType[]		interfaces =  CClassType.EMPTY;
  CParseClassContext	context = CParseClassContext.getInstance();
  TokenReference	sourceRef = buildTokenReference();
}
:
  "interface" ident:IDENT
  ( "extends" interfaces = jNameList[] )?
  jInterfaceBody[context]
    {
      self = new JInterfaceDeclaration(sourceRef,
				       mods,
				       ident.getText(),
				       interfaces,
				       context.getFields(),
				       context.getMethods(),
				       context.getInnerClasses(),			       
				       context.getBody(),
				       getJavadocComment(),
				       getStatementComment());
      context.release();
    }
;

// JLS 9.1.4 Interface Body and Member Declarations
jInterfaceBody [CParseClassContext context]
:
  LCURLY
  (
    jInterfaceBodyDeclaration[context]
  |
    SEMI
      { reportTrouble(new CWarning(buildTokenReference(), "stray-semicolon", null)); }
  )*
  RCURLY
;

jInterfaceBodyDeclaration[CParseClassContext context]
{
  int			mods = 0;
}
:
  mods = jModifiers[]
  (
    jTypeMemberDeclaration[context, mods]
  |
    jInnerTypeDeclaration[context, mods]
  )
;

// Production used to factor type
jTypeMemberDeclaration [CParseClassContext context, int mods]
{
  CType			type;
}
:
  type = jType[]
  (
    jMethodDeclaration[context, mods, type]
  |
    jFieldDeclaration[context, mods, type]
  )
;

// ICS: Extensions to class and interface body declarations
jInnerTypeDeclaration[CParseClassContext context, int mods]
{
  JTypeDeclaration	inner = null;
}
:
  (
    inner = jClassDeclaration[mods]
  |
    inner = jInterfaceDeclaration[mods]
  )
  { context.addInnerDeclaration(inner); }
;

// !!!JLS
// A type specification is a type name with possible brackets afterwards
// (which would make it an array type).
// A type name. which is either a (possibly qualified) class name or
// a primitive type
jType []
  returns [CType self = null]
{
  String		name = null;
  int			bounds = 0;
}
:
  (
    name = jName[]
      { self = CClassType.lookup(name); }
  |
    self = jPrimitiveType []
  )
  ( 
    options { warnWhenFollowAmbig = false; } :
    LBRACK RBRACK { bounds += 1; } 
  )*
    {
      if (bounds > 0) {
	self = new CArrayType(self, bounds);
      }
    }
;

// !!!JLS The primitive types.
jPrimitiveType []
  returns [CType self = null]
:
  "boolean" { self = CStdType.Boolean; }
|
  "byte" { self = CStdType.Byte; }
|
  "char" { self = CStdType.Char; }
|
  "double" { self = CStdType.Double; }
|
  "float" { self = CStdType.Float; }
|
  "int" { self = CStdType.Integer; }
|
  "long" { self = CStdType.Long; }
|
  "short" { self = CStdType.Short; }
|
  "void" { self = CStdType.Void; }
;

// JLS 8.6 Constructor Declaration
jConstructorDeclaration [CParseClassContext context, int mods]
{
  JFormalParameter[]	parameters;
  CClassType[]		throwsList = CClassType.EMPTY;
  Vector		body = VectorCache.request(VectorCache.BIG);
  JStatement		stmt;
  TokenReference	sourceRef = buildTokenReference();
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  ident:IDENT
  parameters = jFormalParameters[JLocalVariable.DES_PARAMETER]
  ( "throws" throwsList = jNameList[] )?
  LCURLY
  (
    ( jExplicitConstructorInvocation[] ) =>
      stmt = jExplicitConstructorInvocation[]
        { body.addElement(stmt); }
  )?
  ( stmt = jBlockStatement[] { body.addElement(stmt); } )*
  RCURLY
    {
      context.addMethodDeclaration(new JConstructorDeclaration(sourceRef,
							       mods,
							       ident.getText(),
							       parameters,
							       throwsList,
							       new JConstructorBlock(sourceRef, 
										     (JStatement[])at.dms.compiler.Utils.vectorToArray(body, JStatement.class)),
							       javadoc,
							       comments));
      VectorCache.release(body);
    }
;

// JLS 8.3 Field Declaration
jFieldDeclaration [CParseClassContext context, int mods, CType type]
{
  JVariableDefinition	field;
  TokenReference	sourceRef = buildTokenReference();
}
:
  field = jVariableDeclarator[mods, type]
    { context.addFieldDeclaration(new JFieldDeclaration(sourceRef, field, getJavadocComment(),
							getStatementComment())); }
  (
    COMMA field = jVariableDeclarator[mods, type]
      { context.addFieldDeclaration(new JFieldDeclaration(sourceRef, field, getJavadocComment(),
							  getStatementComment())); }
  )*
  SEMI
;

// !!!JLS
// Declaration of a variable.  This can be a class/instance variable,
//   or a local variable in a method
// It can also include possible initialization.
jVariableDeclarator [int mods, CType type]
  returns [JVariableDefinition self = null]
{
  int			bounds = 0;
  JExpression		expr = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  ident:IDENT
  ( LBRACK RBRACK { bounds += 1; } )*
  ( ASSIGN expr = jVariableInitializer[] )?
    {
      if (bounds > 0) {
	reportTrouble(new CWarning(sourceRef, "old-style-array-bounds", null));
	type = new CArrayType(type, bounds);
      }

	self = new JVariableDefinition(sourceRef, mods, type, ident.getText(), expr);
      }
;


// JLS 8.4 Method Declaration
jMethodDeclaration [CParseClassContext context, int mods, CType type]
{
  JFormalParameter[]	parameters;
  CClassType[]		throwsList = CClassType.EMPTY;
  int			bounds = 0;
  JBlock		block = null;
  TokenReference	sourceRef = buildTokenReference();
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  ident:IDENT
  parameters = jFormalParameters[JLocalVariable.DES_PARAMETER]
  ( LBRACK RBRACK { bounds += 1; } )*
  ( "throws" throwsList = jNameList[] )?
  ( block = jBlock[] | SEMI )
    {
      if (bounds > 0) {
	reportTrouble(new CWarning(sourceRef, "old-style-array-bounds", null));
	type = new CArrayType(type, bounds);
      }

      context.addMethodDeclaration(new JMethodDeclaration(sourceRef,
							  mods,
							  type,
							  ident.getText(),
							  parameters,
							  throwsList,
							  block,
							  javadoc,
							  comments));
    }
;

// !!!JLS
jFormalParameters [int desc]
 returns [JFormalParameter[] self = JFormalParameter.EMPTY]
{
  JFormalParameter		elem = null;
  Vector			vect = VectorCache.request(VectorCache.SMALL);
}
:
  LPAREN
  (
   elem = jFormalParameter[desc]
      { vect.addElement(elem); }
    ( COMMA elem = jFormalParameter[desc] { vect.addElement(elem); } )*
    {
      self = (JFormalParameter[])at.dms.compiler.Utils.vectorToArray(vect, JFormalParameter.class); 
      VectorCache.release(vect);
    }
  )?
  RPAREN
;

// !!!JLS
jFormalParameter [int desc]
 returns [JFormalParameter self = null]
{
  boolean	isFinal = false;
  int		bounds = 0;
  CType		type;
  TokenReference	sourceRef = buildTokenReference();
}
:
  ( "final" { isFinal = true; } )?
  type = jType[]
  ident:IDENT
  ( LBRACK RBRACK { bounds += 1; } )*
    {
      if (bounds > 0) {
	reportTrouble(new CWarning(sourceRef, "old-style-array-bounds", null));
	type = new CArrayType(type, bounds);
      }

      self = new JFormalParameter(sourceRef, desc, type, ident.getText(), isFinal);
    }
;

// ----------------------------------------------------------------------
// IMPLEMENTATION HELPERS
// ----------------------------------------------------------------------

// A (possibly-qualified) java name.
jName []
  returns [String self = null]
{
  StringBuffer buffer = null;
}
:
  i:IDENT 
  (
    options {warnWhenFollowAmbig = false;} :
    DOT j:IDENT { (buffer == null ? (buffer = new StringBuffer(i.getText())) : buffer).append('/').append(j.getText()); }
  )*
    { self = buffer == null ? i.getText() : buffer.toString(); }
;

jNameList []
  returns [CClassType[] self = null]
{
  String	name;
  Vector			vect = VectorCache.request(VectorCache.SMALL);
}
:
  name = jName[] { vect = new Vector(); vect.addElement(CClassType.lookup(name)); }
  ( COMMA name = jName[] { vect.addElement(CClassType.lookup(name)); } )*
{
  self = (CClassType[])at.dms.compiler.Utils.vectorToArray(vect, CClassType.class);
  VectorCache.release(vect);
}
;

// ######################################################################
// ######################################################################
// ######################################################################
// ######################################################################


jVariableInitializer []
  returns [JExpression self = null]
:
  self = jArrayInitializer[]
|
  self = jExpression[]
;

jArrayInitializer []
  returns [JArrayInitializer self = null]
{
  JExpression		expr = null;
  Vector		vect = VectorCache.request(VectorCache.SMALL);
  TokenReference	sourceRef = buildTokenReference();
}
:
  LCURLY
  (
    expr = jVariableInitializer[]
      { vect.addElement(expr); }
    (
      // CONFLICT: does a COMMA after an initializer start a new
      // initializer or start the option ',' at end?
      // ANTLR generates proper code by matching the comma as soon as possible.
      options { warnWhenFollowAmbig = false; } :

      COMMA expr = jVariableInitializer[]
        { vect.addElement(expr); }
    )*
    (
      COMMA
        { reportTrouble(new CWarning(sourceRef, "stray-comma", null)); }
    )?
  )?
  RCURLY
    {
       self = new JArrayInitializer(sourceRef, 
				   (JExpression[])at.dms.compiler.Utils.vectorToArray(vect, JExpression.class)); 
      VectorCache.release(vect);
  }
;

jExplicitConstructorInvocation []
  returns [JStatement self = null]
{
  JExpression[]		args = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "this" args = jArguments[] SEMI
    { self = new JExplicitConstructorInvocation(sourceRef, null, "this", args, getStatementComment()); }
|
  //8.6.5 Constructor Body !!! correspond to java-lang-spec
  "super" args = jArguments[] SEMI
    { self = new JExplicitConstructorInvocation(sourceRef, null, "super", args, getStatementComment()); }
  //( ( jPrimaryExpression[] DOT )? "super" LPAREN ) =>
  //( jPrimaryExpression[] DOT )? "super" jArguments[] SEMI
;

//
// This expansion has been written this way instead of:
//   jAssignment[] | jConditionalExpression[]
// for performance reasons.
// However, it is a weakening of the grammar for it allows the LHS of
// assignments to be any conditional expression whereas it can only be
// a primary expression.  Consider adding a semantic predicate to work
// around this.
//

jExpression []
  returns [JExpression self = null]
{
  int			operator = -1;
  JExpression		right;
}
:
  self = jConditionalExpression[]
  (
    operator = jAssignmentOperator[] right = jExpression[]
      { self = JAssignmentExpression.create(self.getTokenReference(), operator, self, right); }
  )?
;

jAssignmentOperator []
  returns [int self = -1]
:
  ASSIGN { self = Constants.OPE_SIMPLE; }
|
  PLUS_ASSIGN { self = Constants.OPE_PLUS; }
|
  MINUS_ASSIGN { self = Constants.OPE_MINUS; }
|
  STAR_ASSIGN { self = Constants.OPE_STAR; }
|
  SLASH_ASSIGN { self = Constants.OPE_SLASH; }
|
  PERCENT_ASSIGN { self = Constants.OPE_PERCENT; }
|
  SR_ASSIGN { self = Constants.OPE_SR; }
|
  BSR_ASSIGN { self = Constants.OPE_BSR; }
|
  SL_ASSIGN { self = Constants.OPE_SL; }
|
  BAND_ASSIGN { self = Constants.OPE_BAND; }
|
  BXOR_ASSIGN { self = Constants.OPE_BXOR; }
|
  BOR_ASSIGN { self = Constants.OPE_BOR; }
;


jConditionalExpression []
  returns [JExpression self = null]
{
  JExpression		middle;
  JExpression		right;
}
:
  self = jConditionalOrExpression[]
  (
    QUESTION middle = jExpression[] COLON right = jConditionalExpression[]
      { self = new JConditionalExpression(self.getTokenReference(), self, middle, right); }
  )?
;

jConditionalOrExpression []
  returns [JExpression self = null]
{
  JExpression		right;
}
:
  self = jConditionalAndExpression[]
  (
    LOR right = jConditionalAndExpression[]
      { self = new JConditionalOrExpression(self.getTokenReference(), self, right); }
  )*
;

jConditionalAndExpression []
  returns [JExpression self = null]
{
  JExpression		right;
}
:
  self = jInclusiveOrExpression[]
  (
    LAND right = jInclusiveOrExpression[]
      { self = new JConditionalAndExpression(self.getTokenReference(), self, right); }
  )*
;

jInclusiveOrExpression []
  returns [JExpression self = null]
{
  JExpression		right;
}
:
  self = jExclusiveOrExpression[]
  (
    BOR right = jExclusiveOrExpression[]
      { self = new JBitwiseExpression(self.getTokenReference(), Constants.OPE_BOR, self, right); }
  )*
;

jExclusiveOrExpression []
  returns [JExpression self = null]
{
  JExpression		right;
}
:
  self = jAndExpression[]
  (
    BXOR right = jAndExpression[]
      { self = new JBitwiseExpression(self.getTokenReference(), Constants.OPE_BXOR, self, right); }
  )*
;

jAndExpression []
  returns [JExpression self = null]
{
  JExpression		right;
}
:
  self = jEqualityExpression[]
  (
    BAND right = jEqualityExpression[]
      { self = new JBitwiseExpression(self.getTokenReference(), Constants.OPE_BAND, self, right); }
  )*
;

jEqualityExpression []
  returns [JExpression self = null]
{
  int			operator = -1;
  JExpression		right;
}
:
  self = jInstanceOfExpression[]
  (
    (
      EQUAL { operator = Constants.OPE_EQ; }
    |
      NOT_EQUAL { operator = Constants.OPE_NE; }
    )
    right = jInstanceOfExpression[]
      { self = new JEqualityExpression(self.getTokenReference(), operator, self, right); }
  )*
;

jInstanceOfExpression []
  returns [JExpression self = null]
{
  CType			type;
}
:
  self = jRelationalExpression[]
  (
    "instanceof" type = jType[]
      { self = new JInstanceofExpression(self.getTokenReference(), self, type); }
  )?
;

jRelationalExpression []
  returns [JExpression self = null]
{
  int			operator = -1;
  JExpression		right;
}
:
  self = jShiftExpression[]
  (
    (
      LT { operator = Constants.OPE_LT; }
    |
      GT { operator = Constants.OPE_GT; }
    |
      LE { operator = Constants.OPE_LE; }
    |
      GE { operator = Constants.OPE_GE; }
    )
    right = jShiftExpression[]
      { self = new JRelationalExpression(self.getTokenReference(), operator, self, right); }
  )*
;

jShiftExpression []
  returns [JExpression self = null]
{
  int			operator = -1;
  JExpression		right;
}
:
  self = jAdditiveExpression[]
  (
    (
      SL { operator = Constants.OPE_SL; }
    |
      SR { operator = Constants.OPE_SR; }
    |
      BSR { operator = Constants.OPE_BSR; }
    )
    right = jAdditiveExpression[]
      { self = new JShiftExpression(self.getTokenReference(), operator, self, right); }
  )*
;

jAdditiveExpression []
  returns [JExpression self = null]
{
  int		oper = Constants.OPE_PLUS;
  JExpression		right;
}
:
  self = jMultiplicativeExpression[]
  (
    (
      PLUS { oper = Constants.OPE_PLUS; }
    |
      MINUS { oper = Constants.OPE_MINUS; }
    )
    right = jMultiplicativeExpression[]
      {
	if (oper == Constants.OPE_PLUS) {
	  self = new JAddExpression(self.getTokenReference(), self, right);
	} else {
	  //if (oper == Constants.OPE_MINUS)
	  self = new JMinusExpression(self.getTokenReference(), self, right);
	}
      }
  )*
;

jMultiplicativeExpression []
  returns [JExpression self = null]
{
  int			type = 0;
  JExpression		right;
}
:
  self = jUnaryExpression[]
  (
    (
      STAR { type = 1; }
    |
      SLASH { type = 2; }
    |
      PERCENT { type = 3; }
    )
    right = jUnaryExpression[]
      {
	if (type == 1) {
          self = new JMultExpression(self.getTokenReference(), self, right);
	} else if (type == 2) {
          self = new JDivideExpression(self.getTokenReference(), self, right);
	} else {
          self = new JModuloExpression(self.getTokenReference(), self, right);
	}
      }
  )*
;

jUnaryExpression []
  returns [JExpression self = null]
{
  int			operator = -1;
}
:
  (
    PLUS { operator = Constants.OPE_PLUS; }
  |
    MINUS { operator = Constants.OPE_MINUS; }
  )
  self = jUnaryExpression[]
    { self = new JUnaryExpression(self.getTokenReference(), operator, self); }
|
  self = jPreIncrementExpression[]
|
  self = jPreDecrementExpression[]
|
  self = jUnaryExpressionNotPlusMinus[]
;

jPreIncrementExpression []
  returns [JExpression self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  INC self = jPrimaryExpression[]
    { self = new JPrefixExpression(sourceRef, Constants.OPE_PREINC, self); }
;

jPreDecrementExpression []
  returns [JExpression self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  DEC self = jPrimaryExpression[]
    { self = new JPrefixExpression(sourceRef, Constants.OPE_PREDEC, self); }
;

jCastLookahead []
{
  Object	warn = null;

}
:
  LPAREN warn = jPrimitiveType[]
|
  LPAREN warn = jName[] 
  (
    LBRACK RBRACK
  |
    RPAREN ( BNOT | LNOT | LPAREN | IDENT | "this" | "super" | "new" | warn = jLiteral[] )
  )
;

jUnaryExpressionNotPlusMinus []
  returns [JExpression self = null]
{
  int			operator = -1;
}
:
  (
    BNOT { operator = Constants.OPE_BNOT; }
  |
    LNOT { operator = Constants.OPE_LNOT; }
  )
  self = jUnaryExpression[]
    { self = new JUnaryExpression(self.getTokenReference(), operator, self); }
|
  ( jCastLookahead[] ) =>
  self = jCastExpression[]
|
  self = jPostfixExpression[]
;

// This production is to determine lookahead only.  The LOOKAHEAD specifications
// below are not used, but they are there just to indicate that we know about
// this.

jPostfixExpression []
  returns [JExpression self = null]
:
  self = jPrimaryExpression[]
  (
    INC
      { self = new JPostfixExpression(buildTokenReference(), Constants.OPE_POSTINC, self); }
  |
    DEC
      { self = new JPostfixExpression(buildTokenReference(), Constants.OPE_POSTDEC, self); }
  )?
;

jCastExpression []
  returns [JCastExpression self = null]
{
  JExpression		expr;
  CType			dest;
  TokenReference	sourceRef = buildTokenReference();
}
:
  LPAREN 
    dest = jType[] 
  RPAREN 
    (
      {dest == null || dest.isPrimitive()}? expr = jUnaryExpression[]
    |
      expr = jUnaryExpressionNotPlusMinus[]
    )
  { self = new JCastExpression(sourceRef, expr, dest); }
;

jPrimaryExpression []
  returns [JExpression self = null]
:
  self = jPrimaryPrefix[] ( self = jPrimarySuffix[self] )*
;

jPrimaryPrefix []
  returns [JExpression self = null]
{
  JExpression	expr = null;
  CType		type = null;
  int		bounds = 0;
  String	name;
}
:
  self = jLiteral[]
|
  "this" { self = new JThisExpression(buildTokenReference() ); }
|
  "super" DOT id:IDENT
    { self = new JNameExpression(buildTokenReference(),
				 new JSuperExpression(buildTokenReference() ),
				 id.getText()); }
|
  LPAREN expr = jExpression[] RPAREN
    { self = new JParenthesedExpression(buildTokenReference(), expr); }
|
  self = jAllocationExpression[null]
|
  name = jName[]
    (
      options { warnWhenFollowAmbig = false; } :     
      ( 
         options { warnWhenFollowAmbig = false; } :
         LBRACK RBRACK { bounds += 1; } 
      )*
      DOT "class"
       {
         type = CClassType.lookup(name);
         type = bounds > 0 ? new CArrayType(type, bounds) : type;
         self = new JClassExpression(buildTokenReference(), type);
       }
    )?
   { self = self == null ? new JNameExpression(buildTokenReference(), name, true) : self; }
|
    type = jPrimitiveType[] DOT "class"
      { self = new JClassExpression(buildTokenReference(), type); }
;

jPrimarySuffix [JExpression prefix]
  returns [JExpression self = null]
{
  JExpression	expr = null;
  JExpression[]	args;
  TokenReference	sourceRef = buildTokenReference();
}
:
  DOT "this" { self = new JThisExpression(sourceRef, prefix); }
|
  DOT self = jAllocationExpression[prefix]
|
  LBRACK expr = jExpression[] RBRACK
    { self = new JArrayAccessExpression(sourceRef, prefix, expr); }
|
  DOT id1:IDENT
    { self = new JNameExpression(sourceRef, prefix, id1.getText()); }
|
  args = jArguments[]
    { self = new JMethodCallExpression(sourceRef, prefix, args); }
;

jLiteral []
  returns [JExpression self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  i:INTEGER_LITERAL
    { self = new JOrdinalLiteral(sourceRef, i.getText()); }
|
  r:REAL_LITERAL
    { self = new JRealLiteral(sourceRef, r.getText()); }
|
  c:CHARACTER_LITERAL
    { self = new JCharLiteral(sourceRef, c.getText()); }
|
  s:STRING_LITERAL
    {     self = new JStringLiteral(sourceRef, s.getText()); }
|
  "true"
    { self = new JBooleanLiteral(sourceRef, true); }
|
  "false"
    { self = new JBooleanLiteral(sourceRef, false); }
|
  "null"
    { self = new JNullLiteral(sourceRef ); }
;

jArguments []
  returns [JExpression[] self = JExpression.EMPTY]
:
  LPAREN ( self = jArgumentList[] )? RPAREN
;

jArgumentList []
  returns [JExpression[] self = null]
{
  JExpression		expr;
  Vector		vect = VectorCache.request(VectorCache.SMALL);
}
:
  expr = jExpression[]
    { vect.addElement(expr); }
  (
    COMMA expr = jExpression[]
      { vect.addElement(expr); }
  )*
{
  self = (JExpression[])at.dms.compiler.Utils.vectorToArray(vect, JExpression.class);
  VectorCache.release(vect);
}
;

jAllocationExpression [JExpression expr]
  returns [JExpression self = null]
{
  CType			type = null;
  JClassDeclaration	decl = null;
  JArrayDimsAndInits	arr = null;
  String		name = null;
  JExpression[]		args;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "new"
  (
    type = jPrimitiveType[] arr = jArrayDimsAndInits[]
      { self = new JNewArrayExpression(sourceRef, type, arr); }
  |
    name = jName[]
    (
      arr = jArrayDimsAndInits[]
        { self = new JNewArrayExpression(sourceRef, CClassType.lookup(name), arr); }
    |
      args = jArguments[] 
      (
        decl = jInnerClassBody[name] 
        { self = new JNewAnonymousClassExpression(sourceRef, CClassType.lookup(name), expr, args, decl); }        
      |
        { self = new JNewObjectExpression(sourceRef, CClassType.lookup(name), expr, args); }        
      )
    )
  )
;

//
// The third LOOKAHEAD specification below is to parse to PrimarySuffix
// if there is an expression between the "[...]".
//
jArrayDimsAndInits []
  returns [JArrayDimsAndInits self = null]
{
  boolean		expr = false;
  boolean		unspec = false;
  Vector		vect = VectorCache.request(VectorCache.SMALL);
  JExpression		exp  = null;
  JArrayInitializer	init = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
 (
   options { warnWhenFollowAmbig = false; } :
   LBRACK
    ( { !unspec }? exp = jExpression[] { expr = true; } | { unspec = true; } )
   RBRACK
     { vect.addElement(exp); exp = null; }
 )+
 ({ expr == false }? init = jArrayInitializer[] | )
 //!!! warning okay, try to hide
   { 
     self =  new JArrayDimsAndInits(sourceRef, 
				    (JExpression[])at.dms.compiler.Utils.vectorToArray(vect, JExpression.class), 
				    init); 
      VectorCache.release(vect);     
    }
;

jStatement []
  returns [JStatement self = null]
:
(
  self = jEmptyStatement[]
|
  self = jLabeledStatement[]
|
  self = jBlock[]
|
  self = jExpressionStatement[]
|
  self = jSwitchStatement[]
|
  self = jWhileStatement[]
|
  self = jDoStatement[]
|
  self = jForStatement[]
|
  self = jBreakStatement[]
|
  self = jContinueStatement[]
|
  self = jReturnStatement[]
|
  self = jThrowStatement[]
|
  self = jSynchronizedStatement[]
|
  self = jTryStatement[]
|
  self = jIfStatement[]
)
;

// JLS 14.6: Labeled Statement
jLabeledStatement []
  returns [JStatement self = null]
{
  JStatement		stmt;
  TokenReference	sourceRef = buildTokenReference();
}
:
  label:IDENT COLON stmt = jStatement[]
    { self = new JLabeledStatement(sourceRef, label.getText(), stmt, getStatementComment()); }
;

// JLS 14.2: Block
jBlock []
  returns [JBlock self = null]
{
  Vector		body = VectorCache.request(VectorCache.BIG);
  JStatement		stmt;
  TokenReference	sourceRef = buildTokenReference();
}
:
  LCURLY
  (
    stmt = jBlockStatement[]
      { body.addElement(stmt); }
  )*
  RCURLY
    {
      self = new JBlock(sourceRef, (JStatement[])at.dms.compiler.Utils.vectorToArray(body, JStatement.class), getStatementComment());
      VectorCache.release(body);
    }
;

// JLS 14.2: Block
jClassBlock []
  returns [JClassBlock self = null]
{
  Vector		body = VectorCache.request(VectorCache.BIG);
  JStatement		stmt;
  TokenReference	sourceRef = buildTokenReference();
}
:
  LCURLY
  (
    stmt = jBlockStatement[]
      { body.addElement(stmt); }
  )* RCURLY
    {
      self = new JClassBlock(sourceRef, 
			     (JStatement[])at.dms.compiler.Utils.vectorToArray(body, JStatement.class)); 
      VectorCache.release(body);
    }
;

jBlockStatement []
  returns [JStatement self = null]
{
  JTypeDeclaration type;
}
:
  "final" self = jLocalVariableDeclaration[at.dms.kjc.Constants.ACC_FINAL] SEMI
|
  ( jType[] IDENT ) =>
  self = jLocalVariableDeclaration[0] SEMI
|
  self = jStatement[]
    {
      if (self instanceof JEmptyStatement) {
	reportTrouble(new CWarning(self.getTokenReference(), "stray-semicolon", null));
      }
    }
|
  (
    type = jClassDeclaration[0]
  |
    type = jInterfaceDeclaration[0]
  )
  { self = new JTypeDeclarationStatement(type.getTokenReference(), type); }
;

// JLS 14.3: Local Variable Declaration Statement
jLocalVariableDeclaration [int mods]
  returns [JVariableDeclarationStatement self = null]
{
  CType			type;
  Vector		vars = VectorCache.request(VectorCache.SMALL);
  JVariableDefinition	decl;
  TokenReference	sourceRef = buildTokenReference();
}
:
  type = jType[]
  decl = jVariableDeclarator[mods, type]
    { vars.addElement(decl); }
  (
    COMMA decl = jVariableDeclarator[mods, type]
      { vars.addElement(decl); }
  )*
    {
      self = new JVariableDeclarationStatement(sourceRef, 
					       (JVariableDefinition[])at.dms.compiler.Utils.vectorToArray(vars, JVariableDefinition.class),
					       getStatementComment());
      VectorCache.release(vars);
    }
;

// JLS 14.5: Empty Statement
jEmptyStatement []
  returns [JEmptyStatement self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  SEMI
    { self = new JEmptyStatement(sourceRef, getStatementComment()); }
;

// JLS 14.7: Expression Statement
jExpressionStatement []
  returns [JExpressionStatement self = null]
{
  JExpression		expr;
  TokenReference	sourceRef = buildTokenReference();
}
:
  expr = jStatementExpression[] SEMI
    { self = new JExpressionStatement(sourceRef, expr, getStatementComment()); }
;

//
// The last expansion of this production accepts more than the legal
// Java expansions for StatementExpression.  This expansion does not
// use PostfixExpression for performance reasons.
//
jStatementExpression []
  returns [JExpression self = null]
{
  int			operator = -1;
  JExpression		right;
}
:
  self = jPreIncrementExpression[]
|
  self = jPreDecrementExpression[]
|
  self = jPrimaryExpression[]
  (
    INC
      { self = new JPostfixExpression(buildTokenReference(), Constants.OPE_POSTINC, self); }
  |
    DEC
      { self = new JPostfixExpression(buildTokenReference(), Constants.OPE_POSTDEC, self); }
  |
    operator = jAssignmentOperator[] right = jExpression[]
      { self = JAssignmentExpression.create(self.getTokenReference(), operator, self, right); }
  )?
;

// JLS 14.9: Switch Statement
jSwitchStatement []
  returns [JStatement self = null]
{
  JExpression		expr;
  Vector		body = VectorCache.request(VectorCache.SMALL);

  Vector		labels = Constants.VECTOR_EMPTY;
  Vector		stmts = Constants.VECTOR_EMPTY;

  JSwitchLabel		label;
  JStatement		stmt;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "switch" LPAREN expr = jExpression[] RPAREN
  LCURLY
  (
      {
        labels = new Vector();
	stmts  = new Vector();
      }
    (
      // CONFLICT: to which "cases" does the statement bind?
      // ANTLR generates proper code: it groups the many
      // "case"/"default" labels together then follows them with the statements
      options { warnWhenFollowAmbig = false; } :

      label = jSwitchLabel[] { labels.addElement(label); }
    )+
    ( stmt = jBlockStatement[] { stmts.addElement(stmt); } )*
      { body.addElement(new JSwitchGroup(sourceRef,
					 (JSwitchLabel[])at.dms.compiler.Utils.vectorToArray(labels, JSwitchLabel.class), 
					 (JStatement[])at.dms.compiler.Utils.vectorToArray(stmts, JStatement.class))); }
  )*
  RCURLY
    { 
      self = new JSwitchStatement(sourceRef,
				  expr, 
				  (JSwitchGroup[])at.dms.compiler.Utils.vectorToArray(body, JSwitchGroup.class),
   				  getStatementComment());
      VectorCache.release(body);
    }
;

jSwitchLabel []
  returns [JSwitchLabel self = null]
{
  JExpression		expr = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "case" expr = jExpression[] COLON
    { self = new JSwitchLabel(sourceRef, expr); }
|
  "default" COLON
    { self = new JSwitchLabel(sourceRef, null); }
;

// JLS 14.8: If Statement
jIfStatement []
  returns [JIfStatement self = null]
{
  JExpression		cond;
  JStatement		thenClause;
  JStatement		elseClause = null;
  TokenReference	sourceRef = buildTokenReference();
  int			count = 0;
}
:
  "if" LPAREN cond = jExpression[] RPAREN
  thenClause = jStatement[]
  (
    // CONFLICT: the old "dangling-else" problem...
    // ANTLR generates proper code matching as soon as possible,
    // binding dangling else's to the innermost if statement
    // Hush warning.
    options { warnWhenFollowAmbig = false; } :

    {
      if (count++ > 0) {
        break;
      }
    }
    // !!! Putain d'ANTLR, REMOVE WHEN BUG CORRECTED

    "else" elseClause = jStatement[]
  )*
    {
      if (! (thenClause instanceof JBlock)) {
	reportTrouble(new CWarning(sourceRef, "enclose-if-then-in-block", null));
      }
      if (elseClause != null && !(elseClause instanceof JBlock || elseClause instanceof JIfStatement)) {
	reportTrouble(new CWarning(sourceRef, "enclose-if-else-in-block", null));
      }
      self = new JIfStatement(sourceRef, cond, thenClause, elseClause, getStatementComment());
    }
;

// JLS 14.10: While Statement
jWhileStatement []
  returns [JWhileStatement self = null]
{
  JExpression		cond;
  JStatement		body;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "while" LPAREN cond = jExpression[] RPAREN body = jStatement[]
    {
      if (!(body instanceof JBlock || body instanceof JEmptyStatement)) {
	reportTrouble(new CWarning(sourceRef, "enclose-loop-body-in-block", null));
      }
      self = new JWhileStatement(sourceRef, cond, body, getStatementComment());
    }
;

// JLS 14.11: Do Statement
jDoStatement []
  returns [JDoStatement self = null]
{
  JExpression		cond;
  JStatement		body;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "do" body = jStatement[] "while" LPAREN cond = jExpression[] RPAREN SEMI
    {
      if (! (body instanceof JBlock)) {
	reportTrouble(new CWarning(sourceRef, "enclose-loop-body-in-block", null));
      }
      self = new JDoStatement(sourceRef, cond, body, getStatementComment());
    }
;

jForStatement []
  returns [JStatement self = null]
{
  JStatement		init = null;
  JExpression		cond = null;
  JStatement		incr = null;
  JStatement		body = null;
  TokenReference	sourceRef = buildTokenReference();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  "for" LPAREN
    ( init = jForInit[] )? SEMI
    ( cond = jExpression[] )? SEMI
    ( incr = jStatementExpressionList[] )?
  RPAREN body = jStatement[]
    {
      if (!(body instanceof JBlock || body instanceof JEmptyStatement)) {
	reportTrouble(new CWarning(sourceRef, "enclose-loop-body-in-block", null));
      }
      self = new JForStatement(sourceRef, init, cond, incr, body, comments);
    }
;

jForInit []
  returns [JStatement self = null]
:
  "final" self = jLocalVariableDeclaration[at.dms.kjc.Constants.ACC_FINAL]
|
  ( jType[] IDENT ) =>
  self = jLocalVariableDeclaration[0]
|
  self = jStatementExpressionList[]
;

jStatementExpressionList []
  returns [JExpressionListStatement self = null]
{
  JExpression		expr;
  Vector		v = VectorCache.request(VectorCache.SMALL);
  TokenReference	sourceRef = buildTokenReference();
}
:
  expr = jStatementExpression[]
    { v.addElement(expr); }
  (
    COMMA expr = jStatementExpression[]
      { v.addElement(expr); }
  )*
    { self = new JExpressionListStatement(sourceRef, 
					  (JExpression[])at.dms.compiler.Utils.vectorToArray(v, JExpression.class), 
					  getStatementComment()); 
    VectorCache.release(v);
    }
;

// JLS 14.13: Break Statement
jBreakStatement []
  returns [JBreakStatement self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  "break" ( ident:IDENT )? SEMI
    { self = new JBreakStatement(sourceRef, ident == null ? null : ident.getText(), getStatementComment()); }
;

// JLS 14.14: Continue Statement
jContinueStatement []
  returns [JContinueStatement self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  "continue" ( ident:IDENT )? SEMI
    { self = new JContinueStatement(sourceRef, ident == null ? null : ident.getText(), getStatementComment()); }
;

// JLS 14.15: Return Statement
jReturnStatement []
  returns [JReturnStatement self = null]
{
  JExpression		expr = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "return" ( expr = jExpression[] )? SEMI
    { self = new JReturnStatement(sourceRef, expr, getStatementComment()); }
;

// JLS 14.16: Throw Statement
jThrowStatement []
  returns [JStatement self = null]
{
  JExpression		expr;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "throw" expr = jExpression[] SEMI
    { self = new JThrowStatement(sourceRef, expr, getStatementComment()); }
;

// JLS 14.17: Synchronized Statement
jSynchronizedStatement []
  returns [JSynchronizedStatement self = null]
{
  JExpression		cond;
  JStatement		body;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "synchronized" LPAREN cond = jExpression[] RPAREN body = jBlock[]
    { self = new JSynchronizedStatement(sourceRef, cond, body, getStatementComment()); }
;

// JLS 14.18: Try Statement
jTryStatement []
  returns [JStatement self = null]
{
  JBlock		tryClause;
  Vector		catchClauses = VectorCache.request(VectorCache.SMALL);
  JBlock		finallyClause = null;
  JCatchClause		catcher = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "try" tryClause = jBlock[]
  (
    catcher = jCatchClause[]
      { catchClauses.addElement(catcher); }
  )*
  ( "finally" finallyClause = jBlock[] )?
    {
      if (catchClauses.size() > 0) {
	self = new JTryCatchStatement(sourceRef,
				      tryClause,
				      (JCatchClause[])at.dms.compiler.Utils.vectorToArray(catchClauses, JCatchClause.class),
				      finallyClause == null ? getStatementComment() : null);
	VectorCache.release(catchClauses);
      }
      if (finallyClause != null) {
	// If both catch and finally clauses are present,
	// the try-catch is embedded as try clause into a
	// try-finally statement.
	if (self != null) {
	  tryClause = new JBlock(sourceRef, new JStatement[] {self}, null);
	}
	self = new JTryFinallyStatement(sourceRef, tryClause, finallyClause, getStatementComment());
      }

      if (self == null) {
	// try without catch or finally: error
	reportTrouble(new PositionedError(sourceRef, "try-nocatch", null));
	self = tryClause;
      }
    }
;

jCatchClause []
  returns [JCatchClause self = null]
{
  JFormalParameter	param;
  JBlock		body;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "catch" LPAREN param = jFormalParameter[JLocalVariable.DES_CATCH_PARAMETER] RPAREN body = jBlock[]
    { self = new JCatchClause(sourceRef, param, body); }
;
