/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: AsmError.java,v 1.10 1999/10/08 11:19:02 vgp Exp $
 */

package at.dms.ksm;

import antlr.ParserException;
import at.dms.util.Message;
import at.dms.compiler.TokenReference;

/**
 * Error thrown on problems encountered while running the assembler itself.
 */
public class AsmError extends ParserException {

  // --------------------------------------------------------------------
  // CONSTRUCTORS
  // --------------------------------------------------------------------

  /**
   * An error with with an arbitrary number of parameters
   * @param	where		the reference to token where error happen
   * @param	key		the message ident to be displayed
   * @param	params		the array of parameters
   */
  public AsmError(TokenReference where, String key, Object[] params) {
    super(key);
    this.where		= where;
    this.message	= new Message(key, params);
  }

  /**
   * An error with two parameters
   * @param	where		the reference to token where error happen
   * @param	key		the message ident to be displayed
   * @param	param1		the first parameter
   * @param	param2		the second parameter
   */
  public AsmError(TokenReference where, String key, Object param1, Object param2) {
    this(where, key, new Object[] { param1, param2 });
  }

  /**
   * An error with one parameter
   * @param	where		the reference to token where error happen
   * @param	key		the message ident to be displayed
   * @param	param		the parameter
   */
  public AsmError(TokenReference where, String key, Object param) {
    this(where, key, new Object[] { param });
  }

  /**
   * An error without parameters
   * @param	where		the reference to token where error happen
   * @param	key		the message ident to be displayed
   */
  public AsmError(TokenReference where, String key) {
    this(where, key, null);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   *
   */
  public String getMessageKey() {
    return message.getKey();
  }

  /**
   * Return the string explaining the error and the token reference
   */
  public String getErrorMessage() {
    if (where != null) {
      return where.getFile() + ":" + where.getLine() + ": " + message.getMessage();
    } else {
      return message.getMessage();
    }
  }

  /**
   *
   */
  public String getMessage() {
    return getErrorMessage();
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  protected TokenReference	where;
  protected Message		message;
}
