/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Main.java,v 1.26 1999/10/16 23:25:35 vgp Exp $
 */

package at.dms.ksm;

import gnu.getopt.LongOpt;
import gnu.getopt.Getopt;
import at.dms.util.Message;

/**
 * This class is the entry point for the KOPI assembler.
 */
public class Main {

  // --------------------------------------------------------------------
  // CONSTRUCTORS
  // --------------------------------------------------------------------

  /**
   * Only main can construct Main
   */
  private Main(String[] args) {
    parseArgs(args);

    if (sourceFiles == null) {
      printUsage();
      System.exit(1);
    } else {
      boolean	errorsFound = false;

      for (int i = 0; i < sourceFiles.length; i++) {
	if (optionVerbose) {
	  System.err.println("Processing " + sourceFiles[i] + ".");
	}

	try {
	  Assembler		asm = new Assembler();

	  asm.assemble(sourceFiles[i], destination);
	} catch (AsmError e) {
	  System.err.println(e.getMessage());
	  errorsFound = true;
	}
      }

      System.exit(errorsFound ? 1 : 0);
    }
  }

  // --------------------------------------------------------------------
  // ENTRY POINT
  // --------------------------------------------------------------------

  /**
   * Entry point to the assembler
   */
  public static void main(String[] args) {
    Message.addBundle("ksm");

    new Main(args);
  }

  // --------------------------------------------------------------------
  // ACCESSORS
  // --------------------------------------------------------------------

  /*
   * Parse command line arguments.
   */
  private void parseArgs(String[] args) {
    Getopt	g = new Getopt("ksm", args, shortopts, longopts, true);

  _loop_:
    for (;;) {
      switch (g.getopt()) {
      case 'd':
	this.destination = g.getOptarg();
	break;

      case 'h':
	printUsage();
	System.exit(0);

      case 'V':
	printVersion();
	break;

      case 'v':
	optionVerbose = true;
	break;

      case -1:
	break _loop_;
      }
    }

    if (args.length > g.getOptind()) {
      this.sourceFiles = new String[args.length - g.getOptind()];
      System.arraycopy(args, g.getOptind(), this.sourceFiles, 0, this.sourceFiles.length);
    }
  }

  private static void printUsage() {
    System.err.println(Message.format("usage-line-ksm", null));
    System.err.println(Message.format("usage-verbose", null));
    System.err.println();
    System.err.println(Message.format("info1", null));
    System.err.println(Message.format("info2", null));
    System.err.println(Message.format("info3", null));
    System.err.println(Message.format("info4", null));
  }

  private static void printVersion() {
    System.err.println("KOPI Assembler Version: " + VERSION);
  }

  // --------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // --------------------------------------------------------------------

  private static final String	VERSION		= "1.3D";

  private static final String	shortopts = "d:hVv";
  private static final
    LongOpt[]			longopts = {
      new LongOpt("destination", LongOpt.REQUIRED_ARGUMENT, null, 'd'),
      new LongOpt("help", LongOpt.NO_ARGUMENT, null, 'h'),
      new LongOpt("version", LongOpt.NO_ARGUMENT, null, 'V'),
      new LongOpt("verbose", LongOpt.NO_ARGUMENT, null, 'v')
    };

  private String		destination = ".";
  private boolean		optionVerbose;
  private String[]		sourceFiles;
}
