/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MethodBody.java,v 1.14 1999/09/08 15:37:48 graf Exp $
 */

package at.dms.ksm;

import java.util.Hashtable;
import java.util.Vector;

import at.dms.classfile.AccessorContainer;
import at.dms.classfile.AccessorTransformer;
import at.dms.classfile.BadAccessorException;
import at.dms.classfile.CodeInfo;
import at.dms.classfile.HandlerInfo;
import at.dms.classfile.Instruction;
import at.dms.classfile.InstructionAccessor;
import at.dms.classfile.JumpInstruction;
import at.dms.classfile.LocalVariableInfo;
import at.dms.classfile.LineNumberInfo;
import at.dms.classfile.SwitchInstruction;

/**
 * This class represent the definition of a method body
 */
public class MethodBody {

  // --------------------------------------------------------------------
  // ACCESSORS
  // --------------------------------------------------------------------

  /**
   * Add an instruction at the end of the instruction list
   */
  public void addInstruction(Instruction insn) {
    insns.addElement(insn);
  }

  /**
   * Add a label at the end of the instruction list
   *
   * @exception	at.dms.ksm.AsmError	an assembler error
   */
  public void addLabel(String name) throws AsmError {
    if (labels.contains(name)) {
      throw new AsmError(null, "label-redefined", name);
    }

    labels.put(name, new Integer(insns.size()));
  }

  /**
   * Add an exception handler info
   */
  public void addHandler(String thrown,
			 LabelReference start,
			 LabelReference end,
			 LabelReference handler)
  {
    handlers.addElement(new HandlerInfo(start, end, handler, thrown));
  }

  /**
   * Add a line number info
   */
  public void addLineNumber(int line) {
    String	label;

    // create a unique label (syntactically illegal)
    label = "'" + lineNumberId++;
    try {
      addLabel(label);
    } catch (AsmError e) {
      throw new RuntimeException("INTERNAL ERROR");
    }
    lines.addElement(new LineNumberInfo((short)line, new LabelReference(label)));
  }

  /**
   * Add a local variable info
   */
  public void addVariable(String name,
			  String type,
			  LabelReference start,
			  LabelReference end,
			  short slot)
  {
    variables.addElement(new LocalVariableInfo(start, end, name, type, slot));
  }

  // --------------------------------------------------------------------
  // GENERATE A CODE INFO ATTRIBUTE
  // --------------------------------------------------------------------


  /**
   * Returns the instruction at specified label.
   * @param	name		the label name
   */
  public Instruction resolveLabel(String name) throws AsmError {
    Integer	addr = (Integer)labels.get(name);

    if (addr == null) {
      throw new AsmError(null, "undefined-label", name);
    }
    if (addr.intValue() >= insns.size()) {
      throw new AsmError(null, "undefined-label", name);	// !!! no instruction at label
    }
    return (Instruction)insns.elementAt(addr.intValue());
  }

  /**
   * Generate a classfile code info, do not optimize
   *
   * @exception	at.dms.ksm.AsmError	an assembler error
   */
  public CodeInfo genCodeInfo() throws AsmError {
    Instruction[]		insns;
    HandlerInfo[]		handlers;
    LineNumberInfo[]		lines;
    LocalVariableInfo[]		variables;

    this.insns.copyInto(insns = new Instruction[this.insns.size()]);
    this.handlers.copyInto(handlers = new HandlerInfo[this.handlers.size()]);
    this.lines.copyInto(lines = new LineNumberInfo[this.lines.size()]);
    this.variables.copyInto(variables = new LocalVariableInfo[this.variables.size()]);

    CodeInfo		codeInfo = new CodeInfo(insns,
						handlers,
						lines,
						variables);
    try {
      AccessorTransformer	transformer = new AccessorTransformer() {
	  public InstructionAccessor transform(InstructionAccessor accessor,
					       AccessorContainer container)
	    throws BadAccessorException
	  {
	    // the only accessors to resolve are label references
	    try {
	      return resolveLabel(((LabelReference)accessor).getName());
	    } catch (AsmError e) {
	      throw new BadAccessorException(e.getMessage());
	    }
	  }
	};

      codeInfo.transformAccessors(transformer);
    } catch (BadAccessorException e) {
      throw new AsmError(null, e.getMessage());
    }

    //!!! graf 990903 remove asap
    for (int i = 0; i < insns.length; i++) {
      if (insns[i] instanceof SwitchInstruction) {
	((SwitchInstruction)insns[i]).selectSwitchType();
      }
    }
    //!!! graf 990903 remove asap

    return codeInfo;
  }

  // --------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // --------------------------------------------------------------------

  private Vector		insns = new Vector();
  private Vector		handlers = new Vector();
  private Vector		lines = new Vector();
  private Vector		variables = new Vector();

  private Hashtable		labels = new Hashtable();

  private int			lineNumberId = 0;		// to create unique labels
}

