/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: LexgenError.java,v 1.2 1999/08/23 20:58:07 graf Exp $
 */

package at.dms.lexgen;

import at.dms.util.Message;

/**
 * Error thrown on problems encountered while running the assembler itself.
 */
public class LexgenError extends Exception {

  // --------------------------------------------------------------------
  // CONSTRUCTORS
  // --------------------------------------------------------------------

  /**
   * An error with with an arbitrary number of parameters
   * @param	key		the message ident to be displayed
   * @param	params		the array of parameters
   */
  public LexgenError(String key, Object[] params) {
    super(key);
    this.message	= new Message(key, params);
  }

  /**
   * An error with two parameters
   * @param	key		the message ident to be displayed
   * @param	param1		the first parameter
   * @param	param2		the second parameter
   */
  public LexgenError(String key, Object param1, Object param2) {
    this(key, new Object[] { param1, param2 });
  }

  /**
   * An error with one parameters
   * @param	key		the message ident to be displayed
   * @param	param		the parameter
   */
  public LexgenError(String key, Object param) {
    this(key, new Object[] { param });
  }

  /**
   * An error with two parameters
   * @param	key		the message ident to be displayed
   */
  public LexgenError(String key) {
    this(key, null);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   *
   */
  public String getKey() {
    return message.getKey();
  }

  /**
   * Return the string explaining the error and the token reference
   */
  public String getErrorMessage() {
    return message.getMessage();
  }

  /**
   *
   */
  public String getMessage() {
    return getErrorMessage();
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private Message		message;
}
