/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CMacroJavaPrettyPrinter.java,v 1.27 1999/10/28 14:03:12 vgp Exp $
 */

package at.dms.mkjc;

import java.util.StringTokenizer;
import at.dms.compiler.JavaStyleComment;
import at.dms.compiler.TabbedPrintWriter;
import at.dms.kjc.*;

/**
 * This class implements a macro pretty printer that generate the equivalent code of
 * parsing the java code that is given in parameter, just add checking...
 */
public class CMacroJavaPrettyPrinter extends CJavaPrettyPrinter {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * construct a pretty printer object for java code
   * @param	p		the printwriter into the code is generated
   */
  public CMacroJavaPrettyPrinter(String fileName) {
    super();
    try {
      this.p = new TabbedPrintWriter(
	 new java.io.BufferedWriter(new java.io.FileWriter(fileName.substring(0, fileName.indexOf('.')) +  ".java")));
    } catch (Exception e) {
      e.printStackTrace();
      System.err.println("cant write: " + fileName + ": " + e);
    }
  }

  // ----------------------------------------------------------------------
  // TYPE DECLARATION
  // ----------------------------------------------------------------------

  /**
   * prints a compilation unit
   */
  public void printCompilationUnit(JPackageName packageName,
				   JPackageImport[] importedPackages,
				   JClassImport[] importedClasses,
				   JTypeDeclaration[] typeDeclarations) {
    println("// Generated by mkjc, macro compiler. DO NOT EDIT //");
    super.printCompilationUnit(packageName,
			       importedPackages,
			       importedClasses,
			       typeDeclarations);
  }

  // ----------------------------------------------------------------------
  // TYPE DECLARATION
  // ----------------------------------------------------------------------

  /**
   * prints a class declaration
   */
  public void printClassDeclaration(int modifiers,
				    String ident,
				    String superName,
				    CClassType[] interfaces,
				    JPhylum[] body,
				    JMethodDeclaration[] methods) {

    newLine();
    p.print(CModifier.toString(modifiers));
    p.print("class " + ident);

    if (superName != null) {
      println(" extends " + superName);
    }

    if (interfaces.length != 0) {
      p.print(" implements ");
      for (int i = interfaces.length - 1; i >= 0; i--) {
	p.print((i == 0 ? "" : ", ") + interfaces[i]);
      }
    }

    println(" {");
    pos += TAB_SIZE;
    for (int i = 0; i < body.length ; i++) {
      body[i].genJavaCode(this);
    }
    for (int i = 0; i < methods.length ; i++) {
      methods[i].genJavaCode(this);
    }
    pos -= TAB_SIZE;

    println("}");
  }

  /**
   * prints a method declaration
   */
  public void printMethodDeclaration(int modifiers,
				     CType returnType,
				     String ident,
				     JFormalParameter[] parameters,
				     CClassType[] exceptions,
				     JBlock body) {
    if (ident.equals("<clinit>") || ident.equals("Block$")) {
      return; // we do not want generated methods in source code
    }

    newLine();
    p.print(CModifier.toString(modifiers));
    p.print(returnType.toString());
    p.print(" ");
    p.print(ident);
    p.print("(");
    for (int i = 0; i < parameters.length; i++) {
      if (i != 0) {
	print(", ");
      }
      parameters[i].genJavaCode(this);
    }
    p.print(") {");
    p.println();

    if (((CClassType)returnType).getQualifiedName().equals("at/dms/kjc/JStatement")) {
      p.print("return ");

      flushBlock = true;
      flushStatement = false;
      inInterface = false;
      if (body != null) {
	body.genJavaCode(this);
      }
      inInterface = true;      
      p.print(";");
    } else if (((CClassType)returnType).getQualifiedName().equals("at/dms/kjc/JBlock")) {
      p.print("return ");

      flushBlock = false;
      flushStatement = false;
      inInterface = false;
      if (body != null) {
	body.genJavaCode(this);
      }
      inInterface = true;      
      p.print(";");
    } else {
      p.print("return ");

      flushStatement = true;
      flushBlock = true;
      inInterface = false;
      if (body != null) {
	body.genJavaCode(this);
      }
      inInterface = true;      

      p.print(";");
    }

    println("\n}");

  }

  // ----------------------------------------------------------------------
  // STATEMENT - we transform statement in generated tree after parsing
  // ----------------------------------------------------------------------

  /**
   * prints a while statement
   */
  public void printWhileStatement(JExpression cond, JStatement body) {
    newLine();
    println("new JWhileStatement(ref ,");
    cond.genJavaCode(this);
    println(", ");
    body.genJavaCode(this);
    println(", null)");
  }

  /**
   * prints a variable declaration statement
   */
  public void printVariableDeclarationStatement(JVariableDefinition[] vars) {
    for (int i = 0; i < vars.length; i++) {
      // only one since kjc has already split elements (1)
      vars[i].genJavaCode(this);
    }
  }

  /**
   * prints a variable declaration statement
   */
  public void printVariableDefinition(int modifiers,
				      CType type,
				      String ident,
				      JExpression expr) { // (1)
    newLine();
    println("new JVariableDeclarationStatement(ref,");
    p.print("new JVariableDefinition(ref, " + modifiers + ", ");
    printType(type);
    p.print(", \"" + ident + "\"");
    p.print(", ");
    if (expr == null) {
      p.print("null");
    } else {
      expr.genJavaCode(this);
    }
    println(")");
    println(", null)");
  }

  /**
   * prints a try statement
   */
  public void printTryCatchStatement(JBlock tryClause, JCatchClause[] catchClauses) {
    newLine();

    println("new JTryCatchStatement(ref , ");
    tryClause.genJavaCode(this);
    p.print(", new JCatchClause[]{");
    for (int i = 0; i < catchClauses.length; i++) {
      p.print(i == 0 ? "" : ", ");
      catchClauses[i].genJavaCode(this);
    }
    p.print("}, null)");
  }

  /**
   * prints a try statement
   */
  public void printTryFinallyStatement(JBlock tryClause, JBlock finallyClause) {
    println("new JTryFinallyStatement(ref , ");
    tryClause.genJavaCode(this);    
    p.print(", ");
    finallyClause.genJavaCode(this);
    p.print(", null)");
  }

  /**
   * prints a throw statement
   */
  public void printThrowStatement(JExpression expr) {
    println("new JThrowStatement(ref,");
    expr.genJavaCode(this);    
    p.print(", null)");
  }

  /**
   * prints a synchronized statement
   */
  public void printSynchronizedStatement(JExpression cond, JStatement body) {
    println("new JSynchronizedStatement(ref,");
    cond.genJavaCode(this);
    p.print(", ");
    body.genJavaCode(this);
    p.print(", null)");
  }

  /**
   * prints a switch statement
   */
  public void printSwitchStatement(JExpression expr, JSwitchGroup[] body) {
    throw new RuntimeException("NOT YET HANDLED");
  }

  /**
   * prints a return statement
   */
  public void printReturnStatement(JExpression expr) {
    //newLine();
    // p.print("ls = new JEmptyStatement(ref, null)");
  }

  /**
   * prints a labeled statement
   */
  public void printLabeledStatement(String label, JStatement stmt) {
    throw new RuntimeException("NOT YET HANDLED");
  }

  /**
   * prints a if statement
   */
  public void printIfStatement(JExpression cond,
			       JStatement thenClause,
			       JStatement elseClause) {
    newLine();
    println("new JIfStatement(ref, ");
    cond.genJavaCode(this);
    p.print(", ");
    thenClause.genJavaCode(this);
    p.print(", ");
    if (elseClause != null) {
      elseClause.genJavaCode(this);
    } else {
      p.print("null");
    }
    p.print(", null)");
  }

  /**
   * prints a for statement
   */
  public void printForStatement(JStatement init,
				JExpression cond,
				JStatement incr,
				JStatement body) {
    throw new RuntimeException("NOT YET HANDLED");
  }

  /**
   * prints an expression statement
   */
  public void printExpressionStatement(JExpression expr) {
    if (!flushStatement) {
      println("new JExpressionStatement(ref, ");
      expr.genJavaCode(this);
      p.print(", null)");
    } else {
      expr.genJavaCode(this);
    }
  }

  /**
   * prints an expression list statement
   */
  public void printExpressionListStatement(JExpression[] expr) {
    throw new RuntimeException("NOT YET HANDLED");
  }

  /**
   * prints a empty statement
   */
  public void printEmptyStatement() {
  }

  /**
   * prints a do statement
   */
  public void printDoStatement(JExpression cond, JStatement body) {
    throw new RuntimeException("NOT YET HANDLED");
  }

  /**
   * prints a continue statement
   */
  public void printContinueStatement(String label) {
    throw new RuntimeException("NOT YET HANDLED");
  }

  /**
   * prints a break statement
   */
  public void printBreakStatement(String label) {
    throw new RuntimeException("NOT YET HANDLED");
  }

  /**
   * prints an expression statement
   */
  public void printBlockStatement(JStatement[] body, JavaStyleComment[] comments) {
    int		c = count++;

    if (!flushBlock) {
      newLine();
      println("new JBlock(ref, new JStatement[] {");
      for (int i = 0; i < body.length; i++) {
	if (!(body[i] instanceof JReturnStatement)) {
	  p.print(i == 0 ? "" : ", ");
	  body[i].genJavaCode(this);
	}
      }  
      p.print("}, null)");
    } else {
      body[0].genJavaCode(this);
    }
  }

  // ----------------------------------------------------------------------
  // EXPRESSION
  // ----------------------------------------------------------------------

  /**
   * prints an unary expression
   */
  public void printUnaryExpression(int oper, JExpression expr) {
    println("new JUnaryExpression(ref, " + oper + ", ");
    expr.genJavaCode(this);
    p.print(")");
  }

  /**
   * prints a type name expression
   */
  public void printTypeNameExpression(CType type) {
    println("new JNameExpression(ref, \"" + type + "\", true)");
  }

  /**
   * prints a this expression
   */
  public void printThisExpression(JExpression prefix) {
    if (prefix != null) {
      p.print("new JNameExpression(ref, ");
      prefix.genJavaCode(this);
      println(", this)");
    } else {
      println("new JThisExpression(ref)");
    }
  }

  /**
   * prints a super expression
   */
  public void printSuperExpression() {
    println("new JSuperExpression(ref)");
  }

  /**
   * prints a shift expression
   */
  public void printShiftExpression(int oper, JExpression left, JExpression right) {
    println("new JShiftExpression(ref, " + oper + ", ");
    left.genJavaCode(this);
    p.print(",");
    right.genJavaCode(this);
    p.print("le)");
  }

  /**
   * prints a shift expression
   */
  public void printRelationalExpression(int oper, JExpression left, JExpression right) {
    println("new JRelationalExpression(ref, " + oper + ", ");
    left.genJavaCode(this);
    p.print(", ");
    right.genJavaCode(this);
    p.print(")");
  }

  /**
   * prints a prefix expression
   */
  public void printPrefixExpression(int oper, JExpression expr) {
    println("new JPrefixExpression(ref, " + oper + ", ");
    expr.genJavaCode(this);
    p.print(")");
  }

  /**
   * prints a postfix expression
   */
  public void printPostfixExpression(int oper, JExpression expr) {
    println("new JPostfixExpression(ref, " + oper + ", ");
    expr.genJavaCode(this);
    p.print(")");
  }

  /**
   * prints a parenthesed expression
   */
  public void printParenthesedExpression(JExpression expr) {
    println("new JParenthesedExpression(ref, ");
    expr.genJavaCode(this);
    p.print(")");
  }

  /**
   * prints an object allocator expression
   */
  public void printNewObjectExpression(CClassType type, JExpression[] params, JClassDeclaration innerBody) {
    p.print("new JNewObjectExpression(ref,");
    printType(type);
    println(", null,new JExpression[]{");
    for (int i = 0; i < params.length; i++) {
      p.print(i == 0 ? "" : ", ");
      params[i].genJavaCode(this);
    }
    p.print("})");
  }

  /**
   * prints an array allocator expression
   */
  public void printNewArrayExpression(CType type, JArrayDimsAndInits dims) {
    p.print("new JNewArrayExpression(ref,");
    printType(type);
    println(")");
    if (dims != null) {
      throw new RuntimeException("NOT YET HANDLED");
    }
  }

  /**
   * prints a name expression
   */
  public void printNameExpression(JExpression prefix, String ident) {
    if (prefix != null) {
      println("new JNameExpression(ref, ");
      prefix.genJavaCode(this);
      p.print(", \"" + ident + "\")");
    } else {
      println("new JNameExpression(ref, \"" + ident + "\")");
    }
  }

  /**
   * prints an array allocator expression
   */
  public void printBinaryExpression(String oper,
				    JExpression left,
				    JExpression right) {
    String name;
    if (oper.equals("+")) {
      name = "JAddExpression";
    } else if (oper.equals("-")) {
      name = "JMinusExpression";
    } else if (oper.equals("*")) {
      name = "JStarExpression";
    } else if (oper.equals("/")) {
      name = "JDivideExpression";
    } else if (oper.equals("%")) {
      name = "JPercentExpression";
    } else {
      throw new RuntimeException("NOT YET HANDLED");
    }
    println("new " + name + "(ref, ");
    left.genJavaCode(this);
    p.print(",");
    right.genJavaCode(this);
    p.print(")");
  }

  /**
   * prints a method declaration
   */
  public void printConstructorDeclaration(int modifiers,
					  String ident,
					  JFormalParameter[] parameters,
					  CClassType[] exceptions,
					  JBlock body) {
    // no thanks
  }

  /**
   * prints a method call expression
   * // WARNING ===> DO NOT CALL RECURSIVLY METHOD CALL / else use tabs vect_tabs
   */
  public void printMethodCallExpression(JExpression prefix,
					String ident,
					JExpression[] args) {

    println("new JMethodCallExpression(ref, ");
    if (prefix != null) {
      println("new JNameExpression(ref, ");
      prefix.genJavaCode(this);
      p.print(", \"" + ident + "\")");
    } else {
      println("new JNameExpression(ref, \"" + ident + "\")");
    }
    p.print(", new JExpression[] {");
    for (int i = 0; i < args.length; i++) {
      p.print(i == 0 ? "" : ", ");
      args[i].genJavaCode(this);
    }
    p.print("})");
  }

  /**
   * prints a local variable expression
   */
  public void printLocalVariableExpression(String ident) {
    println("new JNameExpression(ref, \"" + ident + "\")");
  }

  /**
   * prints an instanceof expression
   */
  public void printInstanceofExpression(JExpression expr, CType dest) {
    p.print("new JInstanceofExpression(ref, ");
    expr.genJavaCode(this);
    p.print(", ");
    printType(dest);
    println(")");
  }

  /**
   * prints an equality expression
   */
  public void printEqualityExpression(int oper,
				      JExpression left,
				      JExpression right) {
    println("new JEqualityExpression(ref, " + oper + ", ");
    left.genJavaCode(this);
    p.print(", ");
    right.genJavaCode(this);
    p.print(")");
  }

  /**
   * prints a conditional expression
   */
  public void printConditionalExpression(JExpression cond,
					 JExpression left,
					 JExpression right) {
    throw new RuntimeException("NOT YET HANDLED");
  }

  /**
   * prints a compound expression
   */
  public void printCompoundAssignmentExpression(int oper,
						JExpression left,
						JExpression right) {
    throw new RuntimeException("NOT YET HANDLED");
  }

  /**
   * prints a field expression
   */
  public void printFieldExpression(JExpression left, String ident) {
    throw new RuntimeException("NOT YET HANDLED");
  }

  /**
   * prints a class expression
   */
  public void printClassExpression(CType type) {
    throw new RuntimeException("NOT YET HANDLED");
  }

  /**
   * prints a cast expression
   */
  public void printCastExpression(JExpression expr, CType type) {
    p.print("new JCastExpression(ref, ");
    expr.genJavaCode(this);
    p.print(", ");
    printType(type);
    println(")");
  }

  /**
   * prints a cast expression
   */
  public void printUnaryPromoteExpression(JExpression expr, CType type) {
    p.print("new JCastExpression(ref, ");
    expr.genJavaCode(this);
    p.print(", ");
    printType(type);
    println(")");
  }

  /**
   * prints a compound assignment expression
   */
  public void printBitwiseExpression(int oper,
				     JExpression left,
				     JExpression right) {
    println("new JBitwiseExpression(ref, " + oper + ", ");
    left.genJavaCode(this);
    p.print(", ");
    right.genJavaCode(this);
    p.print(")");
  }

  /**
   * prints an assignment expression
   */
  public void printAssignmentExpression(JExpression left, JExpression right) {
    println("new JAssignmentExpression(ref, ");
    left.genJavaCode(this);
    p.print(", ");
    right.genJavaCode(this);
    p.print(")");
  }

  /**
   * prints an array length expression
   */
  public void printArrayLengthExpression(JExpression prefix) {
    throw new RuntimeException("NOT YET HANDLED");
  }

  /**
   * prints an array length expression
   */
  public void printArrayAccessExpression(JExpression prefix, JExpression accessor) {
    println("new JArrayAccessExpression(ref, ");
    accessor.genJavaCode(this);
    p.print(", ");
    prefix.genJavaCode(this);
    p.print(")");
  }

  // ----------------------------------------------------------------------
  // UTILS
  // ----------------------------------------------------------------------

  /**
   * prints an array length expression
   */
  public void printSwitchLabel(JExpression expr) {
    if (expr != null) {
      p.print("case ");
      expr.genJavaCode(this);
      p.print(":");
    } else {
      println("default:");
    }
  }

  /**
   * prints an array length expression
   */
  public void printSwitchGroup(JSwitchLabel[] labels, JStatement[] stmts) {
    for (int i = 0; i < labels.length; i++) {
      labels[i].genJavaCode(this);
    }
    for (int i = 0; i < stmts.length; i++) {
      stmts[i].genJavaCode(this);
    }
  }

  /**
   * prints an array length expression
   */
  public void printCatchClause(JFormalParameter exception, JBlock body) {
    println("new JCatchClause(ref, ");
    exception.genJavaCode(this);
    p.print(", ");
    body.genJavaCode(this);
    p.print(")");
  }

  /**
   * prints an array length expression
   */
  public void printStringLiteral(String value) {
    println("new JStringLiteral(ref, \"" + value + ")");
  }

  /**
   * prints an array length expression
   */
  public void printCharLiteral(Character value) {
    println("new JCharLiteral(ref, '" + value + "')");
  }

  /**
   * prints an array length expression
   */
  public void printBooleanLiteral(boolean value) {
    println("new JBooleanLiteral(ref, " + value + ")");
  }

  /**
   * prints an array length expression
   */
  public void printNumberLiteral(Number value) {
    if (value instanceof Float || value instanceof Double) {
      println("new JRealLiteral(ref, " + value + ")");
    } else {
      println("new JOrdinalLiteral(ref, " + value + ")");
    }
  }

  /**
   * prints an array length expression
   */
  public void printNullLiteral() {
    println("new JNullLiteral(ref)");
  }

  /**
   * prints an array length expression
   */
  public void printPackageName(String name) {

    p.print("package " + name + ";");
  }

  /**
   * prints an array length expression
   */
  public void printFormalParameters(boolean isFinal, CType type, String ident) {
    if (inInterface) {
      printFormalParameters1(isFinal, type, ident);
    } else {
      p.print("new JFormalParameter(ref, JLocalVariable.DES_PARAMETER, ");
      printType(type);
      p.print(", \"" + ident + "\", " + (isFinal) + ")");
    }
  }

  /**
   * prints an array length expression
   */
  public void printFormalParameters1(boolean isFinal, CType type, String ident) {
    if (isFinal) {
      p.print("final ");
    }
    p.print(type + " " + ident);
  }

  /**
   * prints an array length expression
   */
  public void printExplicitConstructorInvocation(JExpression prefix,
						 String ident,
						 JExpression[] args) {
    if (prefix != null) {
      prefix.genJavaCode(this);
    }

    p.print(ident);

    p.print("(");
    for (int i = 0; i < args.length; i++) {
      if (i != 0) {
	p.print(", ");
      }
      args[i].genJavaCode(this);
    }
    p.print(")");
  }

  /**
   * prints an array initializer expression
   */
  public void printArrayInitializer(JExpression[] elems) {
    p.print("{");
    for (int i = 0; i < elems.length; i++) {
      if (i != 0) {
	p.print(", ");
      }
      elems[i].genJavaCode(this);
    }
    p.print("}");
  }

  /**
   * prints an array initializer expression
   */
  public void printArrayDimsAndInit(JExpression[] dims, JArrayInitializer init) {
    for (int i = 0; i < dims.length; i++) {
      p.print("[");
      if (dims[i] != null) {
	dims[i].genJavaCode(this);
      }
      p.print("]");
    }
    if (init != null) {
      init.genJavaCode(this);
    }
  }

  /**
   * prints an array initializer expression
   */
  public void printType(CType type) {
    switch (type.getTypeID()) {
    case TID_VOID:
      p.print("CStdType.Void");
      break;
    case TID_BYTE:
      p.print("CStdType.Byte");
      break;
    case TID_CHAR:
      p.print("CStdType.Char");
      break;
    case TID_DOUBLE:
      p.print("CStdType.Double");
      break;
    case TID_FLOAT:
      p.print("CStdType.Float");
      break;
    case TID_INT:
      p.print("CStdType.Integer");
      break;
    case TID_LONG:
      p.print("CStdType.Long");
      break;
    case TID_CLASS:
      p.print("CClassType.lookup(\"" + ((CClassType)type).getQualifiedName().replace('.', '/') + "\")");
      break;
    case TID_SHORT:
      p.print("CStdType.Short");
      break;
    case TID_BOOLEAN:
      p.print("CStdType.Boolean");
      break;
    case TID_ARRAY:
      p.print("new CArrayType(");
      printType(((CArrayType)type).getBaseType());
      p.print(", " + ((CArrayType)type).getArrayBound() + ")");
      break;
    default:
      System.err.println(">>>>" + type);
      fail();
    }
  }

  /**
   *
   */
  public TabbedPrintWriter getPrintWriter() {
    return p;
  }

  private void println(String str) {
    p.print(str);
    p.println();
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private int count = 0;
  private boolean inInterface = true;
  private boolean flushBlock;
  private boolean flushStatement;
}
