/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: MExpression.java,v 1.13 1999/10/08 11:19:03 vgp Exp $
 */

package at.dms.mkjc;

import at.dms.kjc.CBodyContext;
import at.dms.kjc.CJavaPrettyPrinter;
import at.dms.kjc.CType;
import at.dms.kjc.CodeSequence;
import at.dms.kjc.JExpression;
import at.dms.kjc.CExpressionContext;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * Root class for all expressions
 */
public class MExpression extends JExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

 /**
   * Construct a node in the parsing tree
   * @param	where		the line of this node in the source code
   */
  public MExpression(TokenReference where, CType type, int num) {
    super(where);
    this.num = num;
    this.type = type;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   *
   */
  public CType getType() {
    return type;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param	context			the actual context of analyse
   * @return	a pure java expression including promote node
   * @exception	PositionedError			Error catched as soon as possible
   * @exception	UnpositionedError		Error catched as soon as possible
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    try {
      type.checkType(context);
    } catch (UnpositionedError e) {
      throw e.getError(getTokenReference());
    }
    return this;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param p the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    ((CMacroJavaPrettyPrinter)p).getPrintWriter().print("exprs[" + num + "]");
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    throw new RuntimeException();
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private CType			type;
  private int			num;
}
