/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Main.java,v 1.20 1999/08/26 20:21:31 graf Exp $
 */

package at.dms.mkjc;

import java.io.File;
import at.dms.kjc.JCompilationUnit;
import at.dms.kjc.CJavaPrettyPrinter;
import at.dms.kjc.CHandLexer;

/**
 * This class implements the entry point of the Macro Java compiler
 */
public class Main extends at.dms.kjc.Main {

  private Main() {
    optionGenerateJava = true;
    destination = "";
  }

  // ----------------------------------------------------------------------
  // CONSTRUCTOR
  // ----------------------------------------------------------------------

  /**
   * Entry point
   * @param	args		the command line arguments
   */
  public static void main(String[] args) {
    if (instance != null) {
      // only one instance of Main at the same time
      throw new RuntimeException("DO NOT CALL MAIN WHILE THERE IS A COMPILE PROCESS");
    }

    boolean	success;

    instance = new Main();
    success = instance.run(args);
    instance = null;
    err.flush();
    System.exit(success ? 0 : 1);
  }

  /**
   * shows an usage message
   * @param	msg		the error that stop the command line parser
   */
  protected void usage(String msg) {
    System.err.println("mkjc: " + msg);
    System.err.println("usage: mkjc [option]* [-help] <macro-java-files>");
  }

  /**
   * shows a help message
   */
  protected void help() {
    System.err.println("usage: kxjc [option]* [-help] <macro-files>");
    System.err.println("\t -verbose: \tprints out informations during compilation");
  }

  // ----------------------------------------------------------------------
  // PROTECTED METHODS
  // ----------------------------------------------------------------------

  /**
   * parse the argument list and set flags
   * @return parsing error occur
   */
  protected boolean parseArguments(String[] args) {
    for (int i = 0; i < args.length; i += 1) {
      if (args[i].equals("-help")) {
	help();
	System.exit(0);
      } else if (args[i].equals("-verbose")) {
	optionVerbose = true;
      } else if (args[i].endsWith(".macro")) {
	infiles.addElement(args[i]);
      } else {
	usage("unrecognised argument " + args[i]);
	return false;
      }
    }

    return true;
  }

  /**
   * parse the givven file and return a compilation unit
   * side effect: increment error number
   * @param	file		the name of the file (assert exists)
   * @return	the compilation unit defined by this file
   */
  protected JCompilationUnit parseFile(File file) {
    CHandLexer		lexer = null; // to get line number if errors

    try {
      // Create a scanner that reads from the input stream passed to us
      MParseManager parsers = MParseManager.getMParseManager(this, file, file.getName());
      lexer = parsers.getMJavaLexer();

      // start parsing at the compilationUnit rule
      JCompilationUnit unit = parsers.getMJavaParser().jCompilationUnit();

      return unit;
    } catch (Exception e) {
      e.printStackTrace();
      System.err.println("{" + file.getName() + ":" + lexer.getLine() + "} " + e.getMessage());
      errorFound = true;
    }
    return null;
  }

  /**
   * Shows an error/warning and set error flag to true
   */
  protected CJavaPrettyPrinter getPrettyPrinterOverided(String pw) {
    return new CMacroJavaPrettyPrinter(pw);
  }
}
