/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Patterns.java,v 1.3 1999/10/08 11:19:03 vgp Exp $
 */

package at.dms.optimize;

import java.util.Vector;
import at.dms.classfile.AccessorContainer;
import at.dms.classfile.AccessorTransformer;
import at.dms.classfile.BadAccessorException;
import at.dms.classfile.CodeInfo;
import at.dms.classfile.HandlerInfo;
import at.dms.classfile.Instruction;
import at.dms.classfile.InstructionAccessor;
import at.dms.classfile.JumpInstruction;
import at.dms.classfile.LineNumberInfo;
import at.dms.classfile.LocalVariableInfo;
import at.dms.classfile.NoArgInstruction;
import at.dms.classfile.PushLiteralInstruction;
import at.dms.util.Utils;

/**
 * This class is the entry point for the peephole byte code optimizer
 */
public class Patterns implements at.dms.classfile.Constants{

  /**
   * Optimizes the byte code for a single method.
   */
  public static boolean optimize(InstructionHandle handle) {
    int	opcode = handle.getInstruction().getOpcode();

    switch (opcode) {
    case opc_ifeq:
    case opc_ifne:
    case opc_iflt:
    case opc_ifge:
    case opc_ifgt:
    case opc_ifle:
    case opc_ifnull:
    case opc_ifnonnull:
    case opc_if_icmpeq:
    case opc_if_icmpne:
    case opc_if_icmplt:
    case opc_if_icmpge:
    case opc_if_icmpgt:
    case opc_if_icmple:
    case opc_if_acmpeq:
    case opc_if_acmpne:
    case opc_goto_w:
    case opc_goto:
    case opc_jsr:
    case opc_jsr_w:
      // Jump instruction
      return optimizeJump(handle, opcode);
    case opc_dastore:
    case opc_lastore:
    case opc_aastore:
    case opc_bastore:
    case opc_castore:
    case opc_dcmpg:
    case opc_dcmpl:
    case opc_fastore:
    case opc_iastore:
    case opc_lcmp:
    case opc_sastore:
    case opc_dadd:
    case opc_ddiv:
    case opc_dmul:
    case opc_drem:
    case opc_dreturn:
    case opc_dsub:
    case opc_ladd:
    case opc_land:
    case opc_ldiv:
    case opc_lmul:
    case opc_lor:
    case opc_lrem:
    case opc_lreturn:
    case opc_lsub:
    case opc_lxor:
    case opc_pop2:
    case opc_aaload:
    case opc_areturn:
    case opc_athrow:
    case opc_baload:
    case opc_caload:
    case opc_d2f:
    case opc_d2i:
    case opc_fadd:
    case opc_faload:
    case opc_fcmpg:
    case opc_fcmpl:
    case opc_fdiv:
    case opc_fmul:
    case opc_frem:
    case opc_freturn:
    case opc_fsub:
    case opc_iadd:
    case opc_iaload:
    case opc_iand:
    case opc_idiv:
    case opc_imul:
    case opc_isub:
    case opc_irem:
    case opc_ishl:
    case opc_ishr:
    case opc_iushr:
    case opc_ior:
    case opc_ixor:
    case opc_l2f:
    case opc_l2i:
    case opc_lshl:
    case opc_lshr:
    case opc_lushr:
    case opc_monitorenter:
    case opc_monitorexit:
    case opc_pop:
    case opc_saload:
    case opc_ireturn:
    case opc_nop:
    case opc_arraylength:
    case opc_d2l:
    case opc_daload:
    case opc_dneg:
    case opc_f2i:
    case opc_fneg:
    case opc_i2b:
    case opc_i2c:
    case opc_i2f:
    case opc_i2s:
    case opc_ineg:
    case opc_l2d:
    case opc_laload:
    case opc_lneg:
    case opc_return:
    case opc_swap:
    case opc_aconst_null:
    case opc_dup:
    case opc_dup_x1:
    case opc_dup_x2:
    case opc_f2d:
    case opc_f2l:
    case opc_i2d:
    case opc_i2l:
    case opc_dup2:
    case opc_dup2_x1:
    case opc_dup2_x2:
      // NoArgInstruction
      return optimizeNoArgInstruction(handle, opcode);
    case opc_invokevirtual:
    case opc_invokespecial:
    case opc_invokestatic:
      // MethodRefInstruction
      // return optimizeMethodRefInstruction(handle);      
      break;
    case opc_getstatic:
    case opc_putstatic:
    case opc_getfield:
    case opc_putfield:
      // FieldRefInstruction
      // return optimizeFieldRefInstruction(handle);      
      break;
    case opc_aload:
    case opc_aload_0:
    case opc_aload_1:
    case opc_aload_2:
    case opc_aload_3:
    case opc_astore:
    case opc_astore_0:
    case opc_astore_1:
    case opc_astore_2:
    case opc_astore_3:
    case opc_dload:
    case opc_dload_0:
    case opc_dload_1:
    case opc_dload_2:
    case opc_dload_3:
    case opc_dstore:
    case opc_dstore_0:
    case opc_dstore_1:
    case opc_dstore_2:
    case opc_dstore_3:
    case opc_fload:
    case opc_fload_0:
    case opc_fload_1:
    case opc_fload_2:
    case opc_fload_3:
    case opc_fstore:
    case opc_fstore_0:
    case opc_fstore_1:
    case opc_fstore_2:
    case opc_fstore_3:
    case opc_iload:
    case opc_iload_0:
    case opc_iload_1:
    case opc_iload_2:
    case opc_iload_3:
    case opc_istore:
    case opc_istore_0:
    case opc_istore_1:
    case opc_istore_2:
    case opc_istore_3:
    case opc_lload:
    case opc_lload_0:
    case opc_lload_1:
    case opc_lload_2:
    case opc_lload_3:
    case opc_lstore:
    case opc_lstore_0:
    case opc_lstore_1:
    case opc_lstore_2:
    case opc_lstore_3:
    case opc_ret:
      // LocalVarRefInstruction
      // return optimizeLocalVarInstruction(handle);      
      break;
    case opc_iconst_m1:
    case opc_iconst_0:
    case opc_iconst_1:
    case opc_iconst_2:
    case opc_iconst_3:
    case opc_iconst_4:
    case opc_iconst_5:
    case opc_fconst_0:
    case opc_fconst_1:
    case opc_fconst_2:
    case opc_sipush:
    case opc_bipush:
    case opc_ldc:
    case opc_ldc_w:
    case opc_lconst_0:
    case opc_lconst_1:
    case opc_dconst_0:
    case opc_dconst_1:
    case opc_ldc2_w:
      // PushLiteralInstruction
      return optimizePushLiteralInstruction(handle, opcode);      
    }
    
    handle.set();

    return false;
  }

  // --------------------------------------------------------------------
  // JUMP INSTRUCTION
  // --------------------------------------------------------------------

  /**
   * Optimizes the byte code for a single method.
   */
  public static boolean optimizeJump(InstructionHandle handle, int opcode) {
    JumpInstruction	current = (JumpInstruction)handle.getInstruction();

    // Jump to next instruction
    if (current.getTarget() == handle.getNext()) {
      switch (opcode) {
      case opc_if_icmpeq:
      case opc_if_icmpne:
      case opc_if_icmplt:
      case opc_if_icmpgt:
      case opc_if_icmpge:
      case opc_if_icmple:
	handle.replaceBy(new InstructionHandle(new NoArgInstruction(opc_pop2),
					       handle.getPrevious()));
	return true;
      case opc_ifeq:
      case opc_ifne:
	handle.replaceBy(new InstructionHandle(new NoArgInstruction(opc_pop),
					       handle.getPrevious()));
	return true;

      case opc_goto:
	if (handle.remove()) {
	  return true;
	}
      }
    }

    // Jump to goto instruction
    if (((InstructionHandle)current.getTarget()).getInstruction().getOpcode() == opc_goto) {
      ((InstructionHandle)current.getTarget()).removeAccessor(current);
      current.setTarget(((JumpInstruction)((InstructionHandle)current.getTarget()).getInstruction()).getTarget());
      ((InstructionHandle)current.getTarget()).addAccessor(current);

      handle.set();

      return true;
    }

    // Goto to return instruction
    if (opcode == opc_goto && current.getTarget() instanceof NoArgInstruction) {
      NoArgInstruction	target = (NoArgInstruction)((InstructionHandle)current.getTarget()).getInstruction();
      
      if (!target.canComplete()) {
        ((InstructionHandle)current.getTarget()).removeAccessor(current);        
	handle.replaceBy(new InstructionHandle(new NoArgInstruction(target.getOpcode()),
					       handle.getPrevious()));

	return true;
      }
    }

    handle.set();

    return false;
  }

  // --------------------------------------------------------------------
  // NO ARG INSTRUCTION
  // --------------------------------------------------------------------

  /**
   * Optimizes the byte code for a single method.
   */
  public static boolean optimizeNoArgInstruction(InstructionHandle handle, int opcode) {
    if (opcode == opc_nop) {
      if (handle.remove()) {
	return true;
      } else {
	handle.set();
	
	return false;
      }
    }

    handle.set();

    return false;
  }

  // --------------------------------------------------------------------
  // NO ARG INSTRUCTION
  // --------------------------------------------------------------------

  /**
   * Optimizes the byte code for a single method.
   */
  public static boolean optimizePushLiteralInstruction(InstructionHandle handle, int opcode) {
    PushLiteralInstruction	current = (PushLiteralInstruction)handle.getInstruction();
/*
    //    comparison to 0 or 1
    if (handle.getNext().getInstruction() instanceof JumpInstruction &&
	!handle.getNext().isTarget()) {
      JumpInstruction	next = (JumpInstruction)handle.getNext().getInstruction();

      if (opcode == opc_iconst_0) {
	int	newOpcode = -1;

	switch (next.getOpcode()) {
	case opc_if_icmpeq:
	  newOpcode = opc_ifeq;
	  break;
	case opc_if_icmpne:
	  newOpcode = opc_ifne;
	  break;
	case opc_if_icmplt:
	  newOpcode = opc_iflt;
	  break;
	case opc_if_icmpgt:
	  newOpcode = opc_ifgt;
	  break;
	case opc_if_icmpge:
	  newOpcode = opc_ifge;
	  break;
	case opc_if_icmple:
	  newOpcode = opc_ifle;
	  break;
	case opc_ifeq:
	  newOpcode = opc_goto;
	  break;
	case opc_ifne:
	  // delete
	  if (handle.remove() && handle.getNext().remove()) {
	    return true;
	  }
	}

	if (newOpcode != -1) {
	  InstructionHandle	impl = new InstructionHandle(new JumpInstruction(newOpcode, next.getTarget()),
							     handle.getPrevious());

	  ((InstructionHandle)next.getTarget()).addAccessor((JumpInstruction)impl.getInstruction());
	  handle.replaceBy(impl);
	  impl.setNext(handle.getNext().getNext());

	  return true;
	}
      } else if (opcode == opc_iconst_1) {
	if (next.getOpcode() == opc_ifne) {
	  InstructionHandle	impl = new InstructionHandle(new JumpInstruction(opc_goto, next.getTarget()),
							     handle.getPrevious());

	  ((InstructionHandle)next.getTarget()).addAccessor((JumpInstruction)impl.getInstruction());
	  handle.replaceBy(impl);
	  impl.setNext(handle.getNext().getNext());

	  return true;
	} else if (next.getOpcode() == opc_ifeq) {
	  // delete
	  if (handle.remove() && handle.getNext().remove()) {
	    return true;
	  }
	}
      } else if (opcode == opc_aconst_null) {
	if (next.getOpcode() == opc_if_acmpeq) {
	  InstructionHandle	impl = new InstructionHandle(new JumpInstruction(opc_ifnull, next.getTarget()),
							     handle.getPrevious());
	  
	  ((InstructionHandle)next.getTarget()).addAccessor((JumpInstruction)impl.getInstruction());
	  handle.replaceBy(impl);
	  impl.setNext(handle.getNext().getNext());

	  return true;
	} else if (next.getOpcode() == opc_if_acmpne) {
	  InstructionHandle	impl = new InstructionHandle(new JumpInstruction(opc_ifnonnull, next.getTarget()),
							     handle.getPrevious());

	  ((InstructionHandle)next.getTarget()).addAccessor((JumpInstruction)impl.getInstruction());
	  handle.replaceBy(impl);
	  impl.setNext(handle.getNext().getNext());

	  return true;
	}
      }
    }
*/
    handle.set();

    return false;
  }

  // --------------------------------------------------------------------
  // PRIVATE METHODS
  // --------------------------------------------------------------------

  private static int getReverseOpcode(int opcode) {
    switch (opcode) {
    case opc_if_acmpeq:
      return opc_if_acmpne;
    case opc_if_acmpne:
      return opc_if_acmpeq;
    case opc_if_icmpeq:
      return opc_if_icmpne;
    case opc_if_icmpne:
      return opc_if_icmpeq;
    case opc_if_icmplt:
      return opc_if_icmpge;
    case opc_if_icmple:
      return opc_if_icmpgt;
    case opc_if_icmpgt:
      return opc_if_icmple;
    case opc_if_icmpge:
      return opc_if_icmplt;
    case opc_ifeq:
      return opc_ifne;
    case opc_ifne:
      return opc_ifeq;
    case opc_iflt:
      return opc_ifge;
    case opc_ifgt:
      return opc_ifle;
    case opc_ifle:
      return opc_ifgt;
    case opc_ifge:
      return opc_iflt;
    case opc_ifnull:
      return opc_ifnonnull;
    case opc_ifnonnull:
      return opc_ifnull;
    }
    throw new RuntimeException("OPCODE NOT ALLOWED:" + opcode);
  }
}


// OLD
//   // --------------------------------------------------------------------
//   // OPTIMIZING A 5 INSTRUCTION DEPTH BLOCK
//   // --------------------------------------------------------------------
// 
//   /**
//    * Optimize a block of 5 instructions
//    *
//    * Return true if an optimization is made
//    * This method should not modify other instructions
//    */
//   protected boolean optimizer5() {
//     if (isJump()) { // replace boolean branch
//       //    Java code that looks like this:     with this:
//       //         if_icmplt A                      B: if_icmpge X
//       //         iconst_0			     goto Z {if someone branched on goto B}
//       //         goto B				  Z
//       //      A: iconst_1
//       //      B: ifeq X
//       //      Z
//       switch (getOpcode()) {
//       case opc_if_acmpeq:
//       case opc_if_acmpne:
//       case opc_if_icmpeq:
//       case opc_if_icmpne:
//       case opc_if_icmplt:
//       case opc_if_icmple:
//       case opc_if_icmpgt:
//       case opc_if_icmpge:
//       case opc_ifeq:
//       case opc_ifne:
//       case opc_iflt:
//       case opc_ifgt:
//       case opc_ifle:
//       case opc_ifge:
//       case opc_ifnull:
//       case opc_ifnonnull:
// 	boolean		startByZero;
// 	boolean		reverse;
// 	Instruction	pointer = next;
// 	Instruction	gotoIns;
// 
// 	if (pointer.getOpcode() == opc_iconst_0) {
// 	  startByZero = true;
// 	} else if (pointer.getOpcode() == opc_iconst_1) {
// 	  startByZero = false;
// 	} else {
// 	  break;
// 	}
// 
// 	gotoIns = pointer = pointer.next;
// 	if (pointer.getOpcode() != opc_goto) // short one
// 	  break;
// 
// 	pointer = pointer.next;
// 	if (pointer.getOpcode() != opc_iconst_0 && (pointer.getOpcode() != opc_iconst_1)) {
// 	  break;
// 	} else if (pointer.getOpcode() == opc_iconst_0 && startByZero) {
// 	  break;
// 	} else if (pointer.getOpcode() == opc_iconst_1 && !startByZero) {
// 	  break;
// 	}
// 
// 	pointer = pointer.next;
// 	if (gotoIns.getTarget() != pointer) {
// 	  break;
// 	}
// 	if (pointer.getOpcode() == opc_ifeq) {
// 	  reverse = true;
// 	} else if (pointer.getOpcode() == opc_ifne) {
// 	  reverse = false;
// 	} else {
// 	  break;
// 	}
// 
// 	reverse = startByZero ? reverse : !reverse;
// 
// 	if (reverse) {
// 	  impl = new JumpInstruction(getReverseOpcode(getOpcode()), pointer.getTarget());
// 	} else {
// 	  impl = new JumpInstruction(getOpcode(), pointer.getTarget());
// 	}
// 
// 	impl.next = new JumpInstruction(opc_goto, pointer.next);
// 	impl.next.next = gotoIns;
// 	forwardImpl = true;
// 	return true;
//       }
//     }
//     return false;
//   }
// 
//   // --------------------------------------------------------------------
//   // OPTIMIZING A 3 INSTRUCTION DEPTH BLOCK
//   // --------------------------------------------------------------------
// 
//   /**
//    * Optimize a block of 3 instructions
//    *
//    * Return true if an optimization is made
//    * This method should not modify other instructions
//    */
//   protected boolean optimizer3() {
//     if (isJump() && next.isJump() && getTarget() == next.next && !next.isTarget()) {
//       //  ifeq X             ifne Z
//       //  goto Z             X:
//       //X:...           ===>
//       if (((JumpInstruction)next).isGoto()) {
// 	switch (getOpcode()) {
// 	case opc_if_icmpeq:
// 	case opc_if_icmpne:
// 	case opc_if_icmplt:
// 	case opc_if_icmple:
// 	case opc_if_icmpgt:
// 	case opc_if_icmpge:
// 	case opc_ifeq:
// 	case opc_ifne:
// 	case opc_iflt:
// 	case opc_ifgt:
// 	case opc_ifle:
// 	case opc_ifge:
// 	case opc_ifnull:
// 	case opc_ifnonnull:
// 	  impl = new JumpInstruction(getReverseOpcode(getOpcode()), next.getTarget());
// 	  impl.next = next.next;
// 	  return true;
// 	}
//       }
//     } else if (getOpcode() == opc_iconst_m1) {
//       // -1
//       // ixor
//       // ifeq         ==>  jmp
//       if ((next.getOpcode() == opc_ixor) && (next.next.getOpcode() == opc_ifeq)) { // $$$ idem for ne
// 	impl = new JumpInstruction(opc_ifne, next.next.getTarget());
// 	impl.next = next.next.next;
// 	return true;
//       }
//     }
//     return false;
//   }
// 
//   /**
//    * Optimize a block of 3 instructions
//    *
//    * Return true if an optimization is made
//    * This method should not modify other instructions
//    */
//   protected boolean softOptimizer3() {
//     if (isLiteral() && next.isJump()) { // zero or null compare
//       JumpInstruction	middle = (JumpInstruction)next;
//       Instruction	next = middle.getTarget();
//       if (middle.isGoto() && next.isJump()) {
// 	switch (getOpcode()) {
// 	case opc_iconst_0:
// 	  forwardImpl = true;
// 	  switch (next.getOpcode()) {
// 	  case opc_if_icmpeq:
// 	    impl = new JumpInstruction(opc_ifeq, next.getTarget());
// 	    impl.next = next.next;
// 	    impl.next.dump();
// 	    return true;
// 	  case opc_if_icmpne:
// 	    impl = new JumpInstruction(opc_ifne, next.getTarget());
// 	    impl.next = next.next;
// 	    return true;
// 	  case opc_if_icmplt:
// 	    impl = new JumpInstruction(opc_iflt, next.getTarget());
// 	    impl.next = next.next;
// 	    return true;
// 	  case opc_if_icmpgt:
// 	    impl = new JumpInstruction(opc_ifgt, next.getTarget());
// 	    impl.next = next.next;
// 	    return true;
// 	  case opc_if_icmpge:
// 	    impl = new JumpInstruction(opc_ifge, next.getTarget());
// 	    impl.next = next.next;
// 	    return true;
// 	  case opc_if_icmple:
// 	    impl = new JumpInstruction(opc_ifle, next.getTarget());
// 	    impl.next = next.next;
// 	    return true;
// 	  case opc_ifeq:
// 	    impl = new JumpInstruction(opc_goto, next.getTarget());
// 	    impl.next = next.next;
// 	    return true;
// 	  case opc_ifne:
// 	    impl = next.next;
// 	    forwardImpl = true;
// 	    return true;
// 	  }
// 	  forwardImpl = false;
// 	  break;
// 	case opc_iconst_1:
// 	  if (next.getOpcode() == opc_ifne) {
// 	    impl = new JumpInstruction(opc_goto, next.getTarget());
// 	    impl.next = next.next;
// 	    return true;
// 	  } else if (next.getOpcode() == opc_ifeq) {
// 	    impl = next.next;
// 	    forwardImpl = true;
// 	    return true;
// 	  }
// 	  break;
// 	case opc_aconst_null:
// 	  if (next.getOpcode() == opc_if_acmpeq) {
// 	    impl = new JumpInstruction(opc_ifnull, next.getTarget());
// 	    impl.next = next.next;
// 	    return true;
// 	  } else if (next.getOpcode() == opc_if_acmpne) {
// 	    impl = new JumpInstruction(opc_ifnonnull, next.getTarget());
// 	    impl.next = next.next;
// 	    return true;
// 	  }
// 	break;
// 	}
//       }
//     }
// 
//     return false;
//   }
// 
//   // --------------------------------------------------------------------
//   // OPTIMIZING A 2 INSTRUCTION DEPTH BLOCK
//   // --------------------------------------------------------------------
// /*  // DISABLED DU TO const 1; dup; const 1
//     if (getOpcode() == next.getOpcode()) {
//       if (isLiteral()) {
// 	if ((this instanceof PushLiteralInstruction)&& ((PushLiteralInstruction)this).getValue().equals(((PushLiteralInstruction)next).getValue())) {
// 	  Instruction tmp = next.next;
// 	  next = getStack() == 2 ?
// 	    new NoArgInstruction(opc_dup2) :
// 	    new NoArgInstruction(opc_dup);
// 	  next.next = tmp;
// 	  return true;
// 	}
//       } else if ((this instanceof LocalVarInstruction) &&
// 		 ((LocalVarInstruction)this).getIndex() == ((LocalVarInstruction)next).getIndex() &&
// 		 getStack() > 0) {
// 	Instruction tmp = next.next;
// 	next = getStack() == 2 ?
// 	  new NoArgInstruction(opc_dup2) :
// 	  new NoArgInstruction(opc_dup);
// 	next.next = tmp;
// 	return true;
//       }
//     }
// */
//     return false;//softOptimizer3(); //DISABLED DUE TO:  (i != 0 && i != (printFirst ? 0 : 1))
//   }
// 
// 
