/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: DeleteStatement.java,v 1.12 1999/10/12 15:39:48 vgp Exp $
 */

package at.dms.sqlc;

import java.util.Vector;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

public class DeleteStatement extends Statement implements SqlContext {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructor
   * @param	ref		           the token reference for this statement
   * @param	schemaName
   * @param	tableName
   * @param	searchCondition
   */
  public DeleteStatement (TokenReference ref,
			  String schemaName,
			  String tableName,
			  SearchCondition searchCondition)
  {
    super(ref);
    this.schemaName = schemaName;
    this.tableName = tableName;
    this.searchCondition = searchCondition;
  }

  // --------------------------------------------------------------------
  // ACCEPT VISITORS
  // --------------------------------------------------------------------

  /**
   * Accepts a visitor.
   *
   * @param	visitor			the visitor
   */
  public void accept(SqlVisitor visitor) throws PositionedError {
    try {
      parent = visitor.getContext();
      visitor.enter(this);
      visitor.visitDeleteStatement(schemaName, tableName, searchCondition);
      visitor.exit(this);
    } catch (UnpositionedError e) {
      throw e.getError(getTokenReference());
    }
  }

  // ----------------------------------------------------------------------
  // CHECK SQL
  // ----------------------------------------------------------------------

  /**
   * Returns the table reference with alias "alias"
   */
  public TableReference getTableFromAlias(String alias) {
    if (alias.equals(tableName)) {
      return new TableName(getTokenReference(),
			   new SimpleIdentExpression(getTokenReference(), schemaName),
			   new TableAlias(getTokenReference(), tableName, null));
    }
    return null;
  }

  /**
   * Returns all tables defined in current context
   */
  public Vector getTables() {
    Vector tables = new Vector();
    TableReference ref = new TableName(getTokenReference(),
				       new SimpleIdentExpression(getTokenReference(), schemaName),
				       new TableAlias(getTokenReference(), tableName, null));
    tables.addElement(ref);
    return tables;
  }

  /**
   * Returns the parent context
   */
  public SqlContext getParentContext() {
    return parent;
  }

  /**
   * Add an error into the list and eat it
   * This method should be called after a try catch block after catching exception
   * or directly without exception thrown
   * @param	error		the error
   */
  public void reportTrouble(PositionedError trouble) {
    parent.reportTrouble(trouble);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private SqlContext		parent;

  private String		schemaName;
  private String		tableName;
  private SearchCondition	searchCondition;
}
