/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: ExpressionList.java,v 1.15 1999/10/12 15:39:48 vgp Exp $
 */

package at.dms.sqlc;

import java.util.Vector;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

public class ExpressionList extends SqlPhylum {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructor
   * @param	ref		           the token reference for this statement
   * @param	exprs	                   the vector of expression
   */
  public ExpressionList (TokenReference ref, Vector exprs) {
    super(ref);
    this.exprs = exprs;
  }

  // ----------------------------------------------------------------------
  // ACCESSOR
  // ----------------------------------------------------------------------

  /**
   * getExpression
   */
  public Vector getExpression() {
    return exprs;
  }

  /**
   * getElem
   */
  public Expression getElem(int i) {
    return (Expression)exprs.elementAt(i);
  }

  // ----------------------------------------------------------------------
  // FUNCTIONS
  // ----------------------------------------------------------------------

  public boolean equals(Object otherExpressionList) {
   if (otherExpressionList instanceof ExpressionList) {
      Vector	otherExprs = ((ExpressionList)otherExpressionList).getExpression();
      if (exprs.size() == otherExprs.size()) {
	for (int i = 0; i < exprs.size(); i++) {
	  if ((exprs.elementAt(i) instanceof Literal) &&
	      (otherExprs.elementAt(i) instanceof Literal)) {
	    if (!((Literal)exprs.elementAt(i)).equals((Literal)otherExprs.elementAt(i))) {
	      for (int j = 0; j < exprs.size(); j++) {
	       }
	      return false;
	    }
	  } else {
	    System.err.println("Can't compare Expressions that are not literals");
	    return false;
	  }
	}
	return true;
      }
    }
    return false;
  }

  // --------------------------------------------------------------------
  // ACCEPT VISITORS
  // --------------------------------------------------------------------

  /**
   * Accepts a visitor.
   *
   * @param	visitor			the visitor
   */
  public void accept(SqlVisitor visitor) throws PositionedError {
    try {
      visitor.visitExpressionList(exprs);
    } catch (UnpositionedError e) {
      throw e.getError(getTokenReference());
    }
  }

  // ----------------------------------------------------------------------
  // CHECK SQL
  // ----------------------------------------------------------------------

  /**
   * Return the number of elements in the vector
   */
  public int elemNumber() {
    int number = 0;
    for(int i = 0; i < exprs.size(); i++) {
      number++;
    }
    return number;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private Vector      exprs;
}
