/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: FieldNameList.java,v 1.14 1999/10/12 15:39:48 vgp Exp $
 */

package at.dms.sqlc;

import java.util.Vector;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;
import at.dms.compiler.CWarning;

public class FieldNameList extends SqlPhylum {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructor
   * @param	ref		           the token reference for this statement
   * @param	exprs	                   the vector of fields
   */
  public FieldNameList (TokenReference ref, Vector fields) {
    super(ref);
    this.fields = fields;
  }

 // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  public boolean hasColumn(String ident) {
    for(int i = 0; i < fields.size(); i++) {
      if (ident.equals(fields.elementAt(i))) {
	return true;
      }
    }
    return false;
  }

 /**
   * getFields
   */
  public Vector getFields() {
    return fields;
  }

  // ----------------------------------------------------------------------
  // FUNCTIONS
  // ----------------------------------------------------------------------

  public boolean equals(Object otherFieldNameList) {
    if (otherFieldNameList instanceof FieldNameList) {
      Vector	otherFields = ((FieldNameList)otherFieldNameList).getFields();
      if (fields.size() == otherFields.size()) {
	for (int i = 0; i < fields.size(); i++) {
	  if (!fields.elementAt(i).equals(otherFields.elementAt(i))) {
	    return false;
	  }
	}
	return true;
      }
    }
    return false;
  }

  // --------------------------------------------------------------------
  // ACCEPT VISITORS
  // --------------------------------------------------------------------

  /**
   * Accepts a visitor.
   *
   * @param	visitor			the visitor
   */
  public void accept(SqlVisitor visitor) throws PositionedError {
    try {
      visitor.visitFieldNameList(fields);
    } catch (UnpositionedError e) {
      throw e.getError(getTokenReference());
    }
  }

  // ----------------------------------------------------------------------
  // CHECK SQL
  // ----------------------------------------------------------------------

  /**
   * Return the number of elements in the vector
   */
  public int elemNumber() {
    int number = 0;
    for (int i = 0; i < fields.size(); i++) {
      number++;
    }
    return number;
  }

  /**
   * This function check if all the idents are unique
   */
  public boolean checkIdentsAreUnique(SqlContext context) {
    for(int i = 0; i < fields.size(); i++) {
      String field = (String)fields.elementAt(i);
      for (int j = i + 1; j < fields.size(); j++) {
	if (field.equals(fields.elementAt(j))) {
	  context.reportTrouble(new CWarning(getTokenReference(), "x-column-not-unique", field));
	  return false;
	}
      }
    }
    return true;
  }
/*
!!!
  public JExpression checkSQL(SqlContext context, JExpression left, StringBuffer current, TableReference table) {
    for (int i = 0; i < fields.size(); i++) {
      if (i != 0) {
	current.append(", ");
      }
      current.append(fields.elementAt(i));
    }

    // check if the columns are in the table
    //if ((context.getTables()).size() == 1) {
      //TableReference table = (TableReference)(context.getTables()).elementAt(0);
      for(int i = 0; i < fields.size(); i++) {
	if (!table.hasColumn((String)fields.elementAt(i))) {
	  context.reportTrouble(new CWarning(getTokenReference(), "x-column-not-resolvable", fields.elementAt(i)));
	}
      }
   // } else {
   //   context.reportTrouble(new CWarning(getTokenReference(), "x-more-than-one-name"));
  //  }
    return left;
  }
*/

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private Vector      fields;
}
