/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: FieldReference.java,v 1.12 1999/10/12 15:39:48 vgp Exp $
 */

package at.dms.sqlc;

import java.util.Vector;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;
import at.dms.compiler.CWarning;

public class FieldReference extends Expression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructor
   * @param	ref		           the token reference for this statement
   * @param	table	           the table name if any
   * @param	ident	           the field ident
   */
  public FieldReference (TokenReference ref,
			 String table,
			 String ident) {
    super(ref);
    this.table = table;
    this.ident = ident;
  }

  // --------------------------------------------------------------------
  // ACCEPT VISITORS
  // --------------------------------------------------------------------

  /**
   * Accepts a visitor.
   *
   * @param	visitor			the visitor
   */
  public void accept(SqlVisitor visitor) throws PositionedError {
    try {
      visitor.visitFieldReference(table, ident);
    } catch (UnpositionedError e) {
      throw e.getError(getTokenReference());
    }
  }

  // ----------------------------------------------------------------------
  // CHECK SQL
  // ----------------------------------------------------------------------

  /**
   * test if the alias exists // la table a la colonne ident ?
   */
   public static String checkColumnExists(TokenReference ref, SqlContext context, String table, String ident) {
    if (table != null) {
      TableReference  tableReference = context.getTableFromAlias(table);
      if (tableReference != null) {
	if (!tableReference.hasColumn(ident)) {
	  context.reportTrouble(new CWarning(ref, "x-column-not-found", ident, table));
	}
	return tableReference.getTableForColumn(ident);
      } else {
	  context.reportTrouble(new CWarning(ref, "x-alias-not-found", table));
	  return null;
      }
    } else {
    ////////////else verifier taille du vecteur getTables() de  a un a table refe supporte la colonne ident avec hasColumn
      Vector TableList = context.getTables();
      if (TableList.size() == 1) {
	TableReference newTable = (TableReference)TableList.elementAt(0);
	if (!(newTable.hasColumn(ident))) {
	  if (newTable instanceof TableName) {
	    context.reportTrouble(new CWarning(ref, "x-column-not-found", ident, ((TableName)newTable).getTableName()));
	  } else {
	    context.reportTrouble(new CWarning(ref, "x-column-not-correct", ident));
	  }
	  return null;
	}
	return ((TableReference)TableList.elementAt(0)).getTableForColumn(ident);
      } else {
	context.reportTrouble(new CWarning(ref, "x-ident-not-resolvable", ident));
	return null;
      }
    }
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private String	table;
  private String	ident;
}
