/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: SQLPrettyPrinter.java,v 1.23 1999/10/12 15:39:49 vgp Exp $
 */

package at.dms.sqlc;

import java.util.Vector;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.StringTokenizer;

/**
 * This class implements a Java pretty printer
 */
public class SQLPrettyPrinter {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * construct a pretty printer object for java code
   * @param	fileName		the file into the code is generated
   */
  public SQLPrettyPrinter(String fileName) {
    String	outputFile = fileName + ".gen";

    try {
      this.p = new java.io.PrintWriter(new java.io.BufferedWriter(new java.io.FileWriter(outputFile)));
    } catch (Exception e) {
      e.printStackTrace();
      //Main.addError(new CWarning(getTokenReference(), "cant-write", fileName, e));
      System.err.println("cant write " + outputFile + ": " + e);
    }
    this.pos = 0;
  }

  /**
   * construct a pretty printer object for java code
   * @param	fileName		the file into the code is generated
   */
  public SQLPrettyPrinter(java.io.PrintWriter p, CJavaPrettyPrinter java_pp) {
    this.p = p;
    this.pos = 0;
    this.in_pp = java_pp;
  }

  /**
   * Returns the java pretty printer
   */
  public CJavaPrettyPrinter getJavaPrettyPrinter() {
    in_pp.setPos(pos);
    return in_pp;
  }

  /**
   * Close
   */
  public void close() {
    p.close();
  }

  /**
   * Sets the position of tabs
   */
  public void setPos(int pos) {
    this.pos = pos;
  }

  // ----------------------------------------------------------------------
  // STATEMENT
  // ----------------------------------------------------------------------

  /**
   * prints a while statement
   */
  public void printCUnit(Vector elems) {
    for (int i = 0; i < elems.size(); i++) {
      ((Statement)elems.elementAt(i)).genSQLCode(this);
      println();
    }
  }

  /**
   * append SQL code
   */
  public void appendString(String code) {
    print(code);
  }

  /**
   * append SQL code
   */
  public void appendString(String code, int length) {
    appendString(code);
    print(space(length - code.length()));
  }

  // ----------------------------------------------------------------------
  // SELECT STATEMENT
  // ----------------------------------------------------------------------


  /**
   * prints an array length expression
   */
  public void printComment(JComment comment) {
    StringTokenizer	tok = new StringTokenizer(comment.getText(), "\n");
    if (tok.countTokens() == 1) {
      print("// " + tok.nextToken().trim());
      println();
    } else if (tok.countTokens() <= 3) {
      println();
      while (tok.hasMoreTokens()) {
	print("//");
	String  comm = tok.nextToken().trim();
	if (comm.startsWith("*")) {
	  comm = comm.substring(1).trim();
	}
	if (comm.length() == 0) {
	  continue;
	}
	print(" " + comm);
	println();
      }
    } else {
      if (line > 0) {
	println();
      }
      print("/*");
      while (tok.hasMoreTokens()) {
	String comm = tok.nextToken().trim();
	if (comm.startsWith("*")) {
	  comm = comm.substring(1).trim();
	}
	if (comm.length() == 0) {
	  continue;
	}
	println();
	print(" * " + comm);
      }
      println();
      print(" */");
      println();
    }
  }

  /**
   * print SELECT
   */
   public void printSelectStatement(String type,
				    Vector columns,
				    FromClause fromClause,
				    WhereClause whereClause,
				    GroupByClause groupByClause,
				    HavingClause havingClause,
				    SortSpec sortSpec,
				    UpdSpec updateSpec)
  {
    print("SELECT ");

    pos += TAB_SIZE;
    if (type != null) {
      print(type);
      print(" ");
    }
    for(int i = 0; i < columns.size(); i++) {
      if (i != 0) {
	println(",");
      }
      ((SelectElem)columns.elementAt(i)).genSQLCode(this);
    }
    println();
    pos -= TAB_SIZE;

    fromClause.genSQLCode(this);

    if (whereClause != null) {
      whereClause.genSQLCode(this);
    }
    if (groupByClause != null) {
      groupByClause.genSQLCode(this);
    }
    if (havingClause !=null) {
      havingClause.genSQLCode(this);
    }
    if (sortSpec != null) {
      sortSpec.genSQLCode(this);
    }
    if (updateSpec != null) {
      updateSpec.genSQLCode(this);
    }
  }

  /**
   * print SELECT
   */
  public void printSelectExpression(String type,
				    Vector columns,
				    FromClause fromClause,
				    WhereClause whereClause,
				    GroupByClause groupByClause,
				    HavingClause havingClause)
  {
    print("SELECT ");

    pos += TAB_SIZE;
    if (type != null) {
      print(type);
      print(" ");
    }
    for(int i = 0; i < columns.size(); i++) {
      if (i != 0) {
	println(",");
      }
      ((SelectElem)columns.elementAt(i)).genSQLCode(this);
    }
    println();
    pos -= TAB_SIZE;

    fromClause.genSQLCode(this);

    if (whereClause != null) {
      whereClause.genSQLCode(this);
    }
    if (groupByClause != null) {
      groupByClause.genSQLCode(this);
    }
    if (havingClause !=null) {
      havingClause.genSQLCode(this);
    }
 }

  /**
   * print GROUP BY
   * @param	fields	                   the fields refernces
   */
  public void printGroupByClause(Vector fields) {
    print("GROUP BY");
    pos += TAB_SIZE;
    for(int i = 0; i < fields.size(); i++) {
      if (i != 0) {
	print(", ");
      }
      ((FieldReference)fields.elementAt(i)).genSQLCode(this);
    }
    pos -= TAB_SIZE;
    println();
  }

  /**
   * print HAVING
   * @param	condition	           the SqlPhylum for the having
   */
  public void printHavingClause(SearchCondition condition) {
    print("HAVING");
    pos += TAB_SIZE;
    condition.genSQLCode(this);
    pos -= TAB_SIZE;
    println();
  }

  /**
   * print SORTELEM
   * @param	elem
   * @param	isDesc
   */
  public void printSortElem(Expression elem, boolean isDesc) {
    elem.genSQLCode(this);
    if(isDesc) {
      print(" DESC");
    }
  }

  /**
   * print UPDSPEC
   */
  public void printUpdSpec() {
    print("FOR UPDATE");
  }

   /**
   * print SORTSPEC
   */
  public void printSortSpec(Vector elems) {
    print("ORDER BY ");
    pos += TAB_SIZE;
    for(int i = 0; i < elems.size(); i++) {
      if (i != 0) {
	print(", ");
      }
      ((SortElem)elems.elementAt(i)).genSQLCode(this);
    }
    pos -= TAB_SIZE;
  }

  /**
   * print SORTELEM
   */
  public void printSortElem(IntegerLiteral intLiteral,
			    IntegerLiteral exprLiteral,
			    boolean hasAsc,
			    boolean isAsc)
  {
    //!!!println("FOR UPDATE");
  }

/**
   * print UPDATE
   * @param	schemaName	schema name to be print
   * @param	tableName	table name to be print
   * @param	listIdent       vector of listIdent
   * @param	searchCondition search condition to be print
   * @param	asCondition     there is a search condition ?
   */
  public void printUpdateStatement(String schemaName, String tableName, Vector listIdent, SearchCondition searchCondition, boolean hasCondition) {
    print("UPDATE ");

    pos += TAB_SIZE;
    print(schemaName);
    if (tableName != null) {
      print(" ");
      print(tableName);
    }

    println();
    pos -= TAB_SIZE;
    print("SET ");
    pos += TAB_SIZE;

    for(int i = 0; i < listIdent.size(); i++) {
      if (i != 0) {
	println(",");
      }
      ((Assignment)listIdent.elementAt(i)).genSQLCode(this);
    }

    if (hasCondition) {
      println();
      pos -= TAB_SIZE;
      print("WHERE ");
      if (searchCondition != null) {
	pos += TAB_SIZE;
	searchCondition.genSQLCode(this);
	pos -= TAB_SIZE;
      } else {
	print("CURRENT");
      }
    }
  }

  /**
   * print Insert statement
   * @param	ident		type ident of the table
   * @param	fields		the field name list
   * @param	soure		the source of data from clause
   */
  public void printInsertStatement(String ident, FieldNameList fields, InsertSource source) {
    print("INSERT INTO ");
    //pos += TAB_SIZE;
    print(ident);
    pos += TAB_SIZE;
    if (fields != null) {
      print(" (");
      fields.genSQLCode(this);
      print(")");
    }
    println();
    pos -= TAB_SIZE;
    source.genSQLCode(this);
  }

  /**
   * print ExpressionList
   * @param	exprs	a vector of SqlPhylum
   */
  public void printExpressionList(Vector exprs) {
    for(int i = 0; i < exprs.size(); i++) {
      if (i != 0) {
	print(", ");
      }
      ((Expression)exprs.elementAt(i)).genSQLCode(this);
    }
  }

  /**
   * print FieldNameList
   * @param	exprs	a vector of SqlPhylum
   */
  public void printFieldNameList(Vector exprs) {
    for(int i = 0; i < exprs.size(); i++) {
      if (i != 0) {
	print(", ");
      }
      print(exprs.elementAt(i));
    }
  }

  /**
   * print WhereClause
   * @param	condition	a SqlPhylum
   */
  public void printWhereClause(SearchCondition condition) {
    print("WHERE");
    pos += TAB_SIZE;
    condition.genSQLCode(this);
    println();
    pos -= TAB_SIZE;
  }

  /**
   * print NotCondition
   * @param	condition	a SqlPhylum
   */
  public void printNotCondition(SearchCondition condition) {
    print("NOT ");
    condition.genSQLCode(this);
  }

  /**
   * print OnJoinPred
   * @param	condition
   */
  public void printOnJoinPred(SearchCondition condition) {
    print("ON ");
    condition.genSQLCode(this);
  }

  /**
   * print UsingJoinPred
   * @param	list
   */
  public void printUsingJoinPred(FieldNameList list) {
    print("USING (");
    list.genSQLCode(this);
    print(")");
  }

  /**
   * print AndCondition
   * @param	left	the left search condition
   * @param	right	the right search condition
   */
  public void printAndCondition(SearchCondition left, SearchCondition right) {
    left.genSQLCode(this);
    println();
    pos -= TAB_SIZE;
    print("AND");
    pos += TAB_SIZE;
    right.genSQLCode(this);
  }

  /**
   * print OrCondition
   * @param	left	the left search condition
   * @param	right	the right search condition
   */
  public void printOrCondition(SearchCondition left, SearchCondition right) {
    left.genSQLCode(this);
    println();
    pos -= TAB_SIZE;
    print("OR");
    pos += TAB_SIZE;
    right.genSQLCode(this);
  }

  /**
   * print printFieldReferencePrimary
   * @param	fieldRef	           the field reference
   * @param	begin	                   the begining of the subrange
   * @param	end	                   the end of the subrange
   */
  public void printFieldReferencePrimary(Expression fieldRef, Expression begin, Expression end) {
    fieldRef.genSQLCode(this);
    if (begin!= null) {
      print(" SUBRANGE(");
     begin.genSQLCode(this);
      print(", ");
     end.genSQLCode(this);
      print(") ");
   }
  }

  /**
   * print printFieldReference
   * @param	table	           the table name if any
   * @param	ident	           the field ident
   */
  public void printFieldReference(String table, String ident) {
    if (table != null) {
      print(table);
      print(".");
   }
    print(ident);
  }

 /**
   * print printBinaryExpression
   * @param	left	                   the left expression
   * @param	right	                   the right expression
   * @param	opt	                   the operator
   */
  public void printBinaryExpression(Expression left, Expression right, String opt) {
     left.genSQLCode(this);
     print(" ");
     print(opt);
     print(" ");
     right.genSQLCode(this);
  }

 /**
   * print printSimpleIdentExpression
   * @param	ident	                   the ident
   */
  public void printSimpleIdentExpression(String ident) {
    print(ident);
  }

 /**
   * print printUnaryExpression
   * @param	expr	                   the unary expression
   * @param	opt	                   the operator
   */
  public void printUnaryExpression(Expression expr, String opt) {
    print(opt);
    expr.genSQLCode(this);
  }

  /**
   * print FROM
   * @param	tableRefs	a vector of table references
   */
  public void printFromClause(Vector tableRefs) {
    print("FROM");

    pos += TAB_SIZE;

    for(int i = 0; i < tableRefs.size(); i++) {
      if (i != 0) {
	println(",");
      }
      ((SimpleTableReference)tableRefs.elementAt(i)).genSQLCode(this);
    }
    println();

    pos -= TAB_SIZE;
  }

  /**
   * Prints a string
   * @param	text	text to be print
   */
  public void printIdent(String text)  {
    print(text);
  }

  /**
   * print BooleanLiteral
   * @param	value	the boolean literal
   */
  public void printBooleanLiteral(boolean value) {
    print(value ? "TRUE" : "FALSE");
  }

  /**
   * print NullLiteral
   * @param	value	the null literal
   */
  public void printNullLiteral() {
    print("NULL");
  }

  /**
   * print StringLiteral
   * @param	value	the string literal
   */
  public void printStringLiteral(String value) {
    print("'");
    print(value);
    print("'");
  }

  /**
   * print BincharLiteral
   * @param	value	the binchar literal
   */
  public void printBincharLiteral(String value) {
    print(value);
  }

  /**
   * print BitLiteral
   * @param	value	the bit literal
   */
  public void printBitLiteral(String value) {
    print(value);
  }

  /**
   * print ExistsPredicate
   * @param	sub the sub table
   */
  public void printExistsPredicate(SubTableExpression sub) {
    print("EXISTS (");
    sub.genSQLCode(this);
    print(")");
  }

  /**
   * print ValueList
   * @param	value	the list of value
   */
  public void printValueList(ExpressionList expr) {
    print("(");
    expr.genSQLCode(this);
    print(")");
  }

  /**
   * print ArrayPrecision
   * @param	size	a SqlPhylum
   */
  public void printArrayPrecision(IntegerLiteral size) {
    print("(");
    if (size != null) {
      size.genSQLCode(this);
    } else {
      print("*");
    }
    print(")");
  }

  /**
   * print BetweenPredicate
   * @param	start		begining of the range
   * @param	startQualifier	mode (INCLUSIVE | EXCLUSIVE)
   * @param	end		end of the range
   * @param	endQualifier	mode (INCLUSIVE | EXCLUSIVE)
   */
  public void printBetweenPredicate(Expression start,
				    String startQualifier,
				    Expression end,
				    String endQualifier) {
    print(" BETWEEN ");
    start.genSQLCode(this);
    if (startQualifier != null) {
      print(" ");
      print(startQualifier);
    }
    print(" AND ");
    end.genSQLCode(this);
    if (endQualifier != null) {
      print(" ");
      print(endQualifier);
    }
  }

  /**
   * print ValueInPredicate
   * @param	value		value  of the predicate
   */
  public void printValueInPredicate(ExpressionList value) {
    print("  IN (");
    value.genSQLCode(this);
    print(")");
  }

  /**
   * print LikePredicate
   * @param	sens
   * @param	expr
   * @param	escape
   */
  public void printLikePredicate(String sens, Expression expr, JdbcEscape escape) {
    print(" LIKE ");
    if (sens != null) {
      print(" ");
      print(sens);
    }
    print(" ");
    expr.genSQLCode(this);
    if (escape != null) {
      print(" ");
      escape.genSQLCode(this);
    }
  }

  /**
   * print MatchesPredicate
   * @param	sens
   * @param	matchesExpr
   * @param	escapeExpr
   */
  public void printMatchesPredicate(String sens,
				    Expression matchesExpr,
				    Expression escapeExpr) {
    print(" MATCHES ");
    print(sens);
    print(" ");
    matchesExpr.genSQLCode(this);
    if(escapeExpr != null) {
      print(" ESCAPE ");
      escapeExpr.genSQLCode(this);
    }
  }

  /**
   * print MatchesPredicate
   */
  public void printTupleInPredicate(ExpressionList exprs, boolean hasNot, SubTableExpression table) {
    print("[");
    exprs.genSQLCode(this);
    print("]");
    if (hasNot) {
      print(" NOT ");
    }
    print("IN ");
    table.genSQLCode(this);
  }

  /**
   * print ExpressionPredicate
   * @param	expr			the predicate expression
   */
  public void printExpressionPredicate(Expression expr) {
    expr.genSQLCode(this);
  }

  /**
   * print IsPredicate
    */
  public void printIsPredicate(Predicate predicate, boolean hasNot) {
    predicate.genSQLCode(this);
    if(hasNot) {
      print("IS NOT NULL");
    } else {
       print("IS NULL");
    }
  }

  /**
   * print QuantifiedPredicate
    */
  public void printQuantifiedPredicate(String type, SubTableExpression table) {
    print(type);
    table.genSQLCode(this);
  }

  /**
   * print ComparaisonPredicate
    */
  public void printComparaisonPredicate(Predicate left, Predicate right, String operator) {
    left.genSQLCode(this);
    print(" ");
    print(operator);
    print(" ");
    right.genSQLCode(this);
  }

  /**
   * print ExtendedPredicate
    */
  public void printExtendedPredicate(Predicate left,  String operator, Predicate right) {
    left.genSQLCode(this);
    print(" ");
    if (operator != null) {
      print(operator);
      print(" ");
    }
    right.genSQLCode(this);
  }

  /**
   * print JdbcFunction
   * @param	value	the bit literal
   */
  public void printJdbcFunction(String name, ExpressionList params) {
    print("{fn ");
    print(name);
    print("(");
    if (params != null) {
      params.genSQLCode(this);
    }
    print(")}");
 }

  /**
   * print JdbcFunction
   * @param	value	the bit literal
   */
  public void printJdbcEscape(String name) {
    print("{escape ");
    print(name);
    print("}");
 }

  /**
   * print JdbcDateLiteral
   * @param	value	the JDBC date
   */
  public void printJdbcDateLiteral(Date value) {
    print("{d ");
    print(value);
    print("}");
  }

  /**
   * print JdbcTimeLiteral
   * @param	value	the JDBC time
   */
  public void printJdbcTimeLiteral(Time value) {
    print("{t ");
    print(value);
    print("}");
  }

  /**
   * print JdbcTimeStampLiteral
   * @param	value	the JDBC time stamp
   */
  public void printJdbcTimeStampLiteral(Timestamp value) {
    print("{ts ");
    print(value);
    print("}");
  }

  /**
   * print IntegerLiteral
   * @param	value	the integer literal
   */
  public void printIntegerLiteral(Integer value) {
    print(value);
  }

  /**
   * print NumericLiteral
   * @param	value	the numeric literal
   */
  public void printNumericLiteral(java.math.BigDecimal value) {
    print(value);
  }

  /**
   * Prints an ident
   * @param	ident	ident.* to be print if it is not null else only * is print
   */
  public void printSelectElemStar(String ident) {
    if (ident != null) {
      print(ident);
      print(".*");
    } else  {
      print("*");
    }
  }

  /**
   * Prints an column name with is new name is it exists
   * @param	columnName	column name to be print
   * @param	newColumnName	new column name to be print
   */
  public void printSelectElemExpression(Expression columnExpr, String newColumnName) {
    columnExpr.genSQLCode(this);
    if (newColumnName != null) {
      print(" AS ");
      print(newColumnName);
    }
  }

 /**
   * Prints a max min avg sum expression
   * @param	oper	                   the operator
   * @param	type	                   the expression type
   * @param	expr	                   the Sql Expression
   */
  public void printSetFunction(String oper, int type, Expression expr) {
    print(oper);
    print("(");
    switch(type) {
    case SetFunction.TYP_NONE:
      break;
    case SetFunction.TYP_ALL:
      print("ALL ");
      break;
    case SetFunction.TYP_DISTINCT:
      print("DISTINCT ");
      break;
    default:
      throw new RuntimeException("Unexpected type");
    }
    expr.genSQLCode(this);
    print(")");
  }

 /**
   * Prints a count expression
   */
  public void printCountExpression(boolean hasDistinct, Expression expr) {
    if (hasDistinct) {
      print("COUNT(DISTINCT ");
      expr.genSQLCode(this);
      print(")");
    } else {
      print("COUNT(*)");
    }
  }

 /**
   * Prints if expression
   * @param	condition	           the if condition
   * @param	thenExpr	           the Expression for then
   * @param	elseExpr	           the Expression for else
   */
  public void printIfExpression(SearchCondition condition, Expression thenExpr, Expression elseExpr) {
    print("IF");
    pos += TAB_SIZE;
    condition.genSQLCode(this);
    pos -= TAB_SIZE;

    println();
    print("THEN");
    pos += TAB_SIZE;
    thenExpr.genSQLCode(this);
    pos -= TAB_SIZE;

    println();
    print("ELSE");
    pos += TAB_SIZE;
    elseExpr.genSQLCode(this);
    pos -= TAB_SIZE;

    println();
    print("FI");
  }

 /**
   * Prints the SimpleWhenClause
   * @param	whenExpr	           the Sql Expression for the when
   * @param	thenExpr	           the Sql Expression for the else
   */
  public void printSimpleWhenClause(Expression whenExpr, Expression thenExpr) {
    print("WHEN");
    pos += TAB_SIZE;
    whenExpr.genSQLCode(this);
    pos -= TAB_SIZE;
    println();
    print("THEN ");
    pos += TAB_SIZE;
    thenExpr.genSQLCode(this);
    pos -= TAB_SIZE;
  }

 /**
   * Prints the SearchedWhenClause
   * @param	condition	           the condition for the when
   * @param	thenExpr	           the Sql Expression for the else
   */
  public void printSearchedWhenClause(SearchCondition condition, Expression thenExpr) {
    print("WHEN ");
    pos += TAB_SIZE;
    condition.genSQLCode(this);
    pos -= TAB_SIZE;
    println();
    print("THEN ");
    pos += TAB_SIZE;
    thenExpr.genSQLCode(this);
    pos -= TAB_SIZE;
  }

 /**
   * Prints the SimpleCaseExpression
   * @param	expr	                   the Sql Expression for the case
   * @param	conditions	           List of the conditions
   * @param	elseExpr	           the Sql Expression for the else
   */
  public void printSimpleCaseExpression(Expression expr, Vector conditions, Expression elseExpr) {
    print("CASE");
    pos += TAB_SIZE;
    expr.genSQLCode(this);
    pos -= TAB_SIZE;
    println();
    for (int i = 0; i < conditions.size(); i++) {
      ((SimpleWhenClause)conditions.elementAt(i)).genSQLCode(this);
      println();
    }
    if (elseExpr != null) {
      print("ELSE ");
      pos += TAB_SIZE;
      elseExpr.genSQLCode(this);
      pos -= TAB_SIZE;
    println();
    }
    //println();
    print("END");
    println();
  }

 /**
   * Prints the SearchedCaseExpression
   * @param	conditions	           List of the conditions
   * @param	elseExpr	           the Sql Expression for the else
   */
  public void printSearchedCaseExpression(Vector conditions, Expression elseExpr) {
    for (int i = 0; i < conditions.size(); i++) {
      ((SearchedWhenClause)conditions.elementAt(i)).genSQLCode(this);
      println();
    }
    if (elseExpr != null) {
      print("ELSE ");
      pos += TAB_SIZE;
      elseExpr.genSQLCode(this);
      pos -= TAB_SIZE;
      println();
   }
  }

 /**
   * Prints the CaseExpression
   * @param	cases
   */
  public void printCaseExpression(AbstractCaseExpression cases) {
    print("CASE");
    println();
    cases.genSQLCode(this);
    print("END");
  }

  /**
   * Prints ASSIGNMENT
   * @param	ident        	ident to be print
   * @param	expr         	expression to be print
   * @param	searchCondition	search condition to be print
   */
  public void printAssignment(Expression left, Expression right) {
    left.genSQLCode(this);
    print(" = ");
    right.genSQLCode(this);
  }

  /**
   * Prints the CorrespondingSpec
   */
  public void printCorrespondingSpec(FieldNameList fields) {
    print("CORRESPONDING");
    if (fields != null) {
      print(" BY (");
      fields.genSQLCode(this);
      print(")");
    }
  }

  /**
   * Prints the IntersectSpec
   */
  public void printIntersectSpec(CorrespondingSpec spec) {
    print("INTERSECT");
    if (spec != null) {
      print(" ");
      spec.genSQLCode(this);
    }
  }

  /**
   * Prints the unidiff spec
   */
  public void printUnidiffSpec(String oper, CorrespondingSpec spec) {
    print(oper + " ");
    if (spec != null) {
      print(" ");
      spec.genSQLCode(this);
    }
  }

  /**
   * Prints the union
   */
  public void printUnidiffTableTerm(UnidiffSpec spec, SubTable left, SubTable right) {
    left.genSQLCode(this);
    print(" ");
    spec.genSQLCode(this);
    print(" ");
    right.genSQLCode(this);
  }

  /**
   * Prints the union
   */
  public void printUnidiffTableReference(UnidiffSpec spec, TableReference left, TableReference right) {
    left.genSQLCode(this);
    print(" ");
    spec.genSQLCode(this);
    print(" ");
    right.genSQLCode(this);
  }

  /**
   * Prints DELETE
   * @param	schemaName	schema name to be print
   * @param	tableName	table name to be print
   * @param	searchCondition	search condition to be print
   */
  public void printDeleteStatement(String schemaName, String tableName, SearchCondition searchCondition) {
    print("DELETE FROM ");
    print(schemaName);
    if (tableName != null) {
      print(" ");
      print(tableName);
    }
    if (searchCondition != null) {
      println();
      print("WHERE ");
      searchCondition.genSQLCode(this);
    }
  }

  /**
   * printSubTableExpression
   * @param	expr		the table expression
   * @param	expr		the alias
   */
  public void printSubTableExpression(SubTable table) {
    print("(");
    table.genSQLCode(this);
    print(")");
  }

  /**
   * print DefaultValuesInsertSource
   */
  public void printDefaultValuesInsertSource() {
    print("DEFAULT VALUES");
  }

  /**
   * print ValueListInsertSource
   */
  public void printValueListInsertSource(ValueList value) {
    print("VALUES ");
    value.genSQLCode(this);
  }

  /**
   * print TableExpressionInsertSource
   */
  public void printTableExpressionInsertSource(TableReference table) {
    table.genSQLCode(this);
  }

  /**
   * print TableInsertSource
   */
  public void printTableInsertSource(Vector table) {
    print("TABLE (");
    for(int i = 0; i < table.size(); i++) {
      if (i != 0) {
	println(",");
      }
      ((ValueList)table.elementAt(i)).genSQLCode(this);
    }
     print(")");
 }

 /**
   * print TableAlias
   */
  public void printTableAlias(String name, FieldNameList list) {
    if (name != null) {
      print(" ");
      print(name);
    }
    if (list != null) {
      print("(");
      list.genSQLCode(this);
      print(")");
    }
  }


  // ----------------------------------------------------------------------
  // TYPE DEFINITION
  // ----------------------------------------------------------------------

  /**
   * print TinyIntType
   */
  public void printTinyIntType() {
    print("TINYINT");
  }

  /**
   * print SmallIntType
   */
  public void printSmallIntType() {
    print("SMALLINT");
  }

  /**
   * print IntegerType
   */
  public void printIntegerType() {
    print("INTEGER");
  }

  /**
   * print NumericType
   * @param	left		the number of digits
   * @param	right		the position of the comma
   */
  public void printNumericType(IntegerLiteral left, IntegerLiteral right) {
    print("NUMERIC");
    if (left != null) {
      print("(");
      left.genSQLCode(this);
      if (right != null) {
	print(", ");
	right.genSQLCode(this);
      }
      print(")");
    }
  }

  /**
   * print FloatType
   */
  public void printFloatType() {
    print("FLOAT");
  }

  /**
   * print DoubleType
   */
  public void printDoubleType() {
    print("DOUBLE");
  }

  /**
   * print RealType
   */
  public void printRealType() {
    print("REAL");
  }

  /**
   * print CharType
   */
  public void printCharType(ArrayPrecision prec) {
    print("CHAR");
    if (prec != null) {
      prec.genSQLCode(this);
    }
  }

  /**
   * print TypeName
   */
  public void printTypeName(String ident) {
    print(ident);
  }

  /**
   * print BinCharType
   */
  public void printBinCharType(ArrayPrecision prec) {
    print("BINCHAR");
    if (prec != null) {
      prec.genSQLCode(this);
    }
  }

  /**
   * print StringType
   */
  public void printStringType() {
    print("STRING");
  }


  /**
   * print BitsType
   */
  public void printBitsType(ArrayPrecision prec) {
    print("BITS");
    prec.genSQLCode(this);
  }

  /**
   * print BoolType
   */
  public void printBoolType() {
    print("BOOL");
  }


  /**
   * print BlobType
   */
  public void printBlobType() {
    print("BLOB");
  }

  // ----------------------------------------------------------------------
  // TABLE REFERENCE
  // ----------------------------------------------------------------------

  /**
   * printSimpleSubTableReference
   * @param	alias		the alias for this reference
   * @param	expr		the table expression
   */
  public void printSimpleSubTableReference(TableAlias alias, TableReference table) {
    print("(");
    table.genSQLCode(this);
    print(") ");
    alias.genSQLCode(this);
  }

  /**
   * printSubTableExpression
   * @param	expr		the table expression
   */
  public void printSubTableReference(SearchCondition expr) {
    expr.genSQLCode(this);
  }

  /**
   * printSimpleTableReference
   * @param	alias		the alias for this reference
   * @param	expr		the table expression
   */
  public void printSimpleTableReference(TableAlias alias, Expression name, boolean hasTable) {
    if (hasTable) {
      print("TABLE ");
    }
    name.genSQLCode(this);
    if (alias != null) {
      print(" ");
      alias.genSQLCode(this);
    }
  }

  /**
   * printIntersectTableReference
   * @param	left		the left reference
   * @param	right		the right reference
   */
  public void printIntersectTableReference(IntersectSpec spec,
					   TableReference left,
					   TableReference right) {
    left.genSQLCode(this);
    print(" ");
    spec.genSQLCode(this);
    print(" ");
    right.genSQLCode(this);
  }

  /**
   * printIntersectTableTerm
   * @param	left		the left reference
   * @param	right		the right reference
   */
  public void printIntersectTableTerm(IntersectSpec spec,
				      SubTable left,
				      SubTable right) {
    left.genSQLCode(this);
    print(" ");
    spec.genSQLCode(this);
    print(" ");
    right.genSQLCode(this);
  }

  /**
   * printOuterJoin
   * @param	leftTable	the left table reference
   * @param	type		the join type
   * @param	rightTable	the right table reference
   * @param	joinPred	the join pred.
   */
  public void printOuterJoin(TableReference leftTable,
			     String type,
			     TableReference rightTable,
			     JoinPred joinPred) {
    print("{oj ");
    leftTable.genSQLCode(this);
    print(" ");
    print(type);
    print(" OUTER JOIN ");
    rightTable.genSQLCode(this);
    joinPred.genSQLCode(this);
    print("}");
  }

 /**
   * print CastPrimary
   */
  public void printCastPrimary(Expression left, Type right, boolean hasAs) {
    if (hasAs) {
      print(" CAST (");
      left.genSQLCode(this);
      print(" AS ");
      right.genSQLCode(this);
      print(")");
    } else {
      left.genSQLCode(this);
      if (right != null) {
	print(" CAST ");
	right.genSQLCode(this);
      }
    }
  }

  // ----------------------------------------------------------------------
  // PROTECTED METHODS
  // ----------------------------------------------------------------------

  protected void print(String str) {
    checkPos();
    p.print(str);
    column += str.length();
  }

  protected void print(Integer str) {
    print("" + str);
  }

  protected void print(Object str) {
    print("" + str);
  }

  protected void print(Double str) {
    print("" + str);
  }

  protected void println(String str) {
    print(str);
    println();
  }

  protected void println() {
    p.println();
    column = 0;
    line++;
  }

  // ----------------------------------------------------------------------
  // TABS
  // ----------------------------------------------------------------------

  protected void checkPos() {
    if (column < pos) {
      p.print(space(pos - column));
      column = Math.max(column, pos);
    }
  }

  protected String space(int count) {
    if (count <= 0) {
      count = 1;
    }
    return spaceIn(count);
  }

  protected String spaceIn(int count) {
    StringBuffer buf = new StringBuffer();

    for (int i = 0; i < count; i++) {
      buf.append(" ");
    }
    return buf.toString();
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  protected int				TAB_SIZE = 10;
  protected java.io.PrintWriter		p;
  protected int				pos;
  protected int				line;
  protected int				column;
  protected CJavaPrettyPrinter		in_pp;
}
