/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: SQLScanner.java,v 1.35 1999/10/12 15:39:49 vgp Exp $
 */

package at.dms.sqlc;

import antlr.Token;
import at.dms.compiler.CToken;
import at.dms.compiler.Compiler;
import at.dms.compiler.InputBuffer;
import at.dms.compiler.PositionedError;

/**
 * This is an handwritten lexer available for speed reason
 */
public class SQLScanner extends at.dms.compiler.HandLexer {

  /**
   * Constructor for a file in a char[]
   */
  public SQLScanner(Compiler compiler, InputBuffer buffer) {
    super(compiler, buffer);
  }

  // Tokens are: Identifiers, Keywords, Literals, Separators, Operators.
  protected Token getToken() throws PositionedError {
    switch (buffer.data[buffer.pos]) {
      // Separators: (period is a special case)
    case '(':
      buffer.pos++;
      return LPAREN;
    case ')':
      buffer.pos++;
      return RPAREN;
    case '{':
      buffer.pos++;
      return getLCurly();
    case '}':
      buffer.pos++;
      return RCURLY;
    case '[':
      buffer.pos++;
      return LBRACKET;
    case ']':
      buffer.pos++;
      return RBRACKET;
    case ';':
      buffer.pos++;
      return SEMICOLON;
    case ',':
      buffer.pos++;
      return COMMA;

      // Operators (single-char):
    case ':':
      buffer.pos++;
      return COLON;

    case '=':
    case '>':
    case '<':
    case '!':
    case '&':
    case '|':
    case '+':
    case '-':
    case '*':
    case '/':
    case '^':
    case '%':
      return getOperator();
    case '\'':
      return getStringLiteral();
    case '\"':
      return getDelimiter();

      // a period is a special case:
    case '.':
      if (Character.digit(buffer.data[buffer.pos + 1], 10) != -1) {
	return getNumericLiteral();
      } else {
	buffer.pos++;
	return DOT;
      }

    default:
      break;
    }

    switch (buffer.data[buffer.pos]) {
    case 'A':  case 'B':  case 'C':
    case 'D':  case 'E':  case 'F':  case 'G':
    case 'H':  case 'I':  case 'J':  case 'K':
    case 'L':  case 'M':  case 'N':  case 'O':
    case 'P':  case 'Q':  case 'R':  case 'S':
    case 'T':  case 'U':  case 'V':  case 'W':
    case 'X':  case 'Y':  case 'Z':
    case '_':
    case 'a':  case 'b':  case 'c':  case 'd':
    case 'e':  case 'f':  case 'g':  case 'h':
    case 'i':  case 'j':  case 'k':  case 'l':
    case 'm':  case 'n':  case 'o':  case 'p':
    case 'q':  case 'r':  case 's':  case 't':
    case 'u':  case 'v':  case 'w':  case 'x':
    case 'y':  case 'z':
      return getIdentifier();
    case '0':  case '1':  case '2':  case '3':
    case '4':  case '5':  case '6':  case '7':
    case '8':  case '9':
      return getNumericLiteral();
    default:
      throw new PositionedError(getTokenReference(), "illegal-char", "" + buffer.data[buffer.pos]);
    }
  }

  protected boolean readIdent() throws PositionedError {
    boolean allupper = true;

    switch (buffer.data[buffer.pos]) {
    case 'A':  case 'B':  case 'C':
    case 'D':  case 'E':  case 'F':  case 'G':
    case 'H':  case 'I':  case 'J':  case 'K':
    case 'L':  case 'M':  case 'N':  case 'O':
    case 'P':  case 'Q':  case 'R':  case 'S':
    case 'T':  case 'U':  case 'V':  case 'W':
    case 'X':  case 'Y':  case 'Z':
      break;
    case '_':
    case 'a':  case 'b':  case 'c':  case 'd':
    case 'e':  case 'f':  case 'g':  case 'h':
    case 'i':  case 'j':  case 'k':  case 'l':
    case 'm':  case 'n':  case 'o':  case 'p':
    case 'q':  case 'r':  case 's':  case 't':
    case 'u':  case 'v':  case 'w':  case 'x':
    case 'y':  case 'z':
      allupper = false;
      break;
    default:
      throw new PositionedError(getTokenReference(), "illegal-char", "" + buffer.data[buffer.pos]);
    }

    _all:
    while (true) {
      switch (buffer.data[++buffer.pos]) {
      case 'A':  case 'B':  case 'C':
      case 'D':  case 'E':  case 'F':  case 'G':
      case 'H':  case 'I':  case 'J':  case 'K':
      case 'L':  case 'M':  case 'N':  case 'O':
      case 'P':  case 'Q':  case 'R':  case 'S':
      case 'T':  case 'U':  case 'V':  case 'W':
      case 'X':  case 'Y':  case 'Z':
	break;
      case '_':
      case 'a':  case 'b':  case 'c':  case 'd':
      case 'e':  case 'f':  case 'g':  case 'h':
      case 'i':  case 'j':  case 'k':  case 'l':
      case 'm':  case 'n':  case 'o':  case 'p':
      case 'q':  case 'r':  case 's':  case 't':
      case 'u':  case 'v':  case 'w':  case 'x':
      case 'y':  case 'z':
      case '0':  case '1':  case '2':  case '3':
      case '4':  case '5':  case '6':  case '7':
      case '8':  case '9':
	allupper = false;
	break;
      default:
	  break _all;
      }
    }
    return allupper;
  }

  protected Token getIdentifier() throws PositionedError {
    int		start = buffer.pos;
    readIdent();

    Token	tok = lookupKeyword(buffer.data, start, buffer.pos - start);
    if (tok != null) {
      return tok;
    } else {
      return CToken.lookupToken(SQLTokenTypes.STANDARD_ID, buffer.data, start, buffer.pos - start);
    }
  }

  protected /*!!!static*/ Token lookupKeyword(char[] data, int start, int length) {
    return SQLKeywords.lookup(data, start, length);
  }

  public Token getLCurly() {
    switch (buffer.data[buffer.pos]) {
    case 't':
      buffer.pos++;
      if (buffer.data[buffer.pos] == 's') {
	buffer.pos++;
	return JDBC_TIMESTAMP;
      }
      return JDBC_TIME;
    case 'd':
      buffer.pos++;
      return JDBC_DATE;
    case 'o':
      if (buffer.data[buffer.pos + 1] == 'j') {
	buffer.pos += 2;
	return JDBC_OUTERJOIN;
      }
      break;
    case 'f':
      if (buffer.data[buffer.pos + 1] == 'n') {
	buffer.pos += 2;
	return JDBC_FUNCTION;
      }
      break;
    case 'e':
      if (buffer.size - buffer.pos > 5
	  && buffer.data[buffer.pos + 1] == 's'
	  && buffer.data[buffer.pos + 2] == 'c'
	  && buffer.data[buffer.pos + 3] == 'a'
	  && buffer.data[buffer.pos + 4] == 'p'
	  && buffer.data[buffer.pos + 5] == 'e') {
	buffer.pos += 6;
	return JDBC_ESCAPE;
      }
    }
    return LCURLY;
  }

  protected Token getNumericLiteral() throws PositionedError {
    int		i;

    // leading decimal indicates float.
    if (buffer.data[buffer.pos] == '.') {
      return getFloatingPointLiteral();
    }
    // 0x indicates Hex.
    if (buffer.data[buffer.pos] == '0' && buffer.data[buffer.pos + 1] == 'x') {
      return getBinChar();
    }
    if (buffer.data[buffer.pos]=='0' && buffer.data[buffer.pos + 1]=='b') {
      return getBitLit();
    }
    // otherwise scan to first non-numeric
    for (i = buffer.pos; Character.digit(buffer.data[i], 10) != -1; ) {
      i++;
    }
    switch (buffer.data[i]) { // discriminate based on first non-numeric
    case '.':
    case 'e':
    case 'E':
      return getFloatingPointLiteral();
    default:
      return getIntegerLiteral(0, 10);
    }
  }

  protected Token getBinChar() throws PositionedError {
    int		start = buffer.pos;

    buffer.pos += 2;

    while (Character.digit(buffer.data[buffer.pos], 16) != -1) {
      buffer.pos++;
    }
    if (buffer.pos == start + 2) {
      throw new PositionedError(getTokenReference(), "illegal-char", "" + buffer.data[buffer.pos - 1]);
    }

    return new CToken(SQLTokenTypes.BINCHAR_LIT, new String(buffer.data, start, buffer.pos - start));
  }

  protected Token getBitLit() throws PositionedError {
    int		start = buffer.pos;
    buffer.pos += 2;

    while (buffer.data[buffer.pos] == '0' || buffer.data[buffer.pos] == '1') {
      buffer.pos++;
    }

    if (buffer.pos == start + 2) {
      throw new PositionedError(getTokenReference(), "illegal-char", "" + buffer.data[buffer.pos - 1]);
    }

    return new CToken(SQLTokenTypes.BIT_LIT, new String(buffer.data, start, buffer.pos - start));
  }

  protected Token getIntegerLiteral(int prefix, int radix) {
    int		start = buffer.pos;

    buffer.pos += prefix;
    while (Character.digit(buffer.data[buffer.pos],radix) != -1) {
      buffer.pos++;
    }

    return new CToken(SQLTokenTypes.INTEGER_LIT, new String(buffer.data, start, buffer.pos - start));
  }

  protected Token getFloatingPointLiteral() {
    int		start = buffer.pos;

    while (Character.digit(buffer.data[buffer.pos], 10) != -1) {
      buffer.pos++;
    }
    if (buffer.data[buffer.pos] == '.') {
      buffer.pos++;
      while (Character.digit(buffer.data[buffer.pos], 10) != -1) {
	buffer.pos++;
      }
    }
    if (buffer.data[buffer.pos]=='e' || buffer.data[buffer.pos]=='E') {
      buffer.pos++;
      if (buffer.data[buffer.pos]=='+' || buffer.data[buffer.pos]=='-') {
	buffer.pos++;
      }
      while (Character.digit(buffer.data[buffer.pos], 10) != -1) {
	buffer.pos++;
      }
    }
    return new CToken(SQLTokenTypes.NUMERIC_LIT, new String(buffer.data, start, buffer.pos - start));
  }

  protected Token getOperator() {
    char	first = buffer.data[buffer.pos++];
    char	second = buffer.data[buffer.pos];

    switch (first) {
      // doubled operators !!!
    case '|':
      if (second == first) {
	buffer.pos++;
	return DBLPIPE;
      }
      break;
    default:
      break;
    }

    // Check for trailing '='
    if (second=='=') {
      buffer.pos++;
      switch (first) {
      case '<':
	return LE;
      case '>':
	return GE;
      }
    }

    switch (first) {
    case '=':
      return ASSIGN;
    case '+':
      return PLUS;
    case '-':
      return MINUS;
    case '*':
      return STAR;
    case '/':
      return SLASH;
    case '>':
      return GT;
    case '<':
      if (second == '>') {
	buffer.pos++;
	return NE;
      } else {
	return LT;
      }
    default:
      throw new RuntimeException("BAD OP" + first + second);
    }
  }

  protected Token getStringLiteral() {
    int start = buffer.pos++;	// skip opening quote

  _loop_:
    for (;;) {
      switch (buffer.data[buffer.pos++]) {
      case '\n':
	buffer.line++;
	break;
      case '\'':
	if (buffer.data[buffer.pos] != '\'') {
	  break _loop_;
	} else {
	  // two quotes: escaped
	  buffer.pos += 1;
	  break;
	}
      default:
	// nothing to do
      }
    }

    return new CToken(SQLTokenTypes.STRING_LIT, new String(buffer.data, start, buffer.pos - start));
  }

  protected Token getDelimiter() {
    int		start = buffer.pos++;	// skip opening quote

  _loop_:
    for (;;) {
      switch (buffer.data[buffer.pos++]) {
      case '\n':
	buffer.line++;
	break;
      case '"':
	break _loop_;
      default:
	// nothing to do
      }
    }

    return new CToken(SQLTokenTypes.DELIMITER_ID, new String(buffer.data, start, buffer.pos - start));
  }

  // ----------------------------------------------------------------------
  // KEYS
  // ----------------------------------------------------------------------

  // OPERATORS
  private static final CToken	ASSIGN = new CToken(SQLTokenTypes.ASSIGN, "=");
  private static final CToken	COLON = new CToken(SQLTokenTypes.COLON, ":");
  private static final CToken	COMMA = new CToken(SQLTokenTypes.COMMA, ",");
  private static final CToken	DBLPIPE = new CToken(SQLTokenTypes.DBLPIPE, "||");
  private static final CToken	DOT = new CToken(SQLTokenTypes.DOT, ".");
  private static final CToken	GE = new CToken(SQLTokenTypes.GE, ">=");
  private static final CToken	GT = new CToken(SQLTokenTypes.GT, ">");
  private static final CToken	JDBC_DATE = new CToken(SQLTokenTypes.JDBC_DATE, "{d");
  private static final CToken	JDBC_ESCAPE = new CToken(SQLTokenTypes.JDBC_ESCAPE, "{escape");
  private static final CToken	JDBC_FUNCTION = new CToken(SQLTokenTypes.JDBC_FUNCTION, "{fn");
  private static final CToken	JDBC_OUTERJOIN = new CToken(SQLTokenTypes.JDBC_OUTERJOIN, "{oj");
  private static final CToken	JDBC_TIME = new CToken(SQLTokenTypes.JDBC_TIME, "{t");
  private static final CToken	JDBC_TIMESTAMP = new CToken(SQLTokenTypes.JDBC_TIMESTAMP, "{ts");
  private static final CToken	LBRACKET = new CToken(SQLTokenTypes.LBRACKET, "[");
  private static final CToken	LCURLY = new CToken(SQLTokenTypes.LCURLY, "{");
  private static final CToken	LE = new CToken(SQLTokenTypes.LE, "<=");
  private static final CToken	LPAREN = new CToken(SQLTokenTypes.LPAREN, "(");
  private static final CToken	LT = new CToken(SQLTokenTypes.LT, "<");
  private static final CToken	MINUS = new CToken(SQLTokenTypes.MINUS, "-");
  private static final CToken	NE = new CToken(SQLTokenTypes.NE, "<>");
  private static final CToken	PLUS = new CToken(SQLTokenTypes.PLUS, "+");
  private static final CToken	RBRACKET = new CToken(SQLTokenTypes.RBRACKET, "]");
  private static final CToken	RCURLY = new CToken(SQLTokenTypes.RCURLY, "}");
  private static final CToken	RPAREN = new CToken(SQLTokenTypes.RPAREN, ")");
  private static final CToken	SEMICOLON = new CToken(SQLTokenTypes.SEMICOLON, ";");
  private static final CToken	SLASH = new CToken(SQLTokenTypes.SLASH, "/");
  private static final CToken	STAR = new CToken(SQLTokenTypes.STAR, "*");
}
