/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: SqlVisitor.java,v 1.8 1999/10/12 12:58:53 vgp Exp $
 */

package at.dms.sqlc;

import java.util.Vector;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;

import at.dms.compiler.PositionedError;
import at.dms.compiler.UnpositionedError;
import at.dms.compiler.JavaStyleComment;

/**
 * This interface defines SQL phylum visitors.
 */
public interface SqlVisitor {

  /**
   * Visits SELECT
   */
  void visitSelectStatement(String type,
			    Vector columns,
			    FromClause fromClause,
			    WhereClause whereClause,
			    GroupByClause groupByClause,
			    HavingClause havingClause,
			    SortSpec sortSpec,
			    UpdSpec updateSpec)
    throws PositionedError, UnpositionedError;

  /**
   * Visits SELECT
   */
  void visitSelectExpression(String type,
			     Vector columns,
			     FromClause fromClause,
			     WhereClause whereClause,
			     GroupByClause groupByClause,
			     HavingClause havingClause)
    throws PositionedError, UnpositionedError;

  /**
   * Visits GROUP BY
   * @param	fields	                   the fields refernces
   */
  void visitGroupByClause(Vector fields)
    throws PositionedError, UnpositionedError;

  /**
   * Visits HAVING
   * @param	condition	           the SqlPhylum for the having
   */
  void visitHavingClause(SearchCondition condition)
    throws PositionedError, UnpositionedError;

  /**
   * Visits SORTELEM
   * @param	elem
   * @param	isDesc
   */
  void visitSortElem(Expression elem, boolean isDesc)
    throws PositionedError, UnpositionedError;

  /**
   * Visits UPDSPEC
   */
  void visitUpdSpec()
    throws PositionedError, UnpositionedError;

  /**
   * Visits SORTSPEC
   */
  void visitSortSpec(Vector elems)
    throws PositionedError, UnpositionedError;

  /**
   * Visits SORTELEM
   * !!!
  void visitSortElem(IntegerLiteral intLiteral,
		     IntegerLiteral exprLiteral,
		     boolean hasAsc,
		     boolean isAsc)
    throws PositionedError, UnpositionedError;

  /**
   * Visits UPDATE
   * @param	schemaName	schema name
   * @param	tableName	table name
   * @param	listIdent       vector of listIdent
   * @param	searchCondition search condition
   * @param	hasCondition     there is a search condition ?
   */
  void visitUpdateStatement(String schemaName,
			    String tableName,
			    Vector listIdent,
			    SearchCondition searchCondition,
			    boolean hasCondition)
    throws PositionedError, UnpositionedError;

  /**
   * Visits Insert statement
   * @param	ident		type ident of the table
   * @param	fields		the field name list
   * @param	soure		the source of data from clause
   */
  void visitInsertStatement(String ident, FieldNameList fields, InsertSource source)
    throws PositionedError, UnpositionedError;

  /**
   * Visits ExpressionList
   * @param	exprs	a vector of SqlPhylum
   */
  void visitExpressionList(Vector exprs)
    throws PositionedError, UnpositionedError;

  /**
   * Visits FieldNameList
   * @param	exprs	a vector of SqlPhylum
   */
  void visitFieldNameList(Vector exprs)
    throws PositionedError, UnpositionedError;

  /**
   * Visits WhereClause
   * @param	condition	a SqlPhylum
   */
  void visitWhereClause(SearchCondition condition)
    throws PositionedError, UnpositionedError;

  /**
   * Visits NotCondition
   * @param	condition	a SqlPhylum
   */
  void visitNotCondition(SearchCondition condition)
    throws PositionedError, UnpositionedError;

  /**
   * Visits SimpleSearchCondition
   * @param	predicate	a SqlPhylum
   */
  void visitSimpleSearchCondition(Predicate predicate)
    throws PositionedError, UnpositionedError;

  /**
   * Visits OnJoinPred
   * @param	condition
   */
  void visitOnJoinPred(SearchCondition condition)
    throws PositionedError, UnpositionedError;

  /**
   * Visits UsingJoinPred
   * @param	list
   */
  void visitUsingJoinPred(FieldNameList list)
    throws PositionedError, UnpositionedError;

  /**
   * Visits AndCondition
   * @param	left	the left search condition
   * @param	right	the right search condition
   */
  void visitAndCondition(SearchCondition left, SearchCondition right)
    throws PositionedError, UnpositionedError;

  /**
   * Visits OrCondition
   * @param	left	the left search condition
   * @param	right	the right search condition
   */
  void visitOrCondition(SearchCondition left, SearchCondition right)
    throws PositionedError, UnpositionedError;

  /**
   * Visits FieldReferencePrimary
   * @param	fieldRef	           the field reference
   * @param	begin	                   the begining of the subrange
   * @param	end	                   the end of the subrange
   */
  void visitFieldReferencePrimary(Expression fieldRef, Expression begin, Expression end)
    throws PositionedError, UnpositionedError;

  /**
   * Visits FieldReference
   * @param	table	           the table name if any
   * @param	ident	           the field ident
   */
  void visitFieldReference(String table, String ident)
    throws PositionedError, UnpositionedError;

  /**
   * Visits BinaryExpression
   * @param	operator                   the operator
   * @param	left	                   the left expression
   * @param	right	                   the right expression
   */
  void visitBinaryExpression(String operator, Expression left, Expression right)
    throws PositionedError, UnpositionedError;

  /**
   * Visits SimpleIdentExpression
   * @param	ident	                   the ident
   */
  void visitSimpleIdentExpression(String ident)
    throws PositionedError, UnpositionedError;

  /**
   * Visits UnaryExpression
   * @param	expr	                   the unary expression
   * @param	operator                   the operator
   */
  void visitUnaryExpression(String operator, Expression operand)
    throws PositionedError, UnpositionedError;

  /**
   * Visits FROM
   * @param	tableRefs	a vector of table references
   */
  void visitFromClause(Vector tableRefs)
    throws PositionedError, UnpositionedError;

  /**
   * Visits a string
   * @param	text	text
   */
  void visitIdent(String text)
    throws PositionedError, UnpositionedError;

  /**
   * Visits BooleanLiteral
   * @param	value	the boolean literal
   */
  void visitBooleanLiteral(boolean value)
    throws PositionedError, UnpositionedError;

  /**
   * Visits NullLiteral
   * @param	value	the null literal
   */
  void visitNullLiteral()
    throws PositionedError, UnpositionedError;

  /**
   * Visits StringLiteral
   * @param	value	the string literal
   */
  void visitStringLiteral(String value)
    throws PositionedError, UnpositionedError;

  /**
   * Visits BincharLiteral
   * @param	value	the binchar literal
   */
  void visitBincharLiteral(String value)
    throws PositionedError, UnpositionedError;

  /**
   * Visits BitLiteral
   * @param	value	the bit literal
   */
  void visitBitLiteral(String value)
    throws PositionedError, UnpositionedError;

  /**
   * Visits ExistsPredicate
   * @param	sub the sub table
   */
  void visitExistsPredicate(SubTableExpression sub)
    throws PositionedError, UnpositionedError;

  /**
   * Visits ValueList
   * @param	value	the list of value
   */
  void visitValueList(ExpressionList expr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits ArrayPrecision
   * @param	size	a SqlPhylum
   */
  void visitArrayPrecision(IntegerLiteral size)
    throws PositionedError, UnpositionedError;

  /**
   * Visits BetweenPredicate
   * @param	start		begining of the range
   * @param	startQualifier	mode (INCLUSIVE | EXCLUSIVE)
   * @param	end		end of the range
   * @param	endQualifier	mode (INCLUSIVE | EXCLUSIVE)
   */
  void visitBetweenPredicate(Expression start,
			     String startQualifier,
			     Expression end,
			     String endQualifier)
    throws PositionedError, UnpositionedError;

  /**
   * Visits ValueInPredicate
   * @param	value		value  of the predicate
   */
  void visitValueInPredicate(ExpressionList value)
    throws PositionedError, UnpositionedError;

  /**
   * Visits LikePredicate
   * @param	sens
   * @param	expr
   * @param	escape
   */
  void visitLikePredicate(String sens, Expression expr, JdbcEscape escape)
    throws PositionedError, UnpositionedError;

  /**
   * Visits MatchesPredicate
   * @param	sens
   * @param	matchesExpr
   * @param	escapeExpr
   */
  void visitMatchesPredicate(String sens,
			     Expression matchesExpr,
			     Expression escapeExpr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits MatchesPredicate
   */
  void visitTupleInPredicate(ExpressionList exprs, boolean hasNot, SubTableExpression table)
    throws PositionedError, UnpositionedError;

  /**
   * Visits ExpressionPredicate
   * @param	expr			the predicate expression
   */
  void visitExpressionPredicate(Expression expr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits IsPredicate
   */
  void visitIsPredicate(Predicate predicate, boolean hasNot)
    throws PositionedError, UnpositionedError;

  /**
   * Visits QuantifiedPredicate
   */
  void visitQuantifiedPredicate(String type, SubTableExpression table)
    throws PositionedError, UnpositionedError;

  /**
   * Visits ComparaisonPredicate
   */
  void visitComparaisonPredicate(Predicate left, Predicate right, String operator)
    throws PositionedError, UnpositionedError;

  /**
   * Visits ExtendedPredicate
   */
  void visitExtendedPredicate(Predicate left,  String operator, Predicate right)
    throws PositionedError, UnpositionedError;

  /**
   * Visits JdbcFunction
   * @param	value	the bit literal
   */
  void visitJdbcFunction(String name, ExpressionList params)
    throws PositionedError, UnpositionedError;

  /**
   * Visits JdbcFunction
   * @param	value	the bit literal
   */
  void visitJdbcEscape(String name)
    throws PositionedError, UnpositionedError;

  /**
   * Visits JdbcDateLiteral
   * @param	value	the JDBC date
   */
  void visitJdbcDateLiteral(Date value)
    throws PositionedError, UnpositionedError;

  /**
   * Visits JdbcTimeLiteral
   * @param	value	the JDBC time
   */
  void visitJdbcTimeLiteral(Time value)
    throws PositionedError, UnpositionedError;

  /**
   * Visits JdbcTimeStampLiteral
   * @param	value	the JDBC time stamp
   */
  void visitJdbcTimeStampLiteral(Timestamp value)
    throws PositionedError, UnpositionedError;

  /**
   * Visits IntegerLiteral
   * @param	value	the integer literal
   */
  void visitIntegerLiteral(Integer value)
    throws PositionedError, UnpositionedError;

  /**
   * Visits NumericLiteral
   * @param	value	the numeric literal
   */
  void visitNumericLiteral(java.math.BigDecimal value)
    throws PositionedError, UnpositionedError;

  /**
   * Visits an ident
   * @param	ident	ident.*  if it is not null else only * is print
   */
  void visitSelectElemStar(String ident)
    throws PositionedError, UnpositionedError;

  /**
   * Visits an column name with is new name is it exists
   * @param	columnName	column name
   * @param	newColumnName	new column name
   */
  void visitSelectElemExpression(Expression columnExpr, String newColumnName)
    throws PositionedError, UnpositionedError;

  /**
   * Visits a max min avg sum expression
   * @param	oper	                   the operator
   * @param	type	                   the expression type
   * @param	expr	                   the Sql Expression
   */
  void visitSetFunction(String oper, int type, Expression expr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits a count expression
   */
  void visitCountExpression(boolean hasDistinct, Expression expr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits if expression
   * @param	condition	           the if condition
   * @param	thenExpr	           the Expression for then
   * @param	elseExpr	           the Expression for else
   */
  void visitIfExpression(SearchCondition condition, Expression thenExpr, Expression elseExpr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits the SimpleWhenClause
   * @param	whenExpr	           the Sql Expression for the when
   * @param	thenExpr	           the Sql Expression for the else
   */
  void visitSimpleWhenClause(Expression whenExpr, Expression thenExpr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits the SearchedWhenClause
   * @param	condition	           the condition for the when
   * @param	thenExpr	           the Sql Expression for the else
   */
  void visitSearchedWhenClause(SearchCondition condition, Expression thenExpr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits the SimpleCaseExpression
   * @param	expr	                   the Sql Expression for the case
   * @param	conditions	           List of the conditions
   * @param	elseExpr	           the Sql Expression for the else
   */
  void visitSimpleCaseExpression(Expression expr, Vector conditions, Expression elseExpr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits the SearchedCaseExpression
   * @param	conditions	           List of the conditions
   * @param	elseExpr	           the Sql Expression for the else
   */
  void visitSearchedCaseExpression(Vector conditions, Expression elseExpr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits the CaseExpression
   * @param	cases
   */
  void visitCaseExpression(AbstractCaseExpression cases)
    throws PositionedError, UnpositionedError;

  /**
   * Visits ASSIGNMENT
   * @param	ident        	ident
   * @param	expr         	expression
   * @param	searchCondition	search condition
   */
  void visitAssignment(Expression left, Expression right)
    throws PositionedError, UnpositionedError;

  /**
   * Visits the CorrespondingSpec
   */
  void visitCorrespondingSpec(FieldNameList fields)
    throws PositionedError, UnpositionedError;

  /**
   * Visits the IntersectSpec
   */
  void visitIntersectSpec(CorrespondingSpec spec)
    throws PositionedError, UnpositionedError;

  /**
   * Visits the unidiff spec
   */
  void visitUnidiffSpec(String oper, CorrespondingSpec spec)
    throws PositionedError, UnpositionedError;

  /**
   * Visits the union
   */
  void visitUnidiffTableTerm(UnidiffSpec spec, SubTable left, SubTable right)
    throws PositionedError, UnpositionedError;

  /**
   * Visits the union
   */
  void visitUnidiffTableReference(UnidiffSpec spec, TableReference left, TableReference right)
    throws PositionedError, UnpositionedError;

  /**
   * Visits ParenthesedTableReference
   */
  void visitParenthesedTableReference(TableReference expr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits SelectTableReference
   */
  void visitSelectTableReference(SelectExpression expr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits DELETE
   * @param	schemaName	schema name
   * @param	tableName	table name
   * @param	searchCondition	search condition
   */
  void visitDeleteStatement(String schemaName, String tableName, SearchCondition searchCondition)
    throws PositionedError, UnpositionedError;

  /**
   * Visits SubTableExpression
   * @param	expr		the table expression
   * @param	expr		the alias
   */
  void visitSubTableExpression(SubTable table)
    throws PositionedError, UnpositionedError;

  /**
   * Visits DefaultValuesInsertSource
   */
  void visitDefaultValuesInsertSource()
    throws PositionedError, UnpositionedError;

  /**
   * Visits ValueListInsertSource
   */
  void visitValueListInsertSource(ValueList value)
    throws PositionedError, UnpositionedError;

  /**
   * Visits TableExpressionInsertSource
   */
  void visitTableExpressionInsertSource(TableReference table)
    throws PositionedError, UnpositionedError;

  /**
   * Visits TableInsertSource
   */
  void visitTableInsertSource(Vector table)
    throws PositionedError, UnpositionedError;

  /**
   * Visits TableAlias
   */
  void visitTableAlias(String name, FieldNameList list)
    throws PositionedError, UnpositionedError;

  /**
   * Visits TinyIntType
   */
  void visitTinyIntType()
    throws PositionedError, UnpositionedError;

  /**
   * Visits SmallIntType
   */
  void visitSmallIntType()
    throws PositionedError, UnpositionedError;

  /**
   * Visits IntegerType
   */
  void visitIntegerType()
    throws PositionedError, UnpositionedError;

  /**
   * Visits NumericType
   * @param	left		the number of digits
   * @param	right		the position of the comma
   */
  void visitNumericType(IntegerLiteral left, IntegerLiteral right)
    throws PositionedError, UnpositionedError;

  /**
   * Visits FloatType
   */
  void visitFloatType()
    throws PositionedError, UnpositionedError;

  /**
   * Visits DoubleType
   */
  void visitDoubleType()
    throws PositionedError, UnpositionedError;

  /**
   * Visits RealType
   */
  void visitRealType()
    throws PositionedError, UnpositionedError;

  /**
   * Visits CharType
   */
  void visitCharType(ArrayPrecision prec)
    throws PositionedError, UnpositionedError;

  /**
   * Visits TypeName
   */
  void visitTypeName(String ident)
    throws PositionedError, UnpositionedError;

  /**
   * Visits BinCharType
   */
  void visitBinCharType(ArrayPrecision prec)
    throws PositionedError, UnpositionedError;

  /**
   * Visits StringType
   */
  void visitStringType()
    throws PositionedError, UnpositionedError;

  /**
   * Visits BitsType
   */
  void visitBitsType(ArrayPrecision prec)
    throws PositionedError, UnpositionedError;

  /**
   * Visits BoolType
   */
  void visitBoolType()
    throws PositionedError, UnpositionedError;

  /**
   * Visits BlobType
   */
  void visitBlobType()
    throws PositionedError, UnpositionedError;

  /**
   * Visits SimpleSubTableReference
   * @param	alias		the alias for this reference
   * @param	expr		the table expression
   */
  void visitSimpleSubTableReference(TableAlias alias, TableReference table)
    throws PositionedError, UnpositionedError;

  /**
   * Visits SubTableExpression
   * @param	expr		the table expression
   */
  void visitSubTableReference(SearchCondition expr)
    throws PositionedError, UnpositionedError;

  /**
   * Visits SimpleTableReference
   * @param	alias		the alias for this reference
   * @param	expr		the table expression
   */
  void visitSimpleTableReference(Expression name, TableAlias alias)
    throws PositionedError, UnpositionedError;

  /**
   * Visits IntersectTableReference
   * @param	left		the left reference
   * @param	right		the right reference
   */
  void visitIntersectTableReference(IntersectSpec spec,
				    TableReference left,
				    TableReference right)
    throws PositionedError, UnpositionedError;

  /**
   * Visits IntersectTableTerm
   * @param	left		the left reference
   * @param	right		the right reference
   */
  void visitIntersectTableTerm(IntersectSpec spec,
			       SubTable left,
			       SubTable right)
    throws PositionedError, UnpositionedError;

  /**
   * Visits OuterJoin
   * @param	leftTable	the left table reference
   * @param	type		the join type
   * @param	rightTable	the right table reference
   * @param	joinPred	the join pred.
   */
  void visitOuterJoin(TableReference leftTable,
		      String type,
		      TableReference rightTable,
		      JoinPred joinPred)
    throws PositionedError, UnpositionedError;

  /**
   * Visits CastPrimary
   */
  void visitCastPrimary(Expression left, Type right, boolean hasAs)
    throws PositionedError, UnpositionedError;

  // ----------------------------------------------------------------------
  // CONTEXT HANDLING
  // ----------------------------------------------------------------------

  SqlContext getContext();

  void enter(SqlContext context);

  void exit(SqlContext context);
}
