/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that Ssinsertit will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: sql.g,v 1.37 1999/11/01 00:02:55 graf Exp $
 */

header { package at.dms.sqlc; }
{
  import java.util.Vector;

  import at.dms.compiler.Compiler;
  import at.dms.compiler.TokenReference;
  import at.dms.compiler.HandLexer;
  import at.dms.compiler.CWarning;
}

// ----------------------------------------------------------------------
// THE PARSER STARTS HERE
// ----------------------------------------------------------------------

class SQLParser extends Parser;

options {
  k = 2;				// two token lookahead
  importVocab = SQL;			// Call its vocabulary "SQL"
  exportVocab = SQL;			// Call its vocabulary "SQL"
  codeGenMakeSwitchThreshold = 2;	// Some optimizations
  codeGenBitsetTestThreshold = 3;
  defaultErrorHandler = false;		// Don't generate parser error handlers
  superClass="at.dms.compiler.Parser";
}
{
  public SQLParser(Compiler compiler, HandLexer lexer, String file) {
    super(compiler, lexer, MAX_LOOKAHEAD);
  }
}

/**
 * This is the main entry point for the grammar
 */
sCompilationUnit []
  returns [SCompilationUnit self]
{
  self = new SCompilationUnit(buildTokenReference());

  Statement	stmt = null;
}
:
  (
    stmt = sSelectStatement[] { self.addStmt(stmt); }
  |
    stmt = sInsertStatement[] { self.addStmt(stmt); }
  |
    stmt = sUpdateStatement[] { self.addStmt(stmt); }
  |
    stmt = sDeleteStatement[] { self.addStmt(stmt); }
    //|
    //stmt = sTruncateStatement[]
    //|
    //stmt = sTransactionStatement[]
    //|
    //stmt = sLockStatement[]
  |
    SEMICOLON
  )+
  EOF
;

sSelectStatement []
  returns [SelectStatement self = null]
{
  String		type = null;
  Vector		selectList;
  FromClause		fromClause;
  WhereClause		whereClause = null;
  GroupByClause		groupByClause = null;
  HavingClause		havingClause = null;
  SortSpec		sortSpec = null;
  UpdSpec		updateSpec = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "SELECT"
  ( "ALL" { type = "ALL"; } | "DISTINCT" { type = "DISTINCT"; } )?
  selectList = sSelectList[]
  fromClause = sFromClause[]
  ( whereClause = sWhereClause[] )?
  ( groupByClause = sGroupByClause[] )?
  ( havingClause = sHavingClause[] )?
  (
    sortSpec = sSortSpec[]
  |
    updateSpec = sUpdSpec[]
  )?
  {
    self = new SelectStatement(sourceRef,
			       type,
			       selectList,
			       fromClause,
			       whereClause,
			       groupByClause,
			       havingClause,
			       sortSpec,
			       updateSpec);
  }
;

sArrayPrecision []
  returns [ArrayPrecision self = null]
{
  IntegerLiteral	p = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  LPAREN ( p = sIntegerLiteral[] | STAR ) RPAREN
    { self = new ArrayPrecision(sourceRef, p); }
;

sAssignList []
  returns [Vector  self = new Vector()]
{
  Assignment	p;
  TokenReference	sourceRef = buildTokenReference();
}
:
  p = sAssignment[] { self.addElement(p); }
  (
    COMMA p = sAssignment[]
      { self.addElement(p); }
  )*
;

sAssignment []
  returns [Assignment self = null]
{
  String p1;
  Expression p2;
  TokenReference	sourceRef = buildTokenReference();
}
:
  p1 = sIdentifier[]
  ASSIGN p2 = sExpression[]
    { self = new Assignment(sourceRef, new SimpleIdentExpression(sourceRef, p1), p2); }
;

sBetweenPredicate []
  returns [BetweenPredicate self = null]
{
  Expression p1;
  String p2 = null;
  Expression p3;
  String p4 = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "BETWEEN"
  p1 = sExpression[] ( p2 = sBetweenQualifier[] )?
  "AND"
  p3 = sExpression[] ( p4 = sBetweenQualifier[] )?
    { self = new BetweenPredicate(sourceRef, p1, p2, p3, p4); }
;

sBetweenQualifier []
  returns [String self = null]
:
  "INCLUSIVE"
    { self = "INCLUSIVE"; }
|
  "EXCLUSIVE"
    { self = "EXCLUSIVE"; }
;

sBoolOp []
  returns [String self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  "AND"  { self = "AND"; }
|
  "OR" { self = "OR"; }
;

sCaseExpression []
  returns [CaseExpression self = null]
{
  AbstractCaseExpression     p1;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "CASE"
  (
    p1 = sSearchedCaseExpression[]
  |
    p1 = sSimpleCaseExpression[]
  )
  "END"
    { self = new CaseExpression(sourceRef,  p1); }
;

sCorrespondingSpec []
  returns [CorrespondingSpec self = null]
{
  FieldNameList p1 = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "CORRESPONDING"
  ( "BY" LPAREN p1 = sFieldNameList[] RPAREN )?
    { self = new CorrespondingSpec(sourceRef, p1); }
;

sDataType []
  returns [Type self = null]
{
  ArrayPrecision prec= null;
  IntegerLiteral	exp1 = null;
  IntegerLiteral	exp2 = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
(
  "TINYINT" { self = new TinyIntType(sourceRef); }
|
  "SMALLINT" { self = new SmallIntType(sourceRef); }
|
  "INTEGER" { self = new IntegerType(sourceRef); }
|
  "NUMERIC"
  (
    LPAREN exp1 = sIntegerLiteral[] ( COMMA exp2 = sIntegerLiteral[] )? RPAREN
      { self = new NumericType(sourceRef, exp1, exp2); }
  )?
|
   "FLOAT" { self = new FloatType(sourceRef); }
|
   "DOUBLE" { self = new DoubleType(sourceRef); }
|
   "REAL" { self = new RealType(sourceRef); }
|
   "CHAR" ( (sArrayPrecision[])=> prec = sArrayPrecision[] )?
     { self = new CharType(sourceRef, prec); }
|
   "STRING" { self = new StringType(sourceRef); }
|
   "BINCHAR" ( (sArrayPrecision[])=> prec = sArrayPrecision[] )?
     { self = new BinCharType(sourceRef, prec); }
|
   "BITS" prec = sArrayPrecision[] { self = new BitsType(sourceRef, prec); }
|
   "BOOL" { self = new BoolType(sourceRef); }
|
   "BLOB" { self = new BlobType(sourceRef); }
)
;

sDataTypeSpec []
  returns [Type self = null]
{
  String p1;
  TokenReference	sourceRef = buildTokenReference();
}
:
  (
    self = sDataType[]
  |
    p1 = sIdentifier[]
      { self = new TypeName(sourceRef, p1); }
  )
;

sDeleteStatement []
  returns [DeleteStatement self = null]
{
  String p1;
  String p2 = null;
  SearchCondition p3 = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "DELETE" "FROM"
  p1 = sIdentifier[] ( p2 = sIdentifier[] )?
  (
    "WHERE" p3 = sSearchCondition[]
  )?
      {
	self = new DeleteStatement(sourceRef,
	                           p1,
				   p2,
				   p3);
      }
;

sExistsPredicate []
  returns [ExistsPredicate self = null]
{
  SubTableExpression p1;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "EXISTS" p1 = sSubTableExpression[]
    { self = new ExistsPredicate(sourceRef, p1); }
;

sExpression []
  returns [Expression self = null]
{
  Expression		right = null;
  int			op = 0;
  TokenReference	sourceRef = buildTokenReference();
}
:
  self = sFactor[]
  (
    PLUS right = sFactor[]
      { self = new PlusExpression(sourceRef, self, right); }
  |
    MINUS right = sFactor[]
      { self = new MinusExpression(sourceRef, self, right); }
  |
    STAR right = sFactor[]
      { self = new StarExpression(sourceRef, self, right); }
  |
    SLASH right = sFactor[]
      { self = new SlashExpression(sourceRef, self, right); }
  |
    "BITAND" right = sFactor[]
      { self = new BitandExpression(sourceRef, self, right); }
  |
    "BITOR" right = sFactor[]
      { self = new BitorExpression(sourceRef, self, right); }
  |
    DBLPIPE right = sFactor[]
      { self = new DbpipeExpression(sourceRef, self, right); }
  )*
;

sFactor []
  returns [Expression self = null]
{
  int			op;
  TokenReference	sourceRef = buildTokenReference();
}
:
  PLUS self = sPrimary[]
    { self = new UnaryPlusExpression(sourceRef, self); }
|
  MINUS self = sPrimary[]
    { self = new UnaryMinusExpression(sourceRef, self); }
|
  "BITNOT" self = sPrimary[]
    { self = new BitnotExpression(sourceRef, self); }
|
  self = sPrimary[]
;

sExpressionList []
  returns [ExpressionList self = null]
{
  Vector		exprs = new Vector();
  Expression		p;
  TokenReference	sourceRef = buildTokenReference();
}
:
  p = sExpression[] { exprs.addElement(p); }
  ( COMMA p = sExpression[] { exprs.addElement(p); } )*
    { self = new ExpressionList(sourceRef, exprs); }
;

sFieldNameList []
  returns [FieldNameList self = null]
{
  Vector		fields = new Vector();
  String		p;
  TokenReference	sourceRef = buildTokenReference();
}
:
  p = sIdentifier[] { fields.addElement(p); }
  ( COMMA p = sIdentifier[] { fields.addElement(p); } )*
    { self = new FieldNameList(sourceRef, fields); }
;

sFieldReference []
  returns [FieldReference self = null]
{
  String		p1;
  String		p2 = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  p1 = sIdentifier[]
  ( DOT p2 = sIdentifier[] )?
    { self = new FieldReference(sourceRef, p2 == null ? null : p1, p2 == null ? p1 : p2); }
;

sFromClause []
  returns [FromClause self = null]
{
  Vector	 v = new Vector();
  SimpleTableReference p;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "FROM" p = sSimpleTableReference[] { v.addElement(p); }
  (
    COMMA p = sSimpleTableReference[] { v.addElement(p); }
  )*
    { self = new FromClause(sourceRef, v); }
;

sGroupByClause []
  returns [GroupByClause self = null]
{
  Vector	 v = new Vector();
  FieldReference	p;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "GROUP" "BY" p = sFieldReference[] { v.addElement(p); }
  (
    COMMA p = sFieldReference[]
      { v.addElement(p); }
  )*
    { self = new GroupByClause(sourceRef, v); }
;

sHavingClause []
  returns [HavingClause self = null]
{
  SearchCondition	p;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "HAVING" p = sSearchCondition[]
    { self = new HavingClause(sourceRef, p); }
;

sIdentifier []
  returns [String self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  t1:STANDARD_ID { self = t1.getText(); }
|
  t2:DELIMITER_ID { self = t2.getText(); }
;

sIfExpression []
  returns [IfExpression self = null]
{
  SearchCondition    p1;
  Expression            p2;
  Expression            p3;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "IF" p1 = sSearchCondition[]
  "THEN" p2 = sExpression[]
  "ELSE" p3 = sExpression[]
  "FI"
    { self = new IfExpression(sourceRef, p1, p2, p3); }
;

sInsertSource []
  returns [InsertSource self = null]
{
  ValueList		value;
  Vector		list = new Vector();
  TableReference	table;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "VALUES" value = sValueList[]
    { self = new ValueListInsertSource(sourceRef,  value); }
|
  ( "TABLE" LPAREN )=>
  "TABLE" LPAREN
  value = sValueList[] { list.addElement(value); }
  ( COMMA value = sValueList[] { list.addElement(value); } )*
  RPAREN
    { self = new TableInsertSource(sourceRef,  list); }
|
  table = sTableExpression[] { self = new TableExpressionInsertSource(sourceRef, table);}
|
  "DEFAULT" "VALUES" { self = new DefaultValuesInsertSource(sourceRef); }
;

sInsertStatement []
  returns [InsertStatement self = null]
{
  String	p1;
  FieldNameList	p2 = null;
  InsertSource	p3;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "INSERT" "INTO" p1 = sIdentifier[]
  (
    (LPAREN sFieldNameList[])=> LPAREN p2 = sFieldNameList[] RPAREN
  )?
  p3 = sInsertSource[]
    { self = new InsertStatement(sourceRef, p1, p2, p3); }
;

sIntegerLiteral []
  returns [IntegerLiteral self = null]
{
  TokenReference	sourceRef = buildTokenReference();

}
:
  t:INTEGER_LIT
    { self = new IntegerLiteral(sourceRef, new Integer(t.getText())); }
;

sIntersectSpec []
  returns [IntersectSpec self = null]
{
  CorrespondingSpec	p1 = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "INTERSECT" ( p1 = sCorrespondingSpec[] )?
    { self = new IntersectSpec(sourceRef, p1); }
;

sJoinPred []
  returns [JoinPred self = null]
{
  SearchCondition condition;
  FieldNameList list;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "ON" condition = sSearchCondition[]
    { self = new OnJoinPred(sourceRef, condition); }
|
  "USING" LPAREN list = sFieldNameList[] RPAREN
    { self = new UsingJoinPred(sourceRef, list); }
;

sJoinType []
  returns [String self = null]
{
  TokenReference	sourceRef = buildTokenReference();

}
:
    "LEFT" { self =  "LEFT"; }
  |
    "RIGHT" { self = "RIGHT"; }
  |
    "FULL" { self = "FULL"; }
;

sLikePredicate []
  returns [LikePredicate self = null]
{
  String	 p1 = null;
  Expression	 p2;
  JdbcEscape	 p3 = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "LIKE"
  ( p1 = sSensSpec[] ) ?
  p2 = sExpression[]
  ( p3 = sEscapeExpression[] )?
    { self = new LikePredicate(sourceRef, p1, p2, p3); }
;

sLiteral []
  returns [Literal self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  t1:INTEGER_LIT
    { self = new IntegerLiteral(sourceRef, new Integer(t1.getText())); }
|
  t2:NUMERIC_LIT
    { self = new NumericLiteral(sourceRef, new java.math.BigDecimal(t2.getText())); }
//|
//  t3:REAL_LIT
//    { self = new SqlExpr(sourceRef, t3.getText()); }
|
  t4:STRING_LIT
    { self = new StringLiteral(sourceRef, t4.getText()); }
|
  t5:BINCHAR_LIT
    { self = new BincharLiteral(sourceRef, t5.getText()); }
|
  t6:BIT_LIT
    { self = new BitLiteral(sourceRef, t6.getText()); }
|
  "FALSE"
    {
      self = new BooleanLiteral(sourceRef, false);
    }
|
  "TRUE"
    {
      self = new BooleanLiteral(sourceRef, true);
    }
|
  "NULL"
    { self = new NullLiteral(sourceRef); }
|
  JDBC_DATE t7:STRING_LIT RCURLY
    { self = new JdbcDateLiteral(sourceRef, t7.getText()); }
|
  JDBC_TIME t8:STRING_LIT RCURLY
    { self = new JdbcTimeLiteral(sourceRef, t8.getText()); }
|
  JDBC_TIMESTAMP t9:STRING_LIT RCURLY
    { self = new JdbcTimeStampLiteral(sourceRef, t9.getText()); }
;

sMatchesPredicate []
  returns [MatchesPredicate self = null]
{
  String		p1 = null;
  Expression		p2;
  Expression		p3 = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "MATCHES" ( p1 = sSensSpec[] )? p2 = sExpression[]
  ( "ESCAPE" p3 = sExpression[] )?
    { self = new MatchesPredicate(sourceRef, p1, p2, p3); }
;

sPredicate []
  returns [Predicate self = null]
{
  Expression		expr;
  String		s = null;
  Predicate		right = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  self = sTupleInPredicate[]
|
  self = sExistsPredicate[]
|
  expr = sExpression[]
    { self = new ExpressionPredicate(sourceRef, expr); }
  (
    ( "NOT" { s = "NOT"; } )?
    (
      "SUBSET" { s += "SUBSET"; } ( "OF" { s += "OF"; } )?
      expr = sExpression[]
       { right = new ExpressionPredicate(sourceRef, expr); }
    |
      right = sWithNotPredicate[]
    )
    { self = new ExtendedPredicate(sourceRef, self, s, right); }
  |
    self = sWithCompPredicate[self]
  )?
;

sPrimary []
  returns [Expression self = null]
{
  Type right = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  self = sSimplePrimary[]
  ( "CAST" right = sDataTypeSpec[] { self = new CastPrimary(sourceRef, self, right, false); } )?
|
  "CAST" LPAREN self = sSimplePrimary[] "AS" right = sDataTypeSpec[] RPAREN
    { self = new CastPrimary(sourceRef, self, right, true); }
;

sQuantifiedPredicate []
  returns [QuantifiedPredicate self = null]
{
  SubTableExpression    p1;
  String  s = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  (
      "ALL" { s = "ALL"; }
    |
      "ANY" { s = "ANY"; }
    |
      "SOME" { s = "SOME"; }
  )
  p1 = sSubTableExpression[]
    { self = new QuantifiedPredicate(sourceRef, s, p1); }
;

sSearchCondition []
  returns [SearchCondition self = null]
{
  SearchCondition	right = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  self = sBooleanTerm[]
  (
    "OR" right = sBooleanTerm[]
      { self = new OrCondition(sourceRef, self, right); }
  )*
;

sBooleanTerm []
  returns [SearchCondition self = null]
{
  SearchCondition	right = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  self = sBooleanFactor[]
  (
    "AND" right = sBooleanFactor[]
      { self = new AndCondition(sourceRef, self, right); }
  )*
;

sBooleanFactor []
  returns [SearchCondition self = null]
{
  Predicate		pred;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "NOT" pred = sPredicate[]
    { self = new NotCondition(sourceRef, new SimpleSearchCondition(sourceRef, pred)); }
|
  pred = sPredicate[]
    { self = new SimpleSearchCondition(sourceRef, pred); }
;

sSearchedCaseExpression []
  returns [SearchedCaseExpression self = null]
{
  Expression	        p = null;
  Vector                conditions = new Vector();
  SearchedWhenClause    condition = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  ( condition = sSearchedWhenClause[] { conditions.addElement(condition); } )+
  (
    "ELSE" p = sExpression[]
  )?
    {
      self = new SearchedCaseExpression(sourceRef, conditions, p);
    }
;

sSearchedWhenClause []
  returns [SearchedWhenClause self = null]
{
  SearchCondition p1;
  Expression p2;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "WHEN" p1 = sSearchCondition[]
  "THEN" p2 = sExpression[]
    { self = new SearchedWhenClause(sourceRef, p1, p2); }
;

sSelectElem []
  returns [SelectElem self = null]
{
  String s = null;
  Expression p1;
  String s2;
  TokenReference	sourceRef = buildTokenReference();
}
:
  STAR
    { self = new SelectElemStar(sourceRef); }
|
  ( sIdentifier[] DOT STAR )=>
  s = sIdentifier[] DOT STAR
    { self = new SelectElemStar(sourceRef, s); }
|
  p1 = sExpression[]
  (
     "AS" s2 = sIdentifier[] { self = new SelectElemExpression(sourceRef, p1, s2); }
  |
     { self = new SelectElemExpression(sourceRef, p1); }
  )
;

sSelectExpression []
  returns [SelectExpression self = null]
{
  String	type =	null;
  FromClause	fromClause;
  WhereClause   whereClause = null;
  GroupByClause groupByClause = null;
  HavingClause  havingClause = null;
  Vector	selectList;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "SELECT"
  ( "ALL" { type =  "ALL"; } | "DISTINCT" { type = "DISTINCT"; } )?
  selectList = sSelectList[]
  fromClause = sFromClause[]
  ( whereClause = sWhereClause[] )?
  ( groupByClause = sGroupByClause[] )?
  ( havingClause = sHavingClause[] )?
  {
    self = new SelectExpression(sourceRef, type, selectList, fromClause, whereClause, groupByClause, havingClause);
  }
;

sSelectList []
  returns [Vector self = new Vector()]
{
  SelectElem	p;
  TokenReference	sourceRef = buildTokenReference();
}
:
  p = sSelectElem[] { self.addElement(p); }
  (
    COMMA p = sSelectElem[]
      { self.addElement(p); }
  )*
;

sSensSpec []
  returns [String self = null]
:
  "SENSITIVE" { self = "SENSITIVE"; }
  |
  "INSENSITIVE" { self = "INSENSITIVE"; }
;

sSimpleCaseExpression []
  returns [SimpleCaseExpression self = null]
{
  Expression	     p;
  Vector             conditions = new Vector();
  SimpleWhenClause   condition = null;
  Expression	     elseExpr = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "CASE"
  p = sExpression[]
  ( condition = sSimpleWhenClause[] { conditions.addElement(condition); } )+
  (
    "ELSE" elseExpr = sExpression[]
  )?
  "END"
    {
      self = new SimpleCaseExpression(sourceRef, p, conditions, elseExpr);
    }
;

sSimplePrimary []
  returns [Expression self = null]
{
  Expression   p1;
  Expression   p2 = null;
  Expression   p3 = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  self = sLiteral[]
|
  p1 = sFieldReference[]
  (
    "SUBRANGE" LPAREN p2 = sExpression[] COMMA p3 = sExpression[] RPAREN
  )?
    { self = new FieldReferencePrimary(sourceRef, p1,  p2, p3); }
|
  self = sSubTableExpression[]
|
  self = sIfExpression[]
|
  self = sCaseExpression[]
|
  self = sCountExpression[]
|
  self = sSetFunction[]
|
  self = sJdbcFunction[]
;

sSimpleTableReference []
  returns [SimpleTableReference self = null]
{
  TableReference	table;
  TableAlias		tableAlias;
  Expression		tableName;
  TokenReference	sourceRef = buildTokenReference();
}
:
  LPAREN table = sTableExpression[] RPAREN
  tableAlias = sTableAlias[]
    { self = new SimpleSubTableReference(sourceRef, table, tableAlias); }
|
  ( "TABLE" )?
  tableName = sTableName[]
  tableAlias = sTableAlias[]
    { self = new TableName(sourceRef, tableName, tableAlias); }
|
  self = sOuterJoin[]
;

sSimpleWhenClause []
  returns [SimpleWhenClause self = null]
{
  Expression   p1;
  Expression   p2;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "WHEN" p1 = sExpression[] "THEN" p2 = sExpression[]
    { self = new SimpleWhenClause(sourceRef, p1, p2); }
;

sSortElem []
  returns [SortElem self = null]
{
  Expression		p1 = null;
  String		ident = null;
  boolean		isDesc = false;
  TokenReference	sourceRef = buildTokenReference();
}
:
  (
    ident = sIdentifier[] { p1 = new SimpleIdentExpression(sourceRef, ident); }
  |
    p1 = sIntegerLiteral[]
  )
  ( "ASC" | "DESC" { isDesc = true; } )?
    { new SortElem(sourceRef, p1, isDesc); }
;

sSortSpec []
  returns [SortSpec self = null]
{
  SortElem	p;
  Vector        v = new Vector();
  TokenReference	sourceRef = buildTokenReference();
}
:
  "ORDER" "BY" p = sSortElem[] { v.addElement(p); }
  ( COMMA p = sSortElem[] { v.addElement(p); } )*
    { self = new SortSpec(sourceRef, v); }
;

sSubTableExpression []
  returns [SubTableExpression self = null]
{
  SubTable       left;
  UnidiffSpec	 spec;
  SubTable       right;
  TokenReference	sourceRef = buildTokenReference();
}
:
  LPAREN
  left = sSubTableTerm[]
  (
    spec = sUnidiffSpec[] right = sSubTableTerm[]
      { left = new UnidiffTableTerm(sourceRef, spec, left, right); }
  )*
  RPAREN
    { self = new SubTableExpression(sourceRef, left); }
;

sSubTableReference []
  returns [SubTable self = null]
{
  SearchCondition   p1;
  TokenReference	sourceRef = buildTokenReference();
}
:
  p1 = sSearchCondition[]
    { self = new SubTableReference(sourceRef, p1); }
|
  self = sSelectExpression[]
;

sSubTableTerm []
  returns [SubTable self = null]
{
  IntersectSpec   spec;
  SubTable        right;
  TokenReference	sourceRef = buildTokenReference();
}
:
  self = sSubTableReference[]
  (
    spec = sIntersectSpec[] right = sSubTableReference[]
      { self = new IntersectTableTerm(sourceRef, spec, self, right); }
  )*
;

sTableAlias []
  returns [TableAlias self = null]
{
  String		ident = null;
  FieldNameList		list = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  ( ident = sIdentifier[] )?
  ( LPAREN list = sFieldNameList[] RPAREN )?
    { self = new TableAlias(sourceRef, ident, list); }
;

sTableName []
  returns [Expression self = null]
{
  String		ident = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  ident = sIdentifier[]
    { self = new SimpleIdentExpression(sourceRef, ident); }
;

sTupleInPredicate []
  returns [TupleInPredicate self = null]
{
  boolean		hasNot = false;
  ExpressionList	p1;
  SubTableExpression	p2;
  TokenReference	sourceRef = buildTokenReference();
}
:
  LBRACKET p1 = sExpressionList[] RBRACKET
  ( "NOT" { hasNot = true; } )?
  "IN" p2 = sSubTableExpression[]
    { self = new TupleInPredicate(sourceRef, p1, hasNot, p2); }
;

sUnidiffOp []
  returns [String self = null]
:
  "UNION"
    { self = "UNION"; }
  ( "ALL"  { self += " ALL"; } )?
|
  "DIFF"  { self = "DIFF"; }
|
  "EXCEPT" { self = "EXCEPT"; }
;

sUnidiffSpec []
  returns [UnidiffSpec self = null]
{
  String		oper;
  CorrespondingSpec     corr = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  oper = sUnidiffOp[]
  ( corr = sCorrespondingSpec[] )?
    { self = new UnidiffSpec(sourceRef, oper, corr); }
;

sUpdSpec []
  returns [UpdSpec self = null]
{
  String	    oper;
  CorrespondingSpec spec;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "FOR" "UPDATE"
    { self = new UpdSpec(sourceRef); }
;

sUpdateStatement []
  returns [UpdateStatement self = null]
{
  String	p1;
  String	p2 = null;
  Vector	p3;
  SearchCondition	p4 = null;
  TokenReference	sourceRef = buildTokenReference();
  }
:
  "UPDATE" p1 = sIdentifier[] ( p2 = sIdentifier[] )?
  "SET" p3 = sAssignList[]
  (
    (
      "WHERE"
      (
        p4 = sSearchCondition[]
      |
        "CURRENT"
      )
        { self = new UpdateStatement(sourceRef, p1, p2, p3,p4); }
    )
  |
    { self = new UpdateStatement(sourceRef, p1, p2, p3); }
  )
;

sValueInPredicate []
  returns [ValueInPredicate self = null]
{
  ExpressionList	p1;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "IN" LPAREN p1 = sExpressionList[] RPAREN
    { self = new ValueInPredicate(sourceRef, p1); }
;

sValueList []
  returns [ValueList self = null]
{
  ExpressionList	p1;
  TokenReference	sourceRef = buildTokenReference();
}
:
  LPAREN p1 = sExpressionList[] RPAREN
    { self = new ValueList(sourceRef, p1); }
;

sWithCompPredicate[Predicate predicate]
  returns [Predicate self = null]
{
  String		s = null;
  Expression		p = null;
  boolean		hasNot = false;
  Predicate		right = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "IS"
  ( "NOT" { hasNot = true; } )?
  "NULL"
    { self = new IsPredicate(sourceRef, predicate, hasNot); }
|
  (
    LT { s = "<"; }
  |
    LE { s = "<="; }
  |
    GT { s = ">"; }
  |
    GE { s = ">="; }
  |
    ASSIGN { s = "="; }
  |
    NE { s = "<>"; }
  )
  (
    right = sQuantifiedPredicate[]
  |
    p = sExpression[]
    { right = new ExpressionPredicate(sourceRef, p); }
  )
    { self = new ComparaisonPredicate(sourceRef, predicate, right, s); }
;

sWithNotPredicate []
  returns [Predicate self = null]
:
  self = sBetweenPredicate[]
|
  self = sLikePredicate[]
|
  self = sValueInPredicate[]
|
  self = sMatchesPredicate[]
;

sOuterJoin []
  returns [OuterJoin self = null]
{
  TableReference p1;
  String	 p2;
  TableReference p3;
  JoinPred	 p4 = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  JDBC_OUTERJOIN
  p1 = sTableReference[]
  p2 = sJoinType[] "OUTER" "JOIN"
  p3 = sTableReference[]
  ( p4 = sJoinPred[] )?
  RCURLY
   { self = new OuterJoin(sourceRef, p1, p2, p3, p4); }
;

sCountExpression []
  returns [CountExpression self = null]
{
  TokenReference	sourceRef = buildTokenReference();
  Expression		expr = null;
  boolean		hasDistinct = false;
}
:
  "COUNT" LPAREN
  (
    STAR
  |
    "DISTINCT" expr = sExpression[] { hasDistinct = true; }
  )
  RPAREN
    { self = new CountExpression(sourceRef, hasDistinct, expr); }
;

sSetFunction []
  returns [SetFunction self = null]
{
  String		oper = null;
  Expression		expr = null;
  int			type = SetFunction.TYP_NONE;
  TokenReference	sourceRef = buildTokenReference();
}
:
  oper = sSetFunctionName []
  LPAREN
  (
    (
      "ALL" { type = SetFunction.TYP_ALL; }
    |
      "DISTINCT" { type = SetFunction.TYP_DISTINCT; }
    )?
     expr = sExpression[] { self = new SetFunction(sourceRef, oper, type, expr); }
  )
  RPAREN
;

sSetFunctionName []
  returns [String self = null]
:
  "MAX" { self = "MAX"; }
|
  "MIN" { self = "MIN"; }
|
  "AVG" { self = "AVG"; }
|
  "SUM" { self = "SUM"; }
;

sJdbcFunction []
  returns [JdbcFunction self = null]
{
  String		name = null;
  ExpressionList	params = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  JDBC_FUNCTION
  (
    t:STANDARD_ID { name = t.getText(); }
  |
    name = sSetFunctionName[]
      { reportTrouble(new CWarning(sourceRef, "x-set-function-is-sql", name)); }
  )
  LPAREN ( params = sExpressionList[] )? RPAREN
  RCURLY
    { self = new JdbcFunction(sourceRef, name, params); }
;

sEscapeExpression []
  returns [JdbcEscape self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  JDBC_ESCAPE t:STRING_LIT RCURLY
    { self = new JdbcEscape(sourceRef, t.getText()); }
;

sTableExpression []
  returns [TableReference self = null]
{
  UnidiffSpec	spec;
  TableReference      right;
  TokenReference	sourceRef = buildTokenReference();
}
:
  self = sTableTerm[]
  (
    spec = sUnidiffSpec[] right = sTableTerm[]
      { self = new UnidiffTableReference(sourceRef, spec, self, right); }
  )*
;

sTableTerm []
  returns [TableReference self]
{
  IntersectSpec    spec;
  TableReference   right;
  TokenReference	sourceRef = buildTokenReference();
}
:
  self = sTableReference[]
  (
    spec = sIntersectSpec[] right = sTableReference[]
      { self = new IntersectTableReference(sourceRef, spec, self, right); }
  )*
;

sTableReference []
  returns [TableReference self = null]
{
  SelectExpression   p1;
  TableReference sub;
  TokenReference	sourceRef = buildTokenReference();
}
:
  LPAREN sub = sTableExpression[] RPAREN
    { self = new ParenthesedTableReference(sourceRef, sub); }
|
  p1 = sSelectExpression[]
    { self = new SelectTableReference(sourceRef, p1); }
;

sWhereClause []
  returns [WhereClause self = null]
{
  SearchCondition	p;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "WHERE" p = sSearchCondition[]
    { self = new WhereClause(sourceRef, p); }
;

