/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Message.java,v 1.9 1999/09/12 14:33:39 vgp Exp $
 */

package at.dms.util;

import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 * This class represents the root class for all kopic errors
 */
public class Message {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a message with an arbitrary number of parameters
   * @param	key		the message key
   * @param	params		the array of parameters
   */
  public Message(String key, Object[] params) {
    this.key	= key;
    this.params	= params;
  }

  /**
   * Constructs a message with 2 parameters
   * @param	key		the message key
   * @param	param1		the first parameter
   * @param	param2		the second parameter
   */
  public Message(String key, Object param1, Object param2) {
    this(key, new Object[] { param1, param2 });
  }

  /**
   * Constructs a message with 1 parameter
   * @param	key		the message key
   * @param	param		the parameter
   */
  public Message(String key, Object param) {
    this(key, new Object[] { param });
  }

  /**
   * Constructs a message with no parameter
   * @param	key		the message key
   * @param	param		the parameter
   */
  public Message(String key) {
    this(key, null);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Returns the string explaining the error
   */
  public String getMessage() {
    return format(this.key, this.params);
  }

  /**
   * Returns the message key.
   */
  public String getKey() {
    return key;
  }

  /**
   * Returns a message (convenience routine).
   * @param	key		the message key
   * @param	params		the array of parameters
   */
  public static String format(String key, Object[] params) {
    String	format = getString(key);
    if (format == null) {
      return MessageFormat.format("undefined error \"{0}\"", new String[] { key });
    } else {
      try {
	return MessageFormat.format(format, params);
      } catch (Exception e) {
	return format;
      }
    }
  }

  /**
   * Returns a message (convenience routine).
   * @param	key		the message key
   */
  public static String getMessage(String key) {
    return Message.format(key, null);
  }

  // ----------------------------------------------------------------------
  // INITIALIZERS
  // ----------------------------------------------------------------------

  /**
   * Add new error in data base
   */
  public static void addBundle(String name) {
    try {
      bundles.addElement(ResourceBundle.getBundle(name));
    } catch (MissingResourceException e) {
      //!!! signal error
    }
  }


  // ----------------------------------------------------------------------
  // PROTECTED METHODS
  // ----------------------------------------------------------------------

  /**
   * Returns the property
   */
  private static String getString(String key) {
    for (int i = 0; i < bundles.size(); i++) {
      try {
	return ((ResourceBundle)bundles.elementAt(i)).getString(key);
      } catch (MissingResourceException e) {
	// try next one
      }
    }

    return null;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private static Vector		bundles = new Vector();

  private /*final javac does not handle*/ String		key;
  private /*final javac does not handle*/ Object[]		params;
}
