/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: SqlChecker.java,v 1.3 1999/10/28 14:03:13 vgp Exp $
 */

package at.dms.xkjc;

import java.util.Vector;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;

import at.dms.compiler.CWarning;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

import at.dms.sqlc.SqlContext;
import at.dms.sqlc.SqlPhylum;
import at.dms.sqlc.SearchCondition;
import at.dms.kjc.*;

/**
 * This class implements a Java pretty printer
 */
public class SqlChecker extends at.dms.sqlc.SqlChecker implements SqlVisitor {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a new visitor
   */
  public SqlChecker(SqlContext context) {
    super(context);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Returns the current string
   */
  public JExpression toJavaExpression(TokenReference ref) {
    String	text = toString();

    if (expr == null) {
      // only string
      return new JStringLiteral(ref, text);
    } else if (text.length() > 0) {
      // pending sql, add to left expression
      return new JAddExpression(ref, expr, new JStringLiteral(ref, text));
    } else {
      return expr;
    }
  }

  // ----------------------------------------------------------------------
  // VISITORS
  // ----------------------------------------------------------------------

  /**
   * Generates code.
   * @param	context		the sql context that holds the kjc Context
   * @param	left		the SQL expression to be built
   * @param	current		the current string literal to be added to left
   */
  public void visitTypedSelectElem(String ident, JLocalVariable var, SqlPhylum sqlExpr, boolean hasAs)
    throws PositionedError
  {
    if (!hasAs) {
      String	column;
      String	alias;
      int	index = ident.indexOf('.');

      if (index != -1) {
	column = ident.substring(index + 1, ident.length());
	alias = ident.substring(0, index);
      } else {
	column = ident;
	alias = null;
      }

      String table = at.dms.sqlc.FieldReference.checkColumnExists(sqlExpr.getTokenReference(), getContext(), alias, column);
      if (table != null) {
	try {
	  XUtils.checkDatabaseType(var.getType(), table, column);
	} catch (UnpositionedError e) {
	  getContext().reportTrouble(new CWarning(sqlExpr.getTokenReference(), e.getText()));
	}
      }
    }

    sqlExpr.accept(this);
  }
  
  /**
   * Generates java code.
   *
   * @exception	PositionedError		if the check fails
   */
  public void visitIfCondition(String oper, SearchCondition left, JExpression cond, XSqlExpr thenClause, XSqlExpr elseClause)
    throws PositionedError
  {
    TokenReference	ref = left.getTokenReference();
    JavaSqlContext	sqlStatement = getJavaContext(getContext());
    CBodyContext	context = sqlStatement.getBodyContext();

    cond = cond.checkExpression(new CExpressionContext(context));
    if (cond.getType() != CStdType.Boolean) {
      throw new PositionedError(left.getTokenReference(), "conditionalMustBeBoolean", cond.getType());
    }

    left.accept(this);

    JExpression elsePart = null;
    JExpression	thenPart = thenClause.checkExpression(this.getContext());
    thenPart = new JAddExpression(ref,
			      new JStringLiteral(ref, " " + oper + " "),
			      new JCheckedExpression(ref, thenPart));

    if (elseClause != null) {
      elsePart = elseClause.checkExpression(this.getContext());
      elsePart = new JAddExpression(ref,
				    new JStringLiteral(ref, " " + oper + " "),
				    new JCheckedExpression(ref, elsePart));
    } else {
      elsePart = new JStringLiteral(ref, "");
    }

    if (current.length() > 0) {
      // pending sql, add to left expression
      if (expr == null) {
	expr = new JStringLiteral(ref, current.toString());
      } else {
	expr = new JAddExpression(ref, expr, new JStringLiteral(ref, current.toString()));
      }
    }
    current.setLength(0);

    JExpression self = new JConditionalExpression(ref,
						  new JParenthesedExpression(ref, new JCheckedExpression(ref, cond)),
						  thenPart, elsePart);

    expr = new JAddExpression(ref, expr, new JParenthesedExpression(ref, self));
  }

  /**
   * Generates java code.
   *
   * @exception	PositionedError		if the check fails
   */
  public void visitSqlExprJava(JExpression expr, boolean asText)
    throws PositionedError
  {
    JavaSqlContext	sqlStatement = getJavaContext(getContext());
    CBodyContext	context = sqlStatement.getBodyContext();
    TokenReference	ref = expr.getTokenReference();

    expr = expr.checkExpression(new CExpressionContext(context));

    if (current.length() > 0) {
      // pending sql, add to left expression
      if (this.expr == null) {
	this.expr = new JStringLiteral(ref, current.toString() + " ");
      } else {
	this.expr = new JAddExpression(ref, this.expr, new JStringLiteral(ref, current.toString() + " "));
      }
    }
    current.setLength(0);

    if (expr instanceof JNullLiteral) {
      if (!asText) {
	this.expr = new JAddExpression(ref, this.expr, new JStringLiteral(ref, "NULL"));
      } else {
	this.expr = expr;
      }
      return;
    }

    if (!XStdType.isSQLPrimitiveType(expr.getType())) {
      sqlStatement.addBlob(expr);
    }

    if (asText) {
      this.expr = new JAddExpression(ref, this.expr, expr);
    } else {
      this.expr = new JAddExpression(ref,
				this.expr,
				XFactory.parseToSql(ref,
						    null,
						    new JExpression[]{new JCheckedExpression(ref, expr)}));
    }
  }

  // ----------------------------------------------------------------------
  // STATIC UTILITY
  // ----------------------------------------------------------------------

  public static JavaSqlContext getJavaContext(SqlContext ctxt) {
    if (ctxt instanceof JavaSqlContext) {
      return (JavaSqlContext)ctxt;
    }
    while (ctxt != null) {
      ctxt = ctxt.getParentContext();
      if (ctxt instanceof JavaSqlContext) {
	return (JavaSqlContext)ctxt;
      }
    }
    return null;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private JExpression			expr;
}
