/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: SqlPrettyPrinter.java,v 1.4 1999/10/28 14:03:13 vgp Exp $
 */

package at.dms.xkjc;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.StringTokenizer;
import java.util.Vector;

import at.dms.compiler.JavaStyleComment;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TabbedPrintWriter;
import at.dms.sqlc.SqlPhylum;
import at.dms.sqlc.SearchCondition;
import at.dms.kjc.JLocalVariable;
import at.dms.kjc.CJavaPrettyPrinter;
import at.dms.kjc.JExpression;
import at.dms.kjc.JNameExpression;

/**
 * This class implements a Java pretty printer
 */
public class SqlPrettyPrinter extends at.dms.sqlc.SqlPrettyPrinter implements SqlVisitor {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * construct a pretty printer object for java code
   * @param	fileName		the file into the code is generated
   */
  public SqlPrettyPrinter(String fileName) {
    super(fileName);
  }

  /**
   * construct a pretty printer object for java code
   * @param	fileName		the file into the code is generated
   */
  public SqlPrettyPrinter(TabbedPrintWriter p, CJavaPrettyPrinter java_pp) {
    super(p);

    this.in_pp = java_pp;
  }

  /**
   * Returns the java pretty printer
   */
  public CJavaPrettyPrinter getJavaPrettyPrinter() {
    in_pp.setPos(pos);
    return in_pp;
  }

  // ----------------------------------------------------------------------
  // VISITOR
  // ----------------------------------------------------------------------

  /**
   * Generates code.
   * @param	context		the sql context that holds the kjc Context
   * @param	left		the SQL expression to be built
   * @param	current		the current string literal to be added to left
   */
  public void visitTypedSelectElem(String ident, JLocalVariable var, SqlPhylum sqlExpr, boolean hasAs) 
    throws PositionedError
  {
    if (!hasAs) {
      print(var.getType());
      print(" ");
      pos += TAB_SIZE;
      pos += TAB_SIZE;
      print(ident);
      pos -= TAB_SIZE;
      pos -= TAB_SIZE;
    } else {
      sqlExpr.accept(this);
      print(" AS ");
      print(var.getType());
      print(" ");
      pos += TAB_SIZE;
      pos += TAB_SIZE;
      print(ident);
      pos -= TAB_SIZE;
      pos -= TAB_SIZE;
    }
  }
  
  /**
   * Generates java code.
   *
   * @exception	PositionedError		if the check fails
   */
  public void visitIfCondition(String oper, SearchCondition left, JExpression cond, XSqlExpr thenClause, XSqlExpr elseClause) 
    throws PositionedError
  {
    left.accept(this);
    println();
    pos -= TAB_SIZE;
    print(oper);
    pos += TAB_SIZE;
    print("{if (");
    cond.genJavaCode(in_pp);
    print(") ");
    thenClause.getSql().accept(this);
    if (elseClause != null) {
      print(" else ");
      elseClause.getSql().accept(this);
    }
    print("}");
  }

  /**
   * Generates java code.
   *
   * @exception	PositionedError		if the check fails
   */
  public void visitSqlExprJava(JExpression expr, boolean asText) 
    throws PositionedError
  {
    print(":");
    if (asText) {
      print("!");
    }
    print("(");
    expr.genJavaCode(getJavaPrettyPrinter());
    print(")");
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  protected CJavaPrettyPrinter		in_pp;
}
