/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XAssignmentExpression.java,v 1.25 1999/10/08 11:19:12 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.*;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * This class implements the assignment operation
 */
public class XAssignmentExpression extends JAssignmentExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	left		the left operand
   * @param	right		the right operand
   */
  public XAssignmentExpression(TokenReference where,
			       JExpression left,
			       JExpression right) {
    super(where, left, right);
  }

  /**
   *
   */
  public static JExpression create(TokenReference where, int oper, JExpression left, JExpression right) {
    if (oper == OPE_SIMPLE) {
      return new XAssignmentExpression(where, left, right);
    } else {
      return new XCompoundAssignmentExpression(where, oper, left, right);
    }
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param	context		the actual context of analyse
   * @return	a pure java expression including promote node./
   * @exception	PositionedError Error catched as soon as possible
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    context.setLeftSide(true);
    left = left.checkExpression(context);
    check(context, left instanceof CInitializable &&
	  ((CInitializable)left).isLValue(context), "assignment-notlvalue");
    context.setLeftSide(false);

    right = right.checkExpression(context);

    try {
      ((CInitializable)left).initialize(context.getBodyContext());
    } catch (UnpositionedError e) {
      throw e.getError(getTokenReference());
    }

    check(context, (right.getType() instanceof XPrimitiveClassType) ||
	  !(right.getType().isClassType()) ||
	  !(left.getType() instanceof XPrimitiveClassType),
	  "x-assignment-tonotnull", right.getType());

    if (right.getType().isAssignableTo(left.getType())) {

      type = left.getType();
      right = right.convertType(type, context);

      return this;
    } else {
      JExpression	prefix = new JNameExpression(getTokenReference(),
						     "operator$" + XConstants.OPE_SIMPLE);
      JExpression	param;

      if (left.getType().isReference()) {
	param = new JCastExpression(getTokenReference(),
				     new JNullLiteral(getTokenReference()),
				     left.getType());
      } else if (left.getType().isNumeric()) {
	param = new JOrdinalLiteral(getTokenReference(), 0);
      } else if (left.getType() == CStdType.Boolean) {
	param = new JBooleanLiteral(getTokenReference(), true);
      } else {
	param = null;
	check(context, false, "assignment-badtype", right.getType(), left.getType());
      }

      XOverloadedMethodCallExpression	overload = new XOverloadedMethodCallExpression(getTokenReference(),
										       prefix,
										       param != null ?
										       new JExpression[] {param, right} :
										       new JExpression[] {right}
										       );

      try {
	JExpression	convert = XUtils.fetchOverloadedOperator(context, overload);

	if (convert != null) {
	  JAssignmentExpression	assign = new JAssignmentExpression(getTokenReference(),
								   left,
								   convert);

	  JExpression ret = assign.checkExpression(context);
	  return ret;
	}
      } catch (PositionedError error) {
	//error.printStackTrace();
	//context.reportTrouble(error); // prevent user that there is a probleme with overloading
      }
      check(context, false, "assignment-badtype", right.getType(), left.getType());
      return null; //never accessed
    }
  }
}
