/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XCastExpression.java,v 1.17 1999/10/08 11:19:12 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.*;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * This class represents a cast expression '((byte)2)'
 */
public class XCastExpression extends JCastExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a node in the parsing tree.
   * This method is directly called by the parser.
   * @param	where	the line of this node in the source code
   * @param	expr	the expression to be casted
   * @param	dest	the type of this expression after cast
   */
  public XCastExpression(TokenReference where, JExpression expr, CType dest) {
    super(where, expr, dest);
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param	context		the actual context of analyse
   * @return	a pure java expression including promote node
   * @exception	PositionedError		the cast is invalid
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    expr = expr.checkExpression(context);
    try {
      dest.checkType(context);
    } catch (UnpositionedError e) {
      throw e.getError(getTokenReference());
    }

 if (!expr.getType().isCastableTo(dest) || castToPrimitive(expr.getType(), getType())) { // not java castable...
      JExpression	left = null;
      JExpression	prefix = new JNameExpression(getTokenReference(),
						     "operator$" + XConstants.OPE_CAST);
      // may try all explicitly loaded classes
      if (dest.isReference()) {
	left = new JCastExpression(getTokenReference(),
				   new JNullLiteral(getTokenReference()),
				   dest);
      } else if (dest.isNumeric()) {
	left = new JOrdinalLiteral(getTokenReference(), 0);
      } else if (dest == CStdType.Boolean) {
	left = new JBooleanLiteral(getTokenReference(), true);
      } else {
	check(context, false, "assignment-badtype", expr.getType(), dest);
      }

      XOverloadedMethodCallExpression	overload = new XOverloadedMethodCallExpression(getTokenReference(),
										       prefix,
										       new JExpression[] {left, expr});

      try {
	JExpression	convert = XUtils.fetchOverloadedOperator(context, overload);

	if (convert != null) {
	  return convert.checkExpression(context);
	}
      } catch (PositionedError error) {
	//context.reportTrouble(error); // prevent user that there is a probleme with overloading
      }
      check(context, false, "cast-cant", expr.getType(), dest);
    } 

    if (expr.isConstant() && expr.getType().isPrimitive()) {
      if (expr.getType().isNumeric()) {
	return JLiteral.createLiteral(dest, ((JNumberLiteral)expr).getValue()) ;
      } else {
	return expr;
      }
    }
    //if (getType() instanceof XPrimitiveClassType &&
    //!(expr.getType() instanceof XPrimitiveClassType)) {
    //  return ;
    //} else {
    return expr.convertType(dest, context);
    //}
  }

  private boolean castToPrimitive(CType from, CType to) {
    if (!(to instanceof XPrimitiveClassType)) {
      return false;
    }
    if (!(from instanceof CClassType)) {
      return false;
    }

    return ((XPrimitiveClassType)to).getNotNullableClass().equals(from);
  }
}
