/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XCursorDeclaration.java,v 1.25 1999/10/28 14:03:13 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.*;
import at.dms.compiler.TokenReference;
import at.dms.compiler.PositionedError;
import at.dms.compiler.JavaStyleComment;

/**
 * This class represents cursor definition in kopi grammar
 */
public class XCursorDeclaration extends JClassDeclaration {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	context		the context in which this class is defined
   * @param	modifiers	the list of modifiers of this class
   * @param	ident		the short name of this class
   * @param	superName	the name of super class of this class
   * @param	interfaces	the name of this class's interfaces
   * @param	deprecated	is this class deprecated
   */
  public XCursorDeclaration(TokenReference where,
			    int modifiers,
			    String ident,
			    JFieldDeclaration[] fields,
			    JMethodDeclaration[] methods,
			    JTypeDeclaration[] inners,
			    JPhylum[] initializers,
			    XSelectStatement select,
			    JFormalParameter[] parameters,
			    JJavadocComment javadoc,
			    JavaStyleComment[] comments) {
    super(where, 
	  modifiers | XConstants.ACC_CURSOR, 
	  ident == null ? "Cursor" : ident, 
	  XConstants.XKJ_CURSOR,
	  CClassType.EMPTY,
	  fields, 
	  methods, 
	  inners, 
	  initializers, 
	  javadoc,
	  comments);

    this.select = select;
    this.parameters = parameters;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  public void setPositionInBlock(int pos) {
    setIdent(("Cursor" + pos).intern());
  }

  public String getIdent() {
    return ident;
  }

  /**
   *
   */
  public static void addImplicitSelect(CParseClassContext context, XSelectStatement select, JFormalParameter[] parameters) {
    TokenReference	ref = select.getTokenReference();
    XSqlExpr	expr = new XSqlExpr(ref, select.getSql());

    context.addMethodDeclaration(new JMethodDeclaration(ref,
							ACC_PUBLIC + ACC_FINAL,
							CStdType.Void,
							"open",
							parameters,
							new CClassType[]{CClassType.lookup(XConstants.JAV_SQLException)},
							XFactory.parseCursorOpen(ref,
										 null,
										 new JExpression[] {
										   new JThisExpression(ref), expr}),
							null, null));

    XTypedSelectElem[] elems = select.getTypedSelectElem();
    for (int i = 0; i < elems.length; i++) {
      XCursorFieldDeclaration	field =(new XCursorFieldDeclaration(ref,
								    elems[i].getVariableDefinition(),
								    null,
								    null));
      field.setPosition(i);
      context.addFieldDeclaration(field);
    }
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * checkTypeBody
   * Check expression and evaluate and alter context
   * @param context the actual context of analyse
   * @return  a pure java expression including promote node
   * @exception	PositionedError	an error with reference to the source file
   */
  public void checkTypeBody(CContext context) throws PositionedError {
    super.checkTypeBody(context);

    checked = true;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param	p		the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    if (checked) {
      super.genJavaCode(p);
    } else {
      if (select != null) {
	((XJavaPrettyPrinter)p).printImplicitCursorBody(select, parameters);
      } else {
	((XJavaPrettyPrinter)p).printExplicitCursorBody(getFields());
      }
    }
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  private boolean		checked;
  private XSelectStatement	select;
  private JFormalParameter[]	parameters;
}
