/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XCursorField.java,v 1.21 1999/10/08 11:19:12 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.classfile.FieldInfo;
import at.dms.kjc.*;
import at.dms.compiler.TokenReference;

/**
 * This class represents an exported member of a class (fields)
 */
public class XCursorField extends CSourceField {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a field export
   * @param	owner		the owner of this field
   * @param	modifiers	the modifiers on this field
   * @param	ident		the name of this field
   * @param	type		the type of this field
   */
  public XCursorField(CClass owner, int modifiers, String ident, CType type) {
    super(owner, modifiers | XConstants.ACC_CURSOR, ident, type, false);
  }

  /**
   * Constructs a field export
   * @param	owner		the owner of this field
   * @param	fieldInfo	a field info from a class file
   */
  public XCursorField(CClass owner, FieldInfo fieldInfo) {
    super(owner, fieldInfo);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * @param	value		the value known at third pass
   */
  public void setValue(JLiteral value) {
    throw new RuntimeException();
  }

  /**
   * @return	the value of initializer or null
   */
  public JExpression getValue() {
    throw new RuntimeException();
  }

  // ----------------------------------------------------------------------
  // GENERATE ACCESS EXPRESSION
  // ----------------------------------------------------------------------

  public JExpression createAccessExpression(TokenReference ref,
					    JExpression prefix,
					    CExpressionContext context)
  {
    return createAccessExpression(ref, getType(), getPosition(), prefix, context);
  }

  public static JExpression createAccessExpression(TokenReference ref,
						   CType type,
						   int pos,
						   JExpression prefix,
						   CExpressionContext context)
  {
    // find method name
    String	name = null;
    boolean	nullable = false;
    boolean	needcast = false;
    boolean     kopiSerializabe = false;

    switch (type.getTypeID()) {
    case TID_BOOLEAN:
      name = "Boolean";
      break;
    case TID_BYTE:
      name = "Byte";
      break;
    case TID_SHORT:
      name = "Short";
      break;
    case TID_INT:
      name = "Int";
      break;
    case TID_LONG:
      name = "Long";
      break;
    case TID_FLOAT:
      name = "Float";
      break;
    case TID_DOUBLE:
      name = "Double";
      break;
    case TID_CHAR:
      name = "Char";
      break;
    case TID_ARRAY:
      name = "ByteArray";
      break;
    case TID_CLASS:
      // check for primitive data
      nullable = true;
      if (type.equals(XStdType.Boolean)) {
	name = "Boolean";
      } else if (type.equals(XStdType.Byte)) {
	name = "Byte";
      } else if (type.equals(XStdType.Short)) {
	name = "Short";
      } else if (type.equals(XStdType.Int)) {
	name = "Int";
      } else if (type.equals(XStdType.Float)) {
	name = "Float";
      } else if (type.equals(XStdType.Double)) {
	name = "Double";
      } else if (type.equals(XStdType.Char)) {
	name = "Char";
      }
      if (name != null) {
	break;
      }

      if (type.equals(CStdType.String)) {
	name = "String";
      } else if (type.equals(XStdType.Date)) {
	name = "Date";
      } else if (type.equals(XStdType.Month)) {
	name = "Month";
      } else if (type.equals(XStdType.Time)) {
	name = "Time";
      } else if (type.equals(XStdType.Fixed)) {
	name = "Fixed";
      }
      if (name != null) {
	break;
      }

      nullable = false;
      if (type.equals(XStdType.PDate)) {
	name = "Date";
      } else if (type.equals(XStdType.PMonth)) {
	name = "Month";
      } else if (type.equals(XStdType.PTime)) {
	name = "Time";
      } else if (type.equals(XStdType.PFixed)) {
	name = "Fixed";
      }
      if (name != null) {
	break;
      }

      nullable = false;
      if (type.getCClass().descendsFrom(XStdType.KopiSerializable.getCClass())) {
	kopiSerializabe = true;
	// !!! has kopiConstructor ?
	name = "ByteArray";
      } else {
	name = "SerializedObject";
	needcast = true;
      }
      break;
    default:
      throw new RuntimeException("INTERNAL ERROR, TYPE UNKNOWN");
    }

    // !!! SPECIAL CASE FOR INPUT STREAM !!! ???

    String	methodName = "get" + (nullable ? "Nullable" : "") + name;
    JMethodCallExpression expr = new JMethodCallExpression(ref, new JNameExpression(ref, prefix, methodName), new JExpression[] {new JOrdinalLiteral(ref, new Integer(pos + 1))});

    if (kopiSerializabe) {
      return buildKopiSerializableCall(ref, (CClassType)type, expr, prefix, pos);
    } else if (!needcast) {
      return expr;
    } else {
      return new JParenthesedExpression(ref, new JCastExpression(ref, expr, type));
    }
  }

  // ----------------------------------------------------------------------
  // PRIVATE UTILITIES
  // ----------------------------------------------------------------------

  /**
   * Construct the call to kopiSerializabe constructor
   */
  private  static JExpression buildKopiSerializableCall(TokenReference ref,
							CClassType type,
							JExpression data,
							JExpression prefix,
							int pos) {
    JExpression n =  new JNewObjectExpression(ref, type, null, new JExpression[] {data, new JNullLiteral(ref)});
    JMethodCallExpression expr = new JMethodCallExpression(ref, new JNameExpression(ref, prefix, "isNull"), new JExpression[] {new JOrdinalLiteral(ref, new Integer(pos + 1))});
    JExpression tri = new JConditionalExpression(ref, expr, new JNullLiteral(ref), n);
    return new JParenthesedExpression(ref, tri);
  }
}
