/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XExecSqlExpression.java,v 1.16 1999/10/08 11:19:12 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.*;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * This class represents an sql expression in java code
 */
public class XExecSqlExpression extends XExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructor
   */
  public XExecSqlExpression(TokenReference ref, JExpression conn, JExpression cursor, XSqlExpr expr) {
    super(ref);
    this.expr = expr;
    this.conn = conn;
    this.cursor = cursor;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate all, do constant folding as possible
   * Alter context and optimize code
   * @param	context			the actual context of analyse
   * @return	an optimized expression that do the same job than it
   * @exception	PositionedError	an error with reference to the source file
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    if (cursor == null) {
      if (conn == null) {
	check(context, context.getClassContext().getCClass().descendsFrom(XStdType.DBContextHandler.getCClass()),
	      "x-badDefaultContext", context.getClassContext().getCClass().getQualifiedName());
      } else {
	conn = conn.checkExpression(context);
	if (!conn.getType().isAssignableTo(XStdType.Connection)) {
	  check(context, conn.getType().isAssignableTo(XStdType.DBContextHandler),
		"x-badConnection", conn.getType().getCClass().getQualifiedName());
	  // CONSTRUCT THE CONNECTION ACCESS
	  conn = XProtectedStatement.buildConnection(getTokenReference(), conn).checkExpression(context);
	}
      }
    } else {
      check(context, conn == null, "x-bad-cursor-and-conn");
    }

    JExpression expr = this.expr.checkExpression(context);

    TokenReference ref = getTokenReference();
    JExpression		blobs;
    JExpression[]	exprs = this.expr.getBlobs();
    if (exprs.length > 0) {
      JArrayInitializer		init = new JArrayInitializer(ref, exprs);
      JArrayDimsAndInits	arr = new JArrayDimsAndInits(ref, new JExpression[] {null}, init);
      blobs = new JNewArrayExpression(ref, CStdType.Object, arr);
    } else {
      blobs = new JNullLiteral(ref);
    }

    if (cursor == null) {
      if (conn == null) {
	conn = XFactory.parseConnection(ref, null, new JExpression[] {new JThisExpression(ref)});
      }
      return XFactory.parseExecSqlExpression(ref,
					     null,
					     new JExpression[] {conn, expr, blobs}).checkExpression(context);
    } else {
      cursor = new JCheckedExpression(ref, cursor.checkExpression(context));

      return XFactory.parseExecSqlCursorExpression(ref,
						   null,
						   new JExpression[] {cursor, expr, blobs}).checkExpression(context);
    }
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param	p		the printwriter into the code is generated
   */
  public void genXKJCCode(XJavaPrettyPrinter p) {
    p.printExecSqlExpression(expr, conn, cursor);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  XSqlExpr			expr;
  JExpression			conn;
  JExpression			cursor;
}
