/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XFixedLiteral.java,v 1.13 1999/10/08 11:19:12 vgp Exp $
 */

package at.dms.xkjc;

import java.math.BigDecimal;

import at.dms.kjc.*;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * Root class for all expressions
 */
public class XFixedLiteral extends JLiteral {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * @param	where		the line of this node in the source code
   * @param	image		the string representation of this literal
   */
  public XFixedLiteral(TokenReference where, String image) {
    super(where);
    this.image = image;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Compute the type of this expression (called after parsing)
   * @return the type of this expression
   */
  public CType getType() {
    return XStdType.PFixed;
  }

  /**
   * Returns the literal value of an expression
   */
  public boolean isDefault() {
    return false;
  }

  /**
   * get a Number that represents the value of this literal
   * Do not call before checkBody
   */
  public Object getValue() {
    throw new RuntimeException("SHOULD NEVER BE CALLED");
  }

  /**
   * @return is this expression constant ?
   */
  public boolean isConstant() {
    // we are constant but we don't support constant folding
    return false;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param context the actual context of analyse
   * @return  a pure java expression including promote node
   * @exception	PositionedError		if the check fails
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    try {
      new BigDecimal(image);
    } catch (NumberFormatException e) {
      check(context, false, "x-bad-fixed-format", image);
    }

    TokenReference	ref = getTokenReference();
    JExpression		expr = new JNewObjectExpression(ref,
							XStdType.PFixed,
							null, 
							new JExpression[] {new JStringLiteral(ref, image)});

    return expr.checkExpression(context);
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param p the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    ((XJavaPrettyPrinter)p).printLiteral(image);
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    throw new RuntimeException("SHOULD NEVER BE CALLED");
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private String image;
}
