/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XFormalParameter.java,v 1.14 1999/10/08 11:19:12 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.JFormalParameter;
import at.dms.kjc.CType;
import at.dms.kjc.CClassContext;
import at.dms.compiler.TokenReference;

/**
 * This class represents a parameter declaration in the syntax tree
 */
public class XFormalParameter extends JFormalParameter {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	ident		the name of this variable
   * @param	initializer	the initializer
   */
  public XFormalParameter(TokenReference where,
			  int desc,
			  CType type,
			  String ident,
			  boolean isFinal) {
    super(where, ident == null ? DES_GENERATED : desc, type, ident, isFinal);
    if (ident == null) {
      //we add a $ because synthetic generated
      // fields are not pretty printed
      this.name = "anonymous_$";
    }
  }

  // ----------------------------------------------------------------------
  // INTERFACE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Second pass (quick), check interface looks good
   * Exceptions are not allowed here, this pass is just a tuning
   * pass in order to create informations about exported elements
   * such as Classes, Interfaces, Methods, Constructors and Fields
   * >>>>> sub classes must check modifiers and call checkInterface(super)
   * @return true iff sub tree is correct enought to check code
   */
  public CType checkInterface(CClassContext classContext) {
    CType type = super.checkInterface(classContext);
    if (name == "anonymous_$") {
      name = ("anonymous$_" + classContext.getAnonymousIndex()).intern();
    }
    return type;
  }
}
