/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XJavaPrettyPrinter.java,v 1.19 1999/10/28 14:03:13 vgp Exp $
 */

package at.dms.xkjc;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TabbedPrintWriter;
import at.dms.kjc.*;
import at.dms.sqlc.SqlPhylum;

/**
 * This class implements a Java pretty printer
 */
public class XJavaPrettyPrinter extends CJavaPrettyPrinter {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * construct a pretty printer object for java code
   * @param	fileName		the file into the code is generated
   */
  public XJavaPrettyPrinter(String fileName) {
    super();

    if (fileName.indexOf(".java") == -1) {
      fileName = fileName.substring(0, fileName.lastIndexOf(".")) +  ".java";
    } else {
      fileName = fileName + ".gen";
    }

    try {
      this.p = new TabbedPrintWriter(new BufferedWriter(new FileWriter(fileName)));
    } catch (IOException e) {
      e.printStackTrace();
      System.err.println("cant write to " + fileName + ": " + e);
    }
    this.pos = 0;
  }

  /**
   * construct a pretty printer object for java code
   * @param	fileName		the file into the code is generated
   */
  public XJavaPrettyPrinter(TabbedPrintWriter p) {
    this.p = p;
    this.pos = 0;
  }

  /**
   * Returns the SQL pretty printer
   */
  public SqlPrettyPrinter getSQLPrettyPrinter() {
    if (sql_pp == null) {
      sql_pp = new SqlPrettyPrinter(p, this);
    }
    sql_pp.setPos(pos);

    return sql_pp;
  }

  /**
   * Returns the SQL pretty printer
   */
  public SqlPrettyPrinter getSqlPrettyPrinter(boolean inPlace) {
    if (sql_pp == null) {
      sql_pp = new SqlPrettyPrinter(p, this);
    }
    sql_pp.setPos(pos);

    return sql_pp;
  }

  // ----------------------------------------------------------------------
  // STATEMENT
  // ----------------------------------------------------------------------

  /**
   * prints a while statement
   */
  public void printAssertStatement(JExpression cond, JExpression expr) {
    newLine();
    print("#assert (");
    cond.genJavaCode(this);
    if (expr != null) {
      print(", ");
      expr.genJavaCode(this);
    }
    print(")");
  }

  /**
   * prints a while statement
   */
  public void printFailStatement(JExpression expr) {
    newLine();
    print("#fail (");
    if (expr != null) {
      expr.genJavaCode(this);
    }
    print(")");
  }

  /**
   * prints a while statement
   */
  public void printProtectedStatement(JExpression mess, JExpression cont, JBlock body) {
    newLine();
    print("#protected");
    if (cont != null) {
      print(" [");
      cont.genJavaCode(this);
      print("] ");
    }
    print("(");
    if (mess != null) {
      mess.genJavaCode(this);
    }
    print(") ");
    pos += TAB_SIZE;
    body.genJavaCode(this);
    pos -= TAB_SIZE;
  }

  /**
   * prints a local cursor statement
   */
  public void printLocalCursorDeclaration(JExpression cont, XCursorDeclaration decl, Vector defs) {
    newLine();
    print("#cursor ");
    if (cont != null) {
      print("[");
      cont.genJavaCode(this);
      print("] ");
    }
    decl.genJavaCode(this);
    for (int i = 0; i < defs.size(); i++) {
      if (i == 0) {
	print(" ");
      } else {
	print(", ");
      }
      print(defs.elementAt(i));
    }
    print(";");
  }

  /**
   * prints an implicit cursor body
   */
  public void printImplicitCursorBody(XSelectStatement stmt, JFormalParameter[] args) {
    print("(");
    for (int i = 0; i < args.length; i++) {
      if (i != 0) {
	print(", ");
      }
      args[i].genJavaCode(this);
    }
    print(") ");
    print("{");
    newLine();
    stmt.genJavaCode(this);
    newLine();
    print("}");
  }

  /**
   * prints an implicit cursor body
   */
  public void printXSelectStatement(String mod, XTypedSelectElem[] attrs, SqlPhylum body) {
    int	old_pos = pos;

    pos += TAB_SIZE;

/*
    getSQLPrettyPrinter().appendString("SELECT " + (mod == null ? "" : mod));
    for (int i = 0; i < attrs.length; i++) {
      if (i != 0) {
	getSQLPrettyPrinter().appendString(",");
      }
      try {
	attrs[i].accept(getSQLPrettyPrinter());
      } catch (PositionedError err) {
	err.printStackTrace();
      }
    }
    newLine();
*/
    try {
      body.accept(getSQLPrettyPrinter());
    } catch (PositionedError e) {
      e.printStackTrace();
    }
    pos = old_pos;
  }

  /**
   * prints an implicit cursor body
   */
  public void printExplicitCursorBody(JFieldDeclaration[] fields) {
    print("{");
    newLine();
    for (int i = 0; i < fields.length; i++) {
      fields[i].genJavaCode(this);
    }
    newLine();
    print("}");
  }

  /**
   * prints a local cursor statement
   */
  public void printExecSqlExpression(XSqlExpr sql, JExpression conn, JExpression cursor) {
    print("#update ");
    if (conn != null) {
      print("[");
      conn.genJavaCode(this);
      print("] ");
    }
    if (cursor != null) {
      print("(");
      cursor.genJavaCode(this);
      print(") ");
    }
    print("{ ");
    pos += TAB_SIZE;
    sql.genJavaCode(this);
    pos -= TAB_SIZE;
    newLine();
    print("}");
  }

  /**
   * prints a local cursor statement
   */
  public void printExecSqlStatement(JExpression conn, JExpression body, JExpressionListStatement list) {
    newLine();
    print("#execute ");
    if (conn != null) {
      print("[");
      conn.genJavaCode(this);
      print("] ");
    }
    print(" { ");
    pos += TAB_SIZE;
    body.genJavaCode(this);
    newLine();
    print("INTO       ");
    list.genJavaCode(this);
    pos -= TAB_SIZE;
    newLine();
    print("}");
  }

  /**
   * prints a local cursor statement
   */
  public void printLiteral(String text) {
    print(text);
  }

  /**
   * prints an SQL statement
   */
  public void printSQL(SqlPhylum text) {
    int	old_pos = pos;
    newLine();

    try {
      text.accept(getSQLPrettyPrinter());
    } catch (PositionedError e) {
      e.printStackTrace();
    }

    pos = old_pos;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  protected SqlPrettyPrinter		sql_pp;
}
