/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XLocalCursorDeclaration.java,v 1.26 1999/10/28 14:03:13 vgp Exp $
 */

package at.dms.xkjc;

import java.util.Vector;
import at.dms.kjc.*;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;

/**
 * This class represents cursor declaration in code part
 */
public class XLocalCursorDeclaration extends XStatement {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructor
   */
  public XLocalCursorDeclaration(TokenReference ref,
				 XCursorDeclaration d,
				 JavaStyleComment[] comments) {
    super(ref, comments);
    decl = d;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * The connection is explicitly set.
   */
  public void setConnection(JExpression conn) {
    this.conn = conn;
  }

  /**
   *
   */
  public void addDeclaration(String ident) {
    names.addElement(ident);
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check statement and return a pure kjc abstract tree that will be used to code generation.
   *
   * @param	context		the actual context of analyse
   * @exception	PositionedError		if the check fails
   */
  public JStatement checkAndConvert(CBodyContext context) throws PositionedError {
    if (conn == null) {
      check(context, context.getClassContext().getCClass().descendsFrom(XStdType.DBContextHandler.getCClass()),
	    "x-badDefaultContext", context.getClassContext().getCClass().getQualifiedName());
    } else {
      conn = conn.checkExpression(new CExpressionContext(context));
      if (!conn.getType().isAssignableTo(XStdType.Connection)) {
	check(context, conn.getType().isAssignableTo(XStdType.DBContextHandler),
	      "x-badConnection", conn.getType().getCClass().getQualifiedName());
	// CONSTRUCT THE CONNECTION ACCESS
	conn = XProtectedStatement.buildConnection(getTokenReference(), conn).checkExpression(new CExpressionContext(context));
      }
    }

    decl.setPositionInBlock(context.getClassContext().getNextSyntheticIndex());
    JStatement[]	defs = new JStatement[1 + 2 * names.size()];

    defs[0] = new JTypeDeclarationStatement(decl.getTokenReference(), decl);

    // CONSTRUCT TREE
    TokenReference	ref = getTokenReference();
    JVariableDeclarationStatement[] stmts = new JVariableDeclarationStatement[names.size()];
    for (int i = 0; i < stmts.length; i++) {
      String ident = (String)names.elementAt(i);
      defs[1 + i] = new JVariableDeclarationStatement(ref,
						  new XVariableDefinition(ref, ACC_FINAL, CClassType.lookup(decl.getIdent()), ident,
									  new JNewObjectExpression(ref, CClassType.lookup(decl.getIdent()),
												   null, JExpression.EMPTY)), null);
    }

    if (conn == null) {
      conn = XFactory.parseConnection(ref, null, new JExpression[] {new JThisExpression(ref)});
    }

    for (int i = 0; i < stmts.length; i++) {
      defs[1 + stmts.length + i] = new JExpressionStatement(ref,
							    new JMethodCallExpression(ref,
										      new JNameExpression(ref,
													  new JNameExpression(ref, ((JVariableDeclarationStatement)defs[1 + i]).getVars()[0].getIdent()),
													  "setDefaultConnection"),
										      new JExpression[] {conn}),
							    null);
    }
    JCompoundStatement compound = new JCompoundStatement(getTokenReference(), defs);

    compound.checkStatement(context);

    return compound;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param	p		the printwriter into the code is generated
   */
  public void genXKJCCode(XJavaPrettyPrinter p) {
    p.printLocalCursorDeclaration(conn, decl, names);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  XCursorDeclaration		decl;
  JExpression			conn;
  Vector			names = new Vector(); //String
}
