/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XMethodCallExpression.java,v 1.15 1999/10/08 11:19:12 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.*;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * This class represents methods call
 * <pre> methodname(explist) </pre>
 */
public class XMethodCallExpression extends JMethodCallExpression {

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	prefix		an expression that is a field of a class representing a method
   * @param	ident		the method ident
   * @param	args		the argument of the call
   */
  public XMethodCallExpression(TokenReference where,
			       JExpression prefix,
			       JExpression[] args) {
    super(where, prefix, args);
    if (prefix instanceof XNameExpression) {
      ((XNameExpression)prefix).setIsMethodName();
    }
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param context the actual context of analyse
   * @return  a pure java expression including promote node
   * @exception	PositionedError		if the check fails
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    try { // TRY JAVA-MATCH
      return super.checkExpression(context);
    } catch (CMethodNotFoundError ce) {

      // CHECK OVERLOADED
      if (prefix instanceof JNameExpression ||
	  ident == null ||
	  ce.getCaller() != this) {
	// problem in params
	throw ce;
      }

      // CONSTRUCT TYPE INTERFACE
      CType[]	argsType = new CType[args.length];
      for (int i = 0; i < argsType.length; i++) {
	argsType[i] = new XMutableType(args[i].getType());
      }

      if (prefix != null) {
	if (prefix.getType() == null) {
	  throw ce;
	}
	try {
	  method = prefix.getType().getCClass().lookupMethod(ident, argsType);
	} catch (UnpositionedError e) {
	  throw e.getError(getTokenReference());
	}
      } else {
	try {
	  method = context.lookupMethod(ident, argsType);
	} catch (UnpositionedError e) {
	  throw e.getError(getTokenReference());
	}
      }

      if (method == null) {
      	throw ce;
     }

      // check access
      CClass		local = context.getClassContext().getCClass();
      CClass		access = method.getOwner();

      check(context, method.isAccessible(local), "method-noaccess", method.getIdent());
      check(context, access.isAccessible(local), "class-noaccess", access.getIdent());

      if ((prefix == null) && !method.isStatic()) {
	if (access == local) {
	  prefix = new JThisExpression(getTokenReference()).checkExpression(context);
	} else {
	  prefix = new JThisExpression(getTokenReference(), access).checkExpression(context);
	}
      }

      TokenReference ref = getTokenReference();
      argsType = method.getParameters();
      for (int i = 0; i < argsType.length; i++) {
	args[i] = new XCastExpression(ref, args[i], argsType[i]).checkExpression(context);
      }

      return this.checkExpression(context);
    }
  }
}
