/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XMutableType.java,v 1.11 1999/10/08 11:19:12 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.compiler.FastStringBuffer;
import at.dms.compiler.UnpositionedError;
import at.dms.kjc.CType;
import at.dms.kjc.CContext;
import at.dms.kjc.Constants;
import at.dms.kjc.CStdType;

/**
 * Root for type converter
 * Actually this class work only for SQL primitive types
 */
public class XMutableType extends CType implements Constants {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a type signature
   */
  public XMutableType(CType original) {
    super(0);

    this.original = original;

    if (original == CStdType.Byte) {
      code = CODE_SBYTE;
    } else if (original == CStdType.Integer) {
      code = CODE_SINTEGER;
    } else if (original == CStdType.Long) {
      code = CODE_SLONG;
    } else if (original == CStdType.String) {
      code = CODE_SSTRING;
    } else if (original == CStdType.Boolean) {
      code = CODE_SBOOLEAN;
    } else if (original == CStdType.Double) {
      code = CODE_SDOUBLE;
    } else if (original == CStdType.Float) {
      code = CODE_SFLOAT;
    } else if (original == CStdType.Short) {
      code = CODE_SSHORT;
    } else if (original == CStdType.Char) {
      code = CODE_SCHAR;
    } else if (original == XStdType.Boolean) {
      code = CODE_XBOOLEAN;
    } else if (original == XStdType.Byte) {
      code = CODE_XBYTE;
    } else if (original == XStdType.Short) {
      code = CODE_XSHORT;
    } else if (original == XStdType.Int) {
      code = CODE_XINT;
    } else if (original == XStdType.Long) {
      code = CODE_XLONG;
    } else if (original == XStdType.Fixed) {
      code = CODE_XBIGDECIMAL;
    } else if (original == XStdType.Float) {
      code = CODE_XFLOAT;
    } else if (original == XStdType.Double) {
      code = CODE_XDOUBLE;
    } else if (original == XStdType.Char) {
      code = CODE_XCHAR;
    } else if (original == XStdType.Date) {
      code = CODE_XDATE;
    } else if (original == XStdType.Month) {
      code = CODE_XMONTH;
    } else if (original == XStdType.Time) {
      code = CODE_XTIME;
    }
  }

  /**
   * check that type is valid
   * necessary to resolve String into java/lang/String
   * @exception	UnpositionedError	this error will be positioned soon
   */
  public void checkType(CContext context) throws UnpositionedError {

  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * implicitlyCastTo
   * @param	dest	the expression type to cast to
   * @return	cast is valid ?
   */
  public boolean implicitlyCastTo(CType dest) {
    if (original.implicitlyCastTo(dest)) {
      return true;
    }

    // TRY OVERLOADING
    switch (code) {
    case CODE_SBYTE:
      if (!dest.isClassType()) {
	return false;
      }
      return dest.getCClass().descendsFrom(XStdType.Number.getCClass());
    case CODE_SLONG:
      if (!dest.isClassType()) {
	return false;
      }
      return dest.getCClass().descendsFrom(XStdType.Number.getCClass()) &&
	!dest.equals(XStdType.Byte) &&
	!dest.equals(XStdType.Short) &&
	!dest.equals(XStdType.Int);
    case CODE_SINTEGER:
      if (!dest.isClassType()) {
	return false;
      }
      return dest.getCClass().descendsFrom(XStdType.Number.getCClass()) &&
	!dest.equals(XStdType.Byte) && !dest.equals(XStdType.Short);
    case CODE_SBOOLEAN:
      return dest.equals(XStdType.Boolean);
    case CODE_SDOUBLE:
      if (!dest.isClassType()) {
	return false;
      }
      return dest.equals(XStdType.Double) ||
	dest.equals(XStdType.Fixed);
    case CODE_SFLOAT:
      if (!dest.isClassType()) {
	return false;
      }
      return dest.equals(XStdType.Float) ||
	dest.equals(XStdType.Double) ||
	dest.equals(XStdType.Fixed);
    case CODE_SSHORT:
      if (!dest.isClassType()) {
	return false;
      }
      return dest.getCClass().descendsFrom(XStdType.Number.getCClass()) &&
	!dest.equals(XStdType.Byte);
    case CODE_SCHAR:
      return dest.equals(XStdType.Char);
    case CODE_XBOOLEAN:
      return dest == CStdType.Boolean;
    case CODE_XBYTE:
      return dest.isNumeric() || (dest == XStdType.Short || dest == XStdType.Int || dest == XStdType.Long);
    case CODE_XSHORT:
      return dest.isNumeric() && dest != CStdType.Byte || 
	(dest == XStdType.Int || dest == XStdType.Long);
    case CODE_XINT:
      return dest.isNumeric() &&
	dest != CStdType.Byte &&
	dest != CStdType.Short;
    case CODE_XLONG:
      return dest.isNumeric() &&
	dest != CStdType.Byte &&
	dest != CStdType.Short &&
	dest != CStdType.Integer;
    case CODE_XBIGDECIMAL:
      return dest.isNumeric() &&
	dest == CStdType.Double;
    case CODE_XFLOAT:
      return dest == CStdType.Float ||
	dest == CStdType.Double;
    case CODE_XDOUBLE:
      return dest == CStdType.Double;
    case CODE_XCHAR:
      return dest == CStdType.Char;
    case CODE_XDATE:
    case CODE_XMONTH:
    case CODE_XTIME:
    default:
      return false; // can't overload
    }
  }

  // ----------------------------------------------------------------------
  // UNIMPLEMENTED ABSTRACT METHODS
  // ----------------------------------------------------------------------

  /**
   * Transforms this type to a string
   */
  public String toString() {
    return "XMUTABLETYPE:" + original;
  }

  /**
   * Transforms this type to a string
   */
  public String getSignature() {
    throw new RuntimeException();
  }

  /**
   * Transforms this type to a string
   */
  public void appendSignature(FastStringBuffer buffer) {
    throw new RuntimeException();
  }

  /**
   * Returns the size used in stack by value of this type
   */
  public int getSize() {
    throw new RuntimeException();
  }

  /**
   * isCastableTo
   * @param dest  the expression type to cast to
   * @return  cast is valid ?
   */
  public boolean isCastableTo(CType dest) {
    throw new RuntimeException();
  }

  /**
   * isAssignableTo
   * @param dest  the destination type
   * @return  assignment is valid ?
   */
  public boolean isAssignableTo(CType dest) {
    throw new RuntimeException();
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private static final int	CODE_SBYTE		= 0;
  private static final int	CODE_SINTEGER		= 1;
  private static final int	CODE_SLONG		= 2;
  private static final int	CODE_SSTRING		= 3;
  private static final int	CODE_SBOOLEAN		= 4;
  private static final int	CODE_SDOUBLE		= 5;
  private static final int	CODE_SFLOAT		= 6;
  private static final int	CODE_SSHORT		= 7;
  private static final int	CODE_SCHAR		= 8;
  private static final int	CODE_XBOOLEAN		= 9;
  private static final int	CODE_XBYTE		= 10;
  private static final int	CODE_XSHORT		= 11;
  private static final int	CODE_XINT		= 12;
  private static final int	CODE_XLONG		= 13;
  private static final int	CODE_XBIGDECIMAL	= 14;
  private static final int	CODE_XFLOAT		= 15;
  private static final int	CODE_XDOUBLE		= 16;
  private static final int	CODE_XCHAR		= 17;
  private static final int	CODE_XDATE		= 18;
  private static final int	CODE_XMONTH		= 19;
  private static final int	CODE_XTIME		= 20;

  private CType		original;
  private int		code;
}
