/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XNameExpression.java,v 1.11 1999/10/08 11:19:12 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.JNameExpression;
import at.dms.kjc.JExpression;
import at.dms.kjc.JClassFieldExpression;
import at.dms.kjc.CClass;
import at.dms.compiler.TokenReference;

/**
 * A name within an expression
 */
public class XNameExpression extends JNameExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * @param where the line of this node in the source code
   */
  public XNameExpression(TokenReference where, String ident) {
    super(where, ident);
  }

  /**
   * Construct a node in the parsing tree
   * @param where the line of this node in the source code
   */
  public XNameExpression(TokenReference where, String ident, boolean fullyQualified) {
    super(where, ident, fullyQualified);
  }

  /**
   * Construct a node in the parsing tree
   * @param where the line of this node in the source code
   */
  public XNameExpression(TokenReference where, JExpression prefix, String ident) {
    super(where, prefix, ident);
    if (prefix instanceof XNameExpression) {
      ((XNameExpression)prefix).setHasSuffix(true);
    }
  }

  /**
   * Since class field may be overloaded in sub compiler, this method allow
   * you to specifie the type of class field without needed to touch
   * the huge method above !
   */
  protected JClassFieldExpression createClassField(TokenReference ref,
						   JExpression prefix,
						   String ident) {
    if (prefix instanceof XCursorFieldExpression) {
      return new XCursorFieldExpression(ref, prefix, ident, hasSuffix);
    }

    CClass	access = prefix.getType().getCClass();
    if (access.descendsFrom(XStdType.Cursor.getCClass())) {
      return new XCursorFieldExpression(getTokenReference(), prefix, ident, hasSuffix);
    } else {
      return new JClassFieldExpression(ref, prefix, ident);
    }
  }

  /**
   * Since class field may be overloaded in sub compiler, this method allow
   * you to specifie the type of class field without needed to touch
   * the huge method above !
   */
  protected JClassFieldExpression createClassField(TokenReference ref,
						   String ident) {
    return new JClassFieldExpression(ref, ident);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  public void setHasSuffix(boolean suffix) {
    hasSuffix = suffix;
  }

  public void setIsMethodName() {
    if (getPrefix() instanceof XNameExpression) {
      ((XNameExpression)getPrefix()).setHasSuffix(false);
    }
  }

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  private	boolean	hasSuffix;
}
