/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XPrefixExpression.java,v 1.18 1999/10/08 11:19:12 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.*;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * This class represents unary expressions.
 */
public class XPrefixExpression extends JPrefixExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * @param	where		the line of this node in the source code
   * @param	oper		the operator
   * @param	expr		the operand
   */
  public XPrefixExpression(TokenReference where, int oper, JExpression expr) {
    super(where, oper, expr);
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param	context		the actual context of analyse
   * @return	a pure java expression including promote node
   * @exception	PositionedError Error catched as soon as possible
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    PositionedError	typeError = null;

    try {
      try {
	return super.checkExpression(context);
      } catch (PositionedError e) {
	typeError = e;
      }
      return XUtils.fetchUnaryOverloadedOperator(oper,
						 context,
						 expr,
						 getTokenReference(),
						 typeError);
    } catch (Exception e) {
      try {
	return tryExpanded(context).checkExpression(context);
      } catch (PositionedError f) {
	throw typeError;
      }
    }
  }

  // ----------------------------------------------------------------------
  // PRIVATE METHODS
  // ----------------------------------------------------------------------

  private JExpression tryExpanded(CExpressionContext context) throws PositionedError {
    // better check: hasSideEffect() !!!
    check(context, expr instanceof JLocalVariableExpression || expr instanceof JClassFieldExpression, "XXX");
    check(context, expr.getType().isReference(), "XXX");

    TokenReference	ref = getTokenReference();
    JExpression	right = new XCompoundAssignmentExpression(ref, OPE_PLUS,
							  new JCheckedExpression(ref, expr),
							  new JOrdinalLiteral(ref, oper == OPE_PREINC ? 1 : -1));

    return new JParenthesedExpression(ref, right);
  }
}
