/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XProtectedStatement.java,v 1.24 1999/10/28 14:03:13 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.*;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;

/**
 * This class represents cursor declaration in code part
 */
public class XProtectedStatement extends XStatement {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructor
   */
  public XProtectedStatement(TokenReference ref, JavaStyleComment[] comments) {
    super(ref, comments);
  }

  /**
   *
   */
  public void close (JExpression message, JExpression cont, JBlock block) {
    this.mess = mess;
    this.cont = cont;
    this.block = block;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check statement and return a pure kjc abstract tree that will be used to code generation.
   *
   * @param	context		the actual context of analyse
   * @exception	PositionedError		if the check fails
   */
  public JStatement checkAndConvert(CBodyContext context) throws PositionedError {
    TokenReference ref = getTokenReference();

    context = new XProtectedContext(context, this);

    if (cont != null) {
      cont = cont.checkExpression(new CExpressionContext(context));
      check(context, cont.getType().isAssignableTo(XStdType.DBContextHandler),
	    "x-protected-bad-context", cont.getType());
    } else {
      check(context, context.getClassContext().getCClass().descendsFrom(XStdType.DBContextHandler.getCClass()),
	    "x-protected-no-context", context.getClassContext().getCClass().getType());
      cont = new JThisExpression(ref).checkExpression(new CExpressionContext(context));
    }

    if (mess == null) {
      mess = new JNullLiteral(ref);
    }

    JStatement stmt =  new JForStatement(ref,
					 null,
					 null,
					 null,
					 XFactory.parseProtected(ref,
								 new JStatement[] {block,
										   new JBreakStatement(ref, null, null),
										   new JBreakStatement(ref, null, null)},
								 new JExpression[] {new JCheckedExpression(ref ,cont), mess}), null);
    stmt.checkStatement(context);
    context.close(getTokenReference());
    return stmt;
  }

  // ----------------------------------------------------------------------
  // UTILS
  // ----------------------------------------------------------------------

  /**
   * Get the connection from a context handler
   */
  public static JExpression buildConnection(TokenReference ref, JExpression handler) {
    JExpression		expr = new JNameExpression(ref, handler, "getDBContext");
    expr = new JMethodCallExpression(ref, expr, JExpression.EMPTY);
    expr = new JNameExpression(ref, expr, "getDefaultConnection");
    return new JMethodCallExpression(ref, expr, JExpression.EMPTY);
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param	p		the printwriter into the code is generated
   */
  public void genXKJCCode(XJavaPrettyPrinter p) {
    p.printProtectedStatement(mess, cont, block);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private JExpression	mess;
  private JExpression	cont;
  private JBlock	block;
}
