/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XReturnStatement.java,v 1.18 1999/10/28 14:03:14 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.*;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;

/**
 * JLS 14.15: Return Statement
 *
 * A return statement returns control to the invoker of a method or constructor.
 */
public class XReturnStatement extends JReturnStatement {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

 /**
   * Construct a node in the parsing tree
   * @param	where		the line of this node in the source code
   * @param	expr		the expression to return.
   */
  public XReturnStatement(TokenReference where, JExpression expr, JavaStyleComment[] comments) {
    super(where, expr, comments);
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check statement.
   *
   * @param	context		the actual context of analyse
   * @exception	PositionedError		if the check fails
   */
  public void checkStatement(CBodyContext context) throws PositionedError {
    if (expr != null) {
      expr = expr.checkExpression(new CExpressionContext(context));
      if (!expr.getType().implicitlyCastTo(context.getMethodContext().getCMethod().getReturnType())) {
	check(context, false,
	      "return-badtype", expr.getType(), context.getMethodContext().getCMethod().getReturnType());
      } else {
	expr = expr.convertType(context.getMethodContext().getCMethod().getReturnType(), new CExpressionContext(context));
      }
    } else {
      check(context, context.getMethodContext().getCMethod().getReturnType() == CStdType.Void,
	    "return-empty-nonvoid");
    }

    if (expr != null) {
      check(context, !expr.getType().equals(CStdType.Void),
	    "return-badtype", expr.getType(), context.getMethodContext().getCMethod().getReturnType());
    }

    checkProtected(context);

    context.addReturn();
  }

  // ----------------------------------------------------------------------
  // UTILITIES
  // ----------------------------------------------------------------------

  private void checkProtected(CBodyContext context) throws PositionedError {
    XProtectedContext	protectedC = getProtectedContext(context);
    check(context, protectedC == null, "x-protected-return");
    XUnprotectedContext	unprotectedC = getUnprotectedContext(context);
    check(context, unprotectedC == null, "x-protected-return");
  }

  private XProtectedContext getProtectedContext(CContext context) {
    if (context instanceof XProtectedContext) {
      return (XProtectedContext)context;
    } else if (context instanceof CBodyContext) {
      return getProtectedContext(context.getParentContext());
    } else {
      return null;
    }
  }

  private XUnprotectedContext getUnprotectedContext(CContext context) {
    if (context instanceof XUnprotectedContext) {
      return (XUnprotectedContext)context;
    } else if (context instanceof CBodyContext) {
      return getUnprotectedContext(context.getParentContext());
    } else {
      return null;
    }
  }
}
