/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XSelectStatement.java,v 1.15 1999/10/08 11:19:13 vgp Exp $
 */

package at.dms.xkjc;

import java.util.Vector;
import at.dms.kjc.JPhylum;
import at.dms.kjc.CBodyContext;
import at.dms.kjc.CJavaPrettyPrinter;
import at.dms.kjc.CBlockError;
import at.dms.sqlc.*;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * This class represents an SQL select statement
 */
public class XSelectStatement extends JPhylum {

  // ----------------------------------------------------------------------
  // CONSTRUCTOR
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param where the line of this node in the source code
   * @param p1 the modifiers
   * @param p2 a list of typed columns in database
   * @param p3 a SQL expression
    */
  public XSelectStatement(TokenReference where,
			  String p1,
			  Vector p2,
			  FromClause tableRefs,
			  WhereClause whereClause,
			  GroupByClause groupByClause,
			  HavingClause havingClause,
			  SortSpec sortSpec,
			  UpdSpec updateSpec)
  {
    super(where);
    mod = p1;
    attrs = new XTypedSelectElem[p2.size()];
    p2.copyInto(attrs);
    body = new SelectStatement(where,
			       p1,
			       p2,
			       tableRefs,
			       whereClause,
			       groupByClause,
			       havingClause,
			       sortSpec,
			       updateSpec);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   *
   */
  public XTypedSelectElem[] getTypedSelectElem() {
    return attrs;
  }

  /**
   * Returns the sql statement.
   */
  public SqlPhylum getSql() {
    return body;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * This method provide a semantics check after parsing
   * Some optimization may be done here
   * @param context the context of analysis of this node
   * @exception	PositionedError	an error with reference to the source file
   */
  public void checkStatement(CBodyContext context) throws PositionedError {
    for (int i = 0; i < attrs.length; i++) {
      for (int j = i + 1; j < attrs.length; j++) {
	if (attrs[i].getIdent().equals(attrs[j].getIdent())) {
	  throw new CBlockError(getTokenReference(), "x-duplicate-ident", attrs[i].getIdent());
	}
      }
    }
  }

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param p the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    ((XJavaPrettyPrinter)p).printXSelectStatement(mod, attrs, body);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private String		mod;
  private XTypedSelectElem[]	attrs;
  private SqlPhylum		body;
}
