/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XSqlExpr.java,v 1.20 1999/10/09 20:25:32 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.*;
import at.dms.sqlc.SqlPhylum;
import at.dms.sqlc.SqlContext;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * an sql text with java inserted
 */
public class XSqlExpr extends XExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * @param	where		the line of this node in the source code
   */
  public XSqlExpr(TokenReference where, SqlPhylum text) {
    super(where);
    this.text = text;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Compute the type of this expression (called after parsing)
   * @return the type of this expression
   */
  public CType getType() {
    return CStdType.String;
  }

  /**
   * @return true if this expression is constant
   */
  public boolean isConstant() {
    return false;
  }

  /**
   * Returns the SqlStatement
   */
  public SqlPhylum getSql() {
    return text;
  }

  /**
   *
   */
  public JExpression[] getBlobs() {
    return sql.getBlobs();
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate all, do constant folding as possible
   * Alter context and optimize code
   * @param context the actual context of analyse
   * @return  an optimized expression that do the same job than it
   * @exception	PositionedError	an error with reference to the source file
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    SqlChecker		checker = new SqlChecker(sql = new JavaSqlContext(context.getBodyContext()));

    text.accept(checker);

    return checker.toJavaExpression(getTokenReference()).checkExpression(context);
  }

  /**
   * Check expression and evaluate all, do constant folding as possible
   * Alter context and optimize code
   * @param context the actual context of analyse
   * @return  an optimized expression that do the same job than it
   * @exception	PositionedError	an error with reference to the source file
   */
  public JExpression checkExpression(SqlContext sql) throws PositionedError {
    CBodyContext	context = SqlChecker.getJavaContext(sql).getBodyContext();
    SqlChecker		checker = new SqlChecker(sql = new JavaSqlContext(context));

    text.accept(checker);

    return checker.toJavaExpression(getTokenReference()).checkExpression(new CExpressionContext(context));
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param	p		the printwriter into the code is generated
   */
  public void genXKJCCode(XJavaPrettyPrinter p) {
    p.printSQL(text);
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  private SqlPhylum		text;
  private JavaSqlContext	sql;
}
