/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XStdType.java,v 1.21 1999/10/08 11:19:13 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.CClassType;
import at.dms.kjc.CType;
import at.dms.kjc.CStdType;
import at.dms.compiler.Compiler;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * library of XType
 */
public class XStdType extends at.dms.util.Utils implements XConstants {

  // ----------------------------------------------------------------------
  // PRIMITIVE SQL TYPES
  // ----------------------------------------------------------------------

  public static CClassType	Boolean;
  public static CClassType	Byte;
  public static CClassType	Short;
  public static CClassType	Int;
  public static CClassType	Long;
  public static CClassType	Float;
  public static CClassType	Double;
  public static CClassType	Char;
  public static CClassType	Date;
  public static CClassType	Month;
  public static CClassType	Time;
  public static CClassType	Fixed;

  public static XDateType	PDate;
  public static XMonthType	PMonth;
  public static XTimeType	PTime;
  public static XFixedType	PFixed;

  public static CClassType	Number;

  // ----------------------------------------------------------------------
  // KOPI UTILS TYPES
  // ----------------------------------------------------------------------

  public static CClassType	Cursor;
  public static CClassType	Connection;
  public static CClassType	DBContextHandler;
  public static CClassType	KopiSerializable;

  // ----------------------------------------------------------------------
  // INITIALIZERS
  // ----------------------------------------------------------------------

  /**
   * Initialize all constants
   */
  public static void init(Compiler compiler) {

    Cursor = CClassType.lookup(XConstants.XKJ_CURSOR);
    Connection = CClassType.lookup(XConstants.XKJ_CONNECTION);
    DBContextHandler = CClassType.lookup(XConstants.XKJ_DBCONTEXTHANDLER);
    //ProtectedException = CClassType.lookup(XConstants.XKJ_PROTECTEDEXCEPTION);
    KopiSerializable = CClassType.lookup(XConstants.XKJ_KOPISERIALIZABLE);
    Boolean = CClassType.lookup(XConstants.JAV_BOOLEAN);
    Byte = CClassType.lookup(XConstants.JAV_BYTE);
    Short = CClassType.lookup(XConstants.JAV_SHORT);
    Int = CClassType.lookup(XConstants.JAV_INT);
    Long = CClassType.lookup(XConstants.JAV_LONG);
    Float = CClassType.lookup(XConstants.JAV_FLOAT);
    Double = CClassType.lookup(XConstants.JAV_DOUBLE);
    Char = CClassType.lookup(XConstants.JAV_CHAR);
    Date = CClassType.lookup(XConstants.XKJ_DATE);
    Fixed = CClassType.lookup(XConstants.XKJ_FIXED);
    Month = CClassType.lookup(XConstants.XKJ_MONTH);
    Time = CClassType.lookup(XConstants.XKJ_TIME);

    PDate = new XDateType();
    PFixed = new XFixedType();
    PMonth = new XMonthType();
    PTime = new XTimeType();

    Number = CClassType.lookup("java/lang/Number");

    try {
      Boolean.checkType(null);
      Byte.checkType(null);
      Short.checkType(null);
      Int.checkType(null);
      Long.checkType(null);
      Float.checkType(null);
      Double.checkType(null);
      Char.checkType(null);
      Date.checkType(null);
      Fixed.checkType(null);
      Month.checkType(null);
      Time.checkType(null);

      PDate.checkType(null);
      PFixed.checkType(null);
      PMonth.checkType(null);
      PTime.checkType(null);

      Number.checkType(null);

      Cursor.checkType(null);
      Connection.checkType(null);
      DBContextHandler.checkType(null);
      //ProtectedException.checkType(null);
      KopiSerializable.checkType(null);
    } catch (UnpositionedError cue) {
      System.err.println(cue.getError(TokenReference.NO_REF).getErrorMessage());
      compiler.reportTrouble(new PositionedError(TokenReference.NO_REF, "cant-load-xkj.jar"));
    }
  }

  // ----------------------------------------------------------------------
  // PRIMITIVE TYPE
  // ----------------------------------------------------------------------

  public static boolean isSQLPrimitiveType(CType type) {
    if (!type.isReference()) {
      return true;
    }

      if (type.equals(Boolean)) {
	return true;
      }
      if (type.equals(Byte)) {
	return true;
      }
      if (type.equals(Short)) {
	return true;
      }
      if (type.equals(Int)) {
	return true;
      }
      if (type.equals(Long)) {
	return true;
      }
      if (type.equals(Float)) {
	return true;
      }
      if (type.equals(Double)) {
	return true;
      }
      if (type.equals(Char)) {
	return true;
      }
      if (type.equals(Fixed)) {
	return true;
      }
      if (type.equals(Date)) {
	return true;
      }
      if (type.equals(Month)) {
	return true;
      }
      if (type.equals(Time)) {
	return true;
      }
      if (type.equals(PFixed)) {
	return true;
      }
      if (type.equals(PDate)) {
	return true;
      }
      if (type.equals(PMonth)) {
	return true;
      }
      if (type.equals(PTime)) {
	return true;
      }
      if (type.equals(CStdType.String)) {
	return true;
      }

      return false;
  }
}
