/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XTypedSelectElem.java,v 1.18 1999/10/28 14:03:14 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.sqlc.*;
import at.dms.kjc.*;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * This class represents a typed column in a SQL SELECT
 */
public class XTypedSelectElem extends SelectElem {

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	p1		a SQL expression (count(*))
   * @param	p2		the type of this expression or column in database
   * @param	ident		the name for this field
   * @param	pos		the position in cursor
   * @param	context		the context of definition for this context
   */
  public XTypedSelectElem(TokenReference where,
			  Expression p1,
			  CType p2,
			  String ident,
			  int pos)
  {
    super(where);

    this.ident = ident;
    if (p1 == null) {
      sqlExpr = new SimpleIdentExpression(where, ident);
      hasAs = false;
    } else {
      sqlExpr = p1;
      hasAs = true;
    }
    var = new JVariableDefinition(where, XConstants.ACC_CURSOR | Constants.ACC_PUBLIC, p2, ident.replace('.', '$'), null);
    // WE DO NOT CHECK '$' BECAUSE IT IS NOT VALID IN SQL IDENT //
  }

  /**
   *
   */
  public JVariableDefinition getVariableDefinition() {
    return var;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Returns the sql statement.
   */
  public SqlPhylum getSql() {
    return sqlExpr;
  }

  /**
   * Returns the sql ident
   */
  public String getIdent() {
    return ident;
  }

  /**
   * Generate the code in pure java form
   * It is usefull to debug and tune compilation process
   * @param p the printwriter into the code is generated
   */
  public void genJavaCode(CJavaPrettyPrinter p) {
    throw new RuntimeException();
  }

  // --------------------------------------------------------------------
  // ACCEPT VISITORS
  // --------------------------------------------------------------------

  /**
   * Accepts a visitor.
   *
   * @param	visitor			the visitor
   */
  public void accept(at.dms.sqlc.SqlVisitor visitor) throws PositionedError {
    ((SqlVisitor)visitor).visitTypedSelectElem(ident, var, sqlExpr, hasAs);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private String		ident;
  private SqlPhylum		sqlExpr;
  private JVariableDefinition	var;
  private boolean		hasAs;
}
