/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XUtils.java,v 1.41 1999/10/08 11:19:13 vgp Exp $
 */

package at.dms.xkjc;

import java.util.Vector;
import java.util.StringTokenizer;
import at.dms.kjc.*;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

public class XUtils extends at.dms.util.Utils {

  // ----------------------------------------------------------------------
  // FIXED LITERAL
  // ----------------------------------------------------------------------

  /**
   * Returns a Fixed from a string
   */
  public static JExpression fixedLiteral(TokenReference ref, String val) {
    return new JNewObjectExpression(ref, XStdType.Fixed, null, new JExpression[] {new JStringLiteral(ref, val)});
  }

  /**
   * Sets database
   */
  public static void setDatabase(CClass database) {
    if (XUtils.database == null) {
      XUtils.database = new Vector();
    }
    XUtils.database.addElement(database);
  }

  /**
   * Sets database
   */
  public static void setDatabase(String database) {
    XUtils.database = null;
    XUtils.databaseType = new Vector();
    StringTokenizer	token = new StringTokenizer(database, ": ");
    while (token.hasMoreTokens()) {
      String	current = token.nextToken();
      XUtils.databaseType.addElement(CClassType.lookup(current.replace('.', '/')));
    }
    at.dms.sqlc.XUtils.setChecker(new at.dms.sqlc.DBChecker() {
	/**
	 * Checks if table exists
	 */
	public boolean tableExists(String table) {
	  return XUtils.tableExists(table);
	}

	/**
	 * Checks if a column exists in a table
	 */
	public boolean columnExists(String table, String column) {
	  return XUtils.columnExists(table, column);
	}
      });
  }

  // ----------------------------------------------------------------------
  // OVERLOADING CHECK
  // ----------------------------------------------------------------------

  /**
   * Returns a method that overload an operator or null if there is no method
   *
   * @param	operator	the id of the operator
   * @param	context		the context to look at
   * @param	left		the left expression
   * @param	right		the right expression
   * @param	ref		the location in source code
   * @param	typeError	the type error between the two operands
   * @exception	PositionedError the typeError is rethrown if there is no method
   */
  public static JExpression fetchBinaryOverloadedOperator(int operator,
							  CExpressionContext context,
							  JExpression left,
							  JExpression right,
							  TokenReference ref,
							  PositionedError typeError)
    throws PositionedError
  {
    JExpression ret = fetchOverloadedOperator(context, buildBinary(ref, operator, left, right));

    if (ret == null) {
      throw typeError;
    } else {
      return ret;
    }
  }

  /**
   * Returns a method that overload an operator or null if there is no method
   *
   * @param	operator	the id of the operator
   * @param	context		the context to look at
   * @param	expr		the left expression
   * @param	ref		the location in source code
   * @param	typeError	the type error between the two operands
   * @exception	PositionedError the typeError is rethrow if there is
   *		no method
   */
  public static JExpression fetchUnaryOverloadedOperator(int operator,
							 CExpressionContext context,
							 JExpression expr,
							 TokenReference ref,
							 PositionedError typeError)
    throws PositionedError
  {
    JExpression ret = fetchOverloadedOperator(context,  buildUnary(ref, operator, expr));
    if (ret == null) {
      throw typeError;
    } else {
      return ret;
    }
  }

  // ----------------------------------------------------------------------
  // DATABASE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Returns the database type of a Table/column
   */
  public static CType getDatabaseType(String table, String column) {
    if (checkDatabase()) {
      CField	field;
      for (int i = 0; i < database.size(); i++) {
	if ((field = ((CClass)database.elementAt(i)).getField(table + "_" + column)) != null) {
	  return field.getType();
	}
      }
      return null;
    } else {
      return null;
    }
  }

  /**
   * Returns the database type of a Table/column
   */
  public static boolean tableExists(String table) {
    if (checkDatabase()) {
      for (int i = 0; i < database.size(); i++) {
	if (((CClass)database.elementAt(i)).getField(table) != null) {
	  return true;
	}
      }
      return false;
    } else {
      return true;
    }
  }

  /**
   * Returns the database type of a Table/column
   */
  public static boolean columnExists(String table, String column) {
    if (checkDatabase()) {
      for (int i = 0; i <database.size(); i++) {
	if (((CClass)database.elementAt(i)).getField(table + "_" + column) != null) {
	  return true;
	}
      }
      return false;
    } else {
      return true;
    }
  }

  /**
   * Returns the database type of a Table/column
   */
  public static boolean compareDatabaseType(CType expected, String table, String column, boolean lazy) {
    if (checkDatabase()) {
      CType	type = getDatabaseType(table, column);
      if (type == null) {
	return false;
      } else {
	return implicitlyCastTo(type, expected, lazy);
      }
    } else {
      return true;
    }
  }

  /**
   * Returns the database type of a Table/column
   */
  public static void checkDatabaseType(CType expected, String table, String column)
    throws UnpositionedError
  {
    checkDatabaseType(expected, table, column, false);
  }

  /**
   * Returns the database type of a Table/column
   */
  public static void checkDatabaseType(CType expected, String table, String column, boolean lazy)
    throws UnpositionedError
  {
    if (checkDatabase() && !compareDatabaseType(expected, table, column, lazy)) {
      // Find error
      if (!tableExists(table)) {
	throw new UnpositionedError("x-undefined-table", table);
      } else if (!columnExists(table, column)) {
	throw new UnpositionedError("x-undefined-column", column, table);
      } else {
	throw new UnpositionedError("x-bad-database-type",
				    getDatabaseType(table, column) + " " + table + "." + column,
				    expected);
      }
    }
  }

  /**
   * Returns the database type of a Table/column
   */
  public static void checkColumnExists(String table, String column)
    throws UnpositionedError
  {
    if (checkDatabase() && getDatabaseType(table, column) == null) {
      if (!tableExists(table)) {
	throw new UnpositionedError("x-undefined-table", table, null);
      } else {
	throw new UnpositionedError("x-undefined-column", column, table);
      }
    }
  }

  // ----------------------------------------------------------------------
  // PRIVATE METHODS
  // ----------------------------------------------------------------------

  /**
   *
   */
  private static boolean implicitlyCastTo(CType from, CType to, boolean lazy) {
    if (from.isClassType()) {
      if (from.equals(to)) {
	return true;
      }

      if (lazy) {
	if (from.equals(XStdType.Byte)) {
	  if (to == CStdType.Byte || to.equals(XStdType.Byte)) {
	    return true;
	  }
	  if (to == CStdType.Short ||
	      to == CStdType.Integer ||
	      to.equals(XStdType.Short) ||
	      to.equals(XStdType.Int)) {
	    return true;
	  }
	} else if (from.equals(XStdType.Short)) {
	  if (to == CStdType.Short ||
	      to == CStdType.Integer ||
	      to.equals(XStdType.Short) ||
	      to.equals(XStdType.Int)) {
	    return true;
	  }
	}
      }

      if (from.equals(XStdType.PDate) && to.equals(XStdType.Date)) {
	return true;
      }
      if (from.equals(XStdType.PFixed) && to.equals(XStdType.Fixed)) {
	return true;
      }
      if (from.equals(XStdType.PMonth) && to.equals(XStdType.Month)) {
	return true;
      }
      if (from.equals(XStdType.PTime) && to.equals(XStdType.Time)) {
	return true;
      }

      return from.equals(CStdType.Object); // !!! TO BE REMOVED
    } else {
      if (from == CStdType.Boolean) {
	return to == CStdType.Boolean || to == XStdType.Boolean;
      }
      if (from == CStdType.Byte) {
	if (to == CStdType.Byte || to == XStdType.Byte) {
	  return true;
	}
	if (lazy && (to == CStdType.Short ||
		     to == CStdType.Integer ||
		     to.equals(XStdType.Short) ||
		     to.equals(XStdType.Int))) {
	  return true;
	}
      }
      if (from == CStdType.Short) {
	if (to == CStdType.Short || to.equals(XStdType.Short)) {
	  return true;
	}
	if (lazy && (to == CStdType.Integer || to.equals(XStdType.Int))) {
	  return true;
	}
      }
      return to == CStdType.Integer || to.equals(XStdType.Int);
    }
  }

  /**
   *
   */
  private static boolean checkDatabase() {
    if (database != null) {
      return true;
    }
    if (databaseType != null && databaseType.size() != 0) {
      for (int i = 0; i < databaseType.size(); i++) {
	setDatabase(((CClassType)databaseType.elementAt(i)).getCClass());
      }
      return true;
    }
    return false;
  }

  // ----------------------------------------------------------------------
  // INITIALIZATION
  // ----------------------------------------------------------------------

  /**
   *
   */
  public static void initialize(String xkjc_path, boolean overloadEqual) {
    XUtils.overloadEqual = overloadEqual;

    if (xkjc_path != null) {
      at.dms.classfile.ClassPath.parseClassPath(xkjc_path);
    }

    if (xkjc_path == null) {
      initialize(new String[] {
	"at/dms/xoper/XFixed",
	  "at/dms/xoper/XByte",
	  "at/dms/xoper/XShort",
	  "at/dms/xoper/XInteger",
	  "at/dms/xoper/XFloat",
	  "at/dms/xoper/XDouble",
	  "at/dms/xoper/XBoolean",
	  "at/dms/xoper/XDate",
	  "at/dms/xoper/XTime",
	  "at/dms/xoper/XMonth",
	  "at/dms/xoper/XCharacter",
	  "at/dms/xoper/XString"
	  });
    } else {
      // !!! USE USER PATH
      prefixs = new JExpression[0];
    }
  }

  /**
   *
   */
  private static void initialize(String[] xkjc_path) {
    Vector		collect = new Vector();

    try {
      for (int i = 0; i < xkjc_path.length; i++) {
	JTypeNameExpression expr = new JTypeNameExpression(TokenReference.NO_REF, xkjc_path[i]);
	collect.addElement(expr.checkExpression(null));
      }
    } catch (PositionedError e) {
      e.printStackTrace();
      System.err.println(e.getErrorMessage());
    }

    collect.copyInto(prefixs = new JExpression[collect.size()]);
  }

  /**
   * Returns a method that overload an operator or null if there is no method
   *
   * @param	context		the context to look at
   * @param	overload	the method call
   */
  public static JExpression fetchOverloadedOperator(CExpressionContext context,
						    XOverloadedMethodCallExpression overload) {
    JExpression	ret = null;
    boolean	found = true;

    try {
      overload.preCheckExpression(context);
    } catch (PositionedError error) {
      return null; // !!!
    }

    try {
      ret = overload.checkExpression(context);
    } catch (PositionedError error) {
      found = false;
    }

    for (int i = 0; !found && i < prefixs.length; i++) {
      try {
	found = true;
	overload.setPrefix(prefixs[i]);
	ret = overload.checkExpression(context);
      } catch (PositionedError error) {
	found = false;
      }
    }

    return found ? ret : null;
  }

  // ----------------------------------------------------------------------
  // PRIVATE METHODS
  // ----------------------------------------------------------------------

  /**
   * buildUnary
   */
  private static XOverloadedMethodCallExpression buildUnary(TokenReference ref,
							    int operator,
							    JExpression expr) {
    JExpression		prefix = new JNameExpression(ref, ("operator$" + operator).intern());

    return new XOverloadedMethodCallExpression(ref, prefix, new JExpression[] {expr});
  }

  /**
   * buildBinary
   */
  private static XOverloadedMethodCallExpression buildBinary(TokenReference ref,
							     int operator,
							     JExpression left,
							     JExpression right) {
    JExpression		prefix = new JNameExpression(ref, ("operator$" + operator).intern());

    return new XOverloadedMethodCallExpression(ref, prefix, new JExpression[] {left, right});
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private static JExpression[]	prefixs;

  private static Vector		database;
  private static Vector		databaseType;
  public  static boolean	overloadEqual;
}
