/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XVariableDefinition.java,v 1.16 1999/10/08 11:19:13 vgp Exp $
 */

package at.dms.xkjc;

import at.dms.kjc.*;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * This class represents a local variable definition in the syntax tree
 */
public class XVariableDefinition extends JVariableDefinition {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	ident		the name of this variable
   * @param	initializer	the initializer
   */
  public XVariableDefinition(TokenReference where,
			     int modifiers,
			     CType type,
			     String ident,
			     JExpression initializer) {
    super(where, modifiers, type, ident, initializer);
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param	context			the actual context of analyse
   * @exception	PositionedError Error catched as soon as possible
   */
  public void checkBody(CBodyContext context) throws PositionedError {
    try {
      type.checkType(context);

      if (expr != null) {
	// special case for array initializers
	if (expr instanceof JArrayInitializer) {
	  ((JArrayInitializer)expr).setType(type);
	}

	expr = expr.checkExpression(new CExpressionContext(context));

	//	System.err.println(expr.getType() + " / " + type);

	if (expr.getType().isAssignableTo(type)) {
	  expr = expr.convertType(type, new CExpressionContext(context));
	} else if (! (expr instanceof JArrayInitializer)){
	  JExpression	prefix = new JNameExpression(getTokenReference(),
						     "operator$" + XConstants.OPE_SIMPLE);
	  JExpression	left = null;

	  if (getType().isReference()) {
	    left = new JCastExpression(getTokenReference(),
				       new JNullLiteral(getTokenReference()),
				       getType());
	  } else if (getType().isNumeric()) {
	    left = new JOrdinalLiteral(getTokenReference(), 0);
	  } else if (getType() == CStdType.Boolean) {
	    left = new JBooleanLiteral(getTokenReference(), true);
	  } else {
	    check(context, false, "assignment-badtype", getType(), getType());
	  }

	  XOverloadedMethodCallExpression	overload = new XOverloadedMethodCallExpression(getTokenReference(), prefix, new JExpression[] {left, expr});

	  try {
	    JExpression	convert = XUtils.fetchOverloadedOperator(new CExpressionContext(context), overload);

	    if (convert == null) {
	      check(context, convert != null, "var-init-badtype", getIdent(), expr.getType());
	    } else {
	      JExpression ret = convert.checkExpression(new CExpressionContext(context));
	      expr = ret;
	    }
	  } catch (PositionedError ce) {
	    check(context, false, "var-init-badtype", getIdent(), expr.getType());
	  }
        } else {
	  check(context, false, "var-init-badtype", getIdent(), expr.getType());
	}
      }
    } catch (UnpositionedError cue) {
      throw cue.getError(getTokenReference());
    }
  }
}
