header { package at.dms.xkjc; }{
  import java.util.Vector;

  import at.dms.kjc.*;
  import at.dms.compiler.Compiler;
  import at.dms.compiler.CWarning;
  import at.dms.compiler.PositionedError;
  import at.dms.compiler.TokenReference;
  import at.dms.compiler.JavaStyleComment;
  import at.dms.util.VectorCache;
}class XJavaParser extends Parser;

options {
	k=2;
	importVocab=XJava;
	exportVocab=XJava;
	codeGenMakeSwitchThreshold=2;
	codeGenBitsetTestThreshold=3;
	defaultErrorHandler=false;
	superClass="at.dms.kjc.Parser";
}

{
  public XJavaParser(Compiler compiler, XParseManager mgr, CHandLexer lexer, String file) {
    super(compiler, lexer, file, MAX_LOOKAHEAD);
    this.mgr = mgr;
    this.isKopiFile = file.endsWith(".k");
  }

  private XParseManager		mgr;
  private boolean		isKopiFile;
}
jDefaultImport[CParseCompilationUnitContext context] :{
    context.addPackageImport(CParseCompilationUnitContext.JAVA_LANG);
    context.addPackageImport(new JPackageImport(TokenReference.NO_REF, "at/dms/xlib", null));
    context.addPackageImport(new JPackageImport(TokenReference.NO_REF, "at/dms/xtype", null));
  }
;

jMethodDeclaration[CParseClassContext context, int mods, CType type] {
  JFormalParameter[]	parameters;
  CClassType[]		throwsList = CClassType.EMPTY;
  int			bounds = 0;
  JBlock		block = null;
  int			oper = -1;
  TokenReference	sourceRef = buildTokenReference();
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
}
:ident:IDENT ( COLON oper = xOverloadableOperator[] )?
  parameters = jFormalParameters[JLocalVariable.DES_PARAMETER]
  ( LBRACK RBRACK { bounds += 1; } )*
  ( "throws" throwsList = jNameList[] )?
  ( block = jBlock[] | SEMI )
    {
      if (bounds > 0) {
	reportTrouble(new CWarning(sourceRef, "old-style-array-bounds", null));
	type = new CArrayType(type, bounds);
      }

      if (oper == -1) {
        context.addMethodDeclaration(new JMethodDeclaration(sourceRef,
							    mods,
							    type,
							    ident.getText(),
							    parameters,
							    throwsList,
							    block,
							    javadoc,
							    comments));
      } else {
        context.addMethodDeclaration(new XOperatorDeclaration(sourceRef,
							      mods,
							      type,
							      ident.getText(),
							      oper,
							      parameters,
							      throwsList,
							      block,
							      false));
      }
    }
;

jInnerTypeDeclaration[CParseClassContext context, int mods] {
  JTypeDeclaration	inner = null;
}
:(
  inner = jClassDeclaration[mods]
|
  inner = jInterfaceDeclaration[mods]
|
  inner = xCursorDeclaration[mods]
)
  { context.addInnerDeclaration(inner); }
;

jAdditiveExpression[] returns [JExpression self = null]{
  int			oper = Constants.OPE_PLUS;
  JExpression		right;
}
:self = jMultiplicativeExpression[]
  (
    (
      PLUS { oper = Constants.OPE_PLUS; }
    |
      MINUS { oper = Constants.OPE_MINUS; }
    )
    right = jMultiplicativeExpression[]
      {
	if (oper == Constants.OPE_PLUS) {
	  self = new XAddExpression(self.getTokenReference(), self, right);
	} else {
	  //if (oper == Constants.OPE_MINUS)
	  self = new XMinusExpression(self.getTokenReference(), self, right);
	}
      }
  )*
;

jEqualityExpression[] returns [JExpression self = null]{
  int			operator = -1;
  JExpression		right;
}
:self = jInstanceOfExpression[]
  (
    (
      EQUAL { operator = Constants.OPE_EQ; }
    |
      NOT_EQUAL { operator = Constants.OPE_NE; }
    )
    right = jInstanceOfExpression[]
      { self = new XEqualityExpression(self.getTokenReference(), operator, self, right); }
  )*
;

jUnaryExpressionNotPlusMinus[] returns [JExpression self = null]{
  int			operator = -1;
  TokenReference	sourceRef = buildTokenReference();
}
:(
    BNOT { operator = Constants.OPE_BNOT; }
  |
    LNOT { operator = Constants.OPE_LNOT; }
  )
  self = jUnaryExpression[]
    { self = new XUnaryExpression(sourceRef, operator, self); }
|
  ( jCastLookahead[] ) =>
  self = jCastExpression[]
|
  self = jPostfixExpression[]
;

jMultiplicativeExpression[] returns [JExpression self = null]{
  int			operator = -1;
  JExpression		right;
}
:self = jUnaryExpression[]
  (
    (
      STAR { operator = 1; }
    |
      SLASH { operator = 2; }
    |
      PERCENT { operator = 3; }
    )
    right = jUnaryExpression[]
      {
	if (operator == 1) {
          self = new XMultExpression(self.getTokenReference(), self, right);
	} else if (operator == 2) {
          self = new XDivideExpression(self.getTokenReference(), self, right);
	} else {
          self = new XModuloExpression(self.getTokenReference(), self, right);
	}
      }
  )*
;

jPostfixExpression[] returns [JExpression self = null]{
  TokenReference	sourceRef = buildTokenReference();
}
:self = jPrimaryExpression[]
  (
    INC
      { self = new XPostfixExpression(sourceRef, Constants.OPE_POSTINC, self); }
  |
    DEC
      { self = new XPostfixExpression(sourceRef, Constants.OPE_POSTDEC, self); }
  )?
;

jPreIncrementExpression[] returns [JExpression self = null]{
  TokenReference	sourceRef = buildTokenReference();
}
:INC self = jPrimaryExpression[]
    { self = new XPrefixExpression(sourceRef, Constants.OPE_PREINC, self); }
;

jPreDecrementExpression[] returns [JExpression self = null]{
  TokenReference	sourceRef = buildTokenReference();
}
:DEC self = jPrimaryExpression[]
    { self = new XPrefixExpression(sourceRef, Constants.OPE_PREDEC, self); }
;

jStatementExpression[] returns [JExpression self = null]{
  int			operator = -1;
  JExpression		right;
  TokenReference	sourceRef = buildTokenReference();
}
:self = jPreIncrementExpression[]
|
  self = jPreDecrementExpression[]
|
  self = jPrimaryExpression[]
  (
    INC
      { self = new XPostfixExpression(sourceRef, Constants.OPE_POSTINC, self); }
  |
    DEC
      { self = new XPostfixExpression(sourceRef, Constants.OPE_POSTDEC, self); }
  |
    operator = jAssignmentOperator[] right = jExpression[]
      { self = XAssignmentExpression.create(self.getTokenReference(), operator, self, right); }
  )?
;

jShiftExpression[] returns [JExpression self = null]{
  int			operator = -1;
  JExpression		right;
}
:self = jAdditiveExpression[]
  (
    (
      SL { operator = Constants.OPE_SL; }
    |
      SR { operator = Constants.OPE_SR; }
    |
      BSR { operator = Constants.OPE_BSR; }
    )
    right = jAdditiveExpression[]
      { self = new XShiftExpression(self.getTokenReference(), operator, self, right); }
  )*
;

jRelationalExpression[] returns [JExpression self = null]{
  int			operator = -1;
  JExpression		right;
}
:self = jShiftExpression[]
  (
    (
      LT { operator = Constants.OPE_LT; }
    |
      GT { operator = Constants.OPE_GT; }
    |
      LE { operator = Constants.OPE_LE; }
    |
      GE { operator = Constants.OPE_GE; }
    )
    right = jShiftExpression[]
      { self = new XRelationalExpression(self.getTokenReference(), operator, self, right); }
  )*
;

jPrimaryPrefix[] returns [JExpression self = null]{
  JExpression		expr = null;
  CType			type = null;
  TokenReference	sourceRef = buildTokenReference();
}
:self = jLiteral[]
|
  "this" { self = new JThisExpression(sourceRef ); }
|
  "super" DOT id:IDENT
    { self = new XNameExpression(sourceRef,
				 new JSuperExpression(sourceRef ),
				 id.getText()); }
|
  LPAREN expr = jExpression[] RPAREN
    { self = new JParenthesedExpression(sourceRef, expr); }
|
  self = jAllocationExpression[null]
|
  ( jType[] DOT "class" ) =>
    type = jType[] DOT "class"
      { self = new JClassExpression(sourceRef, type); }
|
  id2:IDENT
    (
      options { warnWhenFollowAmbig = false; } :     
      DOT "class"
       { self = new JClassExpression(buildTokenReference(), CClassType.lookup(id2.getText())); }
    )?
   { self = self == null ? new XNameExpression(buildTokenReference(), id2.getText()) : self; }
|
    type = xPrimitiveType[]
    (
      options { warnWhenFollowAmbig = false; } :     
      DOT "class"
       { self = new JClassExpression(buildTokenReference(), type); }
    )?
   { self = self == null ? new XNameExpression(buildTokenReference(), ((CClassType)type).getQualifiedName().replace('/', '.'), true) : self; }
|
    type = jPrimitiveType[] DOT "class"
      { self = new JClassExpression(buildTokenReference(), type); }
;

jPrimarySuffix[JExpression prefix] returns [JExpression self = null]{
  JExpression		expr = null;
  JExpression[]	args;
  TokenReference	sourceRef = buildTokenReference();
}
:( DOT "this" ) =>
  DOT "this" { self = new JThisExpression(sourceRef, prefix); }
|
  DOT self = jAllocationExpression[prefix]
|
  LBRACK expr = jExpression[] RBRACK
    { self = new JArrayAccessExpression(sourceRef, prefix, expr); }
|
  DOT id1:IDENT
    { self = new XNameExpression(sourceRef, prefix, id1.getText()); }
|
  args = jArguments[]
    { self = new XMethodCallExpression(sourceRef, prefix, args); }
;

xPrimitiveType[] returns [CType self = null]:"fixed" { self = XStdType.PFixed; }
|
  "month" { self = XStdType.PMonth; }
|
  "date" { self = XStdType.PDate; }
|
  "time" { self = XStdType.PTime; }
;

jPrimitiveType[] returns [CType self = null]:self = xPrimitiveType[]
|
  self = javaPrimitiveType[]
;

javaPrimitiveType[] returns [CType self = null]:"boolean" { self = CStdType.Boolean; }
|
  "byte" { self = CStdType.Byte; }
|
  "char" { self = CStdType.Char; }
|
  "double" { self = CStdType.Double; }
|
  "float" { self = CStdType.Float; }
|
  "int" { self = CStdType.Integer; }
|
  "long" { self = CStdType.Long; }
|
  "short" { self = CStdType.Short; }
|
  "void" { self = CStdType.Void; }
;

jAllocationExpression[JExpression expr] returns [JExpression self = null]{
  CType			type = null;
  JClassDeclaration	decl = null;
  JArrayDimsAndInits	arr = null;
  String		name = null;
  JExpression[]		args = JExpression.EMPTY;
  TokenReference	sourceRef = buildTokenReference();
}
:"new"
  (
    (javaPrimitiveType[] ) =>
    type = jPrimitiveType[] arr = jArrayDimsAndInits[]
      { self = new JNewArrayExpression(sourceRef, type, arr); }
   |
    (xPrimitiveType[] ) =>
    type = xPrimitiveType[]
    (
      arr = jArrayDimsAndInits[]
        { self = new JNewArrayExpression(sourceRef, (CClassType)type, arr); }
    |
      args = jArguments[]
      (
        decl = jInnerClassBody[name] 
        { self = new JNewAnonymousClassExpression(sourceRef, (CClassType)type, expr, args, decl); }        
      |
        { self = new JNewObjectExpression(sourceRef, (CClassType)type, expr, args); }        
      )
    )
  |
    name = jName[]
    (
      arr = jArrayDimsAndInits[]
        { self = new JNewArrayExpression(sourceRef, CClassType.lookup(name), arr); }
    |
      args = jArguments[]
      (
        decl = jInnerClassBody[name] 
        { self = new JNewAnonymousClassExpression(sourceRef, CClassType.lookup(name), expr, args, decl); }        
      |
        { self = new JNewObjectExpression(sourceRef, CClassType.lookup(name), expr, args); }        
      )
    )
  )
;

xOverloadableOperator[] returns [int self = -1]:EQUAL { self = at.dms.kjc.Constants.OPE_EQ; }
|
  LNOT { self = at.dms.kjc.Constants.OPE_LNOT; }
|
  BNOT { self = at.dms.kjc.Constants.OPE_BNOT; }
|
  NOT_EQUAL { self = at.dms.kjc.Constants.OPE_NE; }
|
  SLASH { self = at.dms.kjc.Constants.OPE_SLASH; }
|
  PLUS { self = at.dms.kjc.Constants.OPE_PLUS; }
|
  MINUS { self = at.dms.kjc.Constants.OPE_MINUS; }
|
  STAR { self = at.dms.kjc.Constants.OPE_STAR; }
|
  PERCENT { self = at.dms.kjc.Constants.OPE_PERCENT; }
|
  SR { self = at.dms.kjc.Constants.OPE_SR; }
|
  BSR { self = at.dms.kjc.Constants.OPE_BSR; }
|
  GE { self = at.dms.kjc.Constants.OPE_GE; }
|
  GT { self = at.dms.kjc.Constants.OPE_GT; }
|
  SL { self = at.dms.kjc.Constants.OPE_SL; }
|
  LE { self = at.dms.kjc.Constants.OPE_LE; }
|
  LT { self = at.dms.kjc.Constants.OPE_LT; }
|
  BXOR { self = at.dms.kjc.Constants.OPE_BXOR; }
|
  BOR { self = at.dms.kjc.Constants.OPE_BOR; }
|
  BAND { self = at.dms.kjc.Constants.OPE_BAND; }
|
  LPAREN RPAREN { self = at.dms.xkjc.XConstants.OPE_CAST; }
|
  ASSIGN { self = at.dms.xkjc.XConstants.OPE_SIMPLE; }
|
  INC STAR { self = at.dms.kjc.Constants.OPE_PREINC; }
|
  DEC STAR { self = at.dms.kjc.Constants.OPE_PREDEC; }
|
  STAR
  (
    INC { self = at.dms.kjc.Constants.OPE_POSTINC; }
  |
    DEC { self = at.dms.kjc.Constants.OPE_POSTDEC; }
  )
;

jInclusiveOrExpression[] returns [JExpression self = null]{
  JExpression		right;
}
:self = jExclusiveOrExpression[]
  (
    BOR right = jExclusiveOrExpression[]
      { self = new XBitwiseExpression(self.getTokenReference(), Constants.OPE_BOR, self, right); }
  )*
;

jExclusiveOrExpression[] returns [JExpression self = null]{
  JExpression		right;
}
:self = jAndExpression[]
  (
    BXOR right = jAndExpression[]
      { self = new XBitwiseExpression(self.getTokenReference(), Constants.OPE_BXOR, self, right); }
  )*
;

jAndExpression[] returns [JExpression self = null]{
  JExpression		right;
}
:self = jEqualityExpression[]
  (
    BAND right = jEqualityExpression[]
      { self = new XBitwiseExpression(self.getTokenReference(), Constants.OPE_BAND, self, right); }
  )*
;

jCastExpression[] returns [JCastExpression self = null]{
  JExpression		expr;
  CType			dest;
  TokenReference	sourceRef = buildTokenReference();
}
:( LPAREN jPrimitiveType[] ) =>
  LPAREN dest = jType[] RPAREN expr = jUnaryExpression[]
    { self = new XCastExpression(sourceRef, expr, dest); }
|
  LPAREN dest = jType[] RPAREN expr = jUnaryExpressionNotPlusMinus[]
    { self = new XCastExpression(sourceRef, expr, dest); }
;

jUnaryExpression[] returns [JExpression self = null]{
  int			operator = -1;
  TokenReference	sourceRef = buildTokenReference();
}
:(
    PLUS { operator = Constants.OPE_PLUS; }
  |
    MINUS { operator = Constants.OPE_MINUS; }
  )
  self = jUnaryExpression[]
    { self = new XUnaryExpression(sourceRef, operator, self); }
|
  self = jPreIncrementExpression[]
|
  self = jPreDecrementExpression[]
|
  self = jUnaryExpressionNotPlusMinus[]
;

jExpression[] returns [JExpression self = null]{
  int			operator = -1;
  JExpression		right;
}
:self = jConditionalExpression[]
   (
     operator = jAssignmentOperator[] right = jExpression[]
       { self = XAssignmentExpression.create(self.getTokenReference(), operator, self, right); }
   )?
;

jVariableDeclarator[int mods, CType type] returns [JVariableDefinition self = null]{
  int			bounds = 0;
  JExpression		expr = null;
  TokenReference	sourceRef = buildTokenReference();
}
:ident:IDENT
  ( LBRACK RBRACK { bounds += 1; } )*
  ( ASSIGN expr = jVariableInitializer[] )?
    {
      if (bounds > 0) {
	reportTrouble(new CWarning(sourceRef, "old-style-array-bounds", null));
	type = new CArrayType(type, bounds);
      }

      self = new XVariableDefinition(sourceRef, mods, type, ident.getText(), expr);
    }
;

jBlockStatement[] returns [JStatement self = null]{
  JTypeDeclaration type;
}
:"final" self = jLocalVariableDeclaration[at.dms.kjc.Constants.ACC_FINAL] SEMI
|
  ( jType[] IDENT ) =>
  self = jLocalVariableDeclaration[0] SEMI
|
  self = jStatement[]
    {
      if (self instanceof JEmptyStatement) {
	reportTrouble(new CWarning(self.getTokenReference(), "stray-semicolon", null));
      }
    }
|
  (
    type = jClassDeclaration[0]
  |
    type = jInterfaceDeclaration[0]
  |
    type = xCursorDeclaration[0]
  )
    { self = new JTypeDeclarationStatement(type.getTokenReference(), type); }
|
  self = xAnonymousCursorDeclaration[]
;

jStatement[] returns [JStatement self = null]:self = jEmptyStatement[]
|
  self = jLabeledStatement[]
|
  self = jBlock[]
|
  self = jExpressionStatement[]
|
  self = jSwitchStatement[]
|
  self = jIfStatement[]
|
  self = jWhileStatement[]
|
  self = jDoStatement[]
|
  self = jForStatement[]
|
  self = jBreakStatement[]
|
  self = jContinueStatement[]
|
  self = jReturnStatement[]
|
  self = jThrowStatement[]
|
  self = jSynchronizedStatement[]
|
  self = jTryStatement[]
|
  self = xAssertStatement[]
|
  self = xFailStatement[]
|
  self = xExecSqlStatement[]
|
  self = xProtectedStatement[]
|
  self = xUnprotectedStatement[]
;

jReturnStatement[] returns [JReturnStatement self = null]{
  JExpression		expr = null;
  TokenReference	sourceRef = buildTokenReference();
}
:"return" ( expr = jExpression[] )? SEMI
    { self = new XReturnStatement(sourceRef, expr, getStatementComment()); }
;

jLiteral[] returns [JExpression self = null]{
  TokenReference	sourceRef = buildTokenReference();
}
:i:INTEGER_LITERAL
    { self = new JOrdinalLiteral(sourceRef, i.getText()); }
|
  r:REAL_LITERAL
    {
      if (isKopiFile) {
        self = new XFixedLiteral(sourceRef, r.getText());
      } else {
        self = new JRealLiteral(sourceRef, r.getText());
      }
    }
|
  c:CHARACTER_LITERAL
    { self = new JCharLiteral(sourceRef, c.getText()); }
|
  s:STRING_LITERAL
   { self = new JStringLiteral(sourceRef, s.getText()); }
|
  "true"
    { self = new JBooleanLiteral(sourceRef, true); }
|
  "false"
    { self = new JBooleanLiteral(sourceRef, false); }
|
  "null"
    { self = new JNullLiteral(sourceRef ); }
;

xFailStatement[] returns [XFailStatement self = null]{
  JExpression		expr = null;
  TokenReference	sourceRef = buildTokenReference();
  JavaStyleComment[]	comments = getStatementComment();
}
:FAIL LPAREN ( expr = jExpression[] )? RPAREN SEMI
    { self = new XFailStatement(sourceRef, expr, comments); }
;

xAssertStatement[] returns [XAssertStatement self = null]{
  JExpression		cond;
  JExpression		expr = null;
  TokenReference	sourceRef = buildTokenReference();
  JavaStyleComment[]	comments = getStatementComment();
}
:ASSERT LPAREN cond = jExpression[] ( COMMA expr = jExpression[] )? RPAREN SEMI
    { self = new XAssertStatement(sourceRef, cond, expr, comments); }
;

jTypeDeclaration[CParseCompilationUnitContext context] {
  int			mods = 0;
  JTypeDeclaration	decl = null;
  TokenReference	sourceRef = buildTokenReference();
}
:(
    mods = jModifiers[]
    (
      decl = jClassDeclaration[mods]
    |
      decl = jInterfaceDeclaration[mods]
    |
      decl = xCursorDeclaration[mods]
    )
      { context.addTypeDeclaration(decl); }
  )
|
  SEMI
    { reportTrouble(new CWarning(sourceRef, "stray-semicolon", null)); }
;

jPrimaryExpression[] returns [JExpression self = null]:( self = jPrimaryPrefix[] ( self = jPrimarySuffix[self] )* )
  |
  self = xExecSqlExpression[]
;

xCursorDeclaration[int mods] returns [XCursorDeclaration self = null]{
  CParseClassContext	context = CParseClassContext.getInstance();
  TokenReference	sourceRef = buildTokenReference();
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
  XSelectStatement	sel = null;
  JFormalParameter[]	parameters = null;
}
:CURSOR_SQL ident:IDENT
  (
    xExplicitCursorBody[context]
  |
    parameters = jFormalParameters[JLocalVariable.DES_PARAMETER]
    sel = xImplicitCursorBody[context]
    {
      XCursorDeclaration.addImplicitSelect(context, sel, parameters);
    }
  )
   { 
     self = new XCursorDeclaration(sourceRef,
				   mods,
				   ident.getText(),
				   context.getFields(),
				   context.getMethods(),
				   context.getInnerClasses(),
				   context.getBody(),
	                           sel,
	                           parameters,
				   javadoc,
				   comments);
     context.release();
   }
;

xAnonymousCursorDeclaration[] returns [XLocalCursorDeclaration self = null]{
  CParseClassContext	context = CParseClassContext.getInstance();
  XCursorDeclaration	curs = null;
  JExpression		ctxt = null;
  CClassType		type = null;
  TokenReference	sourceRef = buildTokenReference();
  XSelectStatement	sel = null;
  JFormalParameter[]	parameters = null;
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
}
:CURSOR_SQL
  ( ctxt = xOptionalContext[] )?
  (
    xExplicitCursorBody[context]
  |
    parameters = jFormalParameters[JLocalVariable.DES_PARAMETER]
    sel = xImplicitCursorBody[context]
    {
      XCursorDeclaration.addImplicitSelect(context, sel, parameters);
    }
  )
     { curs = new XCursorDeclaration(sourceRef,
				     XConstants.ACC_FINAL,
				     null,
				     context.getFields(),
				     context.getMethods(),
				     context.getInnerClasses(),
				     context.getBody(),
				     sel,
			             parameters,	
				     javadoc,
				     comments);
       context.release();

       self = new XLocalCursorDeclaration(sourceRef,
					  curs,
					  comments);
       { self.setConnection(ctxt); } // !!!
     }
    // !!! graf 990918 type not used !!!
    { type = CClassType.lookup(curs.getIdent()); }
  ident:IDENT
    { self.addDeclaration(ident.getText()); }
  ( COMMA
    ident2:IDENT
    { self.addDeclaration(ident2.getText()); }
  )*
  SEMI
;

xImplicitCursorBody[CParseClassContext context] returns [XSelectStatement sel = null]:LCURLY
    {
      sel = mgr.getSQLParser().xSelectStatement();
    }
  //RCURLY
;

jFormalParameter[int desc] returns [JFormalParameter self = null]{
  boolean		isFinal = false;
  int			bounds = 0;
  CType			type;
  String		name = null;
  TokenReference	sourceRef = buildTokenReference();
}
:( "final" { isFinal = true; } )?
  type = jType[]
  ( ident:IDENT { name = ident.getText();} )?
  ( LBRACK RBRACK { bounds += 1; } )*
    {
      if (bounds > 0) {
	reportTrouble(new CWarning(sourceRef, "old-style-array-bounds", null));
	type = new CArrayType(type, bounds);
      }

      self = new XFormalParameter(sourceRef, desc, type, name, isFinal);
    }
;

xExplicitCursorBody[CParseClassContext context] {
  CType			type;
  JVariableDefinition	field;
  TokenReference	sourceRef = buildTokenReference();
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
}
:LCURLY
    (
      type = jType[] field = jVariableDeclarator[XConstants.ACC_CURSOR | XConstants.ACC_PUBLIC, type] SEMI
        { context.addFieldDeclaration(new XCursorFieldDeclaration(sourceRef, field, javadoc, comments)); }
    )+
  RCURLY
;

xOptionalContext[] returns [JExpression self]:LBRACK
  self = jExpression[]
  RBRACK
;

xExpressionInSql[] returns [JExpression self = null]://LPAREN
  self = jExpression[]
  RPAREN
;

xExecSqlExpression[] returns [XExecSqlExpression self = null]{
  JExpression		conn = null;
  JExpression		cursor = null;
  XSqlExpr		expr = null;
  TokenReference	sourceRef = buildTokenReference();
}
:( UPDATE_SQL | DELETE_SQL )
  ( conn = xOptionalContext[] )?
  ( LPAREN cursor = jExpression[] RPAREN )?
  LCURLY
  {
     expr = new XSqlExpr(sourceRef, mgr.getSQLParser().xAnyStatement());
  }
  //RCURLY
    { self = new XExecSqlExpression(sourceRef, conn, cursor, expr); }
;

xExecSqlStatement[] returns [XExecSqlStatement self = null]{
  JExpression		conn = null;
  XSqlExpr		expr = null;
  JExpressionListStatement list = null;
  Vector		types = new Vector();
  TokenReference	sourceRef = buildTokenReference();
  JavaStyleComment[]	comments = getStatementComment();
}
:EXECUTE_SQL
  ( conn = xOptionalContext[] )?
  LCURLY
  {
    expr = new XSqlExpr(sourceRef, mgr.getSQLParser().xSelectIntoStatement(types));
  }
  list = jStatementExpressionList[]
  RCURLY
    { self = new XExecSqlStatement(sourceRef, 
				   conn, 
				   expr, 
				   (CType[])at.dms.compiler.Utils.vectorToArray(types, CType.class),
				   list, 
				   comments); 
    }
;

xProtectedStatement[] returns [XProtectedStatement self = null]{
  JExpression		cont = null;
  JExpression		mess = null;
  JBlock		block;
  TokenReference	sourceRef = buildTokenReference();
  JavaStyleComment[]	comments = getStatementComment();
}
:PROTECTED_SQL
    { self = new XProtectedStatement(sourceRef, comments); }
  ( cont = xOptionalContext[] )?
  LPAREN ( mess = jExpression[] )?
  RPAREN
  block = jBlock[]
    { self.close(mess, cont, block); }
;

xUnprotectedStatement[] returns [XUnprotectedStatement self = null]{
  JExpression		cont = null;
  JExpression		mess = null;
  JBlock		block;
  TokenReference	sourceRef = buildTokenReference();
  JavaStyleComment[]	comments = getStatementComment();
}
:UNPROTECTED_SQL
    { self = new XUnprotectedStatement(sourceRef, comments); }
  ( cont = xOptionalContext[] )?
  LPAREN ( mess = jExpression[] )?
  RPAREN
  block = jBlock[]
    { self.close(mess, cont, block); }
;

// inherited from grammar JavaParser
jCompilationUnit returns [JCompilationUnit self = null]{
  TokenReference		sourceRef = buildTokenReference();
  JPackageName			pack = JPackageName.UNNAMED;
  CParseCompilationUnitContext	context = CParseCompilationUnitContext.getInstance();
}
:( pack = jPackageDeclaration[] )?
    {context.setPackage(pack); }
  jDefaultImport[context]
  ( jImportDeclaration[context] )*
  ( jTypeDeclaration[context] )*

  EOF
{
  self = new JCompilationUnit(sourceRef, 
	                      pack,
			      context.getPackageImports(),
			      context.getClassImports(),
			      context.getTypeDeclarations());
  context.release();
}
;

// inherited from grammar JavaParser
jPackageDeclaration[] returns [JPackageName self = null]{
  String	name = null;
}
:"package" name = jName[] SEMI
    { self = new JPackageName(buildTokenReference(), name, getStatementComment()); }
;

// inherited from grammar JavaParser
jImportDeclaration[CParseCompilationUnitContext cunit] {
  String	name = null;
  boolean	star = false;
}
:"import" name = jName[]
  ( DOT STAR { star = true; } )?
  SEMI
    {
      if (star) {
	cunit.addPackageImport(new JPackageImport(buildTokenReference(), name, getStatementComment()));
      } else {
	cunit.addClassImport(new JClassImport(buildTokenReference(), name, getStatementComment()));
      }
    }
;

// inherited from grammar JavaParser
jModifiers[] returns [int self = 0]{
  int		mod;
}
:(
    mod = jModifier[]
      {
	// !!! warning/error if ((mod & self) != 0)
	self += mod;
      }
  )*
;

// inherited from grammar JavaParser
jModifier[] returns [int self = 0]:"public" { self = at.dms.kjc.Constants.ACC_PUBLIC; }
|
  "protected" { self = at.dms.kjc.Constants.ACC_PROTECTED; }
|
  "private" { self = at.dms.kjc.Constants.ACC_PRIVATE; }
|
  "static" { self = at.dms.kjc.Constants.ACC_STATIC; }
|
  "abstract" { self = at.dms.kjc.Constants.ACC_ABSTRACT; }
|
  "final" { self = at.dms.kjc.Constants.ACC_FINAL; }
|
  "native" { self = at.dms.kjc.Constants.ACC_NATIVE; }
|
  "synchronized" { self = at.dms.kjc.Constants.ACC_SYNCHRONIZED; }
|
  "transient" { self = at.dms.kjc.Constants.ACC_TRANSIENT; }
|
  "volatile" { self = at.dms.kjc.Constants.ACC_VOLATILE; }
;

// inherited from grammar JavaParser
jClassDeclaration[int mods] returns [JClassDeclaration self = null]{
  String		superClass = null;
  CClassType[]		interfaces = CClassType.EMPTY;
  CParseClassContext	context = CParseClassContext.getInstance();
  TokenReference	sourceRef = buildTokenReference();
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
}
:"class" ident:IDENT
  ( "extends" superClass = jName[] )?
  ( "implements" interfaces = jNameList[] )?
  jClassBody[context]
    {
      self = new JClassDeclaration(sourceRef,
				   mods,
				   ident.getText(),
				   superClass,
				   interfaces,
				   context.getFields(),
				   context.getMethods(),
				   context.getInnerClasses(),
				   context.getBody(),
				   javadoc,
				   comments);
      context.release();
    }
;

// inherited from grammar JavaParser
jClassBody[CParseClassContext context] :LCURLY
  (
    jClassBodyDeclaration[context]
  |
    SEMI
      { reportTrouble(new CWarning(buildTokenReference(), "stray-semicolon", null)); }
  )*
  RCURLY
;

// inherited from grammar JavaParser
jInnerClassBody[String name] returns [JClassDeclaration self = null]{
  TokenReference	sourceRef = buildTokenReference();
  CParseClassContext	context = CParseClassContext.getInstance();
}
:LCURLY
  (
    jClassBodyDeclaration[context]
  |
    SEMI
      { reportTrouble(new CWarning(sourceRef, "stray-semicolon", null)); }
  )*
  RCURLY
{
  self = new JClassDeclaration(sourceRef,
			       0,
			       name,
			       null,
			       new CClassType[]{},
			       context.getFields(),
			       context.getMethods(),
			       context.getInnerClasses(),			       
			       context.getBody(),
			       getJavadocComment(),
			       getStatementComment());
  context.release();
}
;

// inherited from grammar JavaParser
jClassBodyDeclaration[CParseClassContext context] {
  int			mods = 0;
  JClassBlock		block = null;
}
:mods = jModifiers[]
  (
    jConstructorDeclaration[context, mods]
  |
    jTypeMemberDeclaration[context, mods]
  |
    jInnerTypeDeclaration[context, mods]
  )
|
  // JLS 8.6: Class and Instance Initializer
  "static" block = jClassBlock[]
    { 
      block.setStatic(true);
      context.addBlockInitializer(block); 
    }
|
  block = jClassBlock[]
    { context.addBlockInitializer(block); }
;

// inherited from grammar JavaParser
jInterfaceDeclaration[int mods] returns [JInterfaceDeclaration self = null]{
  CClassType[]		interfaces =  CClassType.EMPTY;
  CParseClassContext	context = CParseClassContext.getInstance();
  TokenReference	sourceRef = buildTokenReference();
}
:"interface" ident:IDENT
  ( "extends" interfaces = jNameList[] )?
  jInterfaceBody[context]
    {
      self = new JInterfaceDeclaration(sourceRef,
				       mods,
				       ident.getText(),
				       interfaces,
				       context.getFields(),
				       context.getMethods(),
				       context.getInnerClasses(),			       
				       context.getBody(),
				       getJavadocComment(),
				       getStatementComment());
      context.release();
    }
;

// inherited from grammar JavaParser
jInterfaceBody[CParseClassContext context] :LCURLY
  (
    jInterfaceBodyDeclaration[context]
  |
    SEMI
      { reportTrouble(new CWarning(buildTokenReference(), "stray-semicolon", null)); }
  )*
  RCURLY
;

// inherited from grammar JavaParser
jInterfaceBodyDeclaration[CParseClassContext context] {
  int			mods = 0;
}
:mods = jModifiers[]
  (
    jTypeMemberDeclaration[context, mods]
  |
    jInnerTypeDeclaration[context, mods]
  )
;

// inherited from grammar JavaParser
jTypeMemberDeclaration[CParseClassContext context, int mods] {
  CType			type;
}
:type = jType[]
  (
    jMethodDeclaration[context, mods, type]
  |
    jFieldDeclaration[context, mods, type]
  )
;

// inherited from grammar JavaParser
jType[] returns [CType self = null]{
  String		name = null;
  int			bounds = 0;
}
:(
    name = jName[]
      { self = CClassType.lookup(name); }
  |
    self = jPrimitiveType []
  )
  ( 
    options { warnWhenFollowAmbig = false; } :
    LBRACK RBRACK { bounds += 1; } 
  )*
    {
      if (bounds > 0) {
	self = new CArrayType(self, bounds);
      }
    }
;

// inherited from grammar JavaParser
jConstructorDeclaration[CParseClassContext context, int mods] {
  JFormalParameter[]	parameters;
  CClassType[]		throwsList = CClassType.EMPTY;
  Vector		body = VectorCache.request(VectorCache.BIG);
  JStatement		stmt;
  TokenReference	sourceRef = buildTokenReference();
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
}
:ident:IDENT
  parameters = jFormalParameters[JLocalVariable.DES_PARAMETER]
  ( "throws" throwsList = jNameList[] )?
  LCURLY
  (
    ( jExplicitConstructorInvocation[] ) =>
      stmt = jExplicitConstructorInvocation[]
        { body.addElement(stmt); }
  )?
  ( stmt = jBlockStatement[] { body.addElement(stmt); } )*
  RCURLY
    {
      context.addMethodDeclaration(new JConstructorDeclaration(sourceRef,
							       mods,
							       ident.getText(),
							       parameters,
							       throwsList,
							       new JConstructorBlock(sourceRef, 
										     (JStatement[])at.dms.compiler.Utils.vectorToArray(body, JStatement.class)),
							       javadoc,
							       comments));
      VectorCache.release(body);
    }
;

// inherited from grammar JavaParser
jFieldDeclaration[CParseClassContext context, int mods, CType type] {
  JVariableDefinition	field;
  TokenReference	sourceRef = buildTokenReference();
}
:field = jVariableDeclarator[mods, type]
    { context.addFieldDeclaration(new JFieldDeclaration(sourceRef, field, getJavadocComment(),
							getStatementComment())); }
  (
    COMMA field = jVariableDeclarator[mods, type]
      { context.addFieldDeclaration(new JFieldDeclaration(sourceRef, field, getJavadocComment(),
							  getStatementComment())); }
  )*
  SEMI
;

// inherited from grammar JavaParser
jFormalParameters[int desc] returns [JFormalParameter[] self = JFormalParameter.EMPTY]{
  JFormalParameter		elem = null;
  Vector			vect = VectorCache.request(VectorCache.SMALL);
}
:LPAREN
  (
   elem = jFormalParameter[desc]
      { vect.addElement(elem); }
    ( COMMA elem = jFormalParameter[desc] { vect.addElement(elem); } )*
    {
      self = (JFormalParameter[])at.dms.compiler.Utils.vectorToArray(vect, JFormalParameter.class); 
      VectorCache.release(vect);
    }
  )?
  RPAREN
;

// inherited from grammar JavaParser
jName[] returns [String self = null]{
  StringBuffer buffer = null;
}
:i:IDENT 
  (
    options {warnWhenFollowAmbig = false;} :
    DOT j:IDENT { (buffer == null ? (buffer = new StringBuffer(i.getText())) : buffer).append('/').append(j.getText()); }
  )*
    { self = buffer == null ? i.getText() : buffer.toString(); }
;

// inherited from grammar JavaParser
jNameList[] returns [CClassType[] self = null]{
  String	name;
  Vector			vect = VectorCache.request(VectorCache.SMALL);
}
:name = jName[] { vect = new Vector(); vect.addElement(CClassType.lookup(name)); }
  ( COMMA name = jName[] { vect.addElement(CClassType.lookup(name)); } )*
{
  self = (CClassType[])at.dms.compiler.Utils.vectorToArray(vect, CClassType.class);
  VectorCache.release(vect);
}
;

// inherited from grammar JavaParser
jVariableInitializer[] returns [JExpression self = null]:self = jArrayInitializer[]
|
  self = jExpression[]
;

// inherited from grammar JavaParser
jArrayInitializer[] returns [JArrayInitializer self = null]{
  JExpression		expr = null;
  Vector		vect = VectorCache.request(VectorCache.SMALL);
  TokenReference	sourceRef = buildTokenReference();
}
:LCURLY
  (
    expr = jVariableInitializer[]
      { vect.addElement(expr); }
    (
      // CONFLICT: does a COMMA after an initializer start a new
      // initializer or start the option ',' at end?
      // ANTLR generates proper code by matching the comma as soon as possible.
      options { warnWhenFollowAmbig = false; } :

      COMMA expr = jVariableInitializer[]
        { vect.addElement(expr); }
    )*
    (
      COMMA
        { reportTrouble(new CWarning(sourceRef, "stray-comma", null)); }
    )?
  )?
  RCURLY
    {
       self = new JArrayInitializer(sourceRef, 
				   (JExpression[])at.dms.compiler.Utils.vectorToArray(vect, JExpression.class)); 
      VectorCache.release(vect);
  }
;

// inherited from grammar JavaParser
jExplicitConstructorInvocation[] returns [JStatement self = null]{
  JExpression[]		args = null;
  TokenReference	sourceRef = buildTokenReference();
}
:"this" args = jArguments[] SEMI
    { self = new JExplicitConstructorInvocation(sourceRef, null, "this", args, getStatementComment()); }
|
  //8.6.5 Constructor Body !!! correspond to java-lang-spec
  "super" args = jArguments[] SEMI
    { self = new JExplicitConstructorInvocation(sourceRef, null, "super", args, getStatementComment()); }
  //( ( jPrimaryExpression[] DOT )? "super" LPAREN ) =>
  //( jPrimaryExpression[] DOT )? "super" jArguments[] SEMI
;

// inherited from grammar JavaParser
jAssignmentOperator[] returns [int self = -1]:ASSIGN { self = Constants.OPE_SIMPLE; }
|
  PLUS_ASSIGN { self = Constants.OPE_PLUS; }
|
  MINUS_ASSIGN { self = Constants.OPE_MINUS; }
|
  STAR_ASSIGN { self = Constants.OPE_STAR; }
|
  SLASH_ASSIGN { self = Constants.OPE_SLASH; }
|
  PERCENT_ASSIGN { self = Constants.OPE_PERCENT; }
|
  SR_ASSIGN { self = Constants.OPE_SR; }
|
  BSR_ASSIGN { self = Constants.OPE_BSR; }
|
  SL_ASSIGN { self = Constants.OPE_SL; }
|
  BAND_ASSIGN { self = Constants.OPE_BAND; }
|
  BXOR_ASSIGN { self = Constants.OPE_BXOR; }
|
  BOR_ASSIGN { self = Constants.OPE_BOR; }
;

// inherited from grammar JavaParser
jConditionalExpression[] returns [JExpression self = null]{
  JExpression		middle;
  JExpression		right;
}
:self = jConditionalOrExpression[]
  (
    QUESTION middle = jExpression[] COLON right = jConditionalExpression[]
      { self = new JConditionalExpression(self.getTokenReference(), self, middle, right); }
  )?
;

// inherited from grammar JavaParser
jConditionalOrExpression[] returns [JExpression self = null]{
  JExpression		right;
}
:self = jConditionalAndExpression[]
  (
    LOR right = jConditionalAndExpression[]
      { self = new JConditionalOrExpression(self.getTokenReference(), self, right); }
  )*
;

// inherited from grammar JavaParser
jConditionalAndExpression[] returns [JExpression self = null]{
  JExpression		right;
}
:self = jInclusiveOrExpression[]
  (
    LAND right = jInclusiveOrExpression[]
      { self = new JConditionalAndExpression(self.getTokenReference(), self, right); }
  )*
;

// inherited from grammar JavaParser
jInstanceOfExpression[] returns [JExpression self = null]{
  CType			type;
}
:self = jRelationalExpression[]
  (
    "instanceof" type = jType[]
      { self = new JInstanceofExpression(self.getTokenReference(), self, type); }
  )?
;

// inherited from grammar JavaParser
jCastLookahead[] {
  Object	warn = null;

}
:LPAREN warn = jPrimitiveType[]
|
  LPAREN warn = jName[] 
  (
    LBRACK RBRACK
  |
    RPAREN ( BNOT | LNOT | LPAREN | IDENT | "this" | "super" | "new" | warn = jLiteral[] )
  )
;

// inherited from grammar JavaParser
jArguments[] returns [JExpression[] self = JExpression.EMPTY]:LPAREN ( self = jArgumentList[] )? RPAREN
;

// inherited from grammar JavaParser
jArgumentList[] returns [JExpression[] self = null]{
  JExpression		expr;
  Vector		vect = VectorCache.request(VectorCache.SMALL);
}
:expr = jExpression[]
    { vect.addElement(expr); }
  (
    COMMA expr = jExpression[]
      { vect.addElement(expr); }
  )*
{
  self = (JExpression[])at.dms.compiler.Utils.vectorToArray(vect, JExpression.class);
  VectorCache.release(vect);
}
;

// inherited from grammar JavaParser
jArrayDimsAndInits[] returns [JArrayDimsAndInits self = null]{
  boolean		expr = false;
  boolean		unspec = false;
  Vector		vect = VectorCache.request(VectorCache.SMALL);
  JExpression		exp  = null;
  JArrayInitializer	init = null;
  TokenReference	sourceRef = buildTokenReference();
}
:(
   options { warnWhenFollowAmbig = false; } :
   LBRACK
    ( { !unspec }? exp = jExpression[] { expr = true; } | { unspec = true; } )
   RBRACK
     { vect.addElement(exp); exp = null; }
 )+
 ({ expr == false }? init = jArrayInitializer[] | )
 //!!! warning okay, try to hide
   { 
     self =  new JArrayDimsAndInits(sourceRef, 
				    (JExpression[])at.dms.compiler.Utils.vectorToArray(vect, JExpression.class), 
				    init); 
      VectorCache.release(vect);     
    }
;

// inherited from grammar JavaParser
jLabeledStatement[] returns [JStatement self = null]{
  JStatement		stmt;
  TokenReference	sourceRef = buildTokenReference();
}
:label:IDENT COLON stmt = jStatement[]
    { self = new JLabeledStatement(sourceRef, label.getText(), stmt, getStatementComment()); }
;

// inherited from grammar JavaParser
jBlock[] returns [JBlock self = null]{
  Vector		body = VectorCache.request(VectorCache.BIG);
  JStatement		stmt;
  TokenReference	sourceRef = buildTokenReference();
}
:LCURLY
  (
    stmt = jBlockStatement[]
      { body.addElement(stmt); }
  )*
  RCURLY
    {
      self = new JBlock(sourceRef, (JStatement[])at.dms.compiler.Utils.vectorToArray(body, JStatement.class), getStatementComment());
      VectorCache.release(body);
    }
;

// inherited from grammar JavaParser
jClassBlock[] returns [JClassBlock self = null]{
  Vector		body = VectorCache.request(VectorCache.BIG);
  JStatement		stmt;
  TokenReference	sourceRef = buildTokenReference();
}
:LCURLY
  (
    stmt = jBlockStatement[]
      { body.addElement(stmt); }
  )* RCURLY
    {
      self = new JClassBlock(sourceRef, 
			     (JStatement[])at.dms.compiler.Utils.vectorToArray(body, JStatement.class)); 
      VectorCache.release(body);
    }
;

// inherited from grammar JavaParser
jLocalVariableDeclaration[int mods] returns [JVariableDeclarationStatement self = null]{
  CType			type;
  Vector		vars = VectorCache.request(VectorCache.SMALL);
  JVariableDefinition	decl;
  TokenReference	sourceRef = buildTokenReference();
}
:type = jType[]
  decl = jVariableDeclarator[mods, type]
    { vars.addElement(decl); }
  (
    COMMA decl = jVariableDeclarator[mods, type]
      { vars.addElement(decl); }
  )*
    {
      self = new JVariableDeclarationStatement(sourceRef, 
					       (JVariableDefinition[])at.dms.compiler.Utils.vectorToArray(vars, JVariableDefinition.class),
					       getStatementComment());
      VectorCache.release(vars);
    }
;

// inherited from grammar JavaParser
jEmptyStatement[] returns [JEmptyStatement self = null]{
  TokenReference	sourceRef = buildTokenReference();
}
:SEMI
    { self = new JEmptyStatement(sourceRef, getStatementComment()); }
;

// inherited from grammar JavaParser
jExpressionStatement[] returns [JExpressionStatement self = null]{
  JExpression		expr;
  TokenReference	sourceRef = buildTokenReference();
}
:expr = jStatementExpression[] SEMI
    { self = new JExpressionStatement(sourceRef, expr, getStatementComment()); }
;

// inherited from grammar JavaParser
jSwitchStatement[] returns [JStatement self = null]{
  JExpression		expr;
  Vector		body = VectorCache.request(VectorCache.SMALL);

  Vector		labels = Constants.VECTOR_EMPTY;
  Vector		stmts = Constants.VECTOR_EMPTY;

  JSwitchLabel		label;
  JStatement		stmt;
  TokenReference	sourceRef = buildTokenReference();
}
:"switch" LPAREN expr = jExpression[] RPAREN
  LCURLY
  (
      {
        labels = new Vector();
	stmts  = new Vector();
      }
    (
      // CONFLICT: to which "cases" does the statement bind?
      // ANTLR generates proper code: it groups the many
      // "case"/"default" labels together then follows them with the statements
      options { warnWhenFollowAmbig = false; } :

      label = jSwitchLabel[] { labels.addElement(label); }
    )+
    ( stmt = jBlockStatement[] { stmts.addElement(stmt); } )*
      { body.addElement(new JSwitchGroup(sourceRef,
					 (JSwitchLabel[])at.dms.compiler.Utils.vectorToArray(labels, JSwitchLabel.class), 
					 (JStatement[])at.dms.compiler.Utils.vectorToArray(stmts, JStatement.class))); }
  )*
  RCURLY
    { 
      self = new JSwitchStatement(sourceRef,
				  expr, 
				  (JSwitchGroup[])at.dms.compiler.Utils.vectorToArray(body, JSwitchGroup.class),
   				  getStatementComment());
      VectorCache.release(body);
    }
;

// inherited from grammar JavaParser
jSwitchLabel[] returns [JSwitchLabel self = null]{
  JExpression		expr = null;
  TokenReference	sourceRef = buildTokenReference();
}
:"case" expr = jExpression[] COLON
    { self = new JSwitchLabel(sourceRef, expr); }
|
  "default" COLON
    { self = new JSwitchLabel(sourceRef, null); }
;

// inherited from grammar JavaParser
jIfStatement[] returns [JIfStatement self = null]{
  JExpression		cond;
  JStatement		thenClause;
  JStatement		elseClause = null;
  TokenReference	sourceRef = buildTokenReference();
  int			count = 0;
}
:"if" LPAREN cond = jExpression[] RPAREN
  thenClause = jStatement[]
  (
    // CONFLICT: the old "dangling-else" problem...
    // ANTLR generates proper code matching as soon as possible,
    // binding dangling else's to the innermost if statement
    // Hush warning.
    options { warnWhenFollowAmbig = false; } :

    {
      if (count++ > 0) {
        break;
      }
    }
    // !!! Putain d'ANTLR, REMOVE WHEN BUG CORRECTED

    "else" elseClause = jStatement[]
  )*
    {
      if (! (thenClause instanceof JBlock)) {
	reportTrouble(new CWarning(sourceRef, "enclose-if-then-in-block", null));
      }
      if (elseClause != null && !(elseClause instanceof JBlock || elseClause instanceof JIfStatement)) {
	reportTrouble(new CWarning(sourceRef, "enclose-if-else-in-block", null));
      }
      self = new JIfStatement(sourceRef, cond, thenClause, elseClause, getStatementComment());
    }
;

// inherited from grammar JavaParser
jWhileStatement[] returns [JWhileStatement self = null]{
  JExpression		cond;
  JStatement		body;
  TokenReference	sourceRef = buildTokenReference();
}
:"while" LPAREN cond = jExpression[] RPAREN body = jStatement[]
    {
      if (!(body instanceof JBlock || body instanceof JEmptyStatement)) {
	reportTrouble(new CWarning(sourceRef, "enclose-loop-body-in-block", null));
      }
      self = new JWhileStatement(sourceRef, cond, body, getStatementComment());
    }
;

// inherited from grammar JavaParser
jDoStatement[] returns [JDoStatement self = null]{
  JExpression		cond;
  JStatement		body;
  TokenReference	sourceRef = buildTokenReference();
}
:"do" body = jStatement[] "while" LPAREN cond = jExpression[] RPAREN SEMI
    {
      if (! (body instanceof JBlock)) {
	reportTrouble(new CWarning(sourceRef, "enclose-loop-body-in-block", null));
      }
      self = new JDoStatement(sourceRef, cond, body, getStatementComment());
    }
;

// inherited from grammar JavaParser
jForStatement[] returns [JStatement self = null]{
  JStatement		init = null;
  JExpression		cond = null;
  JStatement		incr = null;
  JStatement		body = null;
  TokenReference	sourceRef = buildTokenReference();
  JavaStyleComment[]	comments = getStatementComment();
}
:"for" LPAREN
    ( init = jForInit[] )? SEMI
    ( cond = jExpression[] )? SEMI
    ( incr = jStatementExpressionList[] )?
  RPAREN body = jStatement[]
    {
      if (!(body instanceof JBlock || body instanceof JEmptyStatement)) {
	reportTrouble(new CWarning(sourceRef, "enclose-loop-body-in-block", null));
      }
      self = new JForStatement(sourceRef, init, cond, incr, body, comments);
    }
;

// inherited from grammar JavaParser
jForInit[] returns [JStatement self = null]:"final" self = jLocalVariableDeclaration[at.dms.kjc.Constants.ACC_FINAL]
|
  ( jType[] IDENT ) =>
  self = jLocalVariableDeclaration[0]
|
  self = jStatementExpressionList[]
;

// inherited from grammar JavaParser
jStatementExpressionList[] returns [JExpressionListStatement self = null]{
  JExpression		expr;
  Vector		v = VectorCache.request(VectorCache.SMALL);
  TokenReference	sourceRef = buildTokenReference();
}
:expr = jStatementExpression[]
    { v.addElement(expr); }
  (
    COMMA expr = jStatementExpression[]
      { v.addElement(expr); }
  )*
    { self = new JExpressionListStatement(sourceRef, 
					  (JExpression[])at.dms.compiler.Utils.vectorToArray(v, JExpression.class), 
					  getStatementComment()); 
    VectorCache.release(v);
    }
;

// inherited from grammar JavaParser
jBreakStatement[] returns [JBreakStatement self = null]{
  TokenReference	sourceRef = buildTokenReference();
}
:"break" ( ident:IDENT )? SEMI
    { self = new JBreakStatement(sourceRef, ident == null ? null : ident.getText(), getStatementComment()); }
;

// inherited from grammar JavaParser
jContinueStatement[] returns [JContinueStatement self = null]{
  TokenReference	sourceRef = buildTokenReference();
}
:"continue" ( ident:IDENT )? SEMI
    { self = new JContinueStatement(sourceRef, ident == null ? null : ident.getText(), getStatementComment()); }
;

// inherited from grammar JavaParser
jThrowStatement[] returns [JStatement self = null]{
  JExpression		expr;
  TokenReference	sourceRef = buildTokenReference();
}
:"throw" expr = jExpression[] SEMI
    { self = new JThrowStatement(sourceRef, expr, getStatementComment()); }
;

// inherited from grammar JavaParser
jSynchronizedStatement[] returns [JSynchronizedStatement self = null]{
  JExpression		cond;
  JStatement		body;
  TokenReference	sourceRef = buildTokenReference();
}
:"synchronized" LPAREN cond = jExpression[] RPAREN body = jBlock[]
    { self = new JSynchronizedStatement(sourceRef, cond, body, getStatementComment()); }
;

// inherited from grammar JavaParser
jTryStatement[] returns [JStatement self = null]{
  JBlock		tryClause;
  Vector		catchClauses = VectorCache.request(VectorCache.SMALL);
  JBlock		finallyClause = null;
  JCatchClause		catcher = null;
  TokenReference	sourceRef = buildTokenReference();
}
:"try" tryClause = jBlock[]
  (
    catcher = jCatchClause[]
      { catchClauses.addElement(catcher); }
  )*
  ( "finally" finallyClause = jBlock[] )?
    {
      if (catchClauses.size() > 0) {
	self = new JTryCatchStatement(sourceRef,
				      tryClause,
				      (JCatchClause[])at.dms.compiler.Utils.vectorToArray(catchClauses, JCatchClause.class),
				      finallyClause == null ? getStatementComment() : null);
	VectorCache.release(catchClauses);
      }
      if (finallyClause != null) {
	// If both catch and finally clauses are present,
	// the try-catch is embedded as try clause into a
	// try-finally statement.
	if (self != null) {
	  tryClause = new JBlock(sourceRef, new JStatement[] {self}, null);
	}
	self = new JTryFinallyStatement(sourceRef, tryClause, finallyClause, getStatementComment());
      }

      if (self == null) {
	// try without catch or finally: error
	reportTrouble(new PositionedError(sourceRef, "try-nocatch", null));
	self = tryClause;
      }
    }
;

// inherited from grammar JavaParser
jCatchClause[] returns [JCatchClause self = null]{
  JFormalParameter	param;
  JBlock		body;
  TokenReference	sourceRef = buildTokenReference();
}
:"catch" LPAREN param = jFormalParameter[JLocalVariable.DES_CATCH_PARAMETER] RPAREN body = jBlock[]
    { self = new JCatchClause(sourceRef, param, body); }
;


