/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: xjava.g,v 1.61 1999/10/28 14:03:14 vgp Exp $
 */

// Import the necessary classes
header { package at.dms.xkjc; }
{
  import java.util.Vector;

  import at.dms.kjc.*;
  import at.dms.compiler.Compiler;
  import at.dms.compiler.CWarning;
  import at.dms.compiler.PositionedError;
  import at.dms.compiler.TokenReference;
  import at.dms.compiler.JavaStyleComment;
  import at.dms.util.VectorCache;
}

// ----------------------------------------------------------------------
// THE PARSER STARTS HERE
// ----------------------------------------------------------------------

class XJavaParser extends JavaParser;

options {
  k = 2;				// two token lookahead
  importVocab = XJava;			// Call its vocabulary "Java"
  exportVocab = XJava;			// Call its vocabulary "Java"
  codeGenMakeSwitchThreshold = 2;	// Some optimizations
  codeGenBitsetTestThreshold = 3;
  defaultErrorHandler = false;		// Don't generate parser error handlers
  superClass="at.dms.kjc.Parser";
}
{
  public XJavaParser(Compiler compiler, XParseManager mgr, CHandLexer lexer, String file) {
    super(compiler, lexer, file, MAX_LOOKAHEAD);
    this.mgr = mgr;
    this.isKopiFile = file.endsWith(".k");
  }

  private XParseManager		mgr;
  private boolean		isKopiFile;
}

jDefaultImport[CParseCompilationUnitContext context]
:
  {
    context.addPackageImport(CParseCompilationUnitContext.JAVA_LANG);
    context.addPackageImport(new JPackageImport(TokenReference.NO_REF, "at/dms/xlib", null));
    context.addPackageImport(new JPackageImport(TokenReference.NO_REF, "at/dms/xtype", null));
  }
;

// JLS 8.4 Method Declaration
jMethodDeclaration [CParseClassContext context, int mods, CType type]
{
  JFormalParameter[]	parameters;
  CClassType[]		throwsList = CClassType.EMPTY;
  int			bounds = 0;
  JBlock		block = null;
  int			oper = -1;
  TokenReference	sourceRef = buildTokenReference();
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  ident:IDENT ( COLON oper = xOverloadableOperator[] )?
  parameters = jFormalParameters[JLocalVariable.DES_PARAMETER]
  ( LBRACK RBRACK { bounds += 1; } )*
  ( "throws" throwsList = jNameList[] )?
  ( block = jBlock[] | SEMI )
    {
      if (bounds > 0) {
	reportTrouble(new CWarning(sourceRef, "old-style-array-bounds", null));
	type = new CArrayType(type, bounds);
      }

      if (oper == -1) {
        context.addMethodDeclaration(new JMethodDeclaration(sourceRef,
							    mods,
							    type,
							    ident.getText(),
							    parameters,
							    throwsList,
							    block,
							    javadoc,
							    comments));
      } else {
        context.addMethodDeclaration(new XOperatorDeclaration(sourceRef,
							      mods,
							      type,
							      ident.getText(),
							      oper,
							      parameters,
							      throwsList,
							      block,
							      false));
      }
    }
;

// ICS: Extensions to class and interface body declarations
jInnerTypeDeclaration[CParseClassContext context, int mods]
{
  JTypeDeclaration	inner = null;
}
:
(
  inner = jClassDeclaration[mods]
|
  inner = jInterfaceDeclaration[mods]
|
  inner = xCursorDeclaration[mods]
)
  { context.addInnerDeclaration(inner); }
;

// //////////////////////////////////////////////////////////////////////
// OVERLOADABLE EXPRESSION
// //////////////////////////////////////////////////////////////////////

jAdditiveExpression []
  returns [JExpression self = null]
{
  int			oper = Constants.OPE_PLUS;
  JExpression		right;
}
:
  self = jMultiplicativeExpression[]
  (
    (
      PLUS { oper = Constants.OPE_PLUS; }
    |
      MINUS { oper = Constants.OPE_MINUS; }
    )
    right = jMultiplicativeExpression[]
      {
	if (oper == Constants.OPE_PLUS) {
	  self = new XAddExpression(self.getTokenReference(), self, right);
	} else {
	  //if (oper == Constants.OPE_MINUS)
	  self = new XMinusExpression(self.getTokenReference(), self, right);
	}
      }
  )*
;

jEqualityExpression []
  returns [JExpression self = null]
{
  int			operator = -1;
  JExpression		right;
}
:
  self = jInstanceOfExpression[]
  (
    (
      EQUAL { operator = Constants.OPE_EQ; }
    |
      NOT_EQUAL { operator = Constants.OPE_NE; }
    )
    right = jInstanceOfExpression[]
      { self = new XEqualityExpression(self.getTokenReference(), operator, self, right); }
  )*
;

jUnaryExpressionNotPlusMinus []
  returns [JExpression self = null]
{
  int			operator = -1;
  TokenReference	sourceRef = buildTokenReference();
}
:
  (
    BNOT { operator = Constants.OPE_BNOT; }
  |
    LNOT { operator = Constants.OPE_LNOT; }
  )
  self = jUnaryExpression[]
    { self = new XUnaryExpression(sourceRef, operator, self); }
|
  ( jCastLookahead[] ) =>
  self = jCastExpression[]
|
  self = jPostfixExpression[]
;

jMultiplicativeExpression []
  returns [JExpression self = null]
{
  int			operator = -1;
  JExpression		right;
}
:
  self = jUnaryExpression[]
  (
    (
      STAR { operator = 1; }
    |
      SLASH { operator = 2; }
    |
      PERCENT { operator = 3; }
    )
    right = jUnaryExpression[]
      {
	if (operator == 1) {
          self = new XMultExpression(self.getTokenReference(), self, right);
	} else if (operator == 2) {
          self = new XDivideExpression(self.getTokenReference(), self, right);
	} else {
          self = new XModuloExpression(self.getTokenReference(), self, right);
	}
      }
  )*
;

jPostfixExpression[]
  returns [JExpression self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  self = jPrimaryExpression[]
  (
    INC
      { self = new XPostfixExpression(sourceRef, Constants.OPE_POSTINC, self); }
  |
    DEC
      { self = new XPostfixExpression(sourceRef, Constants.OPE_POSTDEC, self); }
  )?
;

jPreIncrementExpression []
  returns [JExpression self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  INC self = jPrimaryExpression[]
    { self = new XPrefixExpression(sourceRef, Constants.OPE_PREINC, self); }
;

jPreDecrementExpression []
  returns [JExpression self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  DEC self = jPrimaryExpression[]
    { self = new XPrefixExpression(sourceRef, Constants.OPE_PREDEC, self); }
;

//
// The last expansion of this production accepts more than the legal
// Java expansions for StatementExpression.  This expansion does not
// use PostfixExpression for performance reasons.
//
jStatementExpression []
  returns [JExpression self = null]
{
  int			operator = -1;
  JExpression		right;
  TokenReference	sourceRef = buildTokenReference();
}
:
  self = jPreIncrementExpression[]
|
  self = jPreDecrementExpression[]
|
  self = jPrimaryExpression[]
  (
    INC
      { self = new XPostfixExpression(sourceRef, Constants.OPE_POSTINC, self); }
  |
    DEC
      { self = new XPostfixExpression(sourceRef, Constants.OPE_POSTDEC, self); }
  |
    operator = jAssignmentOperator[] right = jExpression[]
      { self = XAssignmentExpression.create(self.getTokenReference(), operator, self, right); }
  )?
;

jShiftExpression []
  returns [JExpression self = null]
{
  int			operator = -1;
  JExpression		right;
}
:
  self = jAdditiveExpression[]
  (
    (
      SL { operator = Constants.OPE_SL; }
    |
      SR { operator = Constants.OPE_SR; }
    |
      BSR { operator = Constants.OPE_BSR; }
    )
    right = jAdditiveExpression[]
      { self = new XShiftExpression(self.getTokenReference(), operator, self, right); }
  )*
;

jRelationalExpression []
  returns [JExpression self = null]
{
  int			operator = -1;
  JExpression		right;
}
:
  self = jShiftExpression[]
  (
    (
      LT { operator = Constants.OPE_LT; }
    |
      GT { operator = Constants.OPE_GT; }
    |
      LE { operator = Constants.OPE_LE; }
    |
      GE { operator = Constants.OPE_GE; }
    )
    right = jShiftExpression[]
      { self = new XRelationalExpression(self.getTokenReference(), operator, self, right); }
  )*
;

jPrimaryPrefix []
  returns [JExpression self = null]
{
  JExpression		expr = null;
  CType			type = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  self = jLiteral[]
|
  "this" { self = new JThisExpression(sourceRef ); }
|
  "super" DOT id:IDENT
    { self = new XNameExpression(sourceRef,
				 new JSuperExpression(sourceRef ),
				 id.getText()); }
|
  LPAREN expr = jExpression[] RPAREN
    { self = new JParenthesedExpression(sourceRef, expr); }
|
  self = jAllocationExpression[null]
|
  ( jType[] DOT "class" ) =>
    type = jType[] DOT "class"
      { self = new JClassExpression(sourceRef, type); }
|
  id2:IDENT
    (
      options { warnWhenFollowAmbig = false; } :     
      DOT "class"
       { self = new JClassExpression(buildTokenReference(), CClassType.lookup(id2.getText())); }
    )?
   { self = self == null ? new XNameExpression(buildTokenReference(), id2.getText()) : self; }
|
    type = xPrimitiveType[]
    (
      options { warnWhenFollowAmbig = false; } :     
      DOT "class"
       { self = new JClassExpression(buildTokenReference(), type); }
    )?
   { self = self == null ? new XNameExpression(buildTokenReference(), ((CClassType)type).getQualifiedName().replace('/', '.'), true) : self; }
|
    type = jPrimitiveType[] DOT "class"
      { self = new JClassExpression(buildTokenReference(), type); }
;

jPrimarySuffix [JExpression prefix]
  returns [JExpression self = null]
{
  JExpression		expr = null;
  JExpression[]	args;
  TokenReference	sourceRef = buildTokenReference();
}
:
  ( DOT "this" ) =>
  DOT "this" { self = new JThisExpression(sourceRef, prefix); }
|
  DOT self = jAllocationExpression[prefix]
|
  LBRACK expr = jExpression[] RBRACK
    { self = new JArrayAccessExpression(sourceRef, prefix, expr); }
|
  DOT id1:IDENT
    { self = new XNameExpression(sourceRef, prefix, id1.getText()); }
|
  args = jArguments[]
    { self = new XMethodCallExpression(sourceRef, prefix, args); }
;

xPrimitiveType []
  returns [CType self = null]
:
  "fixed" { self = XStdType.PFixed; }
|
  "month" { self = XStdType.PMonth; }
|
  "date" { self = XStdType.PDate; }
|
  "time" { self = XStdType.PTime; }
;

jPrimitiveType []
  returns [CType self = null]
:
  self = xPrimitiveType[]
|
  self = javaPrimitiveType[]
;

javaPrimitiveType []
  returns [CType self = null]
:
  "boolean" { self = CStdType.Boolean; }
|
  "byte" { self = CStdType.Byte; }
|
  "char" { self = CStdType.Char; }
|
  "double" { self = CStdType.Double; }
|
  "float" { self = CStdType.Float; }
|
  "int" { self = CStdType.Integer; }
|
  "long" { self = CStdType.Long; }
|
  "short" { self = CStdType.Short; }
|
  "void" { self = CStdType.Void; }
;

jAllocationExpression [JExpression expr]
  returns [JExpression self = null]
{
  CType			type = null;
  JClassDeclaration	decl = null;
  JArrayDimsAndInits	arr = null;
  String		name = null;
  JExpression[]		args = JExpression.EMPTY;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "new"
  (
    (javaPrimitiveType[] ) =>
    type = jPrimitiveType[] arr = jArrayDimsAndInits[]
      { self = new JNewArrayExpression(sourceRef, type, arr); }
   |
    (xPrimitiveType[] ) =>
    type = xPrimitiveType[]
    (
      arr = jArrayDimsAndInits[]
        { self = new JNewArrayExpression(sourceRef, (CClassType)type, arr); }
    |
      args = jArguments[]
      (
        decl = jInnerClassBody[name] 
        { self = new JNewAnonymousClassExpression(sourceRef, (CClassType)type, expr, args, decl); }        
      |
        { self = new JNewObjectExpression(sourceRef, (CClassType)type, expr, args); }        
      )
    )
  |
    name = jName[]
    (
      arr = jArrayDimsAndInits[]
        { self = new JNewArrayExpression(sourceRef, CClassType.lookup(name), arr); }
    |
      args = jArguments[]
      (
        decl = jInnerClassBody[name] 
        { self = new JNewAnonymousClassExpression(sourceRef, CClassType.lookup(name), expr, args, decl); }        
      |
        { self = new JNewObjectExpression(sourceRef, CClassType.lookup(name), expr, args); }        
      )
    )
  )
;

// //////////////////////////////////////////////////////////////////////
// OVERLOADABLE OPERATORS
// //////////////////////////////////////////////////////////////////////

xOverloadableOperator []
  returns [int self = -1]
:
  EQUAL { self = at.dms.kjc.Constants.OPE_EQ; }
|
  LNOT { self = at.dms.kjc.Constants.OPE_LNOT; }
|
  BNOT { self = at.dms.kjc.Constants.OPE_BNOT; }
|
  NOT_EQUAL { self = at.dms.kjc.Constants.OPE_NE; }
|
  SLASH { self = at.dms.kjc.Constants.OPE_SLASH; }
|
  PLUS { self = at.dms.kjc.Constants.OPE_PLUS; }
|
  MINUS { self = at.dms.kjc.Constants.OPE_MINUS; }
|
  STAR { self = at.dms.kjc.Constants.OPE_STAR; }
|
  PERCENT { self = at.dms.kjc.Constants.OPE_PERCENT; }
|
  SR { self = at.dms.kjc.Constants.OPE_SR; }
|
  BSR { self = at.dms.kjc.Constants.OPE_BSR; }
|
  GE { self = at.dms.kjc.Constants.OPE_GE; }
|
  GT { self = at.dms.kjc.Constants.OPE_GT; }
|
  SL { self = at.dms.kjc.Constants.OPE_SL; }
|
  LE { self = at.dms.kjc.Constants.OPE_LE; }
|
  LT { self = at.dms.kjc.Constants.OPE_LT; }
|
  BXOR { self = at.dms.kjc.Constants.OPE_BXOR; }
|
  BOR { self = at.dms.kjc.Constants.OPE_BOR; }
|
  BAND { self = at.dms.kjc.Constants.OPE_BAND; }
|
  LPAREN RPAREN { self = at.dms.xkjc.XConstants.OPE_CAST; }
|
  ASSIGN { self = at.dms.xkjc.XConstants.OPE_SIMPLE; }
|
  INC STAR { self = at.dms.kjc.Constants.OPE_PREINC; }
|
  DEC STAR { self = at.dms.kjc.Constants.OPE_PREDEC; }
|
  STAR
  (
    INC { self = at.dms.kjc.Constants.OPE_POSTINC; }
  |
    DEC { self = at.dms.kjc.Constants.OPE_POSTDEC; }
  )
;

jInclusiveOrExpression []
  returns [JExpression self = null]
{
  JExpression		right;
}
:
  self = jExclusiveOrExpression[]
  (
    BOR right = jExclusiveOrExpression[]
      { self = new XBitwiseExpression(self.getTokenReference(), Constants.OPE_BOR, self, right); }
  )*
;

jExclusiveOrExpression []
  returns [JExpression self = null]
{
  JExpression		right;
}
:
  self = jAndExpression[]
  (
    BXOR right = jAndExpression[]
      { self = new XBitwiseExpression(self.getTokenReference(), Constants.OPE_BXOR, self, right); }
  )*
;

jAndExpression []
  returns [JExpression self = null]
{
  JExpression		right;
}
:
  self = jEqualityExpression[]
  (
    BAND right = jEqualityExpression[]
      { self = new XBitwiseExpression(self.getTokenReference(), Constants.OPE_BAND, self, right); }
  )*
;

jCastExpression []
  returns [JCastExpression self = null]
{
  JExpression		expr;
  CType			dest;
  TokenReference	sourceRef = buildTokenReference();
}
:
  ( LPAREN jPrimitiveType[] ) =>
  LPAREN dest = jType[] RPAREN expr = jUnaryExpression[]
    { self = new XCastExpression(sourceRef, expr, dest); }
|
  LPAREN dest = jType[] RPAREN expr = jUnaryExpressionNotPlusMinus[]
    { self = new XCastExpression(sourceRef, expr, dest); }
;

jUnaryExpression[]
  returns [JExpression self = null]
{
  int			operator = -1;
  TokenReference	sourceRef = buildTokenReference();
}
:(
    PLUS { operator = Constants.OPE_PLUS; }
  |
    MINUS { operator = Constants.OPE_MINUS; }
  )
  self = jUnaryExpression[]
    { self = new XUnaryExpression(sourceRef, operator, self); }
|
  self = jPreIncrementExpression[]
|
  self = jPreDecrementExpression[]
|
  self = jUnaryExpressionNotPlusMinus[]
;

jExpression []
  returns [JExpression self = null]
{
  int			operator = -1;
  JExpression		right;
}
:
   self = jConditionalExpression[]
   (
     operator = jAssignmentOperator[] right = jExpression[]
       { self = XAssignmentExpression.create(self.getTokenReference(), operator, self, right); }
   )?
;

// !!!JLS
// Declaration of a variable.  This can be a class/instance variable,
//   or a local variable in a method
// It can also include possible initialization.
jVariableDeclarator [int mods, CType type]
  returns [JVariableDefinition self = null]
{
  int			bounds = 0;
  JExpression		expr = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  ident:IDENT
  ( LBRACK RBRACK { bounds += 1; } )*
  ( ASSIGN expr = jVariableInitializer[] )?
    {
      if (bounds > 0) {
	reportTrouble(new CWarning(sourceRef, "old-style-array-bounds", null));
	type = new CArrayType(type, bounds);
      }

      self = new XVariableDefinition(sourceRef, mods, type, ident.getText(), expr);
    }
;

jBlockStatement []
  returns [JStatement self = null]
{
  JTypeDeclaration type;
}
:
  "final" self = jLocalVariableDeclaration[at.dms.kjc.Constants.ACC_FINAL] SEMI
|
  ( jType[] IDENT ) =>
  self = jLocalVariableDeclaration[0] SEMI
|
  self = jStatement[]
    {
      if (self instanceof JEmptyStatement) {
	reportTrouble(new CWarning(self.getTokenReference(), "stray-semicolon", null));
      }
    }
|
  (
    type = jClassDeclaration[0]
  |
    type = jInterfaceDeclaration[0]
  |
    type = xCursorDeclaration[0]
  )
    { self = new JTypeDeclarationStatement(type.getTokenReference(), type); }
|
  self = xAnonymousCursorDeclaration[]
;


jStatement []
  returns [JStatement self = null]
:
  self = jEmptyStatement[]
|
  self = jLabeledStatement[]
|
  self = jBlock[]
|
  self = jExpressionStatement[]
|
  self = jSwitchStatement[]
|
  self = jIfStatement[]
|
  self = jWhileStatement[]
|
  self = jDoStatement[]
|
  self = jForStatement[]
|
  self = jBreakStatement[]
|
  self = jContinueStatement[]
|
  self = jReturnStatement[]
|
  self = jThrowStatement[]
|
  self = jSynchronizedStatement[]
|
  self = jTryStatement[]
|
  self = xAssertStatement[]
|
  self = xFailStatement[]
|
  self = xExecSqlStatement[]
|
  self = xProtectedStatement[]
|
  self = xUnprotectedStatement[]
;

// JLS 14.15: Return Statement
jReturnStatement []
  returns [JReturnStatement self = null]
{
  JExpression		expr = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "return" ( expr = jExpression[] )? SEMI
    { self = new XReturnStatement(sourceRef, expr, getStatementComment()); }
;

jLiteral []
  returns [JExpression self = null]
{
  TokenReference	sourceRef = buildTokenReference();
}
:
  i:INTEGER_LITERAL
    { self = new JOrdinalLiteral(sourceRef, i.getText()); }
|
  r:REAL_LITERAL
    {
      if (isKopiFile) {
        self = new XFixedLiteral(sourceRef, r.getText());
      } else {
        self = new JRealLiteral(sourceRef, r.getText());
      }
    }
|
  c:CHARACTER_LITERAL
    { self = new JCharLiteral(sourceRef, c.getText()); }
|
  s:STRING_LITERAL
   { self = new JStringLiteral(sourceRef, s.getText()); }
|
  "true"
    { self = new JBooleanLiteral(sourceRef, true); }
|
  "false"
    { self = new JBooleanLiteral(sourceRef, false); }
|
  "null"
    { self = new JNullLiteral(sourceRef ); }
;

// //////////////////////////////////////////////////////////////////////
// NEW STATEMENTS
// //////////////////////////////////////////////////////////////////////

xFailStatement []
  returns [XFailStatement self = null]
{
  JExpression		expr = null;
  TokenReference	sourceRef = buildTokenReference();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  FAIL LPAREN ( expr = jExpression[] )? RPAREN SEMI
    { self = new XFailStatement(sourceRef, expr, comments); }
;

xAssertStatement []
  returns [XAssertStatement self = null]
{
  JExpression		cond;
  JExpression		expr = null;
  TokenReference	sourceRef = buildTokenReference();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  ASSERT LPAREN cond = jExpression[] ( COMMA expr = jExpression[] )? RPAREN SEMI
    { self = new XAssertStatement(sourceRef, cond, expr, comments); }
;

// //////////////////////////////////////////////////////////////////////
// NEW XJAVA GRAMMAR FOR XSQL
// //////////////////////////////////////////////////////////////////////

// JLS 7.6: Type Declarations
// The syntax for modifiers is relaxed like in the LALR(1) grammar (JLS 19.1.2, 19.7).
// A later stage of compiler analysis then sorts out the precise role of each modifier
// and whether it is permitted in a given context.
jTypeDeclaration [CParseCompilationUnitContext context]
{
  int			mods = 0;
  JTypeDeclaration	decl = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  (
    mods = jModifiers[]
    (
      decl = jClassDeclaration[mods]
    |
      decl = jInterfaceDeclaration[mods]
    |
      decl = xCursorDeclaration[mods]
    )
      { context.addTypeDeclaration(decl); }
  )
|
  SEMI
    { reportTrouble(new CWarning(sourceRef, "stray-semicolon", null)); }
;

jPrimaryExpression []
  returns [JExpression self = null]
:
  ( self = jPrimaryPrefix[] ( self = jPrimarySuffix[self] )* )
  |
  self = xExecSqlExpression[]
;

// //////////////////////////////////////////////////////////////////////
// XSQL ENTRIES
// //////////////////////////////////////////////////////////////////////

xCursorDeclaration [int mods]
  returns [XCursorDeclaration self = null]
{
  CParseClassContext	context = CParseClassContext.getInstance();
  TokenReference	sourceRef = buildTokenReference();
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
  XSelectStatement	sel = null;
  JFormalParameter[]	parameters = null;
}
:
  CURSOR_SQL ident:IDENT
  (
    xExplicitCursorBody[context]
  |
    parameters = jFormalParameters[JLocalVariable.DES_PARAMETER]
    sel = xImplicitCursorBody[context]
    {
      XCursorDeclaration.addImplicitSelect(context, sel, parameters);
    }
  )
   { 
     self = new XCursorDeclaration(sourceRef,
				   mods,
				   ident.getText(),
				   context.getFields(),
				   context.getMethods(),
				   context.getInnerClasses(),
				   context.getBody(),
	                           sel,
	                           parameters,
				   javadoc,
				   comments);
     context.release();
   }
;

xAnonymousCursorDeclaration []
  returns [XLocalCursorDeclaration self = null]
{
  CParseClassContext	context = CParseClassContext.getInstance();
  XCursorDeclaration	curs = null;
  JExpression		ctxt = null;
  CClassType		type = null;
  TokenReference	sourceRef = buildTokenReference();
  XSelectStatement	sel = null;
  JFormalParameter[]	parameters = null;
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  CURSOR_SQL
  ( ctxt = xOptionalContext[] )?
  (
    xExplicitCursorBody[context]
  |
    parameters = jFormalParameters[JLocalVariable.DES_PARAMETER]
    sel = xImplicitCursorBody[context]
    {
      XCursorDeclaration.addImplicitSelect(context, sel, parameters);
    }
  )
     { curs = new XCursorDeclaration(sourceRef,
				     XConstants.ACC_FINAL,
				     null,
				     context.getFields(),
				     context.getMethods(),
				     context.getInnerClasses(),
				     context.getBody(),
				     sel,
			             parameters,	
				     javadoc,
				     comments);
       context.release();

       self = new XLocalCursorDeclaration(sourceRef,
					  curs,
					  comments);
       { self.setConnection(ctxt); } // !!!
     }
    // !!! graf 990918 type not used !!!
    { type = CClassType.lookup(curs.getIdent()); }
  ident:IDENT
    { self.addDeclaration(ident.getText()); }
  ( COMMA
    ident2:IDENT
    { self.addDeclaration(ident2.getText()); }
  )*
  SEMI
;

xImplicitCursorBody [CParseClassContext context]
  returns [XSelectStatement sel = null]
:
  LCURLY
    {
      sel = mgr.getSQLParser().xSelectStatement();
    }
  //RCURLY
;

// !!!JLS
jFormalParameter [int desc]
 returns [JFormalParameter self = null]
{
  boolean		isFinal = false;
  int			bounds = 0;
  CType			type;
  String		name = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  ( "final" { isFinal = true; } )?
  type = jType[]
  ( ident:IDENT { name = ident.getText();} )?
  ( LBRACK RBRACK { bounds += 1; } )*
    {
      if (bounds > 0) {
	reportTrouble(new CWarning(sourceRef, "old-style-array-bounds", null));
	type = new CArrayType(type, bounds);
      }

      self = new XFormalParameter(sourceRef, desc, type, name, isFinal);
    }
;

xExplicitCursorBody [CParseClassContext context]
{
  CType			type;
  JVariableDefinition	field;
  TokenReference	sourceRef = buildTokenReference();
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  LCURLY
    (
      type = jType[] field = jVariableDeclarator[XConstants.ACC_CURSOR | XConstants.ACC_PUBLIC, type] SEMI
        { context.addFieldDeclaration(new XCursorFieldDeclaration(sourceRef, field, javadoc, comments)); }
    )+
  RCURLY
;
/*
// NOT USED  !!!! ????
xCursorFieldDeclaration [JTypeDeclaration decl, int mods, CType type]
{
  JVariableDefinition	field;
  TokenReference	sourceRef = buildTokenReference();
  JJavadocComment	javadoc = getJavadocComment();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  field = jVariableDeclarator[XConstants.ACC_CURSOR | mods, type]
    { decl.addFieldDeclaration(new XCursorFieldDeclaration(sourceRef, decl, field, javadoc, comments)); }
  (
    COMMA field = jVariableDeclarator[XConstants.ACC_CURSOR | mods, type]
      { decl.addFieldDeclaration(new XCursorFieldDeclaration(sourceRef, decl, field, javadoc, comments)); }
  )*
  SEMI
;
*/
xOptionalContext []
  returns [JExpression self]
:
  LBRACK
  self = jExpression[]
  RBRACK
;

xExpressionInSql []
  returns [JExpression self = null]
:
  //LPAREN
  self = jExpression[]
  RPAREN
;

xExecSqlExpression []
  returns [XExecSqlExpression self = null]
{
  JExpression		conn = null;
  JExpression		cursor = null;
  XSqlExpr		expr = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  ( UPDATE_SQL | DELETE_SQL )
  ( conn = xOptionalContext[] )?
  ( LPAREN cursor = jExpression[] RPAREN )?
  LCURLY
  {
     expr = new XSqlExpr(sourceRef, mgr.getSQLParser().xAnyStatement());
  }
  //RCURLY
    { self = new XExecSqlExpression(sourceRef, conn, cursor, expr); }
;

xExecSqlStatement []
  returns [XExecSqlStatement self = null]
{
  JExpression		conn = null;
  XSqlExpr		expr = null;
  JExpressionListStatement list = null;
  Vector		types = new Vector();
  TokenReference	sourceRef = buildTokenReference();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  EXECUTE_SQL
  ( conn = xOptionalContext[] )?
  LCURLY
  {
    expr = new XSqlExpr(sourceRef, mgr.getSQLParser().xSelectIntoStatement(types));
  }
  list = jStatementExpressionList[]
  RCURLY
    { self = new XExecSqlStatement(sourceRef, 
				   conn, 
				   expr, 
				   (CType[])at.dms.compiler.Utils.vectorToArray(types, CType.class),
				   list, 
				   comments); 
    }
;

xProtectedStatement []
  returns [XProtectedStatement self = null]
{
  JExpression		cont = null;
  JExpression		mess = null;
  JBlock		block;
  TokenReference	sourceRef = buildTokenReference();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  PROTECTED_SQL
    { self = new XProtectedStatement(sourceRef, comments); }
  ( cont = xOptionalContext[] )?
  LPAREN ( mess = jExpression[] )?
  RPAREN
  block = jBlock[]
    { self.close(mess, cont, block); }
;

xUnprotectedStatement []
  returns [XUnprotectedStatement self = null]
{
  JExpression		cont = null;
  JExpression		mess = null;
  JBlock		block;
  TokenReference	sourceRef = buildTokenReference();
  JavaStyleComment[]	comments = getStatementComment();
}
:
  UNPROTECTED_SQL
    { self = new XUnprotectedStatement(sourceRef, comments); }
  ( cont = xOptionalContext[] )?
  LPAREN ( mess = jExpression[] )?
  RPAREN
  block = jBlock[]
    { self.close(mess, cont, block); }
;
