/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: xsql.g,v 1.39 1999/10/28 14:03:14 vgp Exp $
 */

header { package at.dms.xkjc; }
{
  import java.util.Vector;

  import at.dms.kjc.*;
  import at.dms.sqlc.*;
  import at.dms.compiler.Compiler;
  import at.dms.compiler.CWarning;
  import at.dms.compiler.HandLexer;
  import at.dms.compiler.JavaStyleComment;
  import at.dms.compiler.TokenReference;
}

// ----------------------------------------------------------------------
// THE PARSER STARTS HERE
// ----------------------------------------------------------------------

class XSQLParser extends SQLParser;

options {
  k = 2;				// two token lookahead
  importVocab = XSQL;			// Call its vocabulary "XSQL"
  exportVocab = XSQL;			// Call its vocabulary "XSQL"
  codeGenMakeSwitchThreshold = 2;	// Some optimizations
  codeGenBitsetTestThreshold = 3;
  defaultErrorHandler = false;		// Don't generate parser error handlers
  superClass="at.dms.compiler.Parser";
}
{
  public XSQLParser(Compiler compiler, XParseManager mgr, HandLexer lexer, String file) {
    super(compiler, lexer, MAX_LOOKAHEAD);
    this.mgr = mgr;
  }

  private XParseManager		mgr;
}

xAnyStatement []
  returns [Statement self = null]
:
  //LCURLY
  (
    self = sDeleteStatement[]
  |
    self = sInsertStatement[]
  |
    self = sUpdateStatement[]
  )
  RCURLY
;

xSelectIntoStatement[Vector types]
 returns [SelectStatement self = null]
{
  String		type = null;
  Vector		selectList;
  FromClause		fromClause;
  WhereClause		whereClause = null;
  GroupByClause		groupByClause = null;
  HavingClause		havingClause = null;
  SortSpec		sortSpec = null;
  UpdSpec		updateSpec = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "SELECT"
  ( "ALL" { type = "ALL"; } | "DISTINCT" { type = "DISTINCT"; } )?
  selectList = xSelectIntoList[types]
  fromClause = sFromClause[]
  ( whereClause = sWhereClause[] )?
  ( groupByClause = sGroupByClause[] )?
  ( havingClause = sHavingClause[] )?
  (
    sortSpec = sSortSpec[]
  |
    updateSpec = sUpdSpec[]
  )?
  {
    self = new SelectStatement(sourceRef,
			       type,
			       selectList,
			       fromClause,
			       whereClause,
			       groupByClause,
			       havingClause,
			       sortSpec,
			       updateSpec);
  }
  "INTO"
;

xSelectIntoList [Vector types]
  returns [Vector self = new Vector()]
{
  SelectElem	p;
  CType		t;
}
:
  t = jType[] p = sSelectElem[]
    { types.addElement(t); self.addElement(new XTypedSqlExpr(p.getTokenReference(), p, t, true)); }
  (
    COMMA t = jType[] p = sSelectElem[]
      { types.addElement(t); self.addElement(new XTypedSqlExpr(p.getTokenReference(), p, t, false)); }
  )*
;

xSelectStatement[]
 returns [XSelectStatement self = null]
{
  String		type = null;
  Vector		selectList;
  FromClause		fromClause;
  WhereClause		whereClause = null;
  GroupByClause		groupByClause = null;
  HavingClause		havingClause = null;
  SortSpec		sortSpec = null;
  UpdSpec		updateSpec = null;
  TokenReference	sourceRef = buildTokenReference();
}
:
  "SELECT"
  ( "ALL" { type = "ALL"; } | "DISTINCT" { type = "DISTINCT"; } )?
  selectList = xTypedSelectList[]
  fromClause = sFromClause[]
  ( whereClause = sWhereClause[] )?
  ( groupByClause = sGroupByClause[] )?
  ( havingClause = sHavingClause[] )?
  (
    sortSpec = sSortSpec[]
  |
    updateSpec = sUpdSpec[]
  )?
  {
    self = new XSelectStatement(sourceRef,
				type,
				selectList,
				fromClause,
				whereClause,
				groupByClause,
				havingClause,
				sortSpec,
				updateSpec);
  }
  RCURLY
;


xTypedSelectList []
  returns [Vector self = new Vector()]
{
  XTypedSelectElem	p;
  int			n = 1;
}
:
  p = xTypedSelectElem[n] { self.addElement(p); n++; }
  ( COMMA p = xTypedSelectElem[n] { self.addElement(p); n++; } )*
;

xTypedSelectElem [int pos]
  returns [XTypedSelectElem self = null]
{
  TokenReference	sourceRef = buildTokenReference();
  Expression		p1 = null;
  String		ident = null;
  CType			p2;
}
:
  ( (sExpression[] "AS")=> p1 = sExpression[] "AS" |) // MAY BE A LITTLE BIT TOO EXPENSIVE ! $$$
  p2 = jType[]
  p3:STANDARD_ID { ident = p3.getText();}
  ( DOT p4:STANDARD_ID { ident += "." + p4.getText(); } )?
    { self = new XTypedSelectElem(sourceRef, p1, p2, ident, pos); }
;

sSortElem[]
 returns [SortElem self = null]
{
  TokenReference	sourceRef = buildTokenReference();
  Expression		p1 = null;
  boolean		ifDesc = false;
  String		s = null;
}
:
  (
    s = sIdentifier[]
      { p1 = new SimpleIdentExpression(sourceRef, s); }
  |
    p1 = sIntegerLiteral[]
  |
    p1 = xJavaExpression[]
  )
  ( "ASC" | "DESC" { ifDesc= true; } )?
    { self = new SortElem(sourceRef, p1, ifDesc); }
;

sTableName[]
  returns [Expression self = null]
{
  String		ident;
  TokenReference	sourceRef = buildTokenReference();
}
:
  (
    ident = sIdentifier[]
      { self = new SimpleIdentExpression(sourceRef, ident); }
  |
    COLON BANG LPAREN
    {
      JExpression	expr = mgr.getXJavaParser().xExpressionInSql();
      self = new SqlExprJava(sourceRef, expr, true);
    }
    //RPAREN
  )
;

sAssignment[]
 returns [Assignment self = null]
{
  Expression		p1 = null;
  String		ident;
  Expression		p2;
  TokenReference	sourceRef = buildTokenReference();
}
:
  (
    ident = sIdentifier[]
      { p1 = new SimpleIdentExpression(sourceRef, ident); }
  |
    COLON BANG LPAREN
      {
        JExpression	expr = mgr.getXJavaParser().xExpressionInSql();
        p1 = new SqlExprJava(sourceRef, expr, true);
      }
    //RPAREN
  )
  ASSIGN p2 = sExpression[]
    { self = new Assignment(sourceRef, p1, p2); }
;

xJavaExpression []
  returns [Expression self = null]
{
  TokenReference	sourceRef = buildTokenReference();
  boolean	asText = false;
  JExpression	expr = null;
  String	str;
}
:
  COLON ( BANG { asText = true; } )?
  (
    str = jName[]
      { expr = new JNameExpression(sourceRef, str, true); }
  |
    LPAREN
    {
      expr = mgr.getXJavaParser().xExpressionInSql();
    }
    //RPAREN
  )
    { self = new SqlExprJava(sourceRef, expr, asText); }
;

xJavaIfExpression []
  returns [JExpression self = null]
:
  {
    self = mgr.getXJavaParser().xExpressionInSql();
  }
;

sSearchCondition []
  returns [SearchCondition self = null]
{
  TokenReference	sourceRef = buildTokenReference();
  Predicate             p;
  String		oper = "";
  boolean		isAnd = false;
  boolean		hasNot = false;
  SearchCondition	right;
}
:
  (
    "NOT" p = sPredicate[]
      {
        self = new SimpleSearchCondition(sourceRef, p);
        self = new NotCondition(sourceRef, self);
      }
  |
    p = sPredicate[]
      {
	self = new SimpleSearchCondition(sourceRef, p);
      }
  )
  (
    (
      "AND" { isAnd = true; oper = "AND"; }
    |
      "OR" { isAnd = false; oper = "OR"; }
    )
    (
      (
        "NOT" {oper += " NOT "; hasNot = true; } )?
        (
          p = sPredicate[]
	    {
	      right = new SimpleSearchCondition(sourceRef, p);
	      if (hasNot) {
		right = new NotCondition(sourceRef, right);
	      }
	      self = isAnd ? (BinarySearchCondition)new AndCondition(sourceRef, self, right)
		: (BinarySearchCondition)new OrCondition(sourceRef, self, right);
	    }
        |
          self = xIfCondition[oper, self]
      )
    )
  )*
;

xIfCondition [String oper, SearchCondition left]
  returns [XIfCondition self = null]
{
  TokenReference	sourceRef = buildTokenReference();
  JExpression		cond;
  Predicate		thenClause;
  Predicate		elseClause = null;
}
:
  KOPI_IF LPAREN cond = xJavaIfExpression[] //RPAREN
  thenClause = sPredicate[]
  ( "else" elseClause = sPredicate[] )?
  RCURLY
    {
      self = new XIfCondition(sourceRef, oper, left, cond,
			      new XSqlExpr(sourceRef, thenClause),
			      elseClause == null ? null : new XSqlExpr(sourceRef, elseClause));
    }
;

sSimplePrimary []
 returns [Expression self = null]
{
  TokenReference	sourceRef = buildTokenReference();
  Expression   p1 = null;
  Expression   p2 = null;
  Expression   p3 = null;
}
:
  self = sLiteral[]
|
  p1 = sFieldReference[]
  (
    "SUBRANGE" LPAREN p2 = sExpression[] COMMA p3 = sExpression[] RPAREN
  )?
    { self = new FieldReferencePrimary(sourceRef, p1, p2, p3); }
|
  self = sSubTableExpression[]
|
  self = sIfExpression[]
|
  self = sCaseExpression[]
|
  self = sCountExpression[]
|
  self = sSetFunction[]
|
  self = sJdbcFunction[]
|
  self = xJavaExpression[]
;

// !!!JLS
// A type specification is a type name with possible brackets afterwards
// (which would make it an array type).
// A type name. which is either a (possibly qualified) class name or
// a primitive type
jType []
  returns [CType self = null]
{
  String	name = null;
  int		bounds = 0;
}
:
  (
    name = jName[]
      { self = CClassType.lookup(name); }
  |
    self = jPrimitiveType []
  )
  ( LBRACKET RBRACKET { bounds += 1; } )*
    {
      if (bounds > 0) {
	self = new CArrayType(self, bounds);
      }
    }
;

jPrimitiveType []
  returns [CType self = null]
:
  "fixed" { self = XStdType.PFixed; }
|
  "month" { self = XStdType.PMonth; }
|
  "date" { self = XStdType.PDate; }
|
  "time" { self = XStdType.PTime; }
|
  "boolean" { self = CStdType.Boolean; }
|
  "byte" { self = CStdType.Byte; }
|
  "char" { self = CStdType.Char; }
|
  "double" { self = CStdType.Double; }
|
  "float" { self = CStdType.Float; }
|
  "int" { self = CStdType.Integer; }
|
  "long" { self = CStdType.Long; }
|
  "short" { self = CStdType.Short; }
|
  "void" { self = CStdType.Void; }
;

// A (possibly-qualified) java name.
jName []
  returns [String self = null]
:
  i:STANDARD_ID { self = i.getText(); }
  (
    options {warnWhenFollowAmbig = false;} :
    DOT j:STANDARD_ID { self += "." + j.getText(); }
  )*
;
