/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Calc.java,v 1.8 1999/06/19 17:18:43 graf Exp $
 */

package at.dms.xlib;

/**
 * This class represents operations on nullable types
 */

public final class Calc {

  /**
   * Dynamic computation of nullable expressions
   */
  public static final java.lang.Integer compute(char op,
						java.lang.Integer l,
						java.lang.Integer r) {
    if (l == null || r == null) {
      return null;
    } else {
      switch (op) {
      case '+':
	return new Integer(l.intValue() + r.intValue());
      case '-':
	return new Integer(l.intValue() - r.intValue());
      case '*':
	return new Integer(l.intValue() * r.intValue());
      case '/':
	return new Integer(l.intValue() / r.intValue());
      case '%':
	return new Integer(l.intValue() % r.intValue());
      default:
	throw new RuntimeException("invalid operator " + op);
      }
    }
  }

  public static final boolean test(java.lang.Integer l,
				   java.lang.Integer r) {
    if (l == null && r == null) {
      return true;
    }
    if (l == null || r == null) {
      return false;
    }
    return l.intValue() == r.intValue();
  }

  public static final java.lang.Integer shift(char op,
					      java.lang.Integer l,
					      java.lang.Integer r) {
    if (l == null || r == null) {
      return null;
    }
      switch (op) {
      case 'l':
	return new java.lang.Integer(l.intValue() << r.intValue());
      case 'r':
	return new java.lang.Integer(l.intValue() >> r.intValue());
      case 'u':
	return new java.lang.Integer(l.intValue() >>> r.intValue());
      default:
	throw new RuntimeException("invalid operator " + op);
      }
  }

  public static final java.lang.Long compute(char op,
					     java.lang.Long l,
					     java.lang.Long r) {
    if (l == null || r == null) {
      return null;
    } else {
      switch (op) {
      case '+':
	return new Long(l.longValue() + r.longValue());
      case '-':
	return new Long(l.longValue() - r.longValue());
      case '*':
	return new Long(l.longValue() * r.longValue());
      case '/':
	return new Long(l.longValue() / r.longValue());
      case '%':
	return new Long(l.longValue() % r.longValue());
      default:
	throw new RuntimeException("invalid operator " + op);
      }
    }
  }

  public static final boolean test(java.lang.Long l,
				   java.lang.Long r) {
    if (l == null && r == null) {
      return true;
    }
    if (l == null || r == null) {
      return false;
    }
    return l.longValue() == r.longValue();
  }

  public static final java.lang.Long shift(char op,
					   java.lang.Long l,
					   java.lang.Integer r) {
    if (l == null || r == null) {
      return null;
    }
      switch (op) {
      case 'l':
	return new java.lang.Long(l.longValue() << r.intValue());
      case 'r':
	return new java.lang.Long(l.longValue() >> r.intValue());
      case 'u':
	return new java.lang.Long(l.longValue() >>> r.intValue());
      default:
	throw new RuntimeException("invalid operator " + op);
      }
  }

  public static final java.math.BigDecimal compute(char op,
						   java.math.BigDecimal l,
						   java.math.BigDecimal r) {
    if (l == null || r == null) {
      return null;
    } else {
      switch (op) {
      case '+':
	return l.add(r);
      case '-':
	return l.subtract(r);
      case '*':
	return l.multiply(r);
      case '/':
	return l.divide(r, java.math.BigDecimal.ROUND_HALF_UP);
      default:
	throw new RuntimeException("invalid operator " + op);
      }
    }
  }

  public static final boolean test(java.math.BigDecimal l,
				   java.math.BigDecimal r) {
    if (l == null && r == null) {
      return true;
    }
    if (l == null || r == null) {
      return false;
    }
    return l.equals(r);
  }

  public static final java.lang.Float compute(char op,
					      java.lang.Float l,
					      java.lang.Float r) {
    if (l == null || r == null) {
      return null;
    } else {
      switch (op) {
      case '+':
	return new Float(l.doubleValue() + r.doubleValue());
      case '-':
	return new Float(l.doubleValue() - r.doubleValue());
      case '*':
	return new Float(l.doubleValue() * r.doubleValue());
      case '/':
	return new Float(l.doubleValue() / r.doubleValue());
      default:
	throw new RuntimeException("invalid operator " + op);
      }
    }
  }

  public static final boolean test(java.lang.Float l,
				   java.lang.Float r) {
    if (l == null && r == null) {
      return true;
    }
    if (l == null || r == null) {
      return false;
    }
    return l.floatValue() == r.floatValue();
  }

  public static final java.lang.Double compute(char op,
					       java.lang.Double l,
					       java.lang.Double r) {
    if (l == null || r == null) {
      return null;
    } else {
      switch (op) {
      case '+':
	return new Double(l.doubleValue() + r.doubleValue());
      case '-':
	return new Double(l.doubleValue() - r.doubleValue());
      case '*':
	return new Double(l.doubleValue() * r.doubleValue());
      case '/':
	return new Double(l.doubleValue() / r.doubleValue());
      default:
	throw new RuntimeException("invalid operator " + op);
      }
    }
  }

  public static final boolean test(java.lang.Double l,
				   java.lang.Double r) {
    if (l == null && r == null) {
      return true;
    }
    if (l == null || r == null) {
      return false;
    }
    return l.doubleValue() == r.doubleValue();
  }

  public static final java.math.BigDecimal shift(char op,
						 java.math.BigDecimal l,
						 java.lang.Integer r) {
    if (l == null || r == null) {
      return null;
    }
    switch (op) {
    case 'l':
      return l.setScale(r.intValue(), java.math.BigDecimal.ROUND_UP);
    case 'r':
      return l.setScale(r.intValue(), java.math.BigDecimal.ROUND_FLOOR);
    case 'u':
      return l.setScale(r.intValue(), java.math.BigDecimal.ROUND_HALF_UP);
    default:
      throw new RuntimeException("invalid operator " + op);
    }
  }

  // --------------------------------------------------------------------
  // Computation methods on nullable booleans
  // --------------------------------------------------------------------

  public static final java.lang.Boolean compute(String op,
						java.lang.Boolean l,
						java.lang.Boolean r) {
    if (op.equals("&")) {
      if (l == null) {
	return (r == null || r.booleanValue()) ?
	  null : java.lang.Boolean.FALSE;
      } else if (l.booleanValue()) {
	return r;
      } else {
	return java.lang.Boolean.FALSE;
      }
    } else if (op.equals("|")) {
      if (l == null) {
	return (r == null || ! r.booleanValue()) ?
	  null : java.lang.Boolean.TRUE;
      } else if (l.booleanValue()) {
	return java.lang.Boolean.TRUE;
      } else {
	return r;
      }
    } else if (op.equals("^")) {
      if (l == null || r == null) {
	return null;
      } else {
	return new java.lang.Boolean(l.booleanValue() ^ r.booleanValue());
      }
    } else {
      throw new RuntimeException("invalid operator " + op);
    }
  }

  public static final boolean test(java.lang.Boolean l,
				   java.lang.Boolean r) {
    if (l == null && r == null) {
      return true;
    }
    if (l == null || r == null) {
      return false;
    }
    return l.booleanValue() == r.booleanValue();
  }

  // --------------------------------------------------------------------
  // Computation methods on date objects
  // --------------------------------------------------------------------

  /**
   * Adds (subtracts) a number of days to (from) a nullable date
   */
  public static final java.sql.Date compute(char op,
					    java.sql.Date l,
					    java.lang.Integer r) {
    if (l == null || r == null) {
      return null;
    } else {
      switch (op) {
      case '+':
	return new java.sql.Date(l.getTime() + r.intValue() * 86400000L);
      case '-':
	return new java.sql.Date(l.getTime() - r.intValue() * 86400000L);
      default:
	throw new RuntimeException("invalid operator " + op);
      }
    }
  }

  /**
   * Returns the number of days between two nullable dates.
   */
  public static final java.lang.Integer compute(char op,
						java.sql.Date l,
						java.sql.Date r) {
    if (l == null || r == null) {
      return null;
    } else {
      switch (op) {
      case '-':
	return new java.lang.Integer((int)((l.getTime() - r.getTime()) / 86400000L));
      default:
	throw new RuntimeException("invalid operator " + op);
      }
    }
  }

  public static final boolean test(java.sql.Date l,
				   java.sql.Date r) {
    if (l == null && r == null) {
      return true;
    }
    if (l == null || r == null) {
      return false;
    }
    return l.equals(r);
  }

  // --------------------------------------------------------------------
  // Computation methods on month objects
  // --------------------------------------------------------------------

  /**
   * Adds (subtracts) a number of days to (from) a date
   */
  public static final at.dms.xlib.Month compute(char op,
						at.dms.xlib.Month l,
						int r) {
    if (l == null) {
      throw new RuntimeException("unexpected null arguments");
    } else {
      switch (op) {
      case '+':
	return new at.dms.xlib.Month(l.getScalar() + r);
      case '-':
	return new at.dms.xlib.Month(l.getScalar() - r);
      default:
	throw new RuntimeException("invalid operator " + op);
      }
    }
  }

  /**
   * Adds (subtracts) a number of days to (from) a nullable date
   */
  public static final Month compute(char op,
				    Month l,
				    java.lang.Integer r) {
    if (l == null || r == null) {
      return null;
    } else {
      switch (op) {
      case '+':
	return new Month(l.getScalar() + r.intValue());
      case '-':
	return new Month(l.getScalar() - r.intValue());
      default:
	throw new RuntimeException("invalid operator " + op);
      }
    }
  }

  /**
   * Returns the number of days between two nullable dates.
   */
  public static final java.lang.Integer compute(char op,
						Month l,
						Month r) {
    if (l == null || r == null) {
      return null;
    } else {
      switch (op) {
      case '-':
	return new java.lang.Integer((int)((l.getMonth() - r.getMonth())));
      default:
	throw new RuntimeException("invalid operator " + op);
      }
    }
  }

  public static final boolean test(Month l,
				   Month r) {
    if (l == null && r == null) {
      return true;
    }
    if (l == null || r == null) {
      return false;
    }
    return l.equals(r);
  }

  /**
   * Compares two date objects.
   *
   * Returns -1, 0, or 1 as the first argument is less than, equal to,
   * or greater than the second argument.
   */
  public static final int compare(Month l, Month r) {
    long	diff = l.getScalar() - r.getScalar();

    return diff == 0 ? 0 : (diff > 0 ? 1 : -1);
  }

  // --------------------------------------------------------------------
  // Computation methods on time objects
  // --------------------------------------------------------------------

  /**
   * Adds (subtracts) a number of days to (from) a nullable time
   */
  public static final java.sql.Time compute(char op,
					    java.sql.Time l,
					    java.lang.Integer r) {
    if (l == null || r == null) {
      return null;
    } else {
      switch (op) {
      case '+':
	return new java.sql.Time(l.getTime() + r.intValue() * 1000L);
      case '-':
	return new java.sql.Time(l.getTime() - r.intValue() * 1000L);
      default:
	throw new RuntimeException("invalid operator " + op);
      }
    }
  }

  /**
   * Returns the number of seconds between two nullable time objects.
   */
  public static final java.lang.Integer compute(char op,
						java.sql.Time l,
						java.sql.Time r) {
    if (l == null || r == null) {
      return null;
    } else {
      switch (op) {
      case '-':
	return new java.lang.Integer((int)((l.getTime() - r.getTime()) / 1000L));
      default:
	throw new RuntimeException("invalid operator " + op);
      }
    }
  }

  // --------------------------------------------------------------------
  // Conversion methods from nullable to primitives types
  // --------------------------------------------------------------------

  public static java.lang.Byte toByte(java.lang.Number n) {
    return n == null ? null : new java.lang.Byte(n.byteValue());
  }

  public static java.lang.Short toShort(java.lang.Number n) {
    return n == null ? null : new java.lang.Short(n.shortValue());
  }

  public static java.lang.Integer toInteger(java.lang.Number n) {
    return n == null ? null : new java.lang.Integer(n.intValue());
  }

  public static java.lang.Long toLong(java.lang.Number n) {
    return n == null ? null : new java.lang.Long(n.longValue());
  }

  public static java.math.BigDecimal toBigDecimal(java.lang.Number n) {
    if (n == null) {
      return null;
    } else if (n instanceof java.lang.Float || n instanceof java.lang.Double) {
      return new java.math.BigDecimal(n.floatValue());
    } else {
      return new java.math.BigDecimal(n.longValue());
    }
  }

  public static java.lang.Float toFloat(java.lang.Number n) {
    return n == null ? null : new java.lang.Float(n.floatValue());
  }

  public static java.lang.Double toDouble(java.lang.Number n) {
    return n == null ? null : new java.lang.Double(n.doubleValue());
  }
}
