/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Connection.java,v 1.15 1999/08/25 18:04:34 graf Exp $
 */

package at.dms.xlib;

import java.sql.Driver;
import java.sql.SQLException;
import java.sql.DriverManager;

/**
 * A kopi connection maintain information about current context, underlying
 * JDBC connection, driver and user.
 */
public class Connection {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Creates a kopi connection from an sql one
   *
   * @param	ctxt		the database context
   * @param	conn		the underlying JDBC connection
   * @param	driver		the JDBC driver for tis connection
   * @param	userName	the name of the user
   */
  public Connection(DBContext ctxt,
		    java.sql.Connection conn,
		    Driver driver,
		    String url,
		    String userName,
		    String pass) {
    this.conn = conn;
    this.ctxt = ctxt;
    this.driver = driver;
    this.url = url;
    this.userName = userName;
    this.pass = pass;

    if (!getUserName().equals("tbadmin")) {
      // get user ID
      try {
	ctxt.startWork();	// !!! BEGIN_SYNC

	Query	query = new Query(this);

	query.open("SELECT ID FROM BENUTZER WHERE Kurzname = '" + getUserName() + "'");

	if (!query.next()) {
	  throw new SQLException("user unknown");
	}

	this.userID = query.getInt(1);

	if (query.next()) {
	  throw new SQLException("different users with same name");
	}

	query.close();

	ctxt.commitWork();
	// manually commit ourself since we are not in the connection list of the context
	conn.commit();
      } catch (java.sql.SQLException e) {
	System.err.println("FATAL ERROR : " + e.getMessage());
	e.printStackTrace();
	throw new RuntimeException(e.getMessage());
      }
    }
 }

  /**
   *
   */
  public DBContext getDBContext() {
    return ctxt;
  }

  /**
   * Interrupt
   */
  public void interrupt() throws SQLException {
    conn.close();
    conn = DriverManager.getConnection(url, userName, pass);
  }

  /**
   *
   */
  public java.sql.Connection getJDBCConnection() {
    if (!ctxt.isInTransaction()) {
      throw new RuntimeException(">>>>>>>>>>> SQL STATEMENT OUTSIDE OF ANY TRANSACTION <<<<<<<<<<<<");
    }
    return conn;
  }

  /**
   *
   */
  public void setTransactionIsolation(int isolation) throws SQLException {
    conn.setTransactionIsolation(isolation);
  }

  /**
   *
   */
  public String getUserName() {
    return userName;
  }

  /**
   *
   */
  public String getPassword() {
    return pass;
  }

  /**
   *
   */
  public int getUserID() {
    return userID;
  }

  // ----------------------------------------------------------------------
  // DRIVER SPECIFIC METHODS
  // ----------------------------------------------------------------------

  /**
   *
   */
  public Driver getDriver() {
    return driver;
  }

  /**
   *
   */
  public DBExceptionConverter getDBExceptionConverter() {
    return DBContext.getDBExceptionConverter(driver);
  }

  /**
   *
   */
  public DBException convert(java.sql.SQLException exc) {
    return DBContext.getDBExceptionConverter(driver).convert(exc);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private	java.sql.Driver		driver;
  private	java.sql.Connection	conn;
  private	DBContext		ctxt;
  private	String			userName;
  private	int			userID;
  private	String			pass;
  private	String			url;
}
