/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: DBContextHandler.java,v 1.7 1999/08/25 18:04:34 graf Exp $
 */

package at.dms.xlib;

/**
 * Interface for object that executes transactions and query to databases.
 */
public interface DBContextHandler {

  /**
   * Sets the database context for this object
   *
   * @param	context		a database context
   */
  void setDBContext(DBContext context);

  /**
   * Returns the database context for this object.
   */
  DBContext getDBContext();

  /**
   * Starts a protected transaction.
   *
   * @param	message		the message to be displayed
   */
  void startProtected(String message);

  /**
   * Commits a protected transaction.
   */
  void commitProtected() throws Exception;

  /**
   * Aborts a protected transaction.
   *
   * @param	reason		the reason for the failure
   */
  void abortProtected(Exception reason) throws Exception;

  /**
   * Returns whether this object handles a transaction at this time.
   */
  boolean inTransaction();
}
