/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: DBTransBaseExceptionConverter.java,v 1.8 1999/06/19 17:18:43 graf Exp $
 */

package at.dms.xlib;

import java.sql.SQLException;

public class DBTransBaseExceptionConverter implements DBExceptionConverter {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructor
   */
  public DBTransBaseExceptionConverter() {
    // add here some informations about driver as needed
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Transforms an SQLException into its corresponding kopi DBException
   *
   * @param	from		the SQLException
   * @return	the corresponding kopi DBException
   */
  public DBException convert(SQLException from) {
    switch (from.getErrorCode()) {
    case 2414:	// LM_DB_LOCKED
    case 2416:	// LOCKINFO_FAULT
    case 2418:	// HARD_LOCKS_NOT_GRANTABLE
    case 2420:	// HARD_LOCK_TIMEOUT
    case 11004: // CT_TROUBLE
    case 16002:	// LOCKS_NOT_GRANTABLE
    case 16003:	// LOCK_TIMEOUT
      return new DBDeadLockException(from);

    case 9147:	// DUPLIC_IN_INDEX
      return parseDuplicateIndex(from);

    case 9146:	// INTEGRITY_VIOL2
      return parseIntegrityViolation(from);

    case 9094:	// INTEGRITY_VIOL1
    case 9148:	// NULL_ERROR
    case 9149:	// NULL_ERROR_POS
    case 9150:	// NULL_ERROR_VIEW
    case 9151:	// REL_TUP_INS
      return new DBConstraintException(from);

    default:
      return new DBUnspecifiedException(from);
    }
  }

  // ----------------------------------------------------------------------
  // PRIVATE UTILITIES METHODS
  // ----------------------------------------------------------------------

  /**
   * Parses a transbase duplicate index exception
   */
  private static DBDuplicateIndexException parseDuplicateIndex(SQLException from) {
    String	mesg = from.getMessage();
    int		index = mesg.lastIndexOf("index ");

    return new DBDuplicateIndexException(from, mesg.substring(index + 6, mesg.length() - 2));
  }

  /**
   * Parses a transbase integrity violation exception
   */
  private static DBForeignKeyException parseIntegrityViolation(SQLException from) {
    String	mesg = from.getMessage();
    int		index1 = mesg.indexOf("'");
    int		index2 = mesg.lastIndexOf("'");

    return new DBForeignKeyException(from, mesg.substring(index1 + 1, index2));
  }
}
