/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: DefaultDBContextHandler.java,v 1.8 1999/08/25 18:04:34 graf Exp $
 */

package at.dms.xlib;

public class DefaultDBContextHandler implements DBContextHandler {

  public DefaultDBContextHandler() {

  }

  public DefaultDBContextHandler(DBContext context) {
    this.context = context;
  }

  /**
   * Sets the database context for this object.
   *
   * @param	context		a database context
   */
  public void setDBContext(DBContext context) {
    this.context = context;
  }

  /**
   * Returns the database context for this object.
   */
  public DBContext getDBContext() {
    return context;
  }

  /**
   * Starts a protected transaction.
   *
   * @param message  the message to be displayed
   */
  public void startProtected(String message) {
    isProtected = true;

    context.startWork();
  }


  /**
   * Commits a protected transaction.
   */
  public void commitProtected() {
    try {
      context.commitWork();
    } catch (java.sql.SQLException e) {
      // !!! ignore it: are we sure that it is really aborted ???
    }

    isProtected = false;
  }


  /**
   * Aborts a protected transaction.
   *
   * @param	reason		the reason for the failure
   */
  public void abortProtected(Exception reason) {
    try {
      context.abortWork();
    } catch (java.sql.SQLException e) {
      //!!! ignore it: are we sure that it is really aborted ???
    }

    isProtected = false;
    throw new RuntimeException("RW: " + reason);
  }

  /**
   * Returns whether this object handles a transaction at this time.
   */
  public boolean inTransaction() {
    return isProtected;
  }

  // --------------------------------------------------------------------
  // DATA MEMBERS
  // --------------------------------------------------------------------

  private DBContext             context;
  private boolean		isProtected;
}


