/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: KopiUtils.java,v 1.16 1999/09/12 14:33:39 vgp Exp $
 */

package at.dms.xlib;

import at.dms.xtype.Date;
import at.dms.xtype.Fixed;
import at.dms.xtype.Month;
import at.dms.xtype.Time;

import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;

/**
 * This class represents kopi fixed types (not null)
 */

public class KopiUtils {
  /**
   * Returns the string representation in SQL/JDBC format of a boolean value.
   */
  public static final String toSql(boolean b) {
    return b ? "TRUE" : "FALSE";
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Boolean value.
   */
  public static final String toSql(Boolean b) {
    return b == null ? nullLiteral : toSql(b.booleanValue());
  }

  /**
   * Returns the string representation in SQL/JDBC format of a long value.
   */
  public static final String toSql(int l) {
    return "" + l;
  }

  /**
   * Returns the string representation in SQL/JDBC format of a long value.
   */
  public static final String toSql(long l) {
    return "" + l;
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Byte value.
   */
  public static final String toSql(Byte l) {
    return l == null ? nullLiteral : l.toString();
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Short value.
   */
  public static final String toSql(Short l) {
    return l == null ? nullLiteral : l.toString();
  }

  /**
   * Returns the string representation in SQL/JDBC format of a string value.
   */
  public static final String toSql(String l) {
    if (l == null) {
      return nullLiteral;
    } else {
      StringBuffer b = new StringBuffer();

      b.append('\'');
      for (int i = 0 ; i < l.length() ; ++i) {
	char c = l.charAt(i);

	if (c == '\'') {
	  b.append((char)'\'');
	}
	b.append(c);
      }
      b.append('\'');
      return b.toString();
    }
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Integer value.
   */
  public static final String toSql(Integer l) {
    return l == null ? nullLiteral : l.toString();
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Long value.
   */
  public static final String toSql(Long l) {
    return l == null ? nullLiteral : l.toString();
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Fixed value.
   */
  public static final String toSql(Fixed f) {
    return f == null ? nullLiteral : f.toSql();
  }

  /**
   * Returns the string representation in SQL/JDBC format of a float value.
   */
  public static final String toSql(float f) {
    return "" + f;
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Float value.
   */
  public static final String toSql(Float f) {
    return f == null ? nullLiteral : f.toString();
  }

  /**
   * Returns the string representation in SQL/JDBC format of a double value.
   */
  public static final String toSql(double d) {
    return "" + d;
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Double value.
   */
  public static final String toSql(Double d) {
    return d == null ? nullLiteral : d.toString();
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Double value.
   */
  public static final String toSql(Object o) {
    return o == null ? nullLiteral : "?";
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Double value.
   */
  public static final String toSql(KopiSerializable ser) {
    return ser == null ? nullLiteral : "?";
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Date value.
   */
  public static final String toSql(Date d) {
    return d == null ? nullLiteral : d.toSql();
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Month value.
   */
  public static final String toSql(Month m) {
    return m == null ? nullLiteral : m.toSql();
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Time value.
   */
  public static final String toSql(Time t) {
    return t == null ? nullLiteral : t.toSql();
  }

  /**
   * Returns the string representation in SQL/JDBC format of a char value.
   */
  public static final String toSql(char c) {
    return "" + c;
  }

  /**
   * Returns the string representation in SQL/JDBC format of a Character value.
   */
  public static final String toSql(Character c) {
    return c == null ? nullLiteral : c.toString();
  }

  /**
   * execute an query and close statement
   */
  public static final int executeUpdate(java.sql.Connection conn, String query, Object[] blobs)
    throws DBException
  {
    try {
      if (Query.traceQueries) {
	System.out.println("UPDATE: " + query);
      }

      java.sql.PreparedStatement	stmt = conn.prepareStatement(query);

      if (blobs != null) {
	try {
	  for (int i = 0; i < blobs.length; i++) {
	    if (blobs[i] instanceof KopiSerializable) {
	      byte[]	data = ((KopiSerializable)blobs[i]).toKopiData();
	      stmt.setBinaryStream(i + 1, new ByteArrayInputStream(data), data.length);
	    } else if (blobs[i] instanceof byte[]) {
	      byte[]	data = (byte[])blobs[i];
	      stmt.setBinaryStream(i + 1, new ByteArrayInputStream(data), data.length);
	    } else {
	      ByteArrayOutputStream ostream = new ByteArrayOutputStream();
	      ObjectOutputStream p = new ObjectOutputStream(ostream);
	      p.writeObject(blobs[i]);
	      p.flush();
	      ostream.close();
	      byte[]	data = ostream.toByteArray();
	      stmt.setBinaryStream(i + 1, new ByteArrayInputStream(data), data.length);
	    }
	  }
	} catch (IOException e) {
	  e.printStackTrace();throw new DBInvalidDataException(e);
	}
      }
      int count = stmt.executeUpdate();
      stmt.close();
      return count;
    } catch (SQLException exc) {
      DBExceptionConverter	converter;
      try {
	converter = DBContext.getDBExceptionConverter(conn.getMetaData().getDriverName());
      } catch (SQLException excs) {
	throw new DBRuntimeException("undefined meta data or driver name " + excs.getMessage());
      }
      throw converter.convert(exc);
    }
  }

  /**
   * execute an query and close statement
   */
  public static final int executeUpdate(at.dms.xlib.Connection conn, String query, Object[] blobs)
    throws DBException
  {
    return executeUpdate(conn.getJDBCConnection(), query, blobs);
  }

  /**
   * execute an query and close statement
   */
  public static final int executeUpdate(java.sql.Connection conn, String query) {
    try {
      if (Query.traceQueries) {
	System.out.println("UPDATE: " + query);
      }

      java.sql.Statement	stmt = conn.createStatement();
      int			count = stmt.executeUpdate(query);

      stmt.close();
      return count;
    } catch (SQLException exc) {
      try {
	throw DBContext.getDBExceptionConverter(conn.getMetaData().getDriverName()).convert(exc);
      } catch (SQLException excs) {
	exc.printStackTrace();
	throw new DBRuntimeException("undefined meta data or driver name" + excs.getMessage() + exc.getMessage());
      }
    }
  }

  /**
   * execute an query and close statement
   */
  public static final int executeUpdate(at.dms.xlib.Connection conn, String query) {
    return executeUpdate(conn.getJDBCConnection(), query);
  }

  /**
   * Insert a new record with ID and TS into the database
   *
   * @param	table		the name of the table
   * @param	columns		the columns of the table
   * @param	values		the values of the corresponding columns
   */
  public static final int executeKopiInsert(java.sql.Connection conn,
					    String table,
					    String[] columns,
					    String[] values)
  {
    StringBuffer	text = new StringBuffer();
    int			id = getNextTableId(conn, table);
    int			ts = (int)(System.currentTimeMillis()/1000);

    text.append("INSERT INTO " + table + "(ID, TS");
    for (int i = 0; i < columns.length; i++) {
      text.append(", " + columns[i]);
    }
    text.append(") VALUES (" + id + ", " + ts);
    for (int i = 0; i < columns.length; i++) {
      text.append(", " + values[i]);
    }
    text.append(")");

    executeUpdate(conn, text.toString());

    return id;
  }

  /**
   * INSERT RECORD IN TABLE
   */
  //  public void insert(String table) throws SQLException {
  //    int		id;
  //    int		pos, i;
  //    char			format[1024], *fp;
  //
  //    for (pos = 0; pos < MAX_ATTRNO - 4 && par_tab[pos] != null; pos += 1)
  //      /* loop */;
  //    ASSERT(pos < MAX_ATTRNO - 4);
  //    ASSERT(pos % 2 == 0);
  //
  //    ///////////////////////////////////////////////////////////////////////
  //    // ID AND TIMESTAMP ARE SET IMPLICITLY
  //    ///////////////////////////////////////////////////////////////////////
  //
  //    pos += 1;
  //    addParam(pos, "ID");
  //    pos += 1;
  //    id = getNextTableId(table);
  //    addParam(pos, id);
  //
  //    pos += 1;
  //    addParam(pos, "TS");
  //    pos += 1;
  //    addParam(pos, (int)time(null));
  //
  //    pos /= 2;
  //
  //
  //    ///////////////////////////////////////////////////////////////////////
  //    // CREATE FORMAT
  //    ///////////////////////////////////////////////////////////////////////
  //
  //    fp = format;
  //
  //    sprintf(fp, "INSERT INTO %s (", table); fp += strlen(fp);
  //
  //    for (i = 0; i < pos; i += 1) {
  //      if (i != 0) {
  //	strcpy(fp, ", "); fp += strlen(fp);
  //      }
  //      sprintf(fp, "$%d", 2*i + 1); fp += strlen(fp);
  //    }
  //
  //    strcpy(fp, ") VALUES ("); fp += strlen(fp);
  //
  //    for (i = 0; i < pos; i += 1) {
  //      if (i != 0) {
  //	strcpy(fp, ", "); fp += strlen(fp);
  //      }
  //      sprintf(fp, "#%d", 2*i + 2); fp += strlen(fp);
  //    }
  //
  //    strcpy(fp, ")");
  //
  //    ///////////////////////////////////////////////////////////////////////
  //    // EXCEUTE QUERY AND SET RESULT
  //    ///////////////////////////////////////////////////////////////////////
  //
  //    Run(format);
  //
  //    for (i = 0; i < MAX_ATTRNO; i += 1)
  //      if (att_tab[i] != null)
  //	DELMEM(att_tab[i]);
  //
  //    att_tab[0] = MakeAtom(id);
  //
  //    return;
  //  }
  //

  ///////////////////////////////////////////////////////////////////////

  /**
   * execute an query and close statement
   */
  public static final int executeKopiInsert(at.dms.xlib.Connection conn, String table, String[] columns, String[] exprs)
  {
    return executeKopiInsert(conn.getJDBCConnection(), table, columns, exprs);
  }

  /*
   * Removes all white space characters from both ends of the specified
   * string and replaces multiple spaces between words by a single one.
   *
   * @param	input		the string to trim
   * @return	the trimmed string
   *
   * !!! Note: This method should be moved to at.dms.xlib.Utils
   */
  public static String trimString(String input) {
    char[]	buffer = new char[input.length()];
    int		bufpos = 0;
    int		state  = 0;

    for (int i = 0; i < input.length(); i++) {
      char	c = input.charAt(i);

      if (Character.isWhitespace(c)) {
	state = (state == 0 ? 0 : 2);
      } else {
	if (state == 2) {
	  buffer[bufpos++] = ' ';
	}
	buffer[bufpos++] = c;
	state = 1;
      }
    }

    return bufpos == 0 ? "" : new String(buffer, 0, bufpos);
  }

  /*
   * Removes all white space characters from the end of the specified
   * string.
   *
   * @param	input		the string to trim
   * @return	the trailed string
   *
   * !!! Note: This method should be moved to at.dms.xlib.Utils
   */
  public static String trailString(String input) {
    int			last = -1;

    for (int i = input.length() - 1; last == -1 && i >= 0; --i) {
      if (! Character.isWhitespace(input.charAt(i))) {
	last = i;
      }
    }

    if (last == -1) {
      return "";
    } else if (last == input.length()) {
      return input;
    } else {
      return input.substring(0, last + 1);
    }
  }

  /*
   * ----------------------------------------------------------------------
   * PRIVATE METHOD
   * ----------------------------------------------------------------------
   */

  /*
   * Returns first free ID of table.
   */
  private static int getNextTableId(java.sql.Connection conn, String table) {
    try {
      Statement		stmt;
      ResultSet		rset;
      int		id;

      stmt = conn.createStatement();
      rset = stmt.executeQuery("SELECT MAX(ID) FROM " + table);
      if (!rset.next()) {
	throw new DBRuntimeException("Database Internal Error");
      }
      id = rset.getObject(1) == null ? 1 : rset.getInt(1) + 1;
      stmt.close();

      return id;
    } catch (SQLException exc) {
      try {
	throw DBContext.getDBExceptionConverter(conn.getMetaData().getDriverName()).convert(exc);
      } catch (SQLException excs) {
	exc.printStackTrace();
	throw new DBRuntimeException("undefined meta data or driver name" + excs.getMessage() + exc.getMessage());
      }
    }
  }

  /*
   * ----------------------------------------------------------------------
   * DATA MEMBERS
   * ----------------------------------------------------------------------
   */
  private static final String nullLiteral = "NULL";
}
