/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Query.java,v 1.15 1999/08/26 20:21:33 graf Exp $
 */

package at.dms.xlib;

import at.dms.xtype.Date;
import at.dms.xtype.Fixed;
import at.dms.xtype.Month;
import at.dms.xtype.Time;
import at.dms.xtype.NotNullDate;
import at.dms.xtype.NotNullFixed;
import at.dms.xtype.NotNullMonth;
import at.dms.xtype.NotNullTime;

import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.util.Vector;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.OptionalDataException;
import java.io.IOException;

public class Query {

  /**
   * Creates a query for database connection <conn>.
   */
  public Query(Connection conn) {
    this.conn = conn;
    this.pars = new Vector();
  }

  /**
   * Creates a query.
   */
  public Query(DBContextHandler env) {
    this(env.getDBContext().getDefaultConnection());
  }

  /**
   * Sets the next parameter to a null value.
   */
  public void addNull() {
    pars.addElement(null);
  }

  /**
   * Sets the next parameter to a BigDecimal value.
   */
  public void addBigDecimal(java.math.BigDecimal value) {
    if (value == null) {
      addNull();
    } else {
      pars.addElement(value);
    }
  }

  /**
   * Sets the next parameter to a boolean value.
   */
  public void addBoolean(boolean value) {
    pars.addElement(new Boolean(value));
  }

  /**
   * Sets the next parameter to a date value.
   */
  public void addDate(java.sql.Date value) {
    if (value == null) {
      addNull();
    } else {
      pars.addElement(value);
    }
  }

  /**
   * Sets the next parameter to a month value.
   */
  public void addMonth(Month value) {
    if (value == null) {
      addNull();
    } else {
      pars.addElement(value);
    }
  }

  /**
   * Sets the next parameter to a int value.
   */
  public void addInt(int value) {
    pars.addElement(new Integer(value));
  }

  /**
   * Sets the next parameter to a string value.
   */
  public void addString(String value) {
    if (value == null) {
      addNull();
    } else {
      pars.addElement(value);
    }
  }

  /**
   * Sets the next parameter to a time value.
   */
  public void addTime(java.sql.Time value) {
    if (value == null) {
      addNull();
    } else {
      pars.addElement(value);
    }
  }

  /**
   *
   */
  public void addBlob(InputStream is) {
    blobs.addElement(is);
  }

  /////////////////////////////////////////////////////////////////////////

  /**
   *
   */
  public void open(String format) throws SQLException {
    try {
      buildText(format, "OPEN");

      stmt = conn.getJDBCConnection().createStatement();
      if (supportsCursorNames()) {
	stmt.setCursorName("C" + nextCursorId++);
      }
      rset = stmt.executeQuery(text);
    } catch (SQLException exc) {
      throw conn.convert(exc);
    }
  }

  /**
   *
   */
  public boolean next() throws SQLException {
    try {
      return rset.next();
    } catch (SQLException exc) {
      throw conn.convert(exc);
    }
  }

  /**
   * Positioned update
   */
  public int update(String format) throws SQLException {
    if (! supportsCursorNames()) {
      throw new SQLException("operation not supported by JDBC driver");
    }

    try {
      Statement	updater;
      int		count;

      buildText(format, "UPDPOS");

      if (blobs.size() != 0) {
	updater = conn.getJDBCConnection().prepareStatement(text + " WHERE CURRENT OF " + rset.getCursorName());
	try {
	  for (int i = 0; i < blobs.size(); i++) {
	    ((PreparedStatement)updater).setBinaryStream(i + 1, (InputStream)blobs.elementAt(i), ((InputStream)blobs.elementAt(i)).available());
	  }
	} catch (IOException e) {
	  throw new RuntimeException(e.getMessage());
	}
	count = ((PreparedStatement)updater).executeUpdate();
	updater.close();
      } else {
	updater = conn.getJDBCConnection().createStatement();
	count = updater.executeUpdate(text + " WHERE CURRENT OF " + rset.getCursorName());
	updater.close();
      }
      return count;
    } catch (SQLException exc) {
      throw conn.convert(exc);
    }
  }

  /**
   * Positioned delete
   */
  public int delete(String format) throws java.sql.SQLException {
    if (! supportsCursorNames()) {
      throw new SQLException("operation not supported by JDBC driver");
    }

    try {
      Statement	updater;
      int		count;

      buildText(format, "DELPOS");

      updater = conn.getJDBCConnection().createStatement();
      count = updater.executeUpdate(text + " WHERE CURRENT OF " + rset.getCursorName());
      updater.close();

      return count;
    } catch (SQLException exc) {
      throw conn.convert(exc);
    }
  }

  /**
   *
   */
  public void close() throws java.sql.SQLException {
    try {
      if (stmt != null) {
	stmt.close();
      }
      stmt = null;
      rset = null;
    } catch (SQLException exc) {
      throw conn.convert(exc);
    }
  }

  /**
   *
   */
  public int run(String format) throws SQLException {
    try {
      buildText(format, "RUN");

      if (blobs.size() != 0) {
	stmt = conn.getJDBCConnection().prepareStatement(text);
	try {
	  for (int i = 0; i < blobs.size(); i++) {
	    ((PreparedStatement)stmt).setBinaryStream(i + 1, (InputStream)blobs.elementAt(i), ((InputStream)blobs.elementAt(i)).available());
	  }
	} catch (IOException e) {
	  throw new RuntimeException(e.getMessage());
	}
	return ((PreparedStatement)stmt).executeUpdate();
      } else {
	stmt = conn.getJDBCConnection().createStatement();
	return stmt.executeUpdate(text);
      }
    } catch (SQLException exc) {
      throw conn.convert(exc);
    }
  }


  public boolean getBoolean(int pos) throws SQLException {
    boolean tmp = rset.getBoolean(pos);
    if (rset.wasNull()) {
      throw new SQLException("null pointer exception");
    }
    return tmp;
  }

  public byte getByte(int pos) throws SQLException {
    byte tmp = rset.getByte(pos);
    if (rset.wasNull()) {
      throw new SQLException("null pointer exception");
    }
    return tmp;
  }

  public short getShort(int pos) throws SQLException {
    short tmp = rset.getShort(pos);
    if (rset.wasNull()) {
      throw new SQLException("null pointer exception");
    }
    return tmp;
  }

  public int getInt(int pos) throws SQLException {
    int tmp = rset.getInt(pos);
    if (rset.wasNull()) {
      throw new SQLException("null pointer exception");
    }
    return tmp;
  }

  public NotNullFixed getFixed(int pos) throws SQLException {
    java.math.BigDecimal tmp = rset.getBigDecimal(pos, 4);
    if (rset.wasNull()) {
      throw new SQLException("null pointer exception");
    }
    return new NotNullFixed(tmp);
  }

  public float getFloat(int pos) throws SQLException {
    float tmp = rset.getFloat(pos);
    if (rset.wasNull()) {
      throw new SQLException("null pointer exception");
    }
    return tmp;
  }

  public double getDouble(int pos) throws SQLException {
    double tmp = rset.getDouble(pos);
    if (rset.wasNull()) {
      throw new SQLException("null pointer exception");
    }
    return tmp;
  }

  public char getChar(int pos) throws SQLException {
    String tmp = rset.getString(pos);
    if (rset.wasNull()) {
      throw new SQLException("null pointer exception");
    }
    return tmp.charAt(0);
  }

  public String getString(int pos) throws SQLException {
    return rset.getString(pos);
  }

  public NotNullTime getTime(int pos) throws SQLException {
    java.sql.Time tmp = rset.getTime(pos);
    if (rset.wasNull()) {
      throw new SQLException("null pointer exception");
    }
    return new NotNullTime(tmp);
  }

  public NotNullDate getDate(int pos) throws SQLException {
    java.sql.Date tmp = rset.getDate(pos);
    if (rset.wasNull()) {
      throw new SQLException("null pointer exception");
    }
    return new NotNullDate(tmp);
  }


  public NotNullMonth getMonth(int pos) throws SQLException {
    int tmp = rset.getInt(pos);
    if (rset.wasNull()) {
      throw new SQLException("null pointer exception");
    }
    return new NotNullMonth(tmp);
  }

  public java.lang.Boolean getNullableBoolean(int pos) throws SQLException {
    boolean tmp = rset.getBoolean(pos);
    return rset.wasNull() ? null : new java.lang.Boolean(tmp);
  }

  public java.lang.Byte getNullableByte(int pos) throws SQLException {
    byte tmp = rset.getByte(pos);
    return rset.wasNull() ? null : new java.lang.Byte(tmp);
  }

  public java.lang.Short getNullableShort(int pos) throws SQLException {
    short tmp = rset.getShort(pos);
    return rset.wasNull() ? null : new java.lang.Short(tmp);
  }

  public java.lang.Integer getNullableInt(int pos) throws SQLException {
    int tmp = rset.getInt(pos);
    return rset.wasNull() ? null : new java.lang.Integer(tmp);
  }

  public Fixed getNullableFixed(int pos) throws SQLException {
    java.math.BigDecimal tmp = rset.getBigDecimal(pos, 4);
    if (rset.wasNull()) {
      return null;
    }
    return new NotNullFixed(tmp);
  }

  public java.lang.Float getNullableFloat(int pos) throws SQLException {
    float tmp = rset.getFloat(pos);
    return rset.wasNull() ? null : new java.lang.Float(tmp);
  }

  public java.lang.Double getNullableDouble(int pos) throws SQLException {
    double tmp = rset.getDouble(pos);
    return rset.wasNull() ? null : new java.lang.Double(tmp);
  }

  public java.lang.Character getNullableChar(int pos) throws SQLException {
    String tmp = rset.getString(pos);
    return rset.wasNull() ? null : new java.lang.Character(tmp.charAt(0));
  }

  public String getNullableString(int pos) throws SQLException {
    return getString(pos);
  }

  public Time getNullableTime(int pos) throws SQLException {
    java.sql.Time tmp = rset.getTime(pos);
    if (rset.wasNull()) {
      return null;
    }
    return new NotNullTime(tmp);
  }

  public Date getNullableDate(int pos) throws SQLException {
    java.sql.Date tmp = rset.getDate(pos);
    if (rset.wasNull()) {
      return null;
    }
    return new NotNullDate(tmp);
  }

  public Month getNullableMonth(int pos) throws SQLException {
    int tmp = rset.getInt(pos);
    if (rset.wasNull()) {
      return null;
    }
    return new NotNullMonth(tmp);
  }

  /**
   * Reads the stream in a byte array
   */
  public byte[] getByteArray(int pos) throws SQLException {
    try {
      InputStream	is = rset.getBinaryStream(pos);
      if (is == null) {
	return null;
      }
      byte[]	b = new byte[is.available()];
      is.read(b);
      return b;
    } catch (java.io.IOException e) {
      throw new RuntimeException("INPUT STREAM BROKEN:" + e.getMessage());
    }
  }

  /**
   * Returns an object stored in database
   */
  public Object getSerializedObject(int pos) throws SQLException {
    InputStream is = rset.getBinaryStream(pos);
    if (is == null) {
      return null;
    }

    Object	obj = null;

    try {
      obj = new ObjectInputStream(is).readObject();
      is.close();
    } catch (OptionalDataException e1) {
      throw new DBInvalidDataException(e1);
    } catch (ClassNotFoundException e2) {
      throw new DBInvalidDataException(e2);
    } catch (IOException e3) {
      throw new DBInvalidDataException(e3);
    }

    return obj;
  }

  /**
   * !!! A HACK FOR ORACLE
   */
  public Object getObject(int pos) throws SQLException {
    Object	obj = rset.getObject(pos);

    if (obj == null) {
      return null;
    }

    if (obj instanceof java.math.BigDecimal) {
      if (rset.getMetaData().getScale(pos) != 0) {
	return new at.dms.xtype.NotNullFixed((java.math.BigDecimal)obj);
      } else {
	return new Integer(((java.math.BigDecimal)obj).intValue());
      }
    }

    if (obj instanceof java.sql.Date) {
      return new at.dms.xtype.NotNullDate((java.sql.Date)obj);
    }

    if (obj instanceof java.sql.Time) {
      return new at.dms.xtype.NotNullTime((java.sql.Time)obj);
    }

    return obj;
  }

  public boolean isNull(int pos) throws java.sql.SQLException {
    return rset.getObject(pos) == null;
  }

  /**
   *
   */
  public static void traceQueries(boolean v) {
    traceQueries = v;
  }

  /**
   *
   */
  public static boolean traceQueries() {
    return traceQueries;
  }

  /*
   * ----------------------------------------------------------------------
   * PRIVATE METHODS
   * ----------------------------------------------------------------------
   */

  /*
   * Returns true if the underlying JDBC connection supports cursor names
   */
  private boolean supportsCursorNames() {
    try {
      return conn.getJDBCConnection().getMetaData().getMaxCursorNameLength() > 0;
    } catch (SQLException e) {
      return false;
    }
  }

  /*
   * Builds the query text.
   */
  private void buildText(String format, String type) {
    final int IN_FORMAT	= 1;	// in format string
    final int IN_QUOTE	= 2;	// in quoted string
    final int IN_SQL	= 3;	// expand param to SQL
    final int IN_TEXT	= 4;	// expand param to text
    final int AT_END	= 5;	// end of string

    int		state;
    int         tp = 0;
    int         fp = 0;

    char[]	fpChars;

    state = IN_FORMAT;

    fpChars = format.toCharArray();

    synchronized(tpChars) {
      do {
	switch (state) {
	case IN_FORMAT:
	  switch (fp == fpChars.length ? '\0' : fpChars[fp]) {
	  case '\0':
	    text = new String(tpChars, 0, tp);
	    state = AT_END;
	    break;

	  case '\'':
	    tpChars[tp++] = fpChars[fp++];
	    state = IN_QUOTE;
	    break;

	  case '#':
	    fp += 1;
	    state = IN_SQL;
	    break;

	  case '$':
	    fp += 1;
	    state = IN_TEXT;
	    break;

	  default:
	    tpChars[tp++] = fpChars[fp++];
	  }
	  break;


	case IN_QUOTE:
	  switch (fp == fpChars.length ? '\0' : fpChars[fp]) {
	  case '\0':
	    throw new RuntimeException("unexpected end of query");

	  case '\'':
	    tpChars[tp++] = fpChars[fp++];
	    state = IN_FORMAT;
	    break;

	  default:
	    tpChars[tp++] = fpChars[fp++];
	  }
	  break;


	case IN_SQL:
	case IN_TEXT:
	  {
	    int	parno = 0;

	    while (fp < fpChars.length  && fpChars[fp] >= '0' && fpChars[fp] <= '9') {
	      parno = 10 * parno + (fpChars[fp++] - '0');
	    }

	    if (state == IN_SQL) {
	      tp = getSql(parno - 1, tp, tpChars);
	    } else {
	      tp = getText(parno - 1, tp, tpChars);
	    }

	    state = IN_FORMAT;
	  }
	  break;
	}
      } while (state != AT_END);
    }
    // release parameters
    pars = new Vector();

    if (traceQueries) {
      System.out.println(type + ": " + text);
    }
  }

  /**
   *
   */
  private int getText(int param, int pos, char[] buffer) {
    Object parameter = pars.elementAt(param);

    if (parameter != null) {
      String	text = parameter.toString();
      char[]	textBuffer = text.toCharArray();

      for (int i = 0; i < textBuffer.length; i++) {
	buffer[pos++] = textBuffer[i];
      }
    }

    return pos;
  }

  /**
   *
   */
  private int getSql(int param, int pos, char[] buffer) {
    Object parameter = pars.elementAt(param);
    String text = "";

    if (parameter == null) {
      text = "";
    } else if (parameter instanceof Fixed) {
      text = KopiUtils.toSql((Fixed)parameter);
    } else if (parameter instanceof Boolean) {
      text = KopiUtils.toSql((Boolean)parameter);
    } else if (parameter instanceof Date) {
      text = KopiUtils.toSql((Date)parameter);
    } else if (parameter instanceof Integer) {
      text = KopiUtils.toSql((Integer)parameter);
    } else if (parameter instanceof String) {
      text = KopiUtils.toSql((String)parameter);
    } else if (parameter instanceof Time) {
      text = KopiUtils.toSql((Time)parameter);
    } else if (parameter instanceof Month) {
      text = KopiUtils.toSql((Month)parameter);
    }

    char[] textBuffer = text.toCharArray();
    for (int i = 0; i < textBuffer.length; i++) {
      buffer[pos++] = textBuffer[i];
    }
    return pos;
  }

  /*
   * ----------------------------------------------------------------------
   * DATA MEMBERS
   * ----------------------------------------------------------------------
   */
  static boolean		traceQueries = false;

  private Vector		pars;
  private String 		text;
  private Vector		blobs = new Vector();

  private Connection		conn;
  private Statement		stmt;
  private ResultSet		rset;

  private static int		nextCursorId = 0;
  private static char[]		tpChars = new char[10000];
}
