/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: XFixed.x,v 1.3 1999/09/09 10:43:45 vgp Exp $
 */

package at.dms.xoper;

import at.dms.xtype.Fixed;
import at.dms.xtype.NotNullFixed;

/**
 * This class implements all overloaded operators for doubles and some utilities
 *
 * The class used to represents month is java.sql.Month from the JDK libs
 * We define here every operators than we want to use on month
 *
 * Overloading of operator allows such expression:
 * <PRE>
 * BigDecimal d2 = 1;
 * double	i = d2 - 1;
 * </PRE>
 */
public class XFixed {

  // ----------------------------------------------------------------------
  // SPECIALS OPERATORS
  // ----------------------------------------------------------------------

  /**
   * Allows conversion (Fixed)"123"
   *
   * Returns the value of the string as an double
   */
  public static Fixed operator:()(String m) {
    if (m == null) {
      return null;
    } else {
      return new fixed(m);
    }
  }

  /**
   * Allows conversion "(Fixed)1"
   *
   * Returns the first day of the month
   */
  public static fixed operator:()(double i) {
    return new fixed(i);
  }

  /**
   * Allows conversion "(Fixed)1"
   *
   * Returns the first day of the month
   */
  public static Fixed operator:()(double i) {
    return new fixed(i);
  }

  /**
   * Allows conversion "(Fixed)1"
   *
   * Returns the first day of the month
   */
  public static Fixed operator:()(Number i) {
    if (i == null) {
      return null;
    } else {
      return new fixed(i.doubleValue());
    }
  }

  /**
   * Allows conversion "(Fixed)1"
   *
   * Returns the first day of the month
   */
  public static fixed operator:()(Number i) {
    if (i == null) {
      throw new NullValueInCastOperator("(fixed)null");
    } else {
      return new fixed(i.doubleValue());
    }
  }

  /**
   * Allows conversion "(fixed)Fixed"
   */
  public static fixed operator:()(Fixed i) {
    if (i == null) {
      throw new NullValueInCastOperator("(fixed)null");
    }

    return fixed.castToNotNull(i);
  }

  // ----------------------------------------------------------------------
  // ASSIGNMENT OPERATOR
  // ----------------------------------------------------------------------

  /**
   * Assignments from double to Fixed
   */
  public static fixed operator:=(double i) {
    return new fixed(i);
  }

  /**
   * Assignments from double to Fixed
   */
  public static Fixed operator:=(double i) {
    return new fixed(i);
  }

  /**
   * Assignments from double to Fixed
   */
  public static Fixed operator:=(Number i) {
    return new fixed(i.doubleValue());
  }

  /**
   * Assignment form Fixed to double
   */
  //  public static double operator:=(Number i) {
  //  return i.doubleValue();
  //}

  // ----------------------------------------------------------------------
  // UNARY OPERATORS  DOUBLES ARE INMUTABLE => the compiler will
  //                  transform i++ into (i += 1)
  // ----------------------------------------------------------------------

  /*
   * Operator "*++"
   *
   * This add a double to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the operand
   * @return	the value for l++
   */
  //public static Fixed operator:*++(Fixed l) {
  //}

  /*
   * Operator "++*"
   *
   * This add a double to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the operand
   * @return	the value for ++l
   */
  //public static Fixed operator:++*(Fixed l) {
  //}

  /*
   * Operator "*--"
   *
   * This substract a double to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the operand
   * @return	the value for l--
   */
  //public static Fixed operator:*--(Fixed l) {
  //}

  /*
   * Operator "--*"
   *
   * This substract a double to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the operand
   * @return	the value for --l
   */
  //public static Fixed operator:--*(Fixed l) {
  //}

  // ----------------------------------------------------------------------
  // BINARY OPERATORS (COMPARISON)
  // ----------------------------------------------------------------------

  /**
   * Operator "=="
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l == r
   */
  public static boolean operator:==(Fixed l, Number r) {
    if (l == null && r == null) {
      return true;
    } else if (l == null || r == null) {
      return false;
    } else {
      return l.equals(r);
    }
  }

  /**
   * Operator "!="
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l == r
   */
  public static boolean operator:!=(Fixed l, Number r) {
    if (l == null && r == null) {
      return false;
    } else if (l == null || r == null) {
      return true;
    } else {
      return !l.equals(r);
    }
  }

  /**
   * Operator ">"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l > r
   */
  public static boolean operator:>(Fixed l, Number r) {
    if (l == null || r == null) {
      throw new NullValueInComparison("" + l + ">" + r);
    }
    return l.doubleValue() > r.doubleValue();
  }

  /**
   * Operator ">="
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l >= r
   */
  public static boolean operator:>=(Fixed l, Number r) {
    if (l == null || r == null) {
      throw new NullValueInComparison("" + l + ">=" + r);
    }
    return l.doubleValue() >= r.doubleValue();
  }

  /**
   * Operator "<"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l < r
   */
  public static boolean operator:<(Fixed l, Number r) {
    if (l == null || r == null) {
      throw new NullValueInComparison("" + l + "<" + r);
    }
    return l.doubleValue() < r.doubleValue();
  }

  /**
   * Operator "<="
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l <= r
   */
  public static boolean operator:<=(Fixed l, Number r) {
    if (l == null || r == null) {
      throw new NullValueInComparison("" + l + "<=" + r);
    }
    return l.doubleValue() <= r.doubleValue();
  }

  /**
   * Operator "=="
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l == r
   */
  public static boolean operator:==(Number l, Fixed r) {
    if (l == null && r == null) {
      return true;
    } else if (l == null || r == null) {
      return false;
    } else {
      return l.equals(r);
    }
  }

  /**
   * Operator "=="
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l == r
   */
  public static boolean operator:!=(Number l, Fixed r) {
    if (l == null && r == null) {
      return false;
    } else if (l == null || r == null) {
      return true;
    } else {
    return !l.equals(r);
    }
  }


  /**
   * Operator ">"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l > r
   */
  public static boolean operator:>(Number l, Fixed r) {
    if (l == null || r == null) {
      throw new NullValueInComparison("" + l + ">" + r);
    }
    return l.doubleValue() > r.doubleValue();
  }

  /**
   * Operator ">="
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l >= r
   */
  public static boolean operator:>=(Number l, Fixed r) {
    if (l == null || r == null) {
      throw new NullValueInComparison("" + l + ">=" + r);
    }
    return l.doubleValue() >= r.doubleValue();
  }

  /**
   * Operator "<"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l < r
   */
  public static boolean operator:<(Number l, Fixed r) {
    if (l == null || r == null) {
      throw new NullValueInComparison("" + l + "<" + r);
    }
    return l.doubleValue() < r.doubleValue();
  }

  /**
   * Operator "<="
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l <= r
   */
  public static boolean operator:<=(Number l, Fixed r) {
    if (l == null || r == null) {
      throw new NullValueInComparison("" + l + "<=" + r);
    }
    return l.doubleValue() <= r.doubleValue();
  }

  /**
   * Operator "=="
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l == r
   */
  public static boolean operator:==(Fixed l, Fixed r) {
    if (l == null && r == null) {
      return true;
    } else if (l == null || r == null) {
      return false;
    } else {
      return l.compareTo(r) == 0;
    }
  }

  /**
   * Operator "=="
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l == r
   */
  public static boolean operator:!=(Fixed l, Fixed r) {
    if (l == null && r == null) {
      return false;
    } else if (l == null || r == null) {
      return true;
    } else {
      return l.compareTo(r) != 0;
    }
  }

  /**
   * Operator ">"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l > r
   */
  public static boolean operator:>(Fixed l, Fixed r) {
    if (l == null || r == null) {
      throw new NullValueInComparison("" + l + ">" + r);
    }
    return l.compareTo(r) == 1;
  }

  /**
   * Operator ">="
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l >= r
   */
  public static boolean operator:>=(Fixed l, Fixed r) {
    if (l == null || r == null) {
      throw new NullValueInComparison("" + l + ">=" + r);
    }
    return l.compareTo(r) != -1;
  }

  /**
   * Operator "<"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l < r
   */
  public static boolean operator:<(Fixed l, Fixed r) {
    if (l == null || r == null) {
      throw new NullValueInComparison("" + l + "<" + r);
    }
    return l.compareTo(r) == -1;
  }

  /**
   * Operator "<="
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l <= r
   */
  public static boolean operator:<=(Fixed l, Fixed r) {
    if (l == null || r == null) {
      throw new NullValueInComparison("" + l + "<=" + r);
    }
    return l.compareTo(r) != 1;
  }

  /**
   * Operator "=="
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l == r
   */
  public static boolean operator:==(double l, Fixed r) {
    if (r == null) {
      return false;
    } else {
      return l == r.doubleValue();
    }
  }

  /**
   * Operator "=="
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l == r
   */
  public static boolean operator:!=(double l, Fixed r) {
    if (r == null) {
      return true;
    } else {
      return l != r.doubleValue();
    }
  }

  /**
   * Operator ">"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l > r
   */
  public static boolean operator:>(double l, Fixed r) {
    if (r == null) {
      throw new NullValueInComparison("" + l + ">" + r);
    }
    return l > r.doubleValue();
  }

  /**
   * Operator ">="
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l >= r
   */
  public static boolean operator:>=(double l, Fixed r) {
    if (r == null) {
      throw new NullValueInComparison("" + l + ">=" + r);
    }
    return l >= r.doubleValue();
  }

  /**
   * Operator "<"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l < r
   */
  public static boolean operator:<(double l, Fixed r) {
    if (r == null) {
      throw new NullValueInComparison("" + l + "<" + r);
    }
    return l < r.doubleValue();
  }

  /**
   * Operator "<="
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l <= r
   */
  public static boolean operator:<=(double l, Fixed r) {
    if (r == null) {
      throw new NullValueInComparison("" + l + "<=" + r);
    }
    return l <= r.doubleValue();
  }

  /**
   * Operator "=="
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l == r
   */
  public static boolean operator:==(Fixed l, double r) {
    if (l == null) {
      return false;
    } else {
      return l.doubleValue() == r;
    }
  }

  /**
   * Operator "=="
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l == r
   */
  public static boolean operator:!=(Fixed l, double r) {
    if (l == null) {
      return true;
    } else {
      return l.doubleValue() != r;
    }
  }

  /**
   * Operator ">"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l > r
   */
  public static boolean operator:>(Fixed l, double r) {
    if (l == null) {
      throw new NullValueInComparison("" + l + ">" + r);
    }
    return l.doubleValue() > r;
  }

  /**
   * Operator ">="
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l >= r
   */
  public static boolean operator:>=(Fixed l, double r) {
    if (l == null) {
      throw new NullValueInComparison("" + l + ">=" + r);
    }
    return l.doubleValue() >= r;
  }

  /**
   * Operator "<"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l < r
   */
  public static boolean operator:<(Fixed l, double r) {
    if (l == null) {
      throw new NullValueInComparison("" + l + "<" + r);
    }
    return l.doubleValue() < r;
  }

  /**
   * Operator "<="
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l <= r
   */
  public static boolean operator:<=(Fixed l, double r) {
    if (l == null) {
      throw new NullValueInComparison("" + l + "<=" + r);
    }
    return l.doubleValue() <= r;
  }

  // ----------------------------------------------------------------------
  // ADD OPERATOR
  // ----------------------------------------------------------------------

  /**
   * Operator "+"
   *
   * This add r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l + r
   */
  public static Fixed operator:+(Fixed l, double r) {
    if (l == null) {
      return null;
    }
    return l.add(new fixed(r));
  }

  /**
   * Operator "+"
   *
   * This add r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l + r
   */
  public static fixed operator:+(fixed l, double r) {
    if (l == null) {
      return null;
    }
    return l.add(new fixed(r));
  }

  /**
   * Operator "+"
   *
   * This add r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l + r
   */
  public static Fixed operator:+(Fixed l, Fixed r) {
    if (l == null || r == null) {
      return null;
    }
    return l.add((NotNullFixed)r);
  }

  /**
   * Operator "+"
   *
   * This add r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l + r
   */
  public static fixed operator:+(fixed l, fixed r) {
    if (l == null || r == null) {
      throw new NullValueInOperation(l + " + " + r);
    }
    return l.add((NotNullFixed)r);
  }

  /**
   * Operator "+"
   *
   * This add l doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l + r
   */
  public static Fixed operator:+(double l, Fixed r) {
    if (r == null) {
      return null;
    }
    return r.add(new fixed(l));
  }

  /**
   * Operator "+"
   *
   * This add l doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l + r
   */
  public static fixed operator:+(double l, fixed r) {
    if (r == null) {
      return null;
    }
    return r.add(new fixed(l));
  }

  /**
   * Operator "+"
   *
   * This add r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l + r
   */
  public static Fixed operator:+(Fixed l, Number r) {
    if (l == null || r == null) {
      return null;
    }
    return l.add(new fixed(r.doubleValue()));
  }

  /**
   * Operator "+"
   *
   * This add r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l + r
   */
  public static Fixed operator:+(Number l, Fixed r) {
    if (l == null || r == null) {
      return null;
    }
    return r.add(new fixed(l.doubleValue()));
  }

  // ----------------------------------------------------------------------
  // MINUS OPERATOR
  // ----------------------------------------------------------------------

  /**
   * Operator "-"
   *
   * This minus r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l - r
   */
  public static Fixed operator:-(Fixed l, double r) {
    if (l == null) {
      return null;
    }
    return l.subtract(new fixed(r));
  }

  /**
   * Operator "-"
   *
   * This minus r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l - r
   */
  public static fixed operator:-(fixed l, double r) {
    if (l == null) {
      return null;
    }
    return l.subtract(new fixed(r));
  }

  /**
   * Operator "-"
   *
   * This minus r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l - r
   */
  public static Fixed operator:-(Fixed l, Fixed r) {
    if (l == null || r == null) {
      return null;
    }
    return l.subtract((NotNullFixed)r);
  }

  /**
   * Operator "-"
   *
   * This minus r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l - r
   */
  public static fixed operator:-(fixed l, fixed r) {
    if (l == null || r == null) {
      throw new NullValueInOperation(l + " - " + r);
    }
    return l.subtract((NotNullFixed)r);
  }

  /**
   * Operator "-"
   *
   * This minus l doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l - r
   */
  public static Fixed operator:-(double l, Fixed r) {
    if (r == null) {
      return null;
    }
    return new fixed(l).setScale(r.getScale()).subtract((NotNullFixed)r);
  }

  /**
   * Operator "-"
   *
   * This minus l doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l - r
   */
  public static fixed operator:-(double l, fixed r) {
    if (r == null) {
      return null;
    }
    return new fixed(l).subtract((NotNullFixed)r);
  }

  /**
   * Operator "-"
   *
   * This minus r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l - r
   */
  public static Fixed operator:-(Fixed l, Number r) {
    if (l == null || r == null) {
      return null;
    }
    return l.subtract(new fixed(r.doubleValue()));
  }

  /**
   * Operator "-"
   *
   * This minus r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l - r
   */
  public static Fixed operator:-(Number l, Fixed r) {
    if (l == null || r == null) {
      return null;
    }
    return new fixed(l.doubleValue()).subtract((NotNullFixed)r);
  }

  // ----------------------------------------------------------------------
  // MULTIPLICATIVE OPERATOR
  // ----------------------------------------------------------------------

  /**
   * Operator "*"
   *
   * This multiplicative r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l * r
   */
  public static Fixed operator:*(Fixed l, double r) {
    if (l == null) {
      return null;
    }
    return l.multiply(new fixed(r));
  }

  /**
   * Operator "*"
   *
   * This multiplicative r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l * r
   */
  public static fixed operator:*(fixed l, double r) {
    if (l == null) {
      return null;
    }
    return l.multiply(new fixed(r));
  }

  /**
   * Operator "*"
   *
   * This multiplicative r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l * r
   */
  public static fixed operator:*(fixed l, fixed r) {
    if (l == null || r == null) {
      throw new NullValueInOperation(l + " * " + r);
    }
    return l.multiply((NotNullFixed)r);
  }

  /**
   * Operator "*"
   *
   * This multiplicative r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l * r
   */
  public static Fixed operator:*(Fixed l, Fixed r) {
    if (l == null || r == null) {
      return null;
    }
    return l.multiply((NotNullFixed)r);
  }

  /**
   * Operator "*"
   *
   * This multiplicative l doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l * r
   */
  public static Fixed operator:*(double l, Fixed r) {
    if (r == null) {
      return null;
    }
    return new fixed(l).multiply((NotNullFixed)r);
  }

  /**
   * Operator "*"
   *
   * This multiplicative l doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l * r
   */
  public static fixed operator:*(double l, fixed r) {
    if (r == null) {
      return null;
    }
    return new fixed(l).multiply((NotNullFixed)r);
  }

  /**
   * Operator "*"
   *
   * This multiplicative r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l * r
   */
  public static Fixed operator:*(Fixed l, Number r) {
    if (l == null || r == null) {
      return null;
    }
    return l.multiply(new fixed(r.doubleValue()));
  }

  /**
   * Operator "*"
   *
   * This multiplicative r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l * r
   */
  public static Fixed operator:*(Number l, Fixed r) {
    if (l == null || r == null) {
      return null;
    }
    return new fixed(l.doubleValue()).multiply((NotNullFixed)r);
  }

  // ----------------------------------------------------------------------
  // DIVIDE OPERATOR
  // ----------------------------------------------------------------------

  /**
   * Operator "/"
   *
   * This divide r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l / r
   */
  public static Fixed operator:/(Fixed l, double r) {
    if (l == null) {
      return null;
    }
    return l.divide(new fixed(r));
  }

  /**
   * Operator "/"
   *
   * This divide r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l / r
   */
  public static fixed operator:/(fixed l, double r) {
    if (l == null) {
      return null;
    }
    return l.divide(new fixed(r));
  }

  /**
   * Operator "/"
   *
   * This divide r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l / r
   */
  public static Fixed operator:/(Fixed l, Fixed r) {
    if (l == null || r == null) {
      return null;
    }
    return l.divide((NotNullFixed)r);
  }

  /**
   * Operator "/"
   *
   * This divide r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l / r
   */
  public static fixed operator:/(fixed l, fixed r) {
    if (l == null || r == null) {
      throw new NullValueInOperation(l + " / " + r);
    }
    return l.divide(r);
  }

  /**
   * Operator "/"
   *
   * This divide l doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l / r
   */
  public static Fixed operator:/(double l, Fixed r) {
    if (r == null) {
      return null;
    }
    return new fixed(l).divide((NotNullFixed)r);
  }

  /**
   * Operator "/"
   *
   * This divide l doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l / r
   */
  public static fixed operator:/(double l, fixed r) {
    if (r == null) {
      return null;
    }
    return new fixed(l).divide((NotNullFixed)r);
  }

  /**
   * Operator "/"
   *
   * This divide r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l / r
   */
  public static Fixed operator:/(Fixed l, Number r) {
    if (l == null || r == null) {
      return null;
    }
    return l.divide(new fixed(r.doubleValue()));
  }

  /**
   * Operator "/"
   *
   * This divide r doubles to the double
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l / r
   */
  public static Fixed operator:/(Number l, Fixed r) {
    if (l == null || r == null) {
      return null;
    }
    return new fixed(l.doubleValue()).divide((NotNullFixed)r);
  }

  // ----------------------------------------------------------------------
  // BINARY OPERATORS (SHIFTS)
  // ----------------------------------------------------------------------

  /**
   * Operator ">>"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l >> r
   */
  public static Fixed operator:>>(Fixed l, Number r) {
    if (l == null || r == null) {
      return null;
    }
    return l.setScale(r.intValue(), java.math.BigDecimal.ROUND_FLOOR);
  }

  /**
   * Operator ">>>"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l >>> r
   */
  public static Fixed operator:>>>(Fixed l, Number r) {
    if (l == null || r == null) {
      return null;
    }
    return l.setScale(r.intValue(), java.math.BigDecimal.ROUND_HALF_UP);
  }

  /**
   * Operator "<<"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l << r
   */
  public static Fixed operator:<<(Fixed l, Number r) {
    if (l == null || r == null) {
      return null;
    }
    return l.setScale(r.intValue(), java.math.BigDecimal.ROUND_UP);
  }

  /**
   * Operator ">>"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l >> r
   */
  public static Fixed operator:>>(Fixed l, int r) {
    if (l == null) {
      return null;
    }
    return l.setScale(r, java.math.BigDecimal.ROUND_FLOOR);
  }

  /**
   * Operator ">>>"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l >>> r
   */
  public static Fixed operator:>>>(Fixed l, int r) {
    if (l == null) {
      return null;
    }
    return l.setScale(r, java.math.BigDecimal.ROUND_HALF_UP);
  }

  /**
   * Operator "<<"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l << r
   */
  public static Fixed operator:<<(Fixed l, int r) {
    if (l == null) {
      return null;
    }
    return l.setScale(r, java.math.BigDecimal.ROUND_UP);
  }
  // ----------------------------------------------------------------------
  // BINARY OPERATORS (SHIFTS) NOT NULL
  // ----------------------------------------------------------------------

  /**
   * Operator ">>"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l >> r
   */
  public static fixed operator:>>(fixed l, int r) {
    if (l == null) {
      throw new NullValueInOperation(l + " >> " + r);
    }
    return l.setScale(r, java.math.BigDecimal.ROUND_FLOOR);
  }

  /**
   * Operator ">>>"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l >>> r
   */
  public static fixed operator:>>>(fixed l, int r) {
    if (l == null) {
      throw new NullValueInOperation(l + " >>> " + r);
    }
    return l.setScale(r, java.math.BigDecimal.ROUND_HALF_UP);
  }

  /**
   * Operator "<<"
   *
   * If one operand is null, this will throw a null pointer exception
   *
   * @param	l		the left operand
   * @param	r		the right operand
   * @return	the value for l << r
   */
  public static fixed operator:<<(fixed l, int r) {
    if (l == null) {
      throw new NullValueInOperation(l + " << " + r);
    }
    return l.setScale(r, java.math.BigDecimal.ROUND_UP);
  }
}

