/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Date.java,v 1.7 1999/08/25 18:04:34 graf Exp $
 */

package at.dms.xtype;

import java.util.GregorianCalendar;
import java.text.SimpleDateFormat;

/**
 * This class represents kopi date types
 */
public class Date extends Type {

  /*package*/ Date(String m) {
    this(java.sql.Date.valueOf(m));
  }

  /*package*/ Date(java.sql.Date date) {
    synchronized(calendar) {
      calendar.setTime(date);

      this.date = new java.sql.Date(calendar.getTime().getTime());
    }
  }

  /*package*/ Date(long millis) {
    this(new java.sql.Date(millis));
  }

  /*package*/ Date(int year, int month, int day) {
    // !!! CHANGE IMPLEMENTATION
    synchronized(calendar) {
      calendar.set(java.util.Calendar.YEAR, year);
      calendar.set(java.util.Calendar.MONTH, month - 1);
      calendar.set(java.util.Calendar.DAY_OF_MONTH, day);

      date = new java.sql.Date(calendar.getTime().getTime());
    }
  }

  // ----------------------------------------------------------------------
  // DEFAULT OPERATIONS
  // ----------------------------------------------------------------------

  // HANDLED BY COMPILER, FOR NOW !!!!

  // ----------------------------------------------------------------------
  // OTHER OPERATIONS
  // ----------------------------------------------------------------------

  /**
   * Returns the month number (starts at 1, ends at 12)
   */
  public int getMonth() {
    int		month;
    synchronized(calendar) {
      calendar.setTime(date);
      month = calendar.get(java.util.Calendar.MONTH) + 1;
    }

    return month;
  }

  /**
   * Returns the year of the month (by example 1999 or may be 2000 on year after)
   */
  public int getYear() {
    int		year;
    synchronized(calendar) {
      calendar.setTime(date);
      year = calendar.get(java.util.Calendar.YEAR);
    }

    return year;
  }

  /**
   * Returns the number of millisc since ...
   */
  public long getTime() {
    return date.getTime();
  }

  /**
   * Sets the date
   */
  public void setTime(long time) {
    date.setTime(time);
  }

  // !!! TO BE REMOVED
  public java.sql.Date getSqlDate() {
    return date;
  }

  // ----------------------------------------------------------------------
  // TYPE IMPLEMENTATION
  // ----------------------------------------------------------------------

  /**
   * Compares two objects
   */
  public boolean equals(Object other) {
    return (other instanceof Date) &&
      ((Date)other).date.equals(date);
  }

  /**
   * Parse the String arguments and return the corresponding value
   */
  public static NotNullDate valueOf(String val) {
    return new NotNullDate(val);
  }

  /**
   * Today's date
   */
  public static NotNullDate now() {
    return new NotNullDate(System.currentTimeMillis());
  }

  /**
   * Format the object depending on the current language
   * @param	locale	the current language
   */
  public String toString(java.util.Locale locale) {
    return locale == java.util.Locale.GERMAN ?
      german.format(date) :
      english.format(date);
  }

  /**
   * Represents the value in sql
   */
  public String toSql() {
    return sql.format(date);
  }

  // --------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // --------------------------------------------------------------------

  private static SimpleDateFormat	german = new SimpleDateFormat("dd.MM.yyyy");
  private static SimpleDateFormat	english = new SimpleDateFormat("yyyy.MM.dd");
  private static SimpleDateFormat	sql = new SimpleDateFormat("'{d 'yyyy'-'MM'-'dd'}'");
  private static GregorianCalendar	calendar = new GregorianCalendar();

  private java.sql.Date			date;

  // --------------------------------------------------------------------
  // CONSTANTS
  // --------------------------------------------------------------------

  public static final NotNullDate	DEFAULT = new NotNullDate(0);
}
