/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Month.java,v 1.6 1999/08/25 18:04:34 graf Exp $
 */

package at.dms.xtype;

/**
 * This class represents kopi month types
 */
public class Month extends Type {

  /**
   * Constructs a Month from its String representation
   */
  /*package*/ Month(String m) {
    int		index = m.indexOf("-");
    int		year = Integer.parseInt(m.substring(0, index));
    int		month = Integer.parseInt(m.substring(index + 1));

    this.scalar = year * 12 + month - 1;
  }

  /**
   * Constructs a Month with a year and a month in this year
   */
  /*package*/ Month(int year, int month) {
    this.scalar = year * 12 + month - 1;
  }

  /**
   * Constructs a Month with the base date
   * DO NOT USE OUTSIDE OF THE LIBRARIE
   */
  /*package*/ Month(int scalar) {
    this.scalar = scalar;
  }

  /**
   * Constructs a Month from the time in millisc since 1904...
   */
  /*package*/ Month(long millis) {
    this(new Date(millis));
  }

  /**
   * Constructs a Month from a Date
   */
  /*package*/ Month(Date date) {
    this(date.getYear(), date.getMonth());
  }

  /**
   * Constructs a Month from a Date
   */
  /*package*/ Month(java.sql.Date date) {
    this(date.getYear(), date.getMonth() + 1);
  }

  // ----------------------------------------------------------------------
  // DEFAULT OPERATIONS
  // ----------------------------------------------------------------------

  // HANDLED BY COMPILER, FOR NOW !!!!

  /**
   * add
   * @param	m		add m months to this Month
   */
  public NotNullMonth add(int m) {
    return new NotNullMonth(scalar + m);
  }

  /**
   * subtract
   * @returns the number of month between two Months
   */
  public Integer subtract(Month m) {
    return m == null ? null : new Integer(scalar - m.scalar);
  }

  /**
   * subtract
   * @returns the number of month between two Months
   */
  public int subtract(NotNullMonth m) {
    return scalar - ((Month)m).scalar;
  }

  // ----------------------------------------------------------------------
  // OTHER OPERATIONS
  // ----------------------------------------------------------------------

  /**
   * Returns the month number (starts at 1, ends at 12)
   */
  public int getMonth() {
    return scalar % 12 + 1; // month to start at 1
  }

  /**
   * Returns the year of the month (by example 1999 or may be 2000 on year after)
   */
  public int getYear() {
    return scalar / 12;
  }

  /**
   * Transforms this month in a date (the first day of the month)
   */
  public NotNullDate getDate() {
    return new NotNullDate(getYear(), getMonth(), 1);
  }

  // ----------------------------------------------------------------------
  // COMPILER METHODS - DO NOT USE OUTSIDE OF THE LIBRARIE
  // ----------------------------------------------------------------------

  /**
   * Sets the base value for this object
   * DO NOT USE OUTSIDE OF THE LIBRARIE
   */
  public void setScalar(int scalar) {
    this.scalar = scalar;
  }

  /**
   * Returns the base value for this object
   * DO NOT USE OUTSIDE OF THE LIBRARIE
   */
  public int getScalar() {
    return scalar;
  }

  // ----------------------------------------------------------------------
  // TYPE IMPLEMENTATION
  // ----------------------------------------------------------------------

  /**
   * Compares two objects
   */
  public boolean equals(Object other) {
    return (other instanceof Month) &&
      ((Month)other).getScalar() == getScalar();
  }

  /**
   * Parse the String arguments and return the corresponding value
   */
  public static NotNullMonth valueOf(String val) {
    return new NotNullMonth(val);
  }

  /**
   * Today's month
   */
  public static NotNullMonth now() {
    return new NotNullMonth(Date.now());
  }

  /**
   * Format the object depending on the current language
   * @param	locale	the current language
   */
  public String toString(java.util.Locale locale) {
    int year = scalar / 12;
    int	month = scalar % 12 + 1;

    if (locale == java.util.Locale.GERMAN) {
      return (month < 10 ? "0" + month : "" + month) + "." + year;
    } else {
      return year + "." + (month < 10 ? "0" + month : "" + month);
    }
  }

  /**
   * Represents the value in sql
   */
  public String toSql() {
    return "{fn MONTH(" + (getScalar() / 12) + ", " + ((getScalar() % 12) + 1) + ")}";
  }

  // --------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // --------------------------------------------------------------------

  private int scalar;

  // --------------------------------------------------------------------
  // CONSTANTS
  // --------------------------------------------------------------------

  public static final NotNullMonth	DEFAULT = new NotNullMonth(0);
}
