/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: Time.java,v 1.6 1999/08/25 18:04:34 graf Exp $
 */

package at.dms.xtype;

import java.util.GregorianCalendar;
import java.text.SimpleDateFormat;

/**
 * This class represents kopi time types
 */
public class Time extends Type {

  /*protected*/ Time(String m) {
    this(java.sql.Time.valueOf(m));
  }

  /*protected*/ Time(java.sql.Time time) {
    this.time = time;
  }

  /*protected*/ Time(int hours, int minutes) {
    this(new java.sql.Time(hours, minutes, 0));
  }

  /*protected*/ Time(long millis) {
    time = new java.sql.Time(millis);
  }

  // ----------------------------------------------------------------------
  // DEFAULT OPERATIONS
  // ----------------------------------------------------------------------

  // HANDLED BY COMPILER, FOR NOW !!!!

  /**
   * Today's time
   */
  public static NotNullTime now() {
    return new NotNullTime(System.currentTimeMillis());
  }

  public NotNullTime add(int mn) {
    return new NotNullTime(getTime() + mn * 60 *  1000L);
  }

  // ----------------------------------------------------------------------
  // OTHER OPERATIONS
  // ----------------------------------------------------------------------

  /**
   * Returns the month number (starts at 1, ends at 12)
   */
  public int getHours() {
    int		hour;
    synchronized(calendar) {
      calendar.setTime(time);
      hour = calendar.get(java.util.Calendar.HOUR_OF_DAY);
    }

    return hour;
  }

  /**
   * Returns the year of the month (by example 1999 or may be 2000 on year after)
   */
  public int getMinutes() {
    int		minute;
    synchronized(calendar) {
      calendar.setTime(time);
      minute = calendar.get(java.util.Calendar.MINUTE);
    }

    return minute;
  }

  /**
   * Returns the number of millisc since ...
   */
  public long getTime() {
    return time.getTime();
  }

  /**
   * Sets the number of millisc since ...
   */
  public void setTime(long t) {
    time.setTime(t);
  }

  // ----------------------------------------------------------------------
  // TYPE IMPLEMENTATION
  // ----------------------------------------------------------------------

  /**
   * Compares two objects
   */
  public boolean equals(Object other) {
    return (other instanceof Time) &&
      ((Time)other).time.equals(time);
  }

  /**
   * Parse the String arguments and return the corresponding value
   */
  public static NotNullTime valueOf(String val) {
    return new NotNullTime(val);
  }

  /**
   * Format the object depending on the current language
   * @param	locale	the current language
   */
  public String toString(java.util.Locale locale) {
    return locale == java.util.Locale.GERMAN ?
      german.format(time) :
      english.format(time);
  }

  /**
   * Represents the value in sql
   */
  public String toSql() {
    return sql.format(time);
  }

  // --------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // --------------------------------------------------------------------

  private static SimpleDateFormat	german = new SimpleDateFormat("HH:mm");
  private static SimpleDateFormat	english = new SimpleDateFormat("HH:mm");
  private static SimpleDateFormat	sql = new SimpleDateFormat("'{t 'HH':'mm':'ss'}'");
  private static GregorianCalendar	calendar = new GregorianCalendar();

  private java.sql.Time			time;

  // --------------------------------------------------------------------
  // CONSTANTS
  // --------------------------------------------------------------------

  public static final Time	DEFAULT = new Time(0);
}
